using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Windows.Forms;

namespace DynoLib
{
	public delegate void RunsChangedHandler(object sender, RunsChangedArgs args);
	/// <summary>
	/// Summary description for BasicGraph.
	/// </summary>
	public class BasicGraph : System.Windows.Forms.UserControl
	{
		#region Member Data
		public event RunsChangedHandler RunsChanged;

		public static int LINEMODE_CURVE = 0;
		public static int LINEMODE_LINE = 1;
		private static int LINEMODE_MAX = 1;

		protected int lineMode = LINEMODE_CURVE;

		protected ArrayList graphs;

		protected Color[] colors = {Color.Yellow, Color.Blue, Color.White, Color.Red, Color.Green};
		protected DoubleBuffer dbuffer;

		protected GraphScaleData scaledata;

		protected ArrayList minIgnoreData; //graph will not display values UNDER these ignore rules
		protected ArrayList maxIgnoreData; //graph will not display values OVER these ignore rules
		
		#endregion

		#region Properties
		public int LineMode
		{
			get{ return lineMode; }
			set
			{
				lineMode = value;
				if(value > LINEMODE_MAX || value < LINEMODE_CURVE)
					lineMode = LINEMODE_CURVE;
			}
		}

		public ArrayList Runs
		{
			get
			{
				ArrayList l = new ArrayList(graphs.Count+5);
				foreach(object o in graphs)
					l.Add(o);
				return l;
			}
		}
		#endregion

		#region gets/sets
		public void SetAlpha(GraphData run, byte alpha, int ind)
		{
			run.SetAlphaChannel(alpha,ind);
		}

		public void SetGraphShowing(GraphData run, bool show, int ind)
		{
			run.SetGraphShowing(show,ind);
		}
		#endregion

		#region Constructor
		public BasicGraph()
		{
			// This call is required by the Windows.Forms Form Designer.
			InitializeComponent();

			scaledata = new GraphScaleData(new PointF(1000f,20f),new PointF(50f,40f),new PointF(0f,0f),new PointF(20000.0f,10000.0f));

			this.Load +=new EventHandler(BasicGraph_Load);
			this.Resize +=new EventHandler(BasicGraph_Resize);

			graphs = new ArrayList();
			dbuffer = new DoubleBuffer();

			this.SetStyle(ControlStyles.UserPaint & ControlStyles.AllPaintingInWmPaint & ControlStyles.DoubleBuffer, true);

			minIgnoreData = null;
			maxIgnoreData = null;
		}
		#endregion

		#region Utilities

		/*
		protected void addLine(GraphData dat, Point ind, Color c)
		{
			GraphLine gl = new GraphLine(this.scaledata,dat.RunData,ind,c);
			int x = scaledata.Indent.X;
			int wid = this.ClientRectangle.Width - x;
			int hei = this.ClientRectangle.Height - scaledata.Indent.Y;
			gl.Location = new Point(x,0);
			gl.Size = new Size(wid,hei);
			lines.Add(gl);
			this.Controls.Add(gl);
		}
		*/

		public GraphData findRun(string name)
		{
			foreach(GraphData run in this.graphs)
				if(run.Name.ToLower() == name.ToLower())
					return run;

			return null;
		}
		public int findRunIndex(string name)
		{
			for(int i = 0; i<this.graphs.Count; i++)
			{
				GraphData run = (GraphData)graphs[i];
				if(run.Name.ToLower() == name.ToLower())
					return i;
			}

			return -1;
		}
		/// <summary>
		/// Picks a default color for the graph
		/// </summary>
		/// <param name="gr">the GraphData object</param>
		/// <param name="index">the index in the arraylist</param>
		protected void pickColor(GraphData gr, int index)
		{
			if(gr.GraphColor == Color.Empty)
				gr.GraphColor = colors[index%colors.Length];
		}

		public void addIgnoreRule(IgnoreRule rule, bool min)
		{
			if(min)
			{
				if(minIgnoreData == null)
					minIgnoreData = new ArrayList();
				minIgnoreData.Add(rule);
			}
			else
			{
				if(maxIgnoreData == null)
					maxIgnoreData = new ArrayList();
				maxIgnoreData.Add(rule);
			}
		}
		public void removeIgnoreRule(IgnoreRule rule, bool min)
		{
			if(min && minIgnoreData != null)
				minIgnoreData.Remove(rule);
			else if(!min && maxIgnoreData != null)
				maxIgnoreData.Add(rule);
		}
		public IgnoreRule getIgnoreRule(int index, bool min)
		{
			ArrayList list = maxIgnoreData;
			if(min)
				list = minIgnoreData;

			if(list == null || index>= list.Count)
				return null;

			return (IgnoreRule)list[index];
		}
		//returns true if the item passes the rules.
		public bool checkIgnoreRules(GraphItem item)
		{
			bool pass = true;
			for(int i = 0; pass && minIgnoreData != null && i<minIgnoreData.Count; i++)
			{
				IgnoreRule r = this.getIgnoreRule(i,true);
				if(r != null && r.Index < item.Data.Length && item.Data[r.Index] < r.Value)
					pass = false;
			}

			for(int j = 0; pass && maxIgnoreData != null && j<maxIgnoreData.Count; j++)
			{
				IgnoreRule r = this.getIgnoreRule(j,false);
				if(r != null && r.Index < item.Data.Length && item.Data[r.Index] > r.Value )
					pass = false;
			}
			return pass;
		}
		#endregion

		#region Events
		protected void OnRunsChanged(GraphData run, bool runAdded)
		{
			if(RunsChanged != null)
				RunsChanged(run, new RunsChangedArgs(runAdded));
		}

		private void BasicGraph_Load(object sender, EventArgs e)
		{
			if(!dbuffer.CanDoubleBuffer())
				dbuffer.CreateDoubleBuffer(null, this.ClientRectangle.Width, this.ClientRectangle.Height, true);
		}

		private void BasicGraph_Resize(object sender, EventArgs e)
		{
			dbuffer.CreateDoubleBuffer(null, this.ClientRectangle.Width, this.ClientRectangle.Height, true);
			Invalidate(); // Force a repaint after has been resized 
		}
		#endregion

		#region Useless Crap
		/// <summary> 
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;
		/// <summary> 
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}
		#endregion

		#region Component Designer generated code
		/// <summary> 
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			components = new System.ComponentModel.Container();
		}
		#endregion
	}
	#region RunsChangedArgs
	public class RunsChangedArgs : EventArgs
	{
		bool added;

		public RunsChangedArgs(bool runAdded)
		{
			added = runAdded;
		}

		public bool RunAdded
		{
			get{return added;}
		}
	}
	#endregion
	#region IgnoreRule
	public class IgnoreRule
	{
		private int m_index;
		private float m_val;

		public IgnoreRule(int index, float val)
		{
			m_index = index;
			m_val = val;
		}

		public int Index
		{
			get{return m_index;}
			set{m_index = value;}
		}
		public float Value
		{
			get{return m_val;}
			set{m_val = value;}
		}
	}
	#endregion
}
