using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Windows.Forms;

namespace DynoLib
{
	/// <summary>
	/// Just a simple class that draws a curve or line.
	/// </summary>
	public class GraphLine : System.Windows.Forms.UserControl
	{
		#region Member Data

		public static int LINEMODE_CURVE = 0;
		public static int LINEMODE_LINE = 1;
		private static int LINEMODE_MAX = 1;
		/// So we know the graph scaling
		protected GraphScaleData scaledata;
		private Point index;
		private float [,] data;
        private int lineMode = 0;

		private Color color;

		#endregion

		#region Properties
		public Point Index
		{
			get{ return index; }
		}
		public Color LineColor
		{
			get{ return color; }
			set{ color = value; }
		}
		public float[,] Data
		{
			get{ return data; }
			set
			{
				if(value != null)
					data = value;
			}
		}
		public int LineMode
		{
			get{ return lineMode; }
			set
			{
				lineMode = value;
				if(value > LINEMODE_MAX || value < LINEMODE_CURVE)
					lineMode = LINEMODE_CURVE;
			}
		}
		#endregion

		public GraphLine(GraphScaleData scaledat, float[,] dat, Point ind, Color c)
		{
			InitializeComponent();

			index = ind;

			scaledata = scaledat;
			color = c;

			if(dat == null)
				throw new NullReferenceException("Line data cannot be null.");

			data = dat;
			this.SetStyle(ControlStyles.SupportsTransparentBackColor,true);
		}

		#region Painting
	//	protected override void OnPaintBackground(PaintEventArgs pe)
	//	{
	//		pe.Graphics.FillRectangle(new SolidBrush(Color.Black), pe.ClipRectangle.X, pe.ClipRectangle.Y, pe.ClipRectangle.Width, pe.ClipRectangle.Height);
	//	}
		protected override void OnPaint( PaintEventArgs pe )
		{
			Pen p = new Pen(color,2F);
            DrawGraphLine(pe.Graphics,p);
			p.Dispose();
		}
		private void DrawGraphLine(Graphics g, Pen p)
		{
			int max = data.GetUpperBound(1)+1;
			PointF[] points = new PointF[max];
			PointF[] points2 = new PointF[max];
			for(int i = 0; i< max; i++)
			{
				points[i] = TranslatePoint(data[index.X,i],data[index.Y,i],true);
				points2[i] = TranslatePoint(data[index.X,i],data[index.Y,i],false);
			}

			if(lineMode == LINEMODE_LINE)
				DrawLine(g,p,points);
			else
			{
				System.Drawing.Drawing2D.GraphicsPath shape = new System.Drawing.Drawing2D.GraphicsPath();
				shape.AddCurve(points);
				shape.AddCurve(points2);
		//		shape.CloseFigure();
		//		shape.AddClosedCurve(points);
				this.Region = new Region(shape);
				this.BackColor = Color.Transparent;//color;
				g.DrawCurve(p,points);
			}
		}

		private PointF TranslatePoint(float x, float y, bool left)
		{
			float off = 2f;
			if(left)
				off = -2f;
			float xx = (float)(/*xAxisIndent +*/ (this.ClientRectangle.Width - scaledata.Indent.X) * x / scaledata.Max.X) + off;
			float yy = (float)this.ClientRectangle.Bottom-(/*yAxisIndent +*/ (this.ClientRectangle.Height - scaledata.Indent.Y) * y / scaledata.Max.Y) + off;

			return new PointF(xx,yy);
		}

		private void DrawLine(Graphics g, Pen p, PointF[] points)
		{
			PointF lastp = points[0];//TranslatePoint(0F,0F);

			foreach(PointF po in points)
			{
				g.DrawLine(p,lastp,po);
				lastp = po;
			}
		}
		#endregion

		#region VStudio Stuff
		/// <summary> 
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

		/// <summary> 
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}
		#endregion

		#region Component Designer generated code
		/// <summary> 
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			components = new System.ComponentModel.Container();
		}
		#endregion
	}
}
