using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Drawing.Text;
using System.Drawing.Drawing2D;
using System.Data;
using System.Windows.Forms;

namespace Gauge
{
	/// <summary>
	/// Summary description for UserControl1.
	/// </summary>
	public class Gauge : System.Windows.Forms.UserControl
	{

		float width = 200.0F;
		float height = 200.0F;
		
		float maxvalue = 100.0F;
		float minvalue = 0.0F;
		float currentvalue = 0.0F;
		int divisions = 10;//.0F;
		float minangle = -35.0F;
		float maxangle = 215.0F;
		int numinterlines = 5;
		string title = "Title";
		string units = "Units";

		Bitmap bmp;

		int numdec = 0;

		Color gaugefacecolor = Color.Black;
		Color gaugenumcolor = Color.White;
		Color titlecolor = Color.Red;
		Color unitscolor = Color.White;
		Color needlecolor = Color.Orange;

		#region Properties

		/// <summary>
		/// Size of the Gauge. They must be square. 
		/// </summary>
		[Category("Layout")]
		public Size GaugeSize
		{
			get{return new Size((int)width,(int)width);}
			set
			{
				width = value.Width;
				height = width;

				this.Size = new Size((int)width,(int)width);
			}
		}
		
		/// <summary>
		/// Max value of the gauge
		/// </summary>
		[Category("Layout")]
		public float MaxValue
		{
			get{return maxvalue;}
			set
			{
				maxvalue = value;
				drawFace();
				Refresh();
			}
		}

		/// <summary>
		/// Number of Decimals to display in the units. i.e. 2 would make the units:
		/// 1.00, 2.00, etc
		/// </summary>
		[Category("Layout")]
		public int NumberOfDecimals
		{
			get{return numdec;}
			set
			{
				numdec = value;
				drawFace();
				Refresh();
			}
		}

		/// <summary>
		/// Minimum value. Usually 0.
		/// </summary>
		[Category("Layout")]
		public float MinValue
		{
			get{return minvalue;}
			set
			{
				minvalue = value;
				drawFace();
				Refresh();
			}
		}

		/// <summary>
		/// The value of the gauge. This refreshes the gauge when set.
		/// </summary>
		[Category("Layout")]
		public float CurrentValue
		{
			get{return currentvalue;}
			set
			{
				currentvalue = value;
				this.Refresh();
			}
		}

		/// <summary>
		/// String for the title of the gauge
		/// </summary>
		[Category("Layout")]
		public string Title
		{
			get{return title;}
			set
			{
				title = value;
				drawFace();
				Refresh();
			}
		}

		/// <summary>
		/// string for the units of the gauge
		/// </summary>
		[Category("Layout")]
		public string Units
		{
			get{return units;}
			set
			{
				units = value;
				drawFace();
				Refresh();
			}
		}
		
		/// <summary>
		/// Number of major headings.
		/// </summary>
		[Category("Layout")]
		public int NumberDivisions
		{
			get{return divisions;}
			set
			{
				divisions = value;
				drawFace();
				Refresh();
			}
		}

		/// <summary>
		/// Number of 'ticks' between each division.
		/// </summary>
		[Category("Layout")]
		public int NumberIntermediateLines
		{
			get{return numinterlines;}
			set
			{
				numinterlines = value;
				drawFace();
				Refresh();
			}
		}
		#endregion

		public Gauge()
		{
			InitializeComponent();

			SetStyle(ControlStyles.UserPaint | 
				ControlStyles.AllPaintingInWmPaint |
				ControlStyles.DoubleBuffer |
				ControlStyles.SupportsTransparentBackColor, true);

			drawFace();
		}

		protected void drawFace()
		{
			bmp = new Bitmap((int)this.width,(int)this.height);

			Graphics g = Graphics.FromImage(bmp);
			g.SmoothingMode = SmoothingMode.AntiAlias;
			g.TextRenderingHint = TextRenderingHint.AntiAlias;

			g.Clear(Color.Transparent);

			using (Brush backgroundBrush = new SolidBrush(gaugefacecolor),
					   titleBrush = new SolidBrush(titlecolor),
					   unitsBrush = new SolidBrush(unitscolor))
			{

				//draw the words 
				g.FillEllipse(backgroundBrush,0,0,(float)width,(float)width);
				System.Drawing.Drawing2D.GraphicsPath gp = new GraphicsPath();
				gp.AddEllipse(0,0,width+1,height+1);
				this.Region = new Region(gp);
				
				StringFormat sf = new StringFormat();
				Font f = new System.Drawing.Font("Impact", (float)(width*0.08), 
					System.Drawing.FontStyle.Italic, System.Drawing.GraphicsUnit.Point, 
					((System.Byte)(0)));
				
				float x = (width/2.0F)-(g.MeasureString(title,f).Width/2.0F)-10.0F;
				float fontlength = g.MeasureString(title,f).Width;
				float y = (height*.7F);
				
				RectangleF rectf = new RectangleF((float)x,(float)y,(float)fontlength,f.GetHeight(g)+2);
				g.DrawString(title,f,titleBrush,rectf,sf);

				//draw the units
				y = y + f.GetHeight(g);// + (this.Font.GetHeight(g)/2.0);

				
				f = new System.Drawing.Font("Impact", (float)(width*0.05F), 
					System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, 
					((System.Byte)(0)));

				x = (width/2.0F)-(g.MeasureString(units,f).Width/2.0F)+10.0F;
				fontlength = g.MeasureString(units,f).Width;

				rectf = new RectangleF((float)x,(float)y,(float)fontlength,f.GetHeight(g)+2.0F);
				g.DrawString(units,f,unitsBrush,rectf,sf);


				
			}
			using (Pen pen = new Pen(gaugenumcolor,1))
			{
				//draw the divisions

				Brush unitsBrush = new SolidBrush(unitscolor);

				float linelength = width*0.075F;
				float interlinelength = width*0.04F;

				float deltaangle = (maxangle-minangle)/(float)divisions;
				
				Font f = new System.Drawing.Font("Impact", (float)(width*0.065F), 
					System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, 
					((System.Byte)(0)));

				float midwidth = width/2.0F;
				for(int i = divisions; i>=0; i--)
				{
					//draw main line
					float radians = (float)((minangle+(deltaangle*i))*Math.PI)/180.0F;
					float x1 = midwidth + (float)Math.Cos(radians)*(midwidth-linelength);
					float y1 = midwidth - (float)Math.Sin(radians)*(midwidth-linelength);

					float x2 = midwidth + (float)Math.Cos(radians)*(midwidth);
					float y2 = midwidth - (float)Math.Sin(radians)*(midwidth);

					g.DrawLine(pen,(float)x1,(float)y1,(float)x2,(float)y2);

					//draw intermediate lines 
					for(int j = 0; j<numinterlines && i != divisions; j++)
					{
						radians = ((minangle+(deltaangle*i) + (deltaangle/(numinterlines+1))*(float)(j+1))*(float)Math.PI)/180.0F;
						float xl1 = midwidth + (float)Math.Cos(radians)*(midwidth-interlinelength);
						float yl1 = midwidth - (float)Math.Sin(radians)*(midwidth-interlinelength);

						float xl2 = midwidth + (float)Math.Cos(radians)*(midwidth);
						float yl2 = midwidth - (float)Math.Sin(radians)*(midwidth);

						g.DrawLine(pen,(float)xl1,(float)yl1,(float)xl2,(float)yl2);
					}
					
					float percenty = 1.0F-(y1-y2)/linelength;

					float yunit = y1 - (f.GetHeight(g)/2.0F)*percenty;//y2+ y1-y2;

					//if(y2>=midwidth)
					//	yunit = y1 - (y2-y1);
					float xunit;
					
					string unit;
					float val;
					if(numdec>0)
					{
						string zeros = "0.";
						for(int j = 0; j<numdec; j++)
							zeros += "0";
						val = (maxvalue-minvalue)/(float)divisions;
						val = (float)(divisions-i)*val + minvalue;
						unit = val.ToString(zeros);
					}
					else
					{
						val = (maxvalue-minvalue)/(float)divisions;
						val = (float)(divisions-i)*val + minvalue;
						unit = val.ToString();
						//unit = ((int)(((divisions-i)/divisions)*maxvalue)).ToString();
					}

					SizeF sz = g.MeasureString(unit,f);
					float percentx = (x1-x2)/linelength;

					xunit = x1 - (sz.Width/2.0F) + percentx*(sz.Width/2.0F);

					g.DrawString(unit,f,unitsBrush,(float)xunit,(float)yunit);
				}//end for
			}//end using

			//paint trim ring thing
			Pen trimbrush = new Pen(Color.DarkGray,2);
			g.DrawEllipse(trimbrush,0,0,(float)width,(float)width);

		}

		protected override void OnPaint(PaintEventArgs e)
		{
			Graphics g = e.Graphics;
			g.SmoothingMode = SmoothingMode.AntiAlias;

			g.DrawImage(bmp,(float)0.0,(float)0.0,(float)this.width,(float)this.height);

			float midwidth = width/2.0F;

			using (Pen pen = new Pen(gaugenumcolor,1))
			{

				Brush needlep = new SolidBrush(needlecolor);
				
				float needlea = (maxangle)-((maxangle-minangle)*((currentvalue-minvalue)/(maxvalue-minvalue)));
				
				float rad = (needlea*(float)Math.PI)/180.0F;
				float cos = (float)Math.Cos(rad);
				float sin = (float)Math.Sin(rad);

				float xn = midwidth + cos*(midwidth+4);
				float yn = midwidth - sin*(midwidth+4);

				Point[] p = new Point[3];
				p[0] = new Point((int)xn,(int)yn);

				rad = ((needlea-90.0F)*(float)Math.PI)/180.0F;

				float xh = midwidth + sin*(2.5F);
				float yh = midwidth + cos*(2.5F);

				p[1] = new Point((int)xh,(int)yh);
				
				rad = ((needlea+90.0F)*(float)Math.PI)/180.0F;
				xh = midwidth - sin*(2.5F);
				yh = midwidth - cos*(2.5F);

				p[2] = new Point((int)xh,(int)yh);

				g.FillPolygon(needlep,p);
			}//end using		
				
		}//end paint
	
		#region Forms crap
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;
		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if( components != null )
					components.Dispose();
			}
			base.Dispose( disposing );
		}
		#endregion

		#region Component Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			// 
			// Gauge
			// 
			this.Name = "Gauge";

		}
		#endregion
	}
}
