using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using BGraphLib;

namespace GraphTest
{
	/// <summary>
	/// Summary description for Form1.
	/// </summary>
	public class Form1 : System.Windows.Forms.Form
	{
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

		private double[,] fuelMap = {
			{1.48, 1.82, 2.15, 2.77, 3.31, 3.85, 4.35, 4.85, 5.31, 5.77, 6.31, 6.85, 8.15, 9.46, 10.77},
			{1.48, 1.82, 2.15, 2.77, 3.31, 3.85, 4.35, 4.85, 5.31, 5.77, 6.31, 6.85, 8.15, 9.46, 10.77},
			{1.49, 1.84, 2.18, 2.83, 3.33, 3.88, 4.38, 4.88, 5.42, 6.04, 6.46, 7.00, 8.38, 9.62, 10.85},
			{1.57, 1.97, 2.38, 2.88, 3.46, 4.00, 4.54, 5.04, 5.54, 6.04, 6.62, 7.15, 8.31, 9.62, 10.92},
			{1.61, 2.05, 2.49, 2.98, 3.52, 4.06, 4.54, 5.08, 5.58, 6.08, 6.62, 7.15, 8.46, 9.62, 10.77},
			{1.62, 2.07, 2.52, 3.12, 3.67, 4.21, 4.77, 5.31, 5.81, 6.35, 6.92, 7.38, 8.54, 9.69, 10.85},
			{1.63, 2.10, 2.56, 3.15, 3.69, 4.21, 4.81, 5.27, 5.85, 6.38, 7.00, 7.46, 8.69, 9.92, 11.15},
			{1.67, 2.15, 2.65, 3.29, 3.88, 4.42, 5.04, 5.58, 6.12, 6.69, 7.27, 7.77, 8.92, 10.23, 11.54},
			{1.68, 2.18, 2.69, 3.25, 3.79, 4.42, 5.00, 5.58, 6.12, 6.69, 7.27, 7.92, 9.08, 10.38, 11.69},
			{1.67, 2.17, 2.67, 3.23, 3.81, 4.37, 5.00, 5.58, 6.15, 6.73, 7.35, 7.85, 9.08, 10.46, 11.85},
			{1.71, 2.24, 2.77, 3.35, 3.98, 4.67, 5.27, 5.88, 6.42, 7.04, 7.62, 8.15, 9.46, 10.77, 12.08},
			{1.71, 2.24, 2.78, 3.40, 4.08, 4.67, 5.31, 5.96, 6.58, 7.19, 7.77, 8.38, 9.69, 11.00, 12.31},
			{1.68, 2.18, 2.69, 3.25, 3.87, 4.46, 5.08, 5.77, 6.38, 6.96, 7.69, 8.31, 9.69, 10.77, 11.85},
			{1.72, 2.26, 2.80, 3.44, 4.06, 4.73, 5.35, 6.04, 6.73, 7.31, 8.12, 9.00, 10.69, 11.31, 11.92},
			{1.84, 2.47, 3.12, 3.79, 4.50, 5.13, 5.92, 6.88, 7.65, 8.42, 9.15, 9.77, 11.38, 12.54, 13.69},
			{1.86, 2.51, 3.16, 3.87, 4.62, 5.38, 6.08, 6.88, 7.73, 8.46, 9.19, 9.77, 11.15, 12.54, 13.92},
			{1.79, 2.39, 3.00, 3.83, 4.63, 5.37, 6.27, 7.12, 7.85, 8.58, 9.31, 9.85, 11.46, 12.23, 13.00}
		};

		private int[] rpmList = {500,600,700,800,900,1100,1300,1600,1900,2200,2600,3100,3700,4400,5300,6300,7400};
		private string[] pressureList = {"-26.0","-24.0","-22.0","-21.0","19.0","18.0","16.0","14.0","13.0","11.0","9.0","8.0","4.0","1.0","1.3"};

		private MapGraphControl mapPlot;

		public Form1()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			this.Resize += new EventHandler(Form1_Resize);
		
			mapPlot = new MapGraphControl();
			mapPlot.setSize(12,12);

			for(int i = 0;i<12;i++)
			{			
				mapPlot.setRow(i,rpmList[i]);
				for(int j = 0;j<12;j++)
				{
					mapPlot.setValue(i,j,fuelMap[j,i]);
				}

			}
			mapPlot.setupHeaders();

//			mapPlot.Map = fuelMap;
//			mapPlot.RpmList = rpmList;
			mapPlot.PressureList = pressureList;
			mapPlot.Width = this.ClientRectangle.Width;
			mapPlot.Height = this.ClientRectangle.Height;
			mapPlot.NodeMoved += new BGraphLib.MapGraphControl.UpdateHandler(mapPlot_NodeMoved);

			this.Controls.Add(mapPlot);
			
		}		

		private void Form1_Resize(object sender, EventArgs e)
		{
			mapPlot.Width = this.ClientRectangle.Width;
			mapPlot.Height = this.ClientRectangle.Height;
			mapPlot.Invalidate();
			
		}

		private void mapPlot_NodeMoved(object sender, GraphNodeDragEventArgs e)
		{
			fuelMap[e.X, e.Y] = e.Value;
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}


		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			// 
			// Form1
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.BackColor = System.Drawing.Color.Black;
			this.ClientSize = new System.Drawing.Size(792, 373);
			this.ForeColor = System.Drawing.Color.White;
			this.Name = "Form1";
			this.Text = "Form1";

		}
		#endregion

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main() 
		{
			Application.Run(new Form1());
		}
	}
}
