#region Licence...
//-----------------------------------------------------------------------------
// Date:	17/10/04    Time: 2:33p
// Module:	AssemblyExecutor.cs
// Classes:	AssemblyExecutor
//			RemoteExecutor
//
// This module contains the definition of the AssemblyExecutor class. Which implements 
// executing 'public static void Main(..)' method of a assembly in a different AddDomain
//
// Written by Oleg Shilo (oshilo@gmail.com)
// Copyright (c) 2004. All rights reserved.
//
// Redistribution and use of this code in source and binary forms, with or without 
// modification, are permitted provided that the following conditions are met:
// 1. Redistributions of source code must retain the above copyright notice, 
//    this list of conditions and the following disclaimer. 
// 2. Neither the name of an author nor the names of the contributors may be used 
//    to endorse or promote products derived from this software without specific 
//    prior written permission.
// 3. This code may be used in compiled form in any way you desire. This
//	  file may be redistributed unmodified by any means PROVIDING it is 
//    not sold for profit without the authors written consent, and 
//    providing that this notice and the authors name is included. 
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
// TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
// PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
// LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
//	Caution: Bugs are expected!
//----------------------------------------------
#endregion 

using System;
using System.IO;
using Microsoft.CSharp;
using System.CodeDom.Compiler;
using System.Reflection;
using System.Security.Policy;
using System.Collections;
using System.Threading;
using System.Windows.Forms;
using System.Text;

namespace csscript
{
	/// <summary>
	/// Executes "public static void Main(..)" of assembly in a separate domain.
	/// </summary>
	public class AssemblyExecutor
	{
		AppDomain appDomain;
		RemoteExecutor remoteExecutor;
		string assemblyFileName;
		public AssemblyExecutor(string fileNname, string domainName)
		{
			assemblyFileName = fileNname;
			AppDomainSetup setup = new AppDomainSetup();
			setup.ApplicationBase = Path.GetDirectoryName(assemblyFileName);
			setup.PrivateBinPath = AppDomain.CurrentDomain.BaseDirectory;
			setup.ApplicationName = Path.GetFileName(Assembly.GetExecutingAssembly().Location);
			setup.ShadowCopyFiles = "true";
			setup.ShadowCopyDirectories = Path.GetDirectoryName(assemblyFileName);
			appDomain = AppDomain.CreateDomain(domainName, null, setup);
	
			remoteExecutor = (RemoteExecutor)appDomain.CreateInstanceFromAndUnwrap(Assembly.GetExecutingAssembly().Location, typeof(RemoteExecutor).ToString());
//			remoteExecutor = (RemoteExecutor)AppDomain.CurrentDomain.CreateInstanceFromAndUnwrap(Assembly.GetExecutingAssembly().Location, typeof(RemoteExecutor).ToString());

		}

		public AssemblyExecutor()
		{
	
//			remoteExecutor = (RemoteExecutor)appDomain.CreateInstanceFromAndUnwrap(Assembly.GetExecutingAssembly().Location, typeof(RemoteExecutor).ToString());
			remoteExecutor = (RemoteExecutor)AppDomain.CurrentDomain.CreateInstanceFromAndUnwrap(Assembly.GetExecutingAssembly().Location, typeof(RemoteExecutor).ToString());

		}

		public AppDomain GetAppDomain()
		{
			return appDomain;
		}

		/// <summary>
		/// invokes some method
		/// </summary>
		/// <param name="methodName">name of method</param>
		/// <param name="args">the arguments</param></param>
		public void Execute(Assembly asm, string methodName, object[] args)
		{
		//	remoteExecutor.ExecuteAssembly(asm,methodName, args);
		}

		/// <summary>
		/// invokes ScriptMain
		/// </summary>
		/// <param name="methodName">name of method</param>
		/// <param name="args">the arguments</param></param>
		public void ExecuteMain(Assembly asm, object[] args)
		{
			remoteExecutor.ExecuteAssemblyScriptMain(asm, args);
		}

		public void Execute(string methodName, object[] args)
		{
			if(assemblyFileName!= null)
				remoteExecutor.ExecuteAssembly(assemblyFileName,methodName, args);
		}

		public void ExecuteMain(object[] args)
		{
			if(assemblyFileName!= null)
				remoteExecutor.ExecuteAssemblyScriptMain(assemblyFileName, args);
		}

		public void Unload()
		{
			AppDomain.Unload(appDomain);
			appDomain = null;
		}
	}

	public class ScriptObject
	{
		object parent;
		object obj;
		public ScriptObject(object uniqueParent, object ob)
		{
			parent = uniqueParent;
			obj = ob;
		}

		public object Parent
		{
			get{return parent;}
		}

		public object Object
		{
			get{return obj;}
		}


	}
	/// <summary>
	/// Invokes static method 'Main' from the assembly.
	/// </summary>
	public class RemoteExecutor: MarshalByRefObject
	{
		string workingDir;
		Assembly asm;
		ArrayList list;

		Type type;

	//	bool hasResoleHandler;
		/// <summary>
		/// AppDomain evant handler. This handler will be called if CLR cannot resolve 
		/// referenced local assemblies 
		/// </summary>
		public Assembly ResolveEventHandler(object sender, ResolveEventArgs args)
		{
			return AssemblyResolver.ResolveAssembly(args.Name, workingDir);

		}

		public string getType()
		{
			return type.FullName;
		}

		/// <summary>
		/// This will be called really before any objects are made.
		/// </summary>
		/// <param name="fname"></param>
		public void LoadAssembly(string fname)
		{
			AppDomain.CurrentDomain.AssemblyResolve += new ResolveEventHandler(ResolveEventHandler);
			
			list = new ArrayList(12); //for the objects
			workingDir = Path.GetDirectoryName(fname);
			asm = Assembly.LoadFile(fname);
		}

		/// <summary>
		/// This will call the constructor with no args and ScriptMain with some args
		/// </summary>
		/// <param name="bindingFlags"></param>
		/// <param name="constructorParams"></param>
		/// <returns></returns>
		public void CreateScriptInstance(object uniqueParent, object[] args)
		{
			type = GetTypeWithMethod(asm,"ScriptMain");

			if (asm == null)
			{
				throw new InvalidOperationException("Could not find" + 
					" owning assembly for type " + type.FullName);
			}
		//	MarshalByRefObject createdInstance = 
		//		asm.CreateInstance(type.FullName)
			//	asm.CreateInstance(type.FullName,
			//	false, bindingFlags, null,null, null, null) 
		//		as MarshalByRefObject;

			object createdInstance = 
				asm.CreateInstance(type.FullName);

			if (createdInstance == null)
			{
				throw new ArgumentException("createdInstance == null");
			}

			list.Add(new ScriptObject(uniqueParent,createdInstance));
			ExecuteAssembly(createdInstance, "ScriptMain",args);
		}

		public static Type GetTypeWithMethod(Assembly asm, string methodName)
		{
			MethodInfo method = null;
			Type type = null;
			foreach (Module m in asm.GetModules())
			{
				foreach (Type t in m.GetTypes())
				{
					BindingFlags bf = BindingFlags.Public | BindingFlags.NonPublic | BindingFlags.InvokeMethod | BindingFlags.Instance ;
					foreach (MemberInfo mi in t.GetMembers(bf))
					{
						if (mi.Name == methodName)
						{
							type = t;
							method = t.GetMethod(methodName, bf);
							break;
						}
						//	if (method != null)
						//	{
						//		type = t;
						//		break;
						//	}
					}
					if (method != null)
						break;
				}
				if (method != null)
					break;
			}
			
			return type;

		}

		#region execute
		/// <summary>
		/// Executes a NON-STATIC method in the script. 
		/// The object must have been created with CreateScriptInstance().
		/// </summary>
		/// <param name="parent">unique parent for the object</param>
		/// <param name="method">Name of method</param>
		/// <param name="args">argument array</param>
		/// <returns>Returns whatever object the function returns</returns>
		public object ExecuteMethod(object parent, string method, object[] args)
		{
			object obj = null; 
			foreach(object so in list)
			{
				if(((ScriptObject)so).Parent == parent)
				{
					obj = ((ScriptObject)so).Object;
				}
			}

			if(obj == null)
				throw new Exception("object with parent not found");

			return ExecuteAssembly(obj,method,args);
		}

		//for the scripting
		public object ExecuteAssembly(object asm, string method, object[] args)
		{
			return type.InvokeMember(method,BindingFlags.InvokeMethod,null,asm,args);
		}

		public void ExecuteAssembly(string filename, string method, object[] args)
		{
//			workingDir = Path.GetDirectoryName(filename);
//			AppDomain.CurrentDomain.AssemblyResolve += new ResolveEventHandler(ResolveEventHandler);
//			Assembly assembly = Assembly.LoadFrom(filename);
			InvokeMethod(method, args);
		}

		public void ExecuteAssemblyScriptMain(Assembly asm, object[] args)
		{
			InvokeStaticMain(asm, args);
		}

		public void ExecuteAssemblyScriptMain(string filename, object[] args)
		{
			workingDir = Path.GetDirectoryName(filename);
			AppDomain.CurrentDomain.AssemblyResolve += new ResolveEventHandler(ResolveEventHandler);
			Assembly assembly = Assembly.LoadFrom(filename);
			InvokeStaticMain(assembly, args);
		}
		#endregion
	
		#region invoke methods
		public void InvokeStaticMain(Assembly compiledAssembly, object[] scriptArgs)
		{
			InvokeMethod("ScriptMain", scriptArgs);
			
		}//end invoke static main

		public object InvokeMethod(string methodName, object[] scriptArgs)
		{
			Assembly compiledAssembly = asm;

			MethodInfo method = null;
			foreach (Module m in compiledAssembly.GetModules())
			{
				foreach (Type t in m.GetTypes())
				{
					BindingFlags bf = BindingFlags.Public | BindingFlags.NonPublic | BindingFlags.InvokeMethod | BindingFlags.Static ;
					foreach (MemberInfo mi in t.GetMembers(bf))
					{
						if (mi.Name == methodName)
						{
							method = t.GetMethod(mi.Name, bf);
						}
						if (method != null)
							break;
					}
					if (method != null)
						break;
				}
				if (method != null)
					break;
			}
			if (method != null)
			{
				if (method.GetParameters().Length != 0)
				{
					return method.Invoke( new object(), scriptArgs);
				}
				else
				{
					return method.Invoke( new object(), null);
				}
			}
			else
			{
				throw new Exception("Cannot find function. Make sure script file contains method: 'public static "+methodName+"(...)'"); 
			}
		}//end invokeMethod
		#endregion

	}//end remoteExecutor
}//end namespace
			
