#region Licence...
//-----------------------------------------------------------------------------
// Date:	10/11/04    Time: 3:00p
// Module:	CSScriptLib.cs
// Classes:	CSScript
//			AppInfo
//
// This module contains the definition of the CSScript class. Which implements 
// compiling C# script engine (CSExecutor). Can be used for hosting C# script engine
// from any CLR application
//
// Written by Oleg Shilo (oshilo@gmail.com)
// Copyright (c) 2004. All rights reserved.
//
// Redistribution and use of this code in source and binary forms, with or without 
// modification, are permitted provided that the following conditions are met:
// 1. Redistributions of source code must retain the above copyright notice, 
//    this list of conditions and the following disclaimer. 
// 2. Neither the name of an author nor the names of the contributors may be used 
//    to endorse or promote products derived from this software without specific 
//    prior written permission.
// 3. This code may be used in compiled form in any way you desire. This
//	  file may be redistributed unmodified by any means PROVIDING it is 
//    not sold for profit without the authors written consent, and 
//    providing that this notice and the authors name is included. 
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
// TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
// PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
// LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
//	Caution: Bugs are expected!
//----------------------------------------------
#endregion

using System;
using System.IO;
using System.Windows.Forms;
using System.Reflection;
using System.Diagnostics;
using System.Collections;

namespace CSScriptLibrary
{
	public delegate void PrintDelegate(string msg);
	/// <summary>
	/// Class CSScript which is implements class library for CSExecutor.
	/// </summary>
	public class CSScript
	{
		#region members
		ArrayList list;
		public ArrayList Scripts
		{
			get{return list;}
		}

		static bool rethrow;
		/// <summary>
		/// Force caught exceptions to be rethrown.
		/// </summary>
		static public bool Rethrow
		{
			get {return rethrow;}
			set {rethrow = value;}
		}
		#endregion 

		public CSScript()
		{
			rethrow = false;
			list = new ArrayList(12);

		}

		#region utils

		public void Unload(string scriptPath)//, Form[] parents)
		{
			foreach(object o in list)
			{
				if(((Script)o).ScriptPath.ToLower() == scriptPath.ToLower())
				{
					((Script)o).unload();//(parents);
					list.Remove(o);
					break;
				}
			}

		}

		public static string getType(string fullMethodName)
		{
			string s = fullMethodName.Substring(0,fullMethodName.LastIndexOf('.'));
			return s;
		}

		public static string getMethod(string fullMethodName)
		{
			return fullMethodName.Substring(fullMethodName.LastIndexOf('.')+1);
		}

		public Script getScript(string fullMethodName)
		{
			string type = getType(fullMethodName);
			foreach(object o in list)
			{
				if(((Script)o).Type == type)
					return ((Script)o);
			}

			return null;
		}
		#endregion

		#region creation
		/// <summary>
		/// finds an already created script & loads it if found. else it compiles and loads.
		/// </summary>
		/// <param name="scriptPath"></param>
		/// <returns></returns>
		public Script CompileAndLoad(string scriptPath)//, Form[] parent)
		{
			try
			{
				int i;
				Script script = null;
				for( i = 0; i<list.Count; i++)
					if( ((Script)list[i]).ScriptPath == scriptPath )
					{
						script = (Script)list[i];
						list.RemoveAt(i);
						script.unload();//(parent);
					}
				
				script = null;
				string compiledScript = csscript.CSExecutor.GetAvailableAssembly(scriptPath);
				
				Random rand = new Random();
				script = CreateAppDomain(Path.GetFileName(scriptPath) + rand.Next(999).ToString());
				script.ScriptPath = scriptPath;
			
				if(compiledScript == null)
				{
					compiledScript = Compile(scriptPath,scriptPath+"c",false);

				}
			//	else
			//		script.ScriptAssembly = Load(compiledScript);

				script.AssemblyPath = compiledScript;

				list.Add(script);

				return script;
			}
			catch(Exception e)
			{
				MessageBox.Show(e.Message + "\n\n" + e.StackTrace);
				return null;
			}
		}

		/// <summary>
		/// Compiles script into assembly with CSExecutor
		/// </summary>
		/// <param name="scriptFile">The name of script file to be compiled.</param>
		/// <param name="assemblyFile">The name of compiled assembly. If set to null a temnporary file name will be used.</param>
		/// <param name="debugBuild">true if debug information should be included in assembly; otherwise, false.</param>
		/// <returns></returns>
		static protected string Compile(string scriptFile, string assemblyFile, bool debugBuild)
		{
			csscript.CSExecutor exec = new csscript.CSExecutor();
			exec.Rethrow = true;
			//returns
			return (string)exec.Compile(scriptFile, assemblyFile, debugBuild)[1];
		}

	

		protected Script CreateAppDomain(string domainName)
		{
	
			AppDomainSetup setup = new AppDomainSetup();

			string pluginDirectory = AppDomain.CurrentDomain.BaseDirectory+"plugins";
			setup.ApplicationBase = AppDomain.CurrentDomain.BaseDirectory;
			setup.PrivateBinPath = 
				Path.GetDirectoryName(pluginDirectory).Substring(
				Path.GetDirectoryName(pluginDirectory).LastIndexOf(
				Path.DirectorySeparatorChar) + 1);
			setup.CachePath = Path.Combine(pluginDirectory, 
				"cache" + Path.DirectorySeparatorChar);
			setup.ShadowCopyFiles = "true";
			setup.ShadowCopyDirectories = pluginDirectory;

		//	setup.ApplicationBase = Application.UserAppDataPath;
		//	setup.PrivateBinPath = "plugins";
		//	setup.ApplicationName = Path.GetFileName(Assembly.GetExecutingAssembly().Location);
		//	setup.ShadowCopyFiles = "true";
		//	setup.ShadowCopyDirectories = AppDomain.CurrentDomain.BaseDirectory+@"\plugins";
			AppDomain appDomain = AppDomain.CreateDomain(domainName, null, setup);
	
			csscript.RemoteExecutor remoteExecutor = (csscript.RemoteExecutor)appDomain.CreateInstanceFromAndUnwrap(Assembly.GetExecutingAssembly().Location, typeof(csscript.RemoteExecutor).ToString());
//			remoteExecutor = (RemoteExecutor)AppDomain.CurrentDomain.CreateInstanceFromAndUnwrap(Assembly.GetExecutingAssembly().Location, typeof(RemoteExecutor).ToString());
			
			return new Script(appDomain, remoteExecutor);
		}

		#endregion

		/// <summary>
		/// Default implementation of displaying application messages.
		/// </summary>
		static void DefaultPrint(string msg)
		{
			MessageBox.Show(msg);
		}
	}

	#region Script class
	public class Script
	{
		private string scriptpath;
		private string asmpath;
		private AppDomain appDomain;

		private ArrayList list;

		/// <summary>
		/// This has all of the objects to be created for the script
		/// It also takes care of the assembly loading and execution
		/// </summary>
		private csscript.RemoteExecutor re;

		public string[] AppliesTo
		{
			get
			{
			//	return appliesTo;
				string str = (string)re.InvokeMethod("AppliesTo", null);
				return str.Split(":".ToCharArray());
			}
		}

		public string ScriptTitle
		{
			get
			{
				return (string)re.InvokeMethod("ScriptTitle", null);
			}
		}

		public string AssemblyPath
		{
			get{return asmpath;}
			set
			{
				asmpath = value;
				re.LoadAssembly(asmpath);
			}
		}

		public string ScriptPath
		{
			get{return scriptpath;}
			set{scriptpath = value;}
		}

		public string Type
		{
			get{return re.getType();}
		}

		public Script(AppDomain ad, csscript.RemoteExecutor rx)
		{
			re = rx;
			appDomain = ad;

			list = new ArrayList(10);

		}

		public void addControl(Control c)
		{
			list.Add(c);
		}

		public void CreateInstance(object uniqueParent, object[] args)
		{
			re.CreateScriptInstance(uniqueParent, args);
		//	string str = (string)ExecuteMethod(uniqueParent, "AppliesTo", null);
		//	appliesTo = str.Split(":".ToCharArray());
		}

		public void unload()//(Form[] parents)
		{
		//	foreach(Form p in parents)
		//	{
				for(int i = 0; i< list.Count; i++)
				{
					Control c = (Control)list[i];
					try
					{
						c.Parent.Controls.Remove(c);
					
				//		p.Controls.Remove(c);
					}
					catch{}

				//	for(int j = 0; j< p.Controls.Count; j++)
				//		if(p.Controls[j] == c)
				//			p.Controls.Remove(.RemoveAt(j);
				}
		//	}
			System.Threading.Thread th = new System.Threading.Thread(new System.Threading.ThreadStart(UnloadThread));
			th.Start();
			MessageBox.Show("thread created");
		//	AppDomain.Unload(appDomain);
		//	appDomain = null;
		}

		private void UnloadThread()
		{
			
			AppDomain.Unload(appDomain);
			appDomain = null;
			MessageBox.Show("thread finished");
		}

		public object ExecuteMethod(object uniqueParent, string method, object[] args)
		{
			return re.ExecuteMethod(uniqueParent, method, args);
		}

	}//end script
	#endregion
}//end namespace

namespace csscript
{
	delegate void PrintDelegate(string msg);

}
