#region Licence...
//-----------------------------------------------------------------------------
// Date:	10/11/04    Time: 3:00p
// Module:	CSScriptLib.cs
// Classes:	CSScript
//			AppInfo
//
// This module contains the definition of the CSScript class. Which implements 
// compiling C# script engine (CSExecutor). Can be used for hosting C# script engine
// from any CLR application
//
// Written by Oleg Shilo (oshilo@gmail.com)
// Copyright (c) 2004. All rights reserved.
//
// Redistribution and use of this code in source and binary forms, with or without 
// modification, are permitted provided that the following conditions are met:
// 1. Redistributions of source code must retain the above copyright notice, 
//    this list of conditions and the following disclaimer. 
// 2. Neither the name of an author nor the names of the contributors may be used 
//    to endorse or promote products derived from this software without specific 
//    prior written permission.
// 3. This code may be used in compiled form in any way you desire. This
//	  file may be redistributed unmodified by any means PROVIDING it is 
//    not sold for profit without the authors written consent, and 
//    providing that this notice and the authors name is included. 
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
// TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
// PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
// LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
//	Caution: Bugs are expected!
//----------------------------------------------
#endregion

using System;
using System.IO;
using System.Windows.Forms;
using System.Reflection;
using System.Diagnostics;
using System.Collections;
using System.Text.RegularExpressions;
using System.Text;

namespace CSScriptLibrary
{
	public delegate void PrintDelegate(string msg);
	/// <summary>
	/// Class CSScript which is implements class library for CSExecutor.
	/// </summary>
	public class CSScript
	{
		#region members
		ArrayList list;
		public ArrayList Scripts
		{
			get{return list;}
		}

		bool separateAppDomain;
		public bool SeparateAppDomain
		{
			get{return separateAppDomain;}
		}

		static bool rethrow;
		/// <summary>
		/// Force caught exceptions to be rethrown.
		/// </summary>
		static public bool Rethrow
		{
			get {return rethrow;}
			set {rethrow = value;}
		}
		#endregion 

		public CSScript()
		{
			separateAppDomain = false;
			rethrow = false;
			list = new ArrayList(12);
		}

		public CSScript(bool separate)
		{
			separateAppDomain = separate;
			rethrow = false;
			list = new ArrayList(12);
		}

		#region utils

		public void Unload(string scriptName)//, Form[] parents)
		{
			foreach(object o in list)
			{
				if(((Script)o).ScriptTitle.ToLower() == scriptName.ToLower())
				{
					((Script)o).unload();//(parents);

					list.Remove(o);
					break;
				}
			}

		}

		public void Sort()
		{
			list.Sort();
		}

		public static string getType(string fullMethodName)
		{
			//error check for 3 dots!!!!!!
			//or just grab 2 strings around 1st dot

			fullMethodName = fullMethodName.Trim();
			Regex method = new Regex(@"^(?<type>\w[0-9a-zA-Z_]*\.\w[0-9a-zA-Z_]*)(\.\w[0-9a-zA-Z_]*)?",RegexOptions.Compiled);
			Match m = method.Match(fullMethodName);
			if(m!=null && m.Success)
				return m.Result(@"${type}");
			else
				return "";
		}

		public static string getMethod(string fullMethodName)
		{
			fullMethodName = fullMethodName.Trim();
			Regex method = new Regex(@"^(\w[0-9a-zA-Z_]*\.\w[0-9a-zA-Z_]*)(\.(?<method>\w[0-9a-zA-Z_]*))",RegexOptions.Compiled);
			Match m = method.Match(fullMethodName);
			if(m!=null && m.Success)
				return m.Result(@"${method}");
			else
				return "";
		}

		public Script getScript(string fullMethodName)
		{
			string type = getType(fullMethodName);
			if(type == "")
				return null;
			foreach(object o in list)
			{
				if(((Script)o).Type == type)
					return ((Script)o);
			}

			return null;
		}
		#endregion

		public bool hasMethod(string fullMethodName)
		{
			Script sc = getScript(fullMethodName);
			if(sc == null)
				return false;
			return sc.hasMethod(getMethod(fullMethodName));
		}

		#region creation
		/// <summary>
		/// finds an already created script & loads it if found. else it compiles and loads.
		/// </summary>
		/// <param name="scriptPath"></param>
		/// <returns></returns>
		public Script CompileAndLoad(string scriptPath)//, Form[] parent)
		{
			try
			{
				int i;
				Script script = null;
				for( i = 0; i<list.Count; i++)
					if( ((Script)list[i]).ScriptPath == scriptPath )
					{
						script = (Script)list[i];
						list.RemoveAt(i);
						script.unload();//(parent);
					}
				
				script = null;
				string compiledScript = csscript.CSExecutor.GetAvailableAssembly(scriptPath);
				
				Random rand = new Random();
				script = CreateAppDomain(Path.GetFileName(scriptPath) + rand.Next(999).ToString());
				script.ScriptPath = scriptPath;
			
				if(compiledScript == null)
				{
					compiledScript = Compile(scriptPath,scriptPath+"c",false);

				}
			//	else
			//		script.ScriptAssembly = Load(compiledScript);

				script.AssemblyPath = compiledScript;

				list.Add(script);

				return script;
			}
			catch(Exception e)
			{
//				Regex re = new Regex(@"\w+\.cs\(\d+,\d+\) : error \w{6}: [\w' \.]+");
//				StringBuilder sb = new StringBuilder();
//				Match m = re.Match(e.Message);
//				while(m.Success)
//				{
//					sb.Append(m.Value+"\n");
//					m = m.NextMatch();
//				}
				MessageBox.Show(e.Message, "Compiler Error", MessageBoxButtons.OK,MessageBoxIcon.Exclamation);// + "\n\n" + e.StackTrace);
				return null;
			}
		}

		/// <summary>
		/// Compiles script into assembly with CSExecutor
		/// </summary>
		/// <param name="scriptFile">The name of script file to be compiled.</param>
		/// <param name="assemblyFile">The name of compiled assembly. If set to null a temnporary file name will be used.</param>
		/// <param name="debugBuild">true if debug information should be included in assembly; otherwise, false.</param>
		/// <returns></returns>
		static protected string Compile(string scriptFile, string assemblyFile, bool debugBuild)
		{
			csscript.CSExecutor exec = new csscript.CSExecutor();
			exec.Rethrow = true;
			//returns
			return (string)exec.Compile(scriptFile, assemblyFile, debugBuild)[1];
		}

	

		protected Script CreateAppDomain(string domainName)
		{
			csscript.RemoteExecutor remoteExecutor = null;
			AppDomain appDomain = null;

			if(this.separateAppDomain)
			{
				AppDomainSetup setup = new AppDomainSetup();

				string pluginDirectory = AppDomain.CurrentDomain.BaseDirectory+"plugins";
				setup.ApplicationBase = AppDomain.CurrentDomain.BaseDirectory;
				setup.PrivateBinPath = 
					Path.GetDirectoryName(pluginDirectory).Substring(
					Path.GetDirectoryName(pluginDirectory).LastIndexOf(
					Path.DirectorySeparatorChar) + 1);
				setup.CachePath = Path.Combine(pluginDirectory, 
					"cache" + Path.DirectorySeparatorChar);
				setup.ShadowCopyFiles = "true";
				setup.ShadowCopyDirectories = pluginDirectory;

				appDomain = AppDomain.CreateDomain(domainName, null, setup);
				
				remoteExecutor = (csscript.RemoteExecutor)appDomain.CreateInstanceFromAndUnwrap(Assembly.GetExecutingAssembly().Location, typeof(csscript.RemoteExecutor).ToString());
			}
			else
				remoteExecutor = (csscript.RemoteExecutor)AppDomain.CurrentDomain.CreateInstanceFromAndUnwrap(Assembly.GetExecutingAssembly().Location, typeof(csscript.RemoteExecutor).ToString());
			
			return new Script(appDomain, remoteExecutor, this.separateAppDomain);
		}

		#endregion

		/// <summary>
		/// Default implementation of displaying application messages.
		/// </summary>
		static void DefaultPrint(string msg)
		{
			MessageBox.Show(msg);
		}
	}

	#region Script class
	public class Script : IComparable
	{
		private string scriptpath;
		private string asmpath;
		private AppDomain appDomain;

		private ArrayList list; //controls
		private ArrayList menus; //menus

		/// <summary>
		/// This has all of the objects to be created for the script
		/// It also takes care of the assembly loading and execution
		/// </summary>
		private csscript.RemoteExecutor re;

		#region properties
		public string[] AppliesTo
		{
			get
			{
				//	return appliesTo;
				string str = (string)re.InvokeMethod("AppliesTo", null);
				return str.Split(":".ToCharArray());
			}
		}

		public string ScriptTitle
		{
			get
			{
				return (string)re.InvokeMethod("ScriptTitle", null);
			}
		}

		public int Priority
		{
			get{
				try
				{
					return (int)re.InvokeMethod("Priority", null);
				}
				catch
				{return 100;}
			}
		}

		public string AssemblyPath
		{
			get{return asmpath;}
			set
			{
				asmpath = value;
				re.LoadAssembly(asmpath);
			}
		}

		public string ScriptPath
		{
			get{return scriptpath;}
			set{scriptpath = value;}
		}

		public string Type
		{
			get{return re.getType();}
		}
		#endregion

		public Script(AppDomain ad, csscript.RemoteExecutor rx, bool separateAppDomain)
		{
			re = rx;

			if(separateAppDomain)
				appDomain = ad;
			else
				appDomain = null;
			
			list = new ArrayList(10);
			menus = new ArrayList(2);
		}

		public bool hasMethod(string methodname)
		{
			return re.hasMethod(methodname);
		}

		public void addControl(Control c)
		{
			if(list!= null)
				list.Add(c);
		}
		public void addMenuItem(MenuItem m)
		{
			if(menus!= null)
				menus.Add(m);
		}

		public bool RemoveInstance(object parent)
		{
			return re.RemoveParentObject(parent);
		}

		public void CreateInstance(object uniqueParent, object[] args)
		{
			re.CreateScriptInstance(uniqueParent, args);
		//	string str = (string)ExecuteMethod(uniqueParent, "AppliesTo", null);
		//	appliesTo = str.Split(":".ToCharArray());
		}

		public void unload()//(Form[] parents)
		{
			for(int i = 0; list != null && i< list.Count; i++)
			{
				Control c = (Control)list[i];
				try
				{
					c.Parent.Controls.Remove(c);
				}
				catch{}
			}

			for(int i = 0; menus != null && i< menus.Count; i++)
			{
				MenuItem m = (MenuItem)menus[i];
				try
				{
					m.Parent.MenuItems.Remove(m);
				}
				catch{}
			}

			if(appDomain != null)
			{
				AppDomain.Unload(appDomain);
				appDomain = null;
				MessageBox.Show("Finally unloaded...");
			}
		}


		public object ExecuteMethod(object uniqueParent, string method, object[] args)
		{
			return re.ExecuteMethod(uniqueParent, method, args);
		}
		public object ExecuteStaticMethod(string method, object[] args)
		{
			return re.InvokeMethod(method, args);
		}

		#region IComparable Members

		public int CompareTo(object obj)
		{
			try
			{
				int l = this.Priority;
				int r = ((Script)obj).Priority;

				if(l<r) 
					return -1;
				else if(l>r) 
					return 1;
				else
					return 0;
			}
			catch
			{
				return 0;
			}
		}

		#endregion
	}//end script
	#endregion
}//end namespace

namespace csscript
{
	delegate void PrintDelegate(string msg);

}
