

using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.IO;
using System.Windows.Forms;

namespace CSScriptLibrary
{
	/// <summary>
	/// Summary description for ScriptManager.
	/// </summary>
	public class ScriptManager : System.Windows.Forms.Form
	{
		protected CSScriptLibrary.CSScript scripting;
		protected bool separateAppDomain;

		protected ArrayList parents;
//		protected ArrayList paths;
		protected static string LOADSCRIPTS = "/plugins/LoadScripts.app";
		protected static string APPDIR = @"*appdir*";

		private System.Windows.Forms.ListBox scripts;
		private System.Windows.Forms.Label label1;
		private System.Windows.Forms.Button close;
		private System.Windows.Forms.Button loadBtn;
		private System.Windows.Forms.ContextMenu contextMenu1;
		private System.Windows.Forms.MenuItem load;
		private System.Windows.Forms.MenuItem reload;
		private System.Windows.Forms.MenuItem unload;
		private System.Windows.Forms.MenuItem menuItem4;
		private System.Windows.Forms.MenuItem selectAll;
		private System.Windows.Forms.OpenFileDialog openFileDialog1;
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

		public ScriptManager(bool separateDomain)
		{
			separateAppDomain = separateDomain;
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			scripting = new CSScriptLibrary.CSScript(separateDomain);
			parents = new ArrayList(10);
		//	paths = new ArrayList(10);

			this.LoadScriptsFromFile();
		}

		private string getScriptsPath()
		{
			return Application.StartupPath + LOADSCRIPTS;
		}

		public CSScript CSScript
		{
			get{return scripting;}
		}

		/// <summary>
		/// calls a static method in ALL the scripts
		/// </summary>
		/// <param name="method"></param>
		/// <param name="args"></param>
		/// <param name="suppressErrors"></param>
		/// <returns></returns>
		public object[] CallStaticMethod(string method, object[] args, bool suppressErrors)
		{
			object[] ret = new object[scripting.Scripts.Count];
			
			for(int i =0; i<ret.Length; i++)
			{
				Script s = (Script)(scripting.Scripts[i]);
				try
				{
					ret[i] = s.ExecuteStaticMethod(method,args);
				}
				catch(Exception ex)
				{
					if(!suppressErrors)
						throw ex;
				}
			}
			return ret;
			
		}
		
		//adds a form to the script manager and creates an instance of 
		//each script that applies to this form.
		public void AddChild(Form ch, string type)
		{
			FormWithType fwt = new FormWithType(ch,type);
			parents.Add(fwt);
			//now instantiate for that Form...
			InstantiateObjects(fwt);
		}

		public void RemoveChild(Form ch)
		{
			//need to remove!!!
			foreach(object o in parents)
			{
				if(((FormWithType)o).Form == ch)
				{
					parents.Remove(o);
					break;
				}
			}

			foreach(Script s in CSScript.Scripts)
			{
				s.RemoveInstance(ch);
			}
		}
	
		public FormWithType GetChild(int index)
		{
			return (FormWithType)(parents[index]);
		}

		public FormWithType[] GetChildArray()
		{
			return (FormWithType[])(parents.ToArray()); //__typeof(Form)
		}

		private bool isProperType(string[] applies, string type)
		{
			for(int j = 0; j< applies.Length; j++)
			{

				applies[j] = applies[j].ToLower();
				if(applies[j].Equals("all") || type.ToLower().IndexOf(applies[j]) >= 0)
				{ //
					return true;
				}//end if
			}//end for

			return false;

		}

		public void InstantiateObjects(FormWithType ch)
		{

			ArrayList  al = this.scripting.Scripts;
			for(int i = 0; i<al.Count; i++)
			{
				CSScriptLibrary.Script sc = (CSScriptLibrary.Script)(al[i]);

				if(this.isProperType(sc.AppliesTo,ch.Type))
				{ 
					object[] args = new object[2];
					args[0] = this.MdiParent;
					args[1] = ch.Form;
					sc.CreateInstance(ch.Form,args);
				}//end if
				
			}

		}

		void PopulateListBox()
		{
			scripting.Sort();
			scripts.Items.Clear();
			foreach( CSScriptLibrary.Script s in scripting.Scripts)
			{
				scripts.Items.Add(s.ScriptTitle);
			}
		}

		//returns script name...
		string CompileAndLoadScript(string path)
		{
			try
			{
				CSScriptLibrary.Script  sc = 
						scripting.CompileAndLoad(path);//, GetChildArray());

				if(sc != null)
				{
					LoadScript(sc);
					return sc.ScriptPath;
				}
				return null;
			}
			catch(Exception e)
			{
				MessageBox.Show(string.Concat(e.Message,"\n\n",e.Source, "\n\n", e.StackTrace));
				return null;
			}
		}

		void LoadScript(CSScriptLibrary.Script  sc)
		{
			object[] args = new object[2];
			args[0] = this.MdiParent;

			//ghetto way to get the appliesTo stuff
		//	sc.CreateInstance(this,args);

			string[] applies = sc.AppliesTo;

			for(int i = 0; i<parents.Count; i++)
			{
				FormWithType fwt = GetChild(i);
				if(!fwt.Form.IsDisposed && isProperType(applies,fwt.Type))
				{
					args[1] = fwt.Form;
					try
					{
						sc.CreateInstance(fwt.Form,args);
					}
					catch{}
				}//end if

			}//end for

			for(int i = 0; i<parents.Count; i++)
			{
				FormWithType fwt = GetChild(i);
				if(fwt.Form.IsDisposed)
				{
					parents.RemoveAt(i);
					i--;
				}//end if

			}//end for
		}//end loadScript

		void LoadScriptsFromFile()
		{
			StreamReader tr = null;
			try
			{
				tr = new StreamReader(getScriptsPath());

				string line = null;
				while((line = tr.ReadLine()) != null)
				{
					line = line.Trim().ToLower();
					
					string apath = Application.StartupPath.ToLower();
					if(line.IndexOf(APPDIR) >= 0)
						line = line.Replace(APPDIR,apath);

					bool found = false;
					try
					{
						foreach(CSScriptLibrary.Script s in scripting.Scripts)
							if(line.Equals(s.ScriptPath.ToLower()))
								found = true;
						if(!found)
						{
							string str = CompileAndLoadScript(line);
						}
//						string str = CompileAndLoadScript(line);
//					
//						for(int j = 0; j<scripts.Items.Count; j++)
//							if(str.ToLower().Equals(
//								((string)(scripts.Items[j])).ToLower()))
//								found = true;
//						if(!found)
//						{
//						//	scripts.Items.Add(str);
//						//	paths.Add(line);
//						}
					}
					catch{}
				}//end while
				tr.Close();

				PopulateListBox();
			}
			catch
			{
				if(tr != null)
					tr.Close();
			}
		}

		void SaveScriptsToFile()
		{
			StreamWriter sw = new StreamWriter(getScriptsPath(),false);

			for(int i = 0; i<scripting.Scripts.Count; i++)
			{
				string s = ((CSScriptLibrary.Script)scripting.Scripts[i]).ScriptPath.ToLower();
				string apath = Application.StartupPath.ToLower();
			//	MessageBox.Show(apath + "\n" + s);
				if(s.IndexOf(apath) >= 0)
					s = s.Replace(apath,APPDIR);
			//	MessageBox.Show(s);
				sw.WriteLine(s);
			}

			sw.Close();

		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.scripts = new System.Windows.Forms.ListBox();
			this.contextMenu1 = new System.Windows.Forms.ContextMenu();
			this.load = new System.Windows.Forms.MenuItem();
			this.reload = new System.Windows.Forms.MenuItem();
			this.unload = new System.Windows.Forms.MenuItem();
			this.menuItem4 = new System.Windows.Forms.MenuItem();
			this.selectAll = new System.Windows.Forms.MenuItem();
			this.label1 = new System.Windows.Forms.Label();
			this.loadBtn = new System.Windows.Forms.Button();
			this.close = new System.Windows.Forms.Button();
			this.openFileDialog1 = new System.Windows.Forms.OpenFileDialog();
			this.SuspendLayout();
			// 
			// scripts
			// 
			this.scripts.Anchor = ((System.Windows.Forms.AnchorStyles)((((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom) 
				| System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right)));
			this.scripts.ContextMenu = this.contextMenu1;
			this.scripts.Location = new System.Drawing.Point(8, 24);
			this.scripts.Name = "scripts";
			this.scripts.SelectionMode = System.Windows.Forms.SelectionMode.MultiExtended;
			this.scripts.Size = new System.Drawing.Size(280, 199);
			this.scripts.TabIndex = 0;
			// 
			// contextMenu1
			// 
			this.contextMenu1.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																						 this.load,
																						 this.reload,
																						 this.unload,
																						 this.menuItem4,
																						 this.selectAll});
			// 
			// load
			// 
			this.load.Index = 0;
			this.load.Shortcut = System.Windows.Forms.Shortcut.CtrlL;
			this.load.Text = "Load";
			this.load.Click += new System.EventHandler(this.load_Click);
			// 
			// reload
			// 
			this.reload.Index = 1;
			this.reload.Shortcut = System.Windows.Forms.Shortcut.CtrlR;
			this.reload.Text = "Reload";
			this.reload.Click += new System.EventHandler(this.reload_Click);
			// 
			// unload
			// 
			this.unload.Index = 2;
			this.unload.Shortcut = System.Windows.Forms.Shortcut.Del;
			this.unload.Text = "Unload";
			this.unload.Click += new System.EventHandler(this.unload_Click);
			// 
			// menuItem4
			// 
			this.menuItem4.Index = 3;
			this.menuItem4.Text = "-";
			// 
			// selectAll
			// 
			this.selectAll.Index = 4;
			this.selectAll.Shortcut = System.Windows.Forms.Shortcut.CtrlA;
			this.selectAll.Text = "Select All";
			this.selectAll.Click += new System.EventHandler(this.selectAll_Click);
			// 
			// label1
			// 
			this.label1.Location = new System.Drawing.Point(8, 8);
			this.label1.Name = "label1";
			this.label1.Size = new System.Drawing.Size(96, 16);
			this.label1.TabIndex = 1;
			this.label1.Text = "Loaded Scripts:";
			// 
			// loadBtn
			// 
			this.loadBtn.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
			this.loadBtn.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.loadBtn.Location = new System.Drawing.Point(120, 232);
			this.loadBtn.Name = "loadBtn";
			this.loadBtn.Size = new System.Drawing.Size(80, 32);
			this.loadBtn.TabIndex = 2;
			this.loadBtn.Text = "Load";
			this.loadBtn.Click += new System.EventHandler(this.loadBtn_Click);
			// 
			// close
			// 
			this.close.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
			this.close.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.close.Location = new System.Drawing.Point(208, 232);
			this.close.Name = "close";
			this.close.Size = new System.Drawing.Size(80, 32);
			this.close.TabIndex = 3;
			this.close.Text = "Close";
			this.close.Click += new System.EventHandler(this.close_Click);
			// 
			// ScriptManager
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(296, 270);
			this.Controls.Add(this.close);
			this.Controls.Add(this.loadBtn);
			this.Controls.Add(this.label1);
			this.Controls.Add(this.scripts);
			this.MaximizeBox = false;
			this.Name = "ScriptManager";
			this.ShowInTaskbar = false;
			this.Text = "Script Manager";
			this.Closing += new System.ComponentModel.CancelEventHandler(this.ScriptManager_Closing);
			this.ResumeLayout(false);

		}
		#endregion

		private void loadBtn_Click(object sender, System.EventArgs e)
		{
			load_Click(null,null);
		}

		private void close_Click(object sender, System.EventArgs e)
		{
			this.Visible = false;
		
		}

		private void load_Click(object sender, System.EventArgs e)
		{
			//"txt files (*.txt)|*.txt|All files (*.*)|*.*"
			this.openFileDialog1.Filter = "C# files (*.cs)|*.cs|All files (*.*)|*.*";

			if(this.openFileDialog1.ShowDialog() == DialogResult.OK)
			{
				for(int i = 0; i<this.openFileDialog1.FileNames.Length; i++)
				{
					bool found = false;
					string fname = this.openFileDialog1.FileNames[i];

			//		if(str != null)
			//		{
						foreach(CSScriptLibrary.Script s in scripting.Scripts)
							if(fname.ToLower().Equals(s.ScriptPath.ToLower()))
								found = true;
						if(!found)
						{
							string str = CompileAndLoadScript(fname);
						//	scripts.Items.Add(str);
						//	paths.Add(fname);
						}
						else
							MessageBox.Show(string.Concat(fname," is already on the list"),"BRE");
			//		}

				}//end for

				PopulateListBox();
				SaveScriptsToFile();
			}
		}

		private void reload_Click(object sender, System.EventArgs e)
		{
			if(!this.separateAppDomain)
			{
				unload_Click(null,null);
				return;
			}
			//TODO: fix this. otherwise it breaks the reloading for the separate app domains
//			for(int i = 0; i<scripts.Items.Count; i++)
//			{
//				if(scripts.GetSelected(i))
//				{
//					CompileAndLoadScript((string)(paths[i]));//(scripts.Items[i)));
//				}
//			}
		
		}

		private void unload_Click(object sender, System.EventArgs e)
		{
			for(int i = 0; i<scripts.Items.Count; i++)
			{
				if(scripts.GetSelected(i))
				{
					if(!this.separateAppDomain)
						MessageBox.Show("You must restart the application in order to properly reload this script.");
					scripting.Unload((string)(scripts.Items[i]));//,this.GetChildArray());
					scripts.Items.RemoveAt(i);
			//		paths.RemoveAt(i);
					i--;
				}
			}

			 this.SaveScriptsToFile();
		}

		private void selectAll_Click(object sender, System.EventArgs e)
		{
			for(int i = 0; i<scripts.Items.Count; i++)
			{
				scripts.SetSelected(i,true);
			}
		}

		private void ScriptManager_Closing(object sender, System.ComponentModel.CancelEventArgs e)
		{
			e.Cancel = true;
			this.Visible = false;
		}

	}

	public class FormWithType
	{
		protected Form form;
		protected string type;

		public FormWithType(Form frm, string type)
		{
			this.type = type;
			form = frm;
		}

		public Form Form
		{
			get{return form;}
		}

		public string Type
		{
			get{return type;}
		}
	}
}
