using System;
using System.Drawing;
using System.Collections;

namespace SimpleGrid
{
	/// <summary>
	/// Summary description for Crosshairs.
	/// </summary>
	public class Crosshairs
	{
		#region members
		private Color xhairColor = Color.FromArgb(200,255,0,0);
		private SolidBrush brush = new SolidBrush(Color.FromArgb(200,255,0,0));
		
		//shoudld be a percentage, 0.0-1.0, of usage of that cell. 
		//It is assumed to be 1.0 at the current cell, and 0.0 at the 
		//next cell in either direction.   
		private PointF interp;
		private Point location;

		private bool enabled = false;
		private bool crosshairs = false; //else phantom cell...
		private bool visible = true;
		#endregion

		#region Properties
		public Color CrossHairColor
		{
			set
			{
				xhairColor = value;
				if(brush != null) brush.Dispose();
				brush = new SolidBrush(xhairColor);
			}
			get{return xhairColor;}
		}
		public Point Location
		{
			set{location = value;}
			get{return location;}
		}
		public PointF Interpolation
		{
			set
			{
				interp = new PointF(checkFloat(value.X,0f,1f), checkFloat(value.Y,0f,1f));
			}
			get{return interp;}
		}
		public bool PaintAsCrosshairs
		{
			set{crosshairs = value;}
			get{return crosshairs;}
		}
		public bool Enabled
		{
			set{enabled = value;}
			get{return enabled;}
		}
		public bool Visible
		{
			set{visible = value;}
			get{return visible;}
		}
		#endregion

		private float checkFloat(float val,float min, float max)
		{
			if(val < min)
				return min;
			if(val>max)
				return max;
			return val;
		}

		public Crosshairs()
		{
			interp = new PointF(1f,1f);
			location = new Point(0,0);
		}

		public void paintCrosshairs(Graphics g, Size headerSize, Size cellSize, Size cells)
		{
			if(Enabled && Visible && Location.X>-1 && Location.Y>-1 && 
				Location.X <cells.Width && Location.Y < cells.Height)
			{
				float rowy = (float)headerSize.Height + ((float)cellSize.Height* ((float)Location.Y + 1.0f*(1.0f-Interpolation.Y)));
				float colx = (float)headerSize.Width + ((float)cellSize.Width* ((float)Location.X + 1.0f*(1.0f-Interpolation.X)));
			
				if(crosshairs)
				{
					//row interp bar...
					g.FillRectangle(brush,(float)headerSize.Width+1f,rowy+1f,
						(float)(cells.Width*cellSize.Width)-1f,(float)cellSize.Height-1f);

					//col interp bar...
					g.FillRectangle(brush,colx+1f,(float)headerSize.Height+1f,
						(float)cellSize.Width-1f,(float)(cells.Height*cellSize.Height)-1f);
				}
				else
					g.FillRectangle(brush,colx+1f,rowy+1f,
						(float)(cellSize.Width)-1f,(float)cellSize.Height-1f);
			}
		}
	}
}
