/*! \mainpage SimpleGrid
 * 
 * SimpleGrid - a simple no-nonsense grid component for the .NET framework. 
 * 
 * (C) UK 2004 Tim Heron
 * 
 * Extensions and re-write (C) 2005 Ben Ogle
 */

using System;
using System.Diagnostics;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;
using System.Text.RegularExpressions;

namespace SimpleGrid
{
	public delegate void GridChangedHandler(object sender, CellEvent e);

	/// <summary>
	/// A simple no-nonsense grid component
	/// </summary>
	public class Grid : System.Windows.Forms.UserControl//ScrollableControlWithScrollEvents
	{

		#region memberData
		DoubleBuffer buff = new DoubleBuffer();

		public const int M_PAINT_ALL = 0;
		public const int M_PAINT_NO_COLORS = 1;
		//		private const int M_REPAINT_BLOCK = 1;
		//		private const int M_REPAINT_MOVE = 2;

		RepaintData repData = new RepaintData();

		public static int MODE_DOUBLE = 0;
		public static int MODE_INT = 1;
		public static int MODE_STRING = 2;

		//		private int repaintmode = M_REPAINT_ALL;

		protected System.ComponentModel.Container components = null;

		
		//string[,] data;
		string[] headers = new string[0];
		string[] rowHeaders = new string[0];
		bool[] headerEditable = new bool[0];
		string[,] info;

		Crosshairs xhairs = new Crosshairs();

		Point mouseover; //the cell the mouse is over -1,x if over a row header x,-1 if over col header
		Color[,] cellColors;
		int columns = 0;
		int rows = 0;
		//int[] columnWidth = new int[0];
		int columnWidth;

		GridSelection currSel;

		//int currSel.SelectedRow = -1;
		//int selectedCol = -1;
		//int currSel.LastSelectedRow = -1;
		//int lastSelectedCol = -1;
		
		Color selectedColor = Color.Black;
		Color selectedForeColor = Color.White;
		int rowHeight = 20;
		int rowOffset = 2;
		int rowsVisible = 0;
		int rowHeaderWidth = 30;

		int mouseovercolumn = -1;

		int mode = 0; //0 for float, 1 for int, 2 for string
		int headermode = 0;
		int paintmode = 0;

		bool onlyread = false;

		//Temporary values for when the user is dragging column widths
		int adjustingColumn = -1;
		private System.Windows.Forms.TextBox editBox;
    
		/// <summary>
		/// Occurs when the user selects a row
		/// </summary>
		public event EventHandler SelectedRowChanged;
		[Category("Property Changed")]
		public event GridChangedHandler CellChanged;
		[Category("Property Changed")]
		public event GridChangedHandler ColumnHeaderChanged;

		#endregion
		
		#region Properties
		public Crosshairs Crosshairs
		{
			get{return xhairs;}
		}	
		/// <summary>
		/// Gets or Sets the size of the grid, data in the grid will be lost if the size changes
		/// </summary>
		[Category("Layout")]
		public Size GridSize
		{
			get { return new Size(columns,rows); }
			set
			{
				if (value.Width >= 0 && value.Height >= 0)
				{
					int c = Math.Min(columns,value.Width);
					int r = Math.Min(rows,value.Height);

					columns = value.Width;
					rows = value.Height;
					string[,] data = info;
					info = new string[columns,rows];

					Color[,] colors = cellColors;
					cellColors = new Color[columns,rows];

					for(int i = 0; i<columns; i++)
						for(int j = 0; j<rows; j++)
							cellColors[i,j] = BackColor;

					for(int i = 0; i<c; i++)
					{
						for(int j = 0; j<r; j++)
						{
							cellColors[i,j] = colors[i,j];
							info[i,j] = data[i,j];
						}
					}

					//Copy the column widths
					//int[] newColumnWidths = new int[columns];
					//Array.Copy(columnWidth,newColumnWidths,Math.Min(columnWidth.Length,newColumnWidths.Length));
					//columnWidth = newColumnWidths;
          
					//Copy the column headers
					string[] newColumnHeaders = new string[columns];
					Array.Copy(headers,newColumnHeaders,Math.Min(headers.Length,newColumnHeaders.Length));
					headers = newColumnHeaders;
			
					//Copy the column header edit bools
					bool[] newHeaderEdit = new bool[columns];
					Array.Copy(headerEditable,newHeaderEdit,Math.Min(headerEditable.Length,newHeaderEdit.Length));
					headerEditable = newHeaderEdit;

					//copy the row headers
					string[] newRowHeaders = new string[rows];
					Array.Copy(rowHeaders, newRowHeaders, Math.Min(rowHeaders.Length, newRowHeaders.Length));
					rowHeaders = newRowHeaders;

					rowsVisible = BoxHeight / rowHeight;
					UpdateScrollBars();
					repData.RepaintAll = true;
					Refresh();
				}
			}
		}

		/// <summary>
		/// Gets or sets the read only attribute
		/// 
		/// Set to true if the grid is read only
		/// </summary>
		public bool ReadOnly
		{
			get 
			{ 
				return onlyread;
			}
			set
			{
				onlyread = value;
				this.Focus();
			}
		}

		/// <summary>
		/// Gets or sets the paint mode of the grid
		/// 
		/// Can paint everything, or omit certain things:\n
		/// M_PAINT_ALL\n
		/// M_PAINT_NO_COLORS\n
		/// </summary>
		public int PaintMode
		{
			get 
			{ 
				return paintmode;
			}
			set
			{
				if(value>0)
					paintmode = 1;
				else if(value <0)
					paintmode = 0;
				else
					paintmode = value;
			}
		}

		public int HeaderMode
		{
			get{return headermode;}
			set
			{
				if(value>1)
					headermode = 2;
				else if(value <0)
					headermode = 0;
				else
					headermode = value;
			}
		}
		/// <summary>
		/// Gets or sets the mode of the grid
		/// 
		/// Can accept integers, floats or text:\n
		/// MODE_INT\n
		/// MODE_DOUBLE\n
		/// MODE_STRING\n
		/// </summary>
		public int Mode
		{
			get 
			{ 
				return mode;
			}
			set
			{
				if(value>1)
					mode = 2;
				else if(value <0)
					mode = 0;
				else
					mode = value;
			}
		}

		int BoxHeight
		{
			get 
			{ 
			//	if(hScrollBar.Visible == true)  
			//		return Height - hScrollBar.Height;
			//	else
					return Height;
			}
		}
    
		int BoxWidth
		{
			get 
			{ 
			//	if(vScrollBar.Visible == true)
			//		return Width - vScrollBar.Width;
			//	else
					return Width;
			}
		}
    
		/// <summary>
		/// Gets or Sets the height in pixels of each row
		/// </summary>
		[Category("Layout")]
		public int RowHeight
		{
			get { return rowHeight; }
			set
			{
				if (value > 0)
					rowHeight = value;

				this.UpdateScrollBars();
				repData.RepaintHeader = true;
				Refresh();
			}
		}

		/// <summary>
		/// Gets or Sets the width in pixels of the row headers
		/// </summary>
		[Category("Layout")]
		public int RowHeaderWidth
		{
			get { return rowHeaderWidth; }
			set
			{
				if (value >= 0)
					rowHeaderWidth = value;

				this.UpdateScrollBars();
				repData.RepaintHeader = true;
				Refresh();
			}
		}
		
		/// <summary>
		/// The background color of a selected cell
		/// </summary>
		[Category("Appearance")]
		public Color SelectedRowColor
		{
			get { return selectedColor; }
			set { 
				selectedColor = value; 
				repData.RepaintSelection = true;
			}
		}
		/// <summary>
		/// The foreground color of a selected row
		/// </summary>
		[Category("Appearance")]
		public Color SelectedForeColor
		{
			get { return selectedForeColor; }
			set { 
				selectedForeColor = value; 
				repData.RepaintSelection = true;
			}
		}
		
		/// <summary>
		/// Gets or Sets the selected row index
		/// </summary>
		[Browsable(false)]
		public int SelectedRow
		{
			get { return currSel.SelectedRow; }
			set
			{ 
				if(value <rows)
				{
					currSel.SelectedRow = value;
					repData.RepaintSelection = true;
					OnSelectedRowChanged(EventArgs.Empty);
					xhairs.Location = new Point(xhairs.Location.X,value);
				}
			}
		}

		/// <summary>
		/// Gets or Sets the selected column index
		/// </summary>
		[Browsable(false)]
		public int SelectedColumn
		{
			get { return currSel.SelectedColumn; }
			set
			{ 
				if(value<columns)
				{
					currSel.SelectedColumn = value;
					repData.RepaintSelection = true;
					OnSelectedRowChanged(EventArgs.Empty);
					xhairs.Location = new Point(value,xhairs.Location.Y);
				}
			}
		}
		/// <summary>
		/// Gets or Sets the selected row index
		/// </summary>
		[Browsable(false)]
		public int LastSelectedRow
		{
			get { return currSel.LastSelectedRow; }
			set
			{ 
				if(value<rows)
				{
					currSel.LastSelectedRow = value;
					repData.RepaintSelection = true;
					OnSelectedRowChanged(EventArgs.Empty);
				}
			}
		}

		/// <summary>
		/// Gets or Sets the selected column index
		/// </summary>
		[Browsable(false)]
		public int LastSelectedColumn
		{
			get { return currSel.LastSelectedColumn; }
			set
			{ 
				if(value<columns)
				{
					currSel.LastSelectedColumn = value;
					repData.RepaintSelection = true;
					OnSelectedRowChanged(EventArgs.Empty);
				}
			}
		}
		#endregion

		#region Constructor
		/// <summary>
		/// A simple no-nonsense grid component
		/// </summary>
		public Grid()
		{
			InitializeComponent();
      
			//Set some styles to add double buffering to our control
			SetStyle(ControlStyles.UserPaint,true);
			SetStyle(ControlStyles.AllPaintingInWmPaint,true);
			SetStyle(ControlStyles.DoubleBuffer,true);

			editBox.Visible = false;
			editBox.Enabled = false;
			//	hScrollBar.Show();//.Visible = true;
			//	UpdateScrollBars();

			this.Load +=new EventHandler(Grid_Load);
			this.Resize +=new EventHandler(Grid_Resize);


			this.TabStop = false;

			currSel = new GridSelection();

			this.Focus();
		}
		#endregion
    
		#region gets & sets
		public void setScrollBars()
		{
			this.UpdateScrollBars();
			Refresh();

		}
		/// <summary>
		/// Sets the width of a column
		/// </summary>
		/// <param name="column">Column index</param>
		/// <param name="width">Width in % of total grid width</param>
		public void setColumnWidth(int column,int width)
		{
			columnWidth = width;
			UpdateScrollBars();
			repData.RepaintAll = true;
			Refresh();
//			if (column >= 0 && column < columns && width >= 0)
//			{
//				columnWidth[column] = width;
//				UpdateScrollBars();
//				repData.RepaintAll = true;
//				Refresh();
//			}
		}
		
		/// <summary>
		/// Gets the width of a column
		/// </summary>
		/// <param name="column">Column index</param>
		/// <returns>Width in % of total grid width</returns>
		public int getColumnWidth(int column)
		{
			return columnWidth;//(column >= 0 && column < columns) ? columnWidth[column] : 0;
		}
		
		/// <summary>
		/// Sets the width of all the columns
		/// </summary>
		/// <param name="column">Column index</param>
		/// <param name="width">Width in % of total grid width</param>
		public void setAllColumnWidths(int width)
		{
			columnWidth = width;
			/*
			if (width >= 0)
			{
				for(int i = 0; i<columns; i++)
					columnWidth[i] = width;
				
				UpdateScrollBars();
				repData.RepaintAll = true;
				Refresh();
			}*/
		}

		public void setHeaderEditable(int column, bool editable)
		{
			if (column >= 0 && column < columns)
				headerEditable[column] = editable;
		}

		public bool getHeaderEditable(int column)
		{
			if (column >= 0 && column < columns)
				return headerEditable[column];
			return false;
		}
		
		/// <summary>
		/// Sets the header text of a column
		/// </summary>
		/// <param name="column">Column index</param>
		/// <param name="header">The header text of the column</param>
		public void setHeader(int column,string header)
		{
			if (column >= 0 && column < columns && header != null)
			{
				repData.RepaintHeader = true;
				headers[column] = header;
			}
		}
	
		/// <summary>
		/// Sets the rowHeader text of some row
		/// </summary>
		/// <param name="row">row index</param>
		/// <param name="rowHeader">The header text of the row</param>
		public void setRowHeader(int row,string rowHeader)
		{
			if (row >= 0 && row < rows && rowHeader != null)
			{
				repData.RepaintHeader = true;
				rowHeaders[row] = rowHeader;
			}
		}

		/// <summary>
		/// Gets the header text of a column
		/// </summary>
		/// <param name="column">Column index</param>
		/// <returns>The header text of the column</returns>
		public string getHeader(int column)
		{
			return (column >= 0 && column < columns) ? headers[column] : "";
		}
	
		/// <summary>
		/// Gets the row header text of a column
		/// </summary>
		/// <param name="column">Row index</param>
		/// <returns>The header text of the row</returns>
		public string getRowHeader(int row)
		{
			return (row >= 0 && row < rows) ? rowHeaders[row] : "";
		}		

		/// <summary>
		/// Sets the (float) value text of a cell
		/// </summary>
		/// <param name="column">Column index</param>
		/// <param name="row">Row index</param>
		/// <param name="value">Value text of the cell</param>
		public void setValue(int column,int row,float val)
		{
			
			if (column >= 0 && column < columns && row >= 0 && row < rows)
			{
				repData.addCell(new Point(column, row));
				info[column,row] = val.ToString("0.00");
			}
		}

		/// <summary>
		/// Sets the (int) value text of a cell
		/// </summary>
		/// <param name="column">Column index</param>
		/// <param name="row">Row index</param>
		/// <param name="value">Value text of the cell</param>
		public void setValue(int column,int row,int val)
		{
			
			if (column >= 0 && column < columns && row >= 0 && row < rows)
			{
				repData.addCell(new Point(column, row));
				info[column,row] = val.ToString();
			}
		}

		/// <summary>
		/// Sets the (text) value text of a cell
		/// </summary>
		/// <param name="column">Column index</param>
		/// <param name="row">Row index</param>
		/// <param name="value">Value text of the cell</param>
		public void setValue(int column,int row,string val)
		{
			if (column >= 0 && column < columns && row >= 0 && row < rows)
			{
				repData.addCell(new Point(column, row));
				info[column,row] = val;
			}
		}
		
		/// <summary>
		/// Gets the float value of a cell
		/// </summary>
		/// <param name="column">Column index</param>
		/// <param name="row">Row index</param>
		/// <returns>Value of the cell</returns>
		public float getValue(int column,int row)
		{
			return (column >= 0 && column < columns && row >= 0 && row < rows && info[column, row] != null && info[column, row].Length>0) ?  System.Convert.ToSingle(info[column,row]) : 0.0F;
		}

		/// <summary>
		/// Gets the int value of a cell
		/// </summary>
		/// <param name="column">Column index</param>
		/// <param name="row">Row index</param>
		/// <returns>Value of the cell</returns>
		public int getIntValue(int column,int row)
		{
			if(column >= 0 && column < columns && row >= 0 && row < rows && info[column, row] != null && info[column, row].Length>0) 
			{
				try
				{
					int i = System.Convert.ToInt32(info[column,row]);
					return i;
				}
				catch
				{
					return 0;
				}
			}
			else
				return 0;
		}

		/// <summary>
		/// Gets the text of a cell
		/// </summary>
		/// <param name="column">Column index</param>
		/// <param name="row">Row index</param>
		/// <returns>Value text of the cell</returns>
		public string getText(int column,int row)
		{
			return (column >= 0 && column < columns && row >= 0 && row < rows) ?  info[column,row] : "";
		}

		/// <summary>
		/// Sets the cell back color
		/// </summary>
		/// <param name="column">Column index</param>
		/// <param name="row">Row index</param>
		/// <param name="color">Color</param>    
		public void setCellColor(int column, int row, Color color)
		{
			cellColors[column,row] = color;
			repData.addCell(new Point(column, row));
		}

		/// <summary>
		/// Gets the cell back color
		/// </summary>
		/// <param name="column">Column index</param>
		/// <param name="row">Row index</param>
		/// <returns>Color</returns>    
		public Color getCellColor(int column, int row)
		{
			if(column >= 0 && column < columns && row >= 0 && row < rows)
				return cellColors[column,row];
			else
				return Color.White;
		}

		int calculateTotalHeight()
		{
			return (this.GridSize.Height+1)*this.rowHeight;
		}

		int calculateTotalWidth()
		{
			//Get the width of the grid
			/*
			int totalWidth = 0;
			for (int column = 0; column < columns; column++)
				totalWidth += columnWidth[column];
		
			totalWidth += rowHeaderWidth;
			
			return totalWidth;
			*/
			return rowHeaderWidth + columnWidth*columns;
		}
		#endregion
		
		#region painting
		protected void paintBackground(Graphics g)
		{
			int totalWidth = calculateTotalWidth();
			int cumulativeWidth = rowHeaderWidth;

			int gridWidth = BoxWidth - 1;
			if(gridWidth < totalWidth)
				gridWidth = totalWidth;

			int colHeight = (rows+1) * rowHeight;
			int height = Height;
			if(height < colHeight)
				height = colHeight;

			Pen gridPen = new Pen(Color.LightGray,1);
			Pen header3dPen = new Pen(System.Drawing.Color.DarkGray,1);
			Pen header3dPen1 = new Pen(Color.White,1);
			Brush backgroundBrush = new SolidBrush(SystemColors.AppWorkspace);
			Brush bgBrush = new SolidBrush(BackColor);
			Brush headerBrush = new SolidBrush(SystemColors.Control);

			//MAIN background
			//Clear the background and draw the header background
			g.FillRectangle(backgroundBrush,0,0,gridWidth,height);
			if(paintmode == M_PAINT_NO_COLORS)
				g.FillRectangle(bgBrush,0,0,totalWidth,colHeight);
			g.FillRectangle(headerBrush,rowHeaderWidth,0,totalWidth-rowHeaderWidth,rowHeight);
			g.FillRectangle(headerBrush,0,0,rowHeaderWidth,(rows+1 ) * rowHeight);
		
			//make headers look 3Dish: draw white line on left side
			g.DrawLine(header3dPen1,0,0,0,(rows+1)*rowHeight);

			//draw white line across top
			g.DrawLine(header3dPen1,0,0,totalWidth,0);

			//GridLines
			//Draw the columns
			cumulativeWidth = rowHeaderWidth;
			for (int column = 0; column < columns; column++)
			{
				g.DrawLine(header3dPen1,cumulativeWidth +1,0,cumulativeWidth +1,rowHeight);
				cumulativeWidth += columnWidth;//[column];
					
				//column grid lines
				g.DrawLine(gridPen,cumulativeWidth ,rowHeight,cumulativeWidth ,colHeight);

				//the '3D' lines for the column headers
				g.DrawLine(header3dPen,cumulativeWidth ,0,cumulativeWidth ,rowHeight);
						
			}
		    
			//Draw the rows
			for (int row = 1; row <= (rows + 1 ); row++)
			{
				int yPos = (row * rowHeight);

				//row gridlines
				g.DrawLine(gridPen,rowHeaderWidth,yPos,totalWidth,yPos);
			  
				//row header 3D lines
				g.DrawLine(header3dPen,0,yPos,rowHeaderWidth,yPos);
				if(row!=1)
					g.DrawLine(header3dPen1,0,yPos+1-rowHeight,rowHeaderWidth,yPos+1-rowHeight);
			}
          
			//Draw a border across the top
			//g.DrawLine(pen,0,0,Width - 1,0);
          
			//draw grey line across right of row headers and bottom of col headers
			//			if(hScrollBar.Visible == true)
			//				g.DrawLine(header3dPen,rowHeaderWidth,0,rowHeaderWidth,(rows+1)*rowHeight - hScrollBar.Height);
			//			else
			g.DrawLine(header3dPen,rowHeaderWidth,0,rowHeaderWidth,(rows+1 )*rowHeight);
			g.DrawLine(header3dPen,0,rowHeight,totalWidth,rowHeight);

			gridPen.Dispose();
			header3dPen.Dispose(); 
			header3dPen1.Dispose();
			backgroundBrush.Dispose();
			bgBrush.Dispose();
			headerBrush.Dispose();
		}

		protected void paintHeaders(Graphics g)
		{
			Brush textBrush = new SolidBrush(ForeColor);
			Brush headerBrush = new SolidBrush(SystemColors.Control);
			int currentRow = (int) -this.AutoScrollPosition.Y/rowHeight;
			StringFormat sf = new StringFormat();
			int pos = rowHeight;
			int cumulativeWidth = rowHeaderWidth;
			//Fill the headers
			for (int column = 0; column < columns; column++)
			{
				g.FillRectangle(headerBrush,cumulativeWidth+2,1,columnWidth-2,rowHeight-2);
				RectangleF rectf = new RectangleF(cumulativeWidth +3,rowOffset,columnWidth,rowHeight);
				g.DrawString(headers[column],Font,textBrush,rectf,sf);
				cumulativeWidth += columnWidth;
			}

			//Fill the rowHeaders
			currentRow = 0;
			if (currentRow < 0)
				currentRow = 0;
			while ( currentRow < rows)
			{
				//cumulativeWidth = 0;
				g.FillRectangle(headerBrush,1,pos + rowOffset+1,rowHeaderWidth-1,rowHeight-3);
				RectangleF rectf = new RectangleF(4,pos + rowOffset,rowHeaderWidth,rowHeight - 2);
				g.DrawString(rowHeaders[currentRow],Font,textBrush,rectf,sf);
				currentRow++;
				    
				pos += rowHeight;
			}
			textBrush.Dispose();
			headerBrush.Dispose();
		}
		protected void paintAllCells(Graphics g)
		{
			StringFormat sf = new StringFormat();
			int pos = rowHeight;
			int cumulativeWidth = rowHeaderWidth;

			int currentRow = (int) -this.AutoScrollPosition.Y/rowHeight;

			Brush textBrush = new SolidBrush(ForeColor);
			Brush selTextBrush = new SolidBrush(selectedForeColor); 
			Brush selectedBrush = new SolidBrush(selectedColor);

			//Fill the body
			pos = rowHeight;
			currentRow = 0;
			if (currentRow < 0)
				currentRow = 0;

			Point[] curp = this.normalizeSelection(true);
			while ( currentRow < rows) //pos < Height &&
			{
				cumulativeWidth = rowHeaderWidth;
				for (int column = 0; column < columns; column++)
				{
					//in the selected cell...
					if (currentRow == currSel.SelectedRow && column == currSel.SelectedColumn)
					{
						//draw the selected box
						g.FillRectangle(selectedBrush,cumulativeWidth+1,pos+1,columnWidth-1,rowHeight-1);
								
						RectangleF rectf = new RectangleF(cumulativeWidth +4,pos + rowOffset+1,columnWidth,rowHeight - 2);
						g.DrawString(info[column,currentRow],Font,selTextBrush,rectf,sf);

						//draw a gray box for around the textBox
						if(editBox.Visible && mouseover.X > -1 && mouseover.Y > -1)
						{
							Brush b = new SolidBrush(Color.LightGray);
							g.FillRectangle(b,cumulativeWidth+1,pos+1,columnWidth-1,rowHeight-1);
							b.Dispose();
						}
					}
					else
					{
						RectangleF rectf = new RectangleF(cumulativeWidth + 4,pos + rowOffset+1,columnWidth,rowHeight - 2);
								
						//draw the other cells
						if(isSelected(curp, column,currentRow))
						{
							g.FillRectangle(selectedBrush,cumulativeWidth+1,pos+1,columnWidth-1,rowHeight-1);
							g.DrawString(info[column,currentRow],Font,selTextBrush,rectf,sf);
						}
						else// if((repaintmode == M_REPAINT_MOVE && this.currSel.LastSelectedColumn == column && this.currSel.LastSelectedRow == currentRow)||repaintmode == M_REPAINT_ALL)
						{
							if(paintmode == M_PAINT_ALL)
							{
								Brush cellBrush = new SolidBrush(cellColors[column,currentRow]);
								g.FillRectangle(cellBrush,cumulativeWidth+1,pos+1,columnWidth-1,rowHeight-1);
								cellBrush.Dispose();
							}
							if(info[column,currentRow] != null)
								g.DrawString(info[column,currentRow],Font,textBrush,rectf,sf);					
						}
							
					}
					cumulativeWidth += columnWidth;
				}
				currentRow++;
				    
				pos += rowHeight;
			} 

			//cb.Dispose();
			textBrush.Dispose();
			selTextBrush.Dispose(); 
			selectedBrush.Dispose();
		}
		protected void paintSelectCells(Graphics g)
		{
			
			Brush textBrush = new SolidBrush(ForeColor);
			Brush selTextBrush = new SolidBrush(selectedForeColor); 
			Brush selectedBrush = new SolidBrush(selectedColor);
			Brush cb = new SolidBrush(BackColor);
			StringFormat sf = new StringFormat();

			Point[] curp = this.normalizeSelection(true); 
			///*
			if(repData.RepaintSelection || currSel.HasSelectionChanged)
			{
				//essentially this part's function:
				//paint all new selected cells
				//dont paint overlap (cells that were selected)
				//enqueue cells that were selected & now not
				Point[] oldp = this.normalizeSelection(false); 

				//Console.WriteLine(curp[0] + " | " +curp[1] + " :: " + oldp[0] + " | "+ oldp[1]);

				int maxrow = Math.Max(Math.Max(curp[1].Y, oldp[1].Y),Math.Max(curp[0].Y, oldp[0].Y));
				int maxcol = Math.Max(Math.Max(curp[1].X, oldp[1].X),Math.Max(curp[0].X, oldp[0].X));
				int row = Math.Min(curp[0].Y, oldp[0].Y);
				if(row<0)
					row = curp[0].Y;

				int pos = rowHeight*(row+1);
				for( ; row > -1 && row<=maxrow; row++ ) //pos < Height &&
				{
					int column = Math.Min(curp[0].X, oldp[0].X);
					if(column<0)
						column = curp[0].X;
					//Console.WriteLine(column + "," + row + " :: " + maxcol + "," + maxrow);
					int cumulativeWidth = rowHeaderWidth + columnWidth*column;
					for(; column > -1 && column <=maxcol; column++)
					{
						if(isSelected(curp,column,row) && !isSelected(oldp,column,row))
						{
							RectangleF rectf = new RectangleF(cumulativeWidth + 4,pos + rowOffset+1,columnWidth,rowHeight - 2);
							g.FillRectangle(selectedBrush,cumulativeWidth+1,
								pos+1,columnWidth-1,rowHeight-1);
							g.DrawString(info[column,row],Font,selTextBrush,rectf,sf);
						}
						if(!isSelected(curp,column,row) && isSelected(oldp,column,row))
							this.repData.addCell(new Point(column,row));//enqueue the cell

						cumulativeWidth += columnWidth;
					}//end col for
					pos += rowHeight;
				}//end row for
			}
			//*/

			///*
			while(repData.getCellCount() > 0)
			{
				Point p = repData.getNextCell();

				if(p.X <0 || p.Y < 0)
					continue;

				int x = RowHeaderWidth + p.X*columnWidth+1;
				int y = (p.Y+1)*rowHeight+1;
				RectangleF rectf = new RectangleF(x+3,y + rowOffset,columnWidth-1,rowHeight - 2);

				if(isSelected(curp,p.X,p.Y))
				{
					g.FillRectangle(selectedBrush,x,y,columnWidth-1,rowHeight-1);
					if(info[p.X,p.Y] != null)
						g.DrawString(info[p.X,p.Y],Font,selTextBrush,rectf,sf);
				}
				else
				{
					if(paintmode == M_PAINT_ALL)
					{
						Brush cellBrush = new SolidBrush(cellColors[p.X,p.Y]);
						g.FillRectangle(cellBrush,x,y,columnWidth-1,rowHeight-1);
						cellBrush.Dispose();
					}
					else
						g.FillRectangle(cb,x,y,columnWidth-1,rowHeight-1);
					if(info[p.X,p.Y] != null)
						g.DrawString(info[p.X,p.Y],Font,textBrush,rectf,sf);	
				}
			}
			
			//draw a gray box for around the textBox
			if(editBox.Visible && mouseover.X > -1 && mouseover.Y > -1)
			{
				Brush b = new SolidBrush(Color.LightGray);
				g.FillRectangle(b,RowHeaderWidth + curp[0].X*columnWidth+1,
					(curp[0].Y+1)*rowHeight+1,columnWidth-1,rowHeight-1);
				b.Dispose();
				repData.addCell(curp[0]);
			}

			//dequeue and paint that cell
			cb.Dispose();
			textBrush.Dispose();
			selTextBrush.Dispose(); 
			selectedBrush.Dispose();
		}

		//points MUST be normalized!!!
		private bool isSelected(Point[] sel, int x, int y)
		{
			return (sel[0].X == x && sel[0].Y == y) || 
				(sel[0].X>-1 && sel[0].Y>-1 && sel[1].X>-1 && sel[1].Y>-1 && x>-1 && y>-1 &&
				x >= sel[0].X && x <= sel[1].X && y >= sel[0].Y && y <= sel[1].Y);
		}
		protected override void OnPaint(PaintEventArgs e)
		{
			
			Graphics g = e.Graphics;

			System.Drawing.Drawing2D.Matrix mx=new System.Drawing.Drawing2D.Matrix(1,0,0,1,AutoScrollPosition.X,AutoScrollPosition.Y);
			e.Graphics.Transform=mx;
			if(repData.RepaintAll)
				paintBackground(buff.g);
			if(repData.RepaintAll || repData.RepaintHeader)
				paintHeaders(buff.g);

			//TODO: here we must check if we repaint all.
			//if not then we paint the selection or the cells.

			if(repData.RepaintAll)
				paintAllCells(buff.g);
			else
				paintSelectCells(buff.g);

			repData.clear();
			currSel.clearOld();

			this.UpdateScrollBars();
			
			buff.Render(g);

			xhairs.paintCrosshairs(g,new Size(this.RowHeaderWidth,this.RowHeight),
									new Size(this.columnWidth,this.rowHeight),
									GridSize);
		}//end paint

		protected override void OnPaintBackground(PaintEventArgs e){}

		#endregion
		
		#region Component Designer generated code
		protected override void Dispose(bool disposing)
		{
			if (disposing && components != null)
				components.Dispose();
			base.Dispose(disposing);
		}
		/// <summary> 
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.AutoScroll = true;

			
		//	this.SetAutoScrollMargin(30,30);
			this.AutoScrollMinSize = new System.Drawing.Size(500,400);
		//	this.VScroll = true;
		//	this.HScroll = true;

			this.editBox = new System.Windows.Forms.TextBox();
			this.SuspendLayout();
			// 
			// editBox
			// 
			this.editBox.BackColor = System.Drawing.SystemColors.ScrollBar;
			this.editBox.BorderStyle = System.Windows.Forms.BorderStyle.None;
			this.editBox.ForeColor = System.Drawing.SystemColors.WindowText;
			this.editBox.Location = new System.Drawing.Point(16, 40);
			this.editBox.Name = "editBox";
			this.editBox.Size = new System.Drawing.Size(64, 13);
			this.editBox.TabIndex = 2;
			this.editBox.TabStop = false;
			this.editBox.Text = "";
			this.editBox.WordWrap = false;
			this.editBox.KeyPress += new System.Windows.Forms.KeyPressEventHandler(this.editBox_KeyPress);
			this.editBox.Leave += new System.EventHandler(this.editBox_Leave);
			// 
			// Grid
			// 
			this.Controls.Add(this.editBox);
			this.Name = "Grid";
			this.KeyPress += new System.Windows.Forms.KeyPressEventHandler(this.Grid_KeyPress);
			this.KeyDown += new System.Windows.Forms.KeyEventHandler(this.Grid_KeyDown);
			this.ResumeLayout(false);

			//this.HorzScrollValueChanged += new System.Windows.Forms.ScrollEventHandler(this.this_HorzScrollValueChanged);
			//this.VertScrollValueChanged += new System.Windows.Forms.ScrollEventHandler(this.this_VertScrollValueChanged);
			
		}
		#endregion
		
		#region Utils
		void UpdateScrollBars()
		{
			rowsVisible = BoxHeight / rowHeight;
			this.AutoScrollMinSize = new Size(calculateTotalWidth(),(this.GridSize.Height+1)*this.rowHeight);
		}

		//extra methods..............

		//if true then it will normalize the most recent selection
		//values else it will normalize the old selection vals;
		//returns upper left in [0] and lower right in [1]
		private Point[] normalizeSelection(bool mostrecent)
		{
			Point first = currSel.OldSelectedCell;
			Point last = currSel.OldLastSelectedCell;
			if(mostrecent)
			{
				first = currSel.SelectedCell;
				last = currSel.LastSelectedCell;
			}
			Point[] p = new Point[2];

			p[0] = new Point(first.X, first.Y);
			p[1] = new Point(last.X, last.Y);

			if(first.X > -1 && last.X > -1 && first.X > last.X)
			{
				int sw = p[0].X;
				p[0].X = p[1].X;
				p[1].X = sw;
			}
			if(first.Y > -1 && last.Y > -1 && first.Y > last.Y)
			{
				int sw = p[0].Y;
				p[0].Y = p[1].Y;
				p[1].Y = sw;
			}
			return p;
		}

		private bool isInSelectedCellRange(int column, int row)
		{
			if(currSel.SelectedRow < 0 || currSel.SelectedColumn < 0 ||
				currSel.LastSelectedRow < 0 || currSel.LastSelectedColumn < 0)
				return false;

			//1st cell above and behind last cell
			if(currSel.SelectedRow <= currSel.LastSelectedRow && currSel.SelectedColumn <= currSel.LastSelectedColumn)
				return column >= currSel.SelectedColumn && column <= currSel.LastSelectedColumn &&
					row >= currSel.SelectedRow && row <= currSel.LastSelectedRow;

			//1st cell below and behind last cell
			if(currSel.SelectedRow >= currSel.LastSelectedRow && currSel.SelectedColumn <= currSel.LastSelectedColumn)
				return column >= currSel.SelectedColumn && column <= currSel.LastSelectedColumn &&
					row <= currSel.SelectedRow && row >= currSel.LastSelectedRow;

			//1st cell below and in front of last cell
			if(currSel.SelectedRow >= currSel.LastSelectedRow && currSel.SelectedColumn >= currSel.LastSelectedColumn)
				return column <= currSel.SelectedColumn && column >= currSel.LastSelectedColumn &&
					row <= currSel.SelectedRow && row >= currSel.LastSelectedRow;

			//1st cell above and in front of last cell
			if(currSel.SelectedRow <= currSel.LastSelectedRow && currSel.SelectedColumn >= currSel.LastSelectedColumn)
				return column <= currSel.SelectedColumn && column >= currSel.LastSelectedColumn &&
					row >= currSel.SelectedRow && row <= currSel.LastSelectedRow;

			return false;
		}

		private bool intValidate(string text)
		{
			int i;
			char[] str = text.ToCharArray();
			for(i = 0; i < text.Length; i++)
			{
				if(str[i] < 48 || str[i] > 57)
					return false;
			}
			
			return true;

		}

		private bool validate(string text)
		{
			int i, deccount = 0;
			char[] str = text.ToCharArray();
			for(i = 0; i < text.Length; i++)
			{
				if((str[i] < 48 || str[i] > 57) && str[i] != 46 && str[i] != 45)
					return false;
				if(i>0 && str[i] == 45)//neg
					return false;
				if(str[i] == 46)
					deccount++;
			}

			if(deccount > 1)
				return false;
			
			return true;

		}

		protected void selectNextCell(int mode)
		{
			//mode == 1 : enter/move down 1
			//mode == 2 : tab/move right 1
			//mode == 3 : enter/move up 1
			//mode == 4 : tab/move left 1

			if(currSel.SelectedRow >= 0 && currSel.SelectedColumn >= 0 && !editBox.Visible)
			{
				int c = currSel.SelectedColumn;
				int r = currSel.SelectedRow;

				if(mode == 1) //down
					r++;
				else if(mode == 2) //right
					c++;
				else if(mode == 3) //up
					r--;
				else if(mode == 4) //left
					c--;

				if(c < 0)
					c = columns-1;
				else if(c > columns-1)
					c = 0;
				else if(r > rows-1)
					r = 0;
				else if(r < 0)
					r = rows-1;

				currSel.LastSelectedColumn = -1;
				currSel.LastSelectedRow = -1;

				currSel.SelectedColumn = c;
				currSel.SelectedRow = r;

				ensureVisible();
				OnSelectedRowChanged(EventArgs.Empty);
			}
		}

		protected void ensureVisible()
		{
			try
			{
			//	bool moved = false;

			//	MessageBox.Show(this.AutoScrollPosition.X.ToString() + ", " + this.AutoScrollPosition.Y.ToString());

				if((this.currSel.SelectedColumn+1)*this.columnWidth+this.RowHeaderWidth+this.AutoScrollPosition.X >= this.BoxWidth)
				{
			//		moved = true;

					int xpos = (this.BoxWidth + 10-((this.currSel.SelectedColumn+2)*this.columnWidth + this.RowHeaderWidth));
					Point p = new Point(-xpos,-this.AutoScrollPosition.Y);// = this.currSel.SelectedColumn+1*this.columnWidth[0] - this.BoxWidth;
					this.AutoScrollPosition = p;
				}
				else if((this.currSel.SelectedColumn)*this.columnWidth+this.RowHeaderWidth+this.AutoScrollPosition.X <= 0)
				{
			//		moved = true;
					this.AutoScrollPosition = new Point((this.currSel.SelectedColumn)*this.columnWidth + this.RowHeaderWidth,-this.AutoScrollPosition.Y);
				}
				else if(((this.currSel.SelectedRow+2)*this.rowHeight + this.AutoScrollPosition.Y) >= this.BoxHeight)
				{
			//		moved = true;
					int ypos = -(this.BoxHeight-(this.currSel.SelectedRow+3)*this.rowHeight);
					this.AutoScrollPosition = new Point(-this.AutoScrollPosition.X,ypos);
				}
				else if((this.currSel.SelectedRow+1)*this.rowHeight+this.AutoScrollPosition.Y <= 0)
				{
			//		moved = true;
					this.AutoScrollPosition = new Point(-this.AutoScrollPosition.X,this.currSel.SelectedRow*this.rowHeight);
				}

//				if(!moved)
//					repaintmode = M_REPAINT_MOVE;

				Refresh();
			}
			catch
			{
			}
		}

		protected void displayBox()
		{
			if(mouseover.Y == -1 && mouseover.X >= 0 && mouseover.X< columns &&
				headerEditable[mouseover.X] && !ReadOnly)
			{
				displayBox(mouseover.X,mouseover.Y);
			}

			else if(currSel.SelectedColumn >= 0 && currSel.SelectedColumn < columns &&
				currSel.SelectedRow >= 0 && currSel.SelectedRow < rows && !ReadOnly)
			{
				displayBox(currSel.SelectedColumn,currSel.SelectedRow);
			}
		}

		private void displayBox(int col, int row)
		{
			int boxX, boxY, boxWidth, boxHeight,i;

			boxX = rowHeaderWidth + this.AutoScrollPosition.X;
			for(i = 0; i<col; i++)
				boxX+=columnWidth;

			boxWidth = columnWidth;

//			if(row < 0)

			boxY = rowHeight*(row+1)+this.AutoScrollPosition.Y;
			boxHeight = rowHeight;

			editBox.Top = boxY+2;
			editBox.Left = boxX+4;
			editBox.Width = boxWidth-4;
			editBox.Height = boxHeight;

			if(col < 0)
				editBox.Text = this.getRowHeader(row);
			else if(row < 0)
				editBox.Text = this.getHeader(col);
			else
				editBox.Text = this.getText(col,row);

			editBox.SelectAll();

			editBox.Visible = true;
			editBox.Enabled = true;
			editBox.Focus();

			Refresh();
		}
		#endregion
		//----------------------------------------

		#region Events

		private void Grid_Load(object sender, EventArgs e)
		{
			int w = Math.Max(this.ClientRectangle.Width,calculateTotalWidth()+2);
			int h = Math.Max(this.ClientRectangle.Height,calculateTotalHeight()+1);
			if(!buff.CanDoubleBuffer())
				//buff.CreateDoubleBuffer(null, calculateTotalWidth(), calculateTotalHeight(), true);
				buff.CreateDoubleBuffer(null, w, h, true);
		}

		private void Grid_Resize(object sender, EventArgs e)
		{
			int w = Math.Max(this.ClientRectangle.Width,calculateTotalWidth()+2);
			int h = Math.Max(this.ClientRectangle.Height,calculateTotalHeight()+1);
			//buff.CreateDoubleBuffer(null, calculateTotalWidth(), calculateTotalHeight(), true);
			buff.CreateDoubleBuffer(null, w, h, true);
			repData.RepaintAll = true;
			Invalidate(); // Force a repaint after has been resized 
		}

		protected override void OnResize(EventArgs e)
		{
			base.OnResize(e);
			UpdateScrollBars();
			Refresh();
			this.Focus();
		}
		
		public int getMouseOverCol()
		{
			return mouseovercolumn;
		}

		protected override void OnMouseMove(MouseEventArgs e)
		{
			base.OnMouseMove(e);
			
			
			int widthtest = rowHeaderWidth + this.AutoScrollPosition.X;

			int i;
			for(i = 0; i < columns && widthtest<e.X; i++)
				widthtest+= columnWidth;

			mouseovercolumn = i-1;

			if (e.Button == MouseButtons.Left)
			{
				if (e.Y > rowHeight && e.X > rowHeaderWidth+this.AutoScrollPosition.X)//then its not in the headers
				{
					int rowClicked = (e.Y / rowHeight) - 1;

					widthtest = rowHeaderWidth +this.AutoScrollPosition.X;

					for(i = 0; i < columns && widthtest<e.X; i++)
						widthtest+= columnWidth;
			

					int currentRow = (e.Y - this.AutoScrollPosition.Y)/rowHeight -1;
					if (currentRow < 0)
						currentRow = 0;
			
					int currentCol = i-1;

					currSel.LastSelectedRow = Math.Min(currentRow,rows-1);
					currSel.LastSelectedColumn = Math.Min(currentCol,columns-1);
				}
				Refresh();
				OnSelectedRowChanged(EventArgs.Empty);
			}
		}
    
		protected override void OnMouseUp(MouseEventArgs e)
		{
			base.OnMouseUp(e);
      
			if (adjustingColumn != -1)
			{
				adjustingColumn = -1;
				Capture = false;
				UpdateScrollBars();
			}
		}
    
		protected override void OnMouseDown(MouseEventArgs e)
		{
			base.OnMouseDown(e);
			editBox.Visible = false;
			editBox.Enabled = false;
			this.Focus();

			//See if the user pressed the button when the mouse was in the header
			if (e.Button == MouseButtons.Left)
			{
				currSel.LastSelectedColumn = -1;
				currSel.LastSelectedRow = -1;

				mouseover = new Point(-2, -2);

				int widthtest = rowHeaderWidth + this.AutoScrollPosition.X;

				int i;

				for(i = 0; i < columns && widthtest<e.X; i++)
					widthtest+= columnWidth;
			
				mouseover.Y = (e.Y -this.AutoScrollPosition.Y)/rowHeight -1;
				if (mouseover.Y < -1)
					mouseover.Y = -2;

				if(i > columns)
				{
					mouseover.Y = -2;
					mouseover.X = -2;
				}
				else
					mouseover.X = i-1;

				if (mouseover.X > -1 && mouseover.Y > -1 && mouseover.X < columns && mouseover.Y < rows)//e.Y > rowHeight && e.X > rowHeaderWidth+this.AutoScrollPosition.X)//then its not in the headers
				{
					currSel.SelectedColumn = mouseover.X;
					currSel.SelectedRow = mouseover.Y;
				}
			}
			Refresh();
      
			OnSelectedRowChanged(EventArgs.Empty);
		}

		protected override void OnDoubleClick(EventArgs e)
		{
			base.OnDoubleClick (e);
			displayBox();
		}

    
		protected virtual void OnSelectedRowChanged(EventArgs e)
		{
			if (SelectedRowChanged != null)
				SelectedRowChanged(this,e);
		}

		protected virtual void OnCellChanged(CellEvent e)
		{
			if (CellChanged != null)
				CellChanged(this,e);
			//	e.
		}

		protected virtual void OnColumnHeaderChanged(CellEvent e)
		{
			if (ColumnHeaderChanged != null)
				ColumnHeaderChanged(this,e);
			//	e.
		}
		
		protected override bool IsInputKey(Keys keyData)
		{
			bool ret = true;

			switch (keyData)
			{
				case Keys.Left:
					break;
				case Keys.Right:
					break;
				case Keys.Up:
					break;
				case Keys.Down:
					break;
				case Keys.Tab:
					break;
				case Keys.Enter:
					break;
				default:
					ret = base.IsInputKey(keyData);
					break;
			}
			return ret;
		}


		private void editBox_Leave(object sender, System.EventArgs e)
		{
			editBox.Visible = false;
		}

		private void editBox_KeyPress(object sender, System.Windows.Forms.KeyPressEventArgs e)
		{
			bool incolheader = mouseover.Y == -1 && headerEditable[mouseover.X]; //|| mouseover.X == -1;
			int md = mode;
			if(incolheader)
				md = headermode;
			
			if(e.KeyChar == (char)13 || e.KeyChar == (char)9)//enter or tab is pressed
			{
				if(incolheader && !headers[mouseover.X].Equals(editBox.Text))
				{
					this.setHeader(mouseover.X,editBox.Text);
					SimpleGrid.CellEvent ce = new CellEvent();
					ce.Column = mouseover.X;
					ce.Row = -1;
					OnColumnHeaderChanged(ce);
				}
				if(!incolheader && (info[currSel.SelectedColumn,currSel.SelectedRow] == null || !info[currSel.SelectedColumn,currSel.SelectedRow].Equals(editBox.Text) ))
				{
					try
					{
						if(mode == 0)
							this.setValue(currSel.SelectedColumn,currSel.SelectedRow,Convert.ToSingle(editBox.Text));
						else if(mode == 1)
							this.setValue(currSel.SelectedColumn,currSel.SelectedRow,(int)Convert.ToSingle(editBox.Text));
							//this.setValue(currSel.SelectedColumn,currSel.SelectedRow,Int32.Parse(editBox.Text));
						else if(mode == 2)
							this.setValue(currSel.SelectedColumn,currSel.SelectedRow,editBox.Text);

						SimpleGrid.CellEvent ce = new CellEvent();
						ce.Column = currSel.SelectedColumn;
						ce.Row = currSel.SelectedRow;
						OnCellChanged(ce);
					}
					catch{}
				}
				if(e.KeyChar == (char)9)
					selectNextCell(2);//tab to next cell
				
				editBox.Visible = false;
				editBox.Enabled = false;
	
				this.Focus();
				Refresh();
			}
			else if(md == MODE_INT)
			{
				if(e.KeyChar <= '9' && e.KeyChar >= '0' || e.KeyChar == 8)
					e.Handled = false;
				else
					e.Handled = true;
			}
			else if(md == MODE_DOUBLE)
			{
				handleDoubleKeyPress(editBox,e,false);
			}

		}

		public static void handleDoubleKeyPress(TextBox box, System.Windows.Forms.KeyPressEventArgs e, bool isSigned)
		{
			string text = box.Text;
			if(box.SelectedText == text)
			text = "";

			char divchar = '.';
			string div = System.Globalization.NumberFormatInfo.CurrentInfo.NumberDecimalSeparator;
			if(e.KeyChar == '.' || e.KeyChar == ',')
				if(div != null && div.Length > 0)
					divchar = (char)div.ToCharArray()[0];

			if(div != ",")
				div = @"\.";

			string regexp = @"[0-9]*"+div+"[0-9]*";
			if(isSigned)
				regexp = @"-?[0-9]*"+div+"[0-9]*";
			Regex re = new Regex(regexp);

			bool neg = false;
			if(isSigned && text.Length == 0)
				neg = true;
						
			bool tst = re.IsMatch(text);
			if((e.KeyChar < '0' || e.KeyChar > '9') && !(e.KeyChar == 8) && (tst || !(e.KeyChar == divchar)) && !(e.KeyChar == 13) && !(neg && e.KeyChar == '-'))
				e.Handled = true;
		}

		private void Grid_KeyPress(object sender, System.Windows.Forms.KeyPressEventArgs e)
		{
			if(e.KeyChar == (char)13 || e.Equals(System.Windows.Forms.Keys.Down))
			{
				e.Handled = true;
				selectNextCell(1);
				Refresh();
			}
			if((int)e.KeyChar == 9)// || e.KeyChar == System.Windows.Forms.Keys.Right)
			{
				e.Handled = true;
				selectNextCell(2);
				Refresh();
			}
			if((int)e.KeyChar > 47 && (int)e.KeyChar < 58)
			{
				e.Handled = true;
				displayBox();
				editBox.Text = e.KeyChar.ToString();
				editBox.Select(1,0);
				Refresh();
			}

		}

		private void Grid_KeyDown(object sender, System.Windows.Forms.KeyEventArgs e)
		{
			//MessageBox.Show("key: " + e.KeyCode);
			if(e.KeyCode == Keys.Right)
			{
				e.Handled = true;
				selectNextCell(2);
				Refresh();
			}
			if(e.KeyCode == Keys.Down)
			{
				e.Handled = true;
				selectNextCell(1);
				Refresh();
			}

			if(e.KeyCode == Keys.Up)
			{
				e.Handled = true;
				selectNextCell(3);
				Refresh();
			}

			if(e.KeyCode == Keys.Left)
			{
				e.Handled = true;
				selectNextCell(4);
				Refresh();
			}
		}
		#endregion
	}
}