using System;
using System.Collections;
using System.Drawing;

namespace SimpleGrid
{

	public class GridSelection
	{
		#region init
		private const int DONT_USE = int.MinValue;
		private Point[] ps;
		private Point[] old;

		bool selChanged;

		public GridSelection()
		{
			ps = new Point[2];
			ps[0] = new Point(-1,-1);
			ps[1] = new Point(-1,-1);

			selChanged = false;

			old = new Point[2];
			old[0] = new Point(-1,-1);
			old[1] = new Point(-1,-1);
		}
		#endregion

		#region Properties

		public Point SelectedCell{
			get{return ps[0];}
			set{
				selChanged = true;
				old[0] = ps[0];
				ps[0] = value;
			}
		}
		public Point LastSelectedCell{
			get{return ps[1];}
			set{
				selChanged = true;
				old[1] = ps[1];
				ps[1] = value;
			}
		}

		public bool HasSelectionChanged{get{return selChanged;}}
		public Point OldSelectedCell{get{return old[0];}}
		public Point OldLastSelectedCell{get{return old[1];}}

		public int SelectedRow{
			get{return ps[0].Y;}
			set{makeNewCell(0,DONT_USE,value);}
		}
		public int SelectedColumn{
			get{return ps[0].X;}
			set{makeNewCell(0,value,DONT_USE);}
		}
		public int LastSelectedRow{
			get{return ps[1].Y;}
			set{makeNewCell(1,DONT_USE,value);}
		}
		public int LastSelectedColumn{
			get{return ps[1].X;}
			set{makeNewCell(1,value,DONT_USE);}
		}

		private void makeNewCell(int index, int x, int y)
		{
			if(x == DONT_USE)
			{
				old[index].Y = ps[index].Y;
				ps[index].Y = y;
			}
			else if(y == DONT_USE)
			{
				old[index].X = ps[index].X;
				ps[index].X = x;
			}
			else
			{
				old[index] = ps[index];
				ps[index] = new Point(x,y);
			}
			selChanged = true;
		}
		#endregion

		public void clearOld(){
			old[0].Y = ps[0].Y;
			old[0].X = ps[0].X;
			old[1].Y = ps[1].Y;
			old[1].X = ps[1].X;
			selChanged = false;
		}
		//public Point[] getCurrentSelection(){return ps;}
		//public Point[] getOldSelection(){return old;}
	}

	public class RepaintData
	{
		private bool all = true;
		private bool header = true;
		private bool selection = true;
		private Queue cells;

		public bool RepaintAll
		{
			get{return all;}
			set{all = value;}
		}

		public bool RepaintHeader
		{
			get{return header;}
			set{header = value;}
		}

		public bool RepaintSelection
		{
			get{return selection;}
			set{selection = value;}
		}

		public RepaintData()
		{
			cells = new Queue();
		}

		public void clear()
		{
			//cells.Clear();
			all = false;
			header = false;
			selection = false;
		}

		public void addCell(Point p)
		{
			cells.Enqueue(p);
		}

		public int getCellCount()
		{
			return cells.Count;
		}
		public Point getNextCell()
		{
			if(cells.Count > 0)
				return (Point)cells.Dequeue();
			else
				return Point.Empty;
		}
	}
	/// <summary>
	/// Summary description for Util.
	/// </summary>
	public class Util
	{
		public Util()
		{
			//
			// TODO: Add constructor logic here
			//
		}
	}
}
