#include "stdafx.h"
#include "common.h"
#include "DatalogOptions.h"
#include "Engine.h"

void bre20::Common::handleDoubleKeyPress(TextBox* box, System::Windows::Forms::KeyPressEventArgs * e, bool isSigned)
{
	String* text = box->Text;
	if(box->SelectedText->Equals(text))
		text = "";

	char divchar = '.';
	String* div = getNumberDivider();
	if(e->KeyChar == '.' || e->KeyChar == ',')
		if(div != NULL && div->Length > 0)
			divchar = (char)div->ToCharArray()[0];

	if(!div->Equals(","))
		div = S"\\.";

	String* regexp = String::Concat(S"[0-9]*",div,S"[0-9]*");
	if(isSigned)
		regexp = String::Concat(S"-?[0-9]*",div,S"[0-9]*");
	Regex * re = new Regex(regexp);

	bool neg = false;
	if(isSigned && text->Length == 0)
		neg = true;
				
	bool tst = re->IsMatch(text);
	if((e->KeyChar < '0' || e->KeyChar > '9') && !(e->KeyChar == 8) && (tst || !(e->KeyChar == divchar)) && !(e->KeyChar == 13) && !(neg && e->KeyChar == '-'))
		e->Handled = true;
}
void bre20::Common::handleIntKeyPress(TextBox* box, System::Windows::Forms::KeyPressEventArgs * e, bool isSigned)
{
	bool neg = false;
	if(isSigned && box->Text->Length==0)
		neg = true;

	if((e->KeyChar < '0' || e->KeyChar > '9') && !(e->KeyChar == 8) && !(e->KeyChar == 13) && !(neg && e->KeyChar == '-'))
		e->Handled = true;
}

String* bre20::Common::openFileDialog(bool open, bool multifile,String* file,String* filter)[]
{	//S"CSV files (*.csv)|*.csv|All files (*.*)|*.*"

	FileDialog* dlg;

	if(open)
		dlg = new OpenFileDialog();
	else
		dlg = new SaveFileDialog();

	if(open && multifile)
		static_cast<OpenFileDialog*>(dlg)->Multiselect = true;

	dlg->Filter = filter;
	if(file != NULL)
		dlg->FileName = file;
	
	if(dlg->ShowDialog() != DialogResult::OK)
	{
//		delete dlg;
		return NULL;
	}

	String* fnames[];
	if(open && multifile)
		fnames = dlg->FileNames;
	else{
		fnames = new String*[1];
		fnames[0] = dlg->FileName;
	}

//	delete dlg;

	return fnames;
}
//matcher is the pattern to match
//matchee is the string to try to match matcher in
bool bre20::Common::isMatch(String* matcher, String* matchee, bool caseSen)
{
	if(matcher == NULL || matchee == NULL)
		return false;

	if(!caseSen)
	{
		matcher = matcher->ToLower();
		matchee = matchee->ToLower();
	}

	String* delim = "|";
	String* strs[] = matcher->Split(delim->ToCharArray());

	for(int i = 0; i< strs->Length; i++)
		if(matchee->IndexOf(strs[i]->Trim()) >= 0)
			return true;

	return false;
}

float bre20::Common::convertToLocale(String* val)
{
	String* div = getNumberDivider();
	if(div->Length <1)
		return 0.0f;

	for(int i = 0; i<val->Length; i++)
	{
		if((val->Chars[i] == '.' || val->Chars[i] == ',') && val->Chars[i] != div->Chars[0])
		{
			val = String::Concat(val->Substring(0,i), div, val->Substring(i+1, val->Length-i-1));
			break;
		}
	}

	return extractDouble(val);
}

float bre20::Common::extractDouble(String* str)
{
	String* div = getNumberDivider();
	if(!div->Equals(","))
		div = S"\\.";

	String* rex = String::Concat(S"-?[0-9]*(", div,S"[0-9]+)?");
	Regex * re = new Regex(rex);

	Match * m= re->Match(str);
	str = m->Value;

	return Convert::ToSingle(str);
}

int bre20::Common::extractInt(String* str)
{
	Regex * re = new Regex(S"-?[0-9]*");

	Match * m= re->Match(str);
	str = m->Value;

	return Convert::ToInt32(str);
}

bool bre20::Common::checkCom(String * com)
{
	bool good = true;

	if(com->Substring(0,3)->Equals("COM")){
		try{
			Convert::ToInt32(com->Substring(3,com->Length - 3));
		}
		catch(...){
			good = false;
		}
	}
	else
		good = false;

	return good;

}

//-------------------------------------------------
float bre20::Common::getAfr(unsigned char cmd , int o2type)
{
	float val;
	//WB processing here...
	if(o2type == DatalogOptions::TENARROW)
	{
		//techedge narrow out.
		float volts __gc[] = {5.1F ,0.8739F ,0.8735F ,0.873F ,0.8725F ,0.8720F ,0.8718F ,
		0.8712F ,0.869F ,0.852F ,0.841F ,0.832F ,0.818F ,0.805F ,0.792F ,0.779F ,0.763F ,
		0.741F ,0.718F ,0.683F ,0.606F ,0.231F ,0.171F ,0.107F ,0.063F ,0.050F ,0.038F ,
		0.033F ,0.031F ,0.0F };
		
		float afr __gc[] = {0.0F,10.78F,10.93F,11.08F,11.24F,11.39F,11.54F,11.69F,11.86F,12.04F,
		12.23F,12.39F,12.62F,12.83F,13.03F,13.21F,13.40F,13.59F,13.82F,14.10F,14.43F,14.83F,
		15.31F,15.85F,16.47F,17.15F,17.90F,18.70F,19.57F,3000.0F};	
	
		val = RomRep::Engine::rawToAfr((int)cmd, volts, afr);
	}
	else if(o2type == DatalogOptions::TEWIDEABND || o2type == DatalogOptions::INNOVATELM1)
		val = (RomRep::Engine::rawToVolts((int)cmd)*2.0F) +9.0F;
	else if(o2type == DatalogOptions::ZEITRONIX)
	{
		float volts __gc[] = {5.1F, 3.27F, 3.12F, 3.0F,  2.96F, 2.80F, 2.65F, 2.5F,  2.34F, 2.18F, 2.02F, 1.87F, 1.71F, 1.56F, 1.4F,  1.24F, 1.09F, 0.93F, 0.78F, 0.62F, 0.46F, 0.31F, 0.15F, 0.0F};
		float afr __gc[] = {3000.0F,21.2F, 19.9F, 18.8F, 18.5F, 16.9F, 15.6F, 14.7F, 14.2F, 13.7F, 13.2F, 12.8F, 12.4F, 12.1F, 11.7F, 11.4F, 11.0F, 10.7F, 10.5F, 10.3F, 10.1F, 9.9F,  9.7F,  0.0F};
		val = (RomRep::Engine::rawToAfr((int)cmd,volts,afr));
	}
	else if(o2type == DatalogOptions::PLX)
	{

 		float volts __gc[] = {5.1F ,4.0F ,3.5F ,3.0F ,2.75F ,2.5F ,2.38F ,2.25F ,2.13F ,2.0F ,
			1.75F ,1.5F ,1.25F ,1.0F ,.75F ,.5F ,0.0F };

		float afr __gc[] = {3000.0F,18.0F,17.0F,16.0F,15.5F,15.0F,14.75F,14.5F,14.25F,14.0F,
			13.5F,13.0F,12.5F,12.0F,11.5F,11.0F,10.0F};
		val = (RomRep::Engine::rawToAfr((int)cmd,volts,afr));
	}
	else
		val = (float)cmd;//txAndRx(cmd, 100); //secondary

	return val;
}

float bre20::Common::interpolate(float keyArray __gc[], int startind, int endind, float first, float last) __gc[]
{
	if(startind >= endind)
		return NULL;

	int len = endind - startind-1;
	float valArray __gc[] = new float __gc[len];

	float range = keyArray[endind] - keyArray[startind];
	float lgKey = keyArray[endind];
	if(range < 0.0f){
		range *= -1.0f;
		lgKey = keyArray[startind];
		float inter = first;
		first = last;
		last = inter;
	}

	for(int i = 0; i<len; i++){
		float perc = (lgKey - keyArray[startind+i+1]) / range;
		valArray[i] = (1.0f - perc) * last + perc * first;
	}

	//testing... We will have an array of ratios: ms/rpm. It should be the same
	//through the whole array. I just look at it through the debugger as they will never
	//be exactly the same.
	//float testArr __gc[] = new float __gc[len+1];
	//for( int i = 1; i<testArr->Length; i++)
	//	testArr[i] = (valArray[i] - valArray[i-1]) / (keyArray[i] - keyArray[i-1]);

	return valArray;
}