#include "stdafx.h"
#include "datalogfilewriter.h"
#include "global.h"

bre20::DatalogFileWriter::DatalogFileWriter()
{
	minrpm = INT_P(S"asMinRpm");
	minspeed = Globals::Props->getIntParam(S"asMinSpeed");
	maxrpm = Globals::Props->getIntParam(S"asMaxRpm");
	maxspeed = Globals::Props->getIntParam(S"asMaxSpeed");
	mintps = Globals::Props->getIntParam(S"asMinTps");
	maxtps = Globals::Props->getIntParam(S"asMaxTps");

	interval = Globals::Props->getIntParam(S"logInterval");

	autoenable = Globals::Props->getBoolParam(S"asEnabled");
	sepfile = Globals::Props->getBoolParam(S"asMultFiles");

	Globals::Props->add_OnPropertyChanged(new PropertyEventHandler(this, prop_Changed));

	fname = NULL;
	tofile = false;
	allgo = false;
	usergo = true;

	writeH = false;

	fileHeader = "";
}

bre20::DatalogFileWriter::~DatalogFileWriter()
{
	close();
}

void bre20::DatalogFileWriter::setFileHeader(String* str)
{
	fileHeader = "";
	if(str != NULL)
		fileHeader = str;
}
void bre20::DatalogFileWriter::appendFileHeader(String* str)
{
	fileHeader = String::Concat(fileHeader,str);
}

///This will only write to the file if the auto start conditions are 
///satisfied or the autostart is disabled.
void bre20::DatalogFileWriter::WriteLine(String* str)
{
	if(((allgo && autoenable) || !autoenable))
		WriteLineNow(str);
}

///This will only write to the file if the auto start conditions are 
///satisfied or the autostart is disabled.
void bre20::DatalogFileWriter::Write(String* str)
{
	if(((allgo && autoenable) || !autoenable))
	{
		checkHasWritten();
		WriteNow(str);
	}
}

void bre20::DatalogFileWriter::WriteNow(String* str)
{
	if(tofile && logout != NULL)
	{
		checkHasWritten();
		logout->Write(str);
	}
}

void bre20::DatalogFileWriter::checkHasWritten()
{
	if(!writeH)
	{
		writeH = true;
		WriteLineNow(interval.ToString());
		WriteLineNow(fileHeader);
	}
}
///This will write regardless of the autostart state.
void bre20::DatalogFileWriter::WriteLineNow(String* str)
{
	if(tofile && logout != NULL)
	{
		logout->WriteLine(str);
		logout->Flush();
	}
}

void bre20::DatalogFileWriter::close()
{
	if(logout != NULL)
	{
		logout->Close();
		logout = NULL;
	}
}

bool bre20::DatalogFileWriter::createFile(String* name)
{
	if(name == NULL)
		return false;

	try
	{
		close();
		logout = System::IO::File::CreateText(name);
		writeH = false;
		return true;
	}
	catch(...)
	{
		return false;
	}
}

///Set the actual file name
void bre20::DatalogFileWriter::setCurrentFile(String* name)
{
	curfname = name;
	FileNameChanged();
}

///Set the suggested file name.
void bre20::DatalogFileWriter::setFile(String* name)
{
	fname = name;
	close();
	if(!sepfile || !autoenable)
	{
		setCurrentFile(fname);
		createFile(fname);
	}
	this->filenum = 1;
}

void bre20::DatalogFileWriter::setFileWriteEnable(bool enable)
{
	tofile = enable;
}

void bre20::DatalogFileWriter::makeDecision()
{
	bool newgo = (rpmgo && speedgo && tpsgo && usergo);

	if(tofile && fname != NULL && sepfile && autoenable)// && newgo != *go)
	{
		if(!allgo && newgo)
		{
			Globals::LogFile->WriteLine(this,String::Concat("DatalogFileFriter: creating new file: ",filenum.ToString()));
			String* s = String::Concat((filenum++).ToString(),".csv");
			setCurrentFile(Path::ChangeExtension(fname,s));
			createFile(curfname);
		}
		else if(allgo && !newgo && logout != NULL)
		{
			close();
			//Globals::LogFile->WriteLine(this,String::Concat("DatalogFileFriter: Closing file: ",(filenum-1).ToString()));
			//logout->Close();
			//logout = NULL;
		}
	}

	allgo = newgo;
}

void bre20::DatalogFileWriter::setCond(int newval, int __gc* val, int __gc* min, 
									   int __gc* max, bool __gc* go)
{
	if(newval>=0) *val = newval;
	else return;

	*go = *val>=*min && *val<=*max;
}

void bre20::DatalogFileWriter::prop_Changed(System::Object *  sender, bre20::PropertyEventArgs *  e)
{
	String * name = e->getName();
	String * val = e->getValue();

	if(name->Equals("asMinRpm"))
		minrpm = Convert::ToInt32(val);
	else if(name->Equals("logInterval"))
		interval = Convert::ToInt32(val);
	else if(name->Equals("asMinSpeed"))
		minspeed = Convert::ToInt32(val);
	else if(name->Equals("asMaxRpm"))
		maxrpm = Convert::ToInt32(val);
	else if(name->Equals("asMaxSpeed"))
		maxspeed = Convert::ToInt32(val);
	else if(name->Equals("asMinTps"))
		mintps = Convert::ToInt32(val);
	else if(name->Equals("asMaxTps"))
		maxtps = Convert::ToInt32(val);
	else if(name->Equals("asEnabled"))
	{
		bool old = autoenable;
		autoenable = Globals::Props->getBoolParam(S"asEnabled");
		if(old != autoenable && autoenable && sepfile)
			setFile(this->fname);
	}
	else if(name->Equals("asMultFiles"))
	{
		bool old = sepfile;
		sepfile = Globals::Props->getBoolParam(S"asMultFiles");
		if(old != sepfile && sepfile)
		{
			allgo = false;
			setFile(this->fname);
		}
	}
}
//these set the conditions 
void bre20::DatalogFileWriter::setRpm(int r)
{
	setCond(r,&rpm,&minrpm, &maxrpm, &rpmgo);
	LOG(this,String::Concat("Drpm \t: ", r.ToString()));
}
void bre20::DatalogFileWriter::setTps(int t)
{
	setCond(t,&tps,&mintps, &maxtps, &tpsgo);
	LOG(this,String::Concat("Dtps \t: ", t.ToString()));
}
void bre20::DatalogFileWriter::setSpeed(int s)
{
	setCond(s,&speed,&minspeed, &maxspeed, &speedgo);
	LOG(this,String::Concat("Dspd \t: ", s.ToString()));
}