
#pragma once
using namespace System;
using namespace System::Windows::Forms;
using namespace System::IO;

//#include "FormWithRules.h"
#include "LoggerData.h"
#include "Properties.h"

namespace bre20
{
	/*!
	The DatalogFileWriter class handles all of the writing to 
	any datalog csv file. It also handles the auto start functionality
	which essentially just disables writing to the log file when the 
	autostart conditions are not met.
	*/
	public __gc class DatalogFileWriter
	{
	private:
		System::IO::StreamWriter * logout;
		int rpm, speed;
		int tps;
		int interval;

		String* fileHeader;

		String* fname;
		String* curfname;
		int filenum;//if sepfile then we create a new file with fname+"_"+filenum+".csv"

		int minrpm, maxrpm;
		int minspeed, maxspeed;
		int mintps, maxtps;

		bool allgo;
		bool usergo; ///this is here so a script writer can add some params
		             ///which can influence the auto start logging.
		             ///It could get hairy if 2 different scripts set this, 
		             /// but maybe its not my problem.

		bool tofile;//if this object should be logging to the file
		bool rpmgo; //true if rpm cond satisfied
		bool tpsgo; //true if tps cond satisfied
		bool speedgo; //true if speed cond satisfied
		bool autoenable;//true if autostart is enabled.
		bool sepfile; //true if we should be starting a new file at each start

		bool writeH; //true if the header data for the current file has been written 

		void setCond(int newval, int __gc* val, int __gc* min, int __gc* max, bool __gc* go);

		void setCurrentFile(String* name);
		bool createFile(String* name);
		void checkHasWritten();

		void prop_Changed(System::Object *  sender, bre20::PropertyEventArgs *  e);
		
		void FileNameChanged(){
			if (OnFileNameChanged != NULL)
				OnFileNameChanged(this, new EventArgs());
		}
	public:
		DatalogFileWriter();
		~DatalogFileWriter();

		__event EventHandler * OnFileNameChanged;

		///This will only write to the file if the auto start conditions are 
		///satisfied or the autostart is disabled.
		void Write(String* str);
		///This will only write to the file if the auto start conditions are 
		///satisfied or the autostart is disabled.
		void WriteLine(String* str);

		///This will write regardless of the autostart state.
		void WriteNow(String* str);
		///This will write regardless of the autostart state.
		void WriteLineNow(String* str);

		void Flush(){if(logout != NULL) logout->Flush();}

		void setFileHeader(String* str);
		void appendFileHeader(String* str);
		String* getFileHeader(){ return fileHeader; };

		///Closes the output file if its got one open. 
		void close();

		///Sets the suggested file name and opens that file.
		void setFile(String* name);
		String* getCurrentFile(){return curfname;}
		String* getFile(){ return fname; }
		///True if you want it to log to a file
		void setFileWriteEnable(bool enable);

		///Sets a 'user' condition that is anded with the rest of the
		///built in conditions to determine if auto logging should turn on.
		///This is for scripts!
		void setUserGoodToGo(bool good){ usergo = good; }
		///Allows you to see if the user condition is set.
		bool getUserGoodToGo(){ return usergo; }

		///This function takes the entered data and makes a 
		///decision whether to turn on or off the file logging.
		///This is called by the Logger \b after the rules have 
		///all been executed. That way we dont get extra stupid 
		///files with data that doesnt satisfy the criteria.
		void makeDecision();

		///
		void setRpm(int r);
		void setTps(int t);
		void setSpeed(int s);

		int getRpm(){return rpm;};
		int getSpeed(){return speed;};
		int getTps(){return tps;};
	};
}