#include "StdAfx.h"
#include "DatalogOptions.h"

bre20::DatalogOptions::DatalogOptions(String* extrao2defs[])
{
	InitializeComponent();

	if(extrao2defs != NULL)
		for(int i = 0; i<extrao2defs->Length; i++)
			this->o2box->Items->Add(extrao2defs[i]);

	o2box->SelectedIndex = 0;

	Globals::Props->NotifyDialogOpened(this,"DatalogOptions");
	Globals::Props->add_OnPropertyChanged(new PropertyEventHandler(this,prop_Changed));

	loadProperties();
}

void bre20::DatalogOptions::loadProperties()
{
	this->rpmbox->Text = Globals::Props->getIntParam(S"asMinRpm").ToString();
	this->speedbox->Text = Globals::Props->getIntParam(S"asMinSpeed").ToString();
	this->rpmstopbox->Text = Globals::Props->getIntParam(S"asMaxRpm").ToString();
	this->tespeedstopbox->Text = Globals::Props->getIntParam(S"asMaxSpeed").ToString();
	this->tpsBox->Text = Globals::Props->getIntParam(S"asMinTps").ToString();
	this->tpsstopbox->Text = Globals::Props->getIntParam(S"asMaxTps").ToString();

	this->chkInterpTrace->Checked = Globals::Props->getBoolParam("logInterpTraceEnabled");
	this->chkMapTInRom->Checked = Globals::Props->getBoolParam("logRomTraceEnabled");
	this->traceEnabled->Checked = Globals::Props->getBoolParam("logTraceEnabled");
	this->traceEnabled_CheckedChanged(NULL,NULL);

	this->asEnabled->Checked = Globals::Props->getBoolParam(S"asEnabled");
	this->asEnabled_CheckedChanged(NULL,NULL);
	this->newFile->Checked = Globals::Props->getBoolParam(S"asMultFiles");


//	String* str = Globals::Props->getParam(S"logUnits");
//	if(str->ToLower()->Equals("metric"))
//		setSpeedLabels(false);
//	else
//		setSpeedLabels(true);
}

bool bre20::DatalogOptions::addTabPage(TabPage* newpage)
{
	if(newpage == NULL)
		return false;

	this->tabControl1->TabPages->Add(newpage);
	return true;
}

void bre20::DatalogOptions::setInterval(int inter)
{
	sampleinterval->Text = inter.ToString();
}
int bre20::DatalogOptions::getInterval()
{
	try
	{
		return System::Convert::ToInt32(sampleinterval->Text);
	}
	catch(...){return 100;}
}

void bre20::DatalogOptions::setO2Type(int type)
{
	if(type>0 && type<=5)
		o2box->SelectedIndex = type;
}

int bre20::DatalogOptions::getO2Type()
{
	return o2box->SelectedIndex;
}

void bre20::DatalogOptions::setUnitsType(int type)
{
	if(type == DatalogOptions::SAE)
		sae1->Checked = true;
	else if(type == DatalogOptions::METRIC)
		metric->Checked = true;

	setSpeedLabels(this->sae1->Checked);
}
int bre20::DatalogOptions::getUnitsType()
{
	if(sae1->Checked == true)
		return 0;
	else if(metric->Checked == true)
		return 1;

	return 0;
}

bool bre20::DatalogOptions::isMapTraceEnabled()
{
	return traceEnabled->Checked;
}

void bre20::DatalogOptions::setTraceType(int type)
{
	if(type == DatalogOptions::NOTRACE)
		mtnothing->Checked = true;
	else if(type == DatalogOptions::AFRTRACE)
		mtafr->Checked = true;
	else if(type == DatalogOptions::KNOCKTRACE)
		mtknock->Checked = true;
}
int bre20::DatalogOptions::getTraceType()
{
	if(mtnothing->Checked)
		return 0;
	else if(mtafr->Checked)
		return 1;
	else if(mtknock->Checked)
		return 2;

	return 0;
}

int bre20::DatalogOptions::getMixtureUnits()
{
	if(this->afrUnit->Checked)
		return 0;
	else
		return 1;
}
float bre20::DatalogOptions::getMultiplyAdjust()
{
	try
	{
		return Convert::ToSingle(compMult->Text);
	}
	catch(...)
	{return 1.0F;}
}
float bre20::DatalogOptions::getAddAdjust()
{
	try
	{
		return Convert::ToSingle(compAdd->Text);
	}
	catch(...)
	{return 0.0F;}
}

void bre20::DatalogOptions::changeUnits(bool std)
{
	setSpeedLabels(std);

	if(std)
	{
		sae1->Checked = true;
	}
	else
	{
		metric->Checked = true;
	}
}
void bre20::DatalogOptions::setSpeedLabels(bool std)
{
	String* sunit = "KPH";
	if(std)
		sunit = "MPH";
	sunit = String::Concat(S"Speed (",sunit,S")");
	this->speedlab->Text = sunit;
	this->speedstoplab->Text = sunit;
}

  ///////////|\\\\\\\\\\\\
 //////////EVENTS\\\\\\\\\\
/////////////|\\\\\\\\\\\\\\

void bre20::DatalogOptions::okbutton_Click(System::Object *  sender, System::EventArgs *  e)
{
	Globals::Props->setParam(this,S"logInterval",this->getInterval().ToString());

	Globals::Props->setParam(this,S"logSensor",o2box->SelectedIndex.ToString());

	Globals::Props->setParam(this,S"logO2AddAdjust",this->getAddAdjust().ToString());
	Globals::Props->setParam(this,S"logO2MultiplyAdjust",this->getMultiplyAdjust().ToString());

	String* units = "sae";
	if(!this->sae1->Checked)
		units = "metric";
	Globals::Props->setParam(this,S"logUnits",units);

	units = "afr";
	if(!afrUnit->Checked)
		units = "lambda";
	Globals::Props->setParam(this,S"logO2Units",units);

	String* trace = S"";
	if(this->mtnothing->Checked)
		trace = S"nothing";
	else if(this->mtknock->Checked)
		trace = S"knock";
	else if(this->mtafr->Checked)
		trace = S"afr";
	Globals::Props->setParam(this,S"logTrace",trace);
	Globals::Props->setParam(this,S"logRomTraceEnabled",this->chkMapTInRom->Checked.ToString());
	Globals::Props->setParam(this,S"logInterpTraceEnabled",this->chkInterpTrace->Checked.ToString());

	Globals::Props->setParam(this,S"logTraceEnabled",traceEnabled->Checked.ToString());

	Globals::Props->setParam(this,S"asMinRpm",this->rpmbox->Text);
	Globals::Props->setParam(this,S"asMinSpeed",this->speedbox->Text);
	Globals::Props->setParam(this,S"asMaxRpm",this->rpmstopbox->Text);
	Globals::Props->setParam(this,S"asMaxSpeed",this->tespeedstopbox->Text);
	Globals::Props->setParam(this,S"asMinTps",this->tpsBox->Text);
	Globals::Props->setParam(this,S"asMaxTps",this->tpsstopbox->Text);

	Globals::Props->setParam(this,S"asEnabled",asEnabled->Checked.ToString());
	Globals::Props->setParam(this,S"asMultFiles",newFile->Checked.ToString());

	this->Close();
}

void bre20::DatalogOptions::o2box_SelectedIndexChanged(System::Object *  sender, System::EventArgs *  e)
{
	if(traceEnabled->Checked)
	{
		if(o2box->SelectedIndex > 0){
			mtnothing->Enabled = true;
			mtafr->Enabled = true;
		}
		else
		{	
			mtnothing->Enabled = true;
			mtnothing->Checked = true;
			mtafr->Enabled = false;
		}
	}
}

void bre20::DatalogOptions::sampleinterval_KeyPress(System::Object *  sender, System::Windows::Forms::KeyPressEventArgs *  e)
{
	Common::handleIntKeyPress(static_cast<TextBox*>(sender),e,false);
}

void bre20::DatalogOptions::DatalogOptions_Load(System::Object *  sender, System::EventArgs *  e)
{
		int tmp = Globals::Props->getIntParam(S"logInterval");
		if(tmp < 0)
			tmp = 500;

		sampleinterval->Text = tmp.ToString();

		tmp = Globals::Props->getIntParam(S"logTraceEnabled");
		if(tmp == 1)
			traceEnabled->Checked = true;
		traceEnabled_CheckedChanged(NULL,NULL);

		tmp = Globals::Props->getIntParam(S"logSensor");
		if(tmp < 0)
			tmp = 0;

		try
		{
			o2box->SelectedIndex = tmp;
		}
		catch(...)
		{
		}

		String* str = Globals::Props->getParam(S"logUnits")->ToLower();
		if(str->Equals("sae"))
			sae1->Checked = true;
		else if(str->Equals("metric"))
			metric->Checked = true;

		changeUnits(sae1->Checked);

		str = Globals::Props->getParam(S"logTrace");

		if(str->Equals("nothing"))
			mtnothing->Checked = true;
		else if(str->Equals("knock"))
			mtknock->Checked = true;
		else if(str->Equals("afr"))
			mtafr->Checked = true;

		float ftmp = Globals::Props->getFloatParam("logO2MultiplyAdjust");
		this->compMult->Text = ftmp.ToString("0.00");
		ftmp = Globals::Props->getFloatParam("logO2AddAdjust");
		this->compAdd->Text = ftmp.ToString("0.00");

		str = Globals::Props->getParam(S"logO2Units");
		lambdaUnit->Checked = true;
		if(str->Equals("afr"))
			afrUnit->Checked = true;
}

void bre20::DatalogOptions::traceEnabled_CheckedChanged(System::Object *  sender, System::EventArgs *  e)
{
	if(traceEnabled->Checked)
		o2box_SelectedIndexChanged(NULL,NULL);
	else
	{
		mtafr->Enabled = false;
		mtnothing->Enabled = false;
	}
}

void bre20::DatalogOptions::prop_Changed(Object * sender, bre20::PropertyEventArgs* e)
{
	if(e->getName()->Equals("logUnits"))
	{
		if(e->getValue()->ToLower()->Equals("metric"))
			changeUnits(false);
		else
			changeUnits(true);
	}
}