

using System;
using System.Windows.Forms;

using bre20;
using RomRep;
//css_reference ..\..\bre.exe;
//css_import ITBFunctionDlg.cs;

namespace BensScripts
{
	public class ITBFunctions
	{
		private child romWind;
		private static int xoverAdd = 0x7ff0;
		private static int maxtpsAdd = 0x7fef;
		private static int mintpsAdd = 0x7fee;

		private int lastXOver;
		private float lastMinTps;
		private float lastMaxTps;
		private ITBFunctionDlg dlg;

		public static object AppliesTo(){return "pr3:pw0";}
		public static object ScriptTitle(){return "ITB Functions by Ben Ogle";}

		//args[1] = child window of type bre20.child
		public object ScriptMain(params object[] args)
		{
			romWind = (child)args[1];
			string str = romWind.getRomType();
			if(Common.isMatch("ITB",str,false))
				romWind.AddMenuItem("Scripts", "ITB Functions...", "BensScripts.ITBFunctions.mnu_click");
			return null;
		}

		private int voltsToRaw(float volts)
		{
			return (int)(255.0f * volts/5.0f);
		}

		private void setMaxTps()
		{
			dlg.MaxTpsVolts = Engine.rawToVolts(romWind.getRom().getByte(maxtpsAdd));
			lastMaxTps = dlg.MaxTpsVolts;
		}
		private void setMinTps()
		{
			dlg.MinTpsVolts = Engine.rawToVolts(romWind.getRom().getByte(mintpsAdd));
			lastMinTps = dlg.MinTpsVolts;
		}
		private void setXOver()
		{
			dlg.XOverCol = ITBMapHelper.getXoverCol(romWind.getRom().getByte(xoverAdd), ITBMapHelper.mapcolumns.Length);
			lastXOver = dlg.XOverCol;
		}
		private void setAll()
		{
			setMaxTps();
			setMinTps();
			setXOver();
		}

		private void btnApply_Click(object sender, System.EventArgs e)
		{
			float fmaxt = dlg.MaxTpsVolts;
			float fmint = dlg.MinTpsVolts;

			if(fmaxt <= fmint)
			{
				setMaxTps();
				setMinTps();
				return;
			}
			else if(fmaxt > 5.0f)
			{
				fmaxt = 5.0f;
				return;
			}

			int maxtps = voltsToRaw(fmaxt);
			int mintps = voltsToRaw(fmint);
			int xovercol = dlg.XOverCol;
			if(xovercol > -1 && xovercol < ITBMapHelper.mapcolumns.Length)
				xovercol = ITBMapHelper.mapcolumns[xovercol];

			if(fmaxt != lastMaxTps)
				romWind.getRom().setByte(maxtpsAdd,maxtps); 
			if(fmint != lastMinTps)
				romWind.getRom().setByte(mintpsAdd,mintps); 
			if(xovercol != lastXOver)
				romWind.getRom().setByte(xoverAdd,xovercol); 

			setAll();
		}

		public object mnu_click(params object[] objs)
		{
			dlg = new ITBFunctionDlg();
			dlg.ApplyClicked += new System.EventHandler(this.btnApply_Click);
			
			setAll();

			dlg.ShowDialog();

			dlg.Dispose();
			dlg = null;
			return null;
		}
	}//end class
}//end namespace