using System;
using System.Windows.Forms;
using System.Reflection;
using System.IO;

using bre20;
using RomRep;
//css_reference ..\bre.exe;
//css_reference ..\SimpleGrid.dll;

namespace BensScripts
{
	public class TargetAfrEdit
	{
		protected Form1 mainWind;
		protected child romWind;
		protected HondaEcu ecu;
		protected GridPage[] gp;
		bool afropt;

		public static object AppliesTo()
		{
			return "pr3:pw0:ng";
		}
		
		public static object ScriptTitle()
		{
			return "Target Afr Edit by Ben Ogle";
		}

		protected static string n()
		{
			return "BensScripts.TargetAfrEdit";
		}

		//args[0] = main window of type bre20.Form1
		//args[1] = child window of type bre20.child
		public object ScriptMain(params object[] args)
		{
			mainWind = (Form1)args[0];
			romWind = (child)args[1];
			ecu = romWind.getRom();

			afropt = false;
			if(Globals.Props.getParam("logO2Units") == "afr")
				afropt = true;

			romWind.subscribeToEvent("OnHeaderChanged", n()+".page_HeaderChange");
			ecu.OnTargetAfrChanged += new RomRep.TargetAfrChangeHandler(this.page_AfrChange);
			Globals.Props.OnPropertyChanged += new PropertyEventHandler(this.props_AfrToLambda);

			gp = new GridPage[2];
			
			string str = "VTEC";
			if(Common.isMatch("NG",romWind.getRom().getType(),false))
				str = "Boost";

			gp[0]= romWind.AddMap("Low Target AFR", ecu.getWidth(), ecu.getHeight(), n()+".fuel_Edit");
			gp[1]= romWind.AddMap(str+" Target AFR", ecu.getWidth(), ecu.getHeight(), n()+".fuel_Edit");

			gp[0].setIndex(0);
			gp[1].setIndex(1);

			gp[0].setMode(RomRep.Map.MODE_FUEL);
			gp[1].setMode(RomRep.Map.MODE_FUEL);

			//set column headers
			for(int ind = 0; ind <2; ind++)
				for(int i = 0; i<ecu.getWidth();i++)
					gp[ind].setColumnHeader(i,ecu.getColHeader(RomRep.Map.MODE_FUEL,ind,i));

			for(int i = 0; i<ecu.getHeight();i++)
			{
				gp[0].setRowHeader(i,ecu.getRowHeader(child.FUELMODE,0,i));
				gp[1].setRowHeader(i,ecu.getRowHeader(child.FUELMODE,1,i));
			}

			fillGrids();

			return null;
		}

		private void fillGrids()
		{
			for(int index = 0; index<2; index++)
			{
				float[,] map = ecu.getLambdaMap(index);
				for(int i = 0; i<ecu.getWidth();i++)
				{
					for(int j = 0; j<ecu.getHeight();j++)
					{
						float val = map[i,j];
						if(!afropt)
							val = Common.calcLambda(val);
						setValue(gp[index],i,j,val);
					}
				}
				gp[index].Refresh();
			}
		}

		private void setValue(GridPage page, int col, int row, float val)
		{
			page.setValue(col,row,val,false);
			if(!afropt)
				val = Common.calcAfr(val);
			romWind.setCellColor(page,col,row,(val-10.0F)*1.6F,child.FUELMODE);
		}
		//the params are:
		//args[0] = main window (the Form1 object)
		//args[1] = rom window (the child object)
		//args[2] = object that sent the event 
		//args[3] = event object (will be base class of EventArgs)
		public object fuel_Edit(params object[] args)
		{
			romWind = (child)args[1];
			ecu = romWind.getRom();
			SimpleGrid.CellEvent ce = (SimpleGrid.CellEvent)args[3];
			GridPage page = (GridPage)args[2];
			int index = page.getIndex();
			int x = ce.Column;
			int y = ce.Row;
			float val = page.getValue(x,y);
			if(!afropt)
				val = Common.calcAfr(val);
			ecu.setAfrCellEx(x, y, val, index);
		//	setValue(page,x,y,ecu.getAfrCell(x,y, index));
			return null;
		}

		private void page_AfrChange(object sender, RomRep.AfrChangeArgs e)
		{
			float val = e.getAfr();
			if(!afropt)
				val = Common.calcLambda(val);
			setValue(gp[e.getIndex()],e.getX(),e.getY(),val);
		}

		private void props_AfrToLambda(object sender, PropertyEventArgs e)
		{
			if(e.getName() != "logO2Units")
				return;

			bool tmpafr = false;
			if(e.getValue() == "afr")
				tmpafr = true;

			if((tmpafr && !afropt) || (!tmpafr && afropt))
			{
				afropt = tmpafr;
				fillGrids();
			}
		}

		//the params are:
		//args[0] = main window (the Form1 object)
		//args[1] = rom window (the child object)
		//args[2] = object that sent the event 
		//args[3] = event object (will be base class of EventArgs)
		public object page_HeaderChange(params object[] args)
		{
			try
			{
				GridPage egp = (GridPage)args[2];
				RomRep.MapHeaderChangeArgs ca = (RomRep.MapHeaderChangeArgs)args[3];

				//ignore the originating event. We just want the updates that 
				//happen as a result of the actual ROM/binary change.
				if((RomRep.CellChangeArgs.Marker)ca.getMarker() == 
					RomRep.CellChangeArgs.Marker.ARGS_SPECIAL)
					return null;

				int x = ca.getX();
				int y = ca.getY();
				bool col = false, all = false;

				if(y < 0){ //col event
					col = true;
					if(y == RomRep.Map.HEADER_EVENT_ALL) 
						all = true;
				}
				else if(x < 0) //row event
					if(x == RomRep.Map.HEADER_EVENT_ALL) 
						all = true;

				if(!all){
					if(col) gp[egp.getIndex()].setColumnHeader(x,ca.getValue()[0]);
					else gp[egp.getIndex()].setRowHeader(y,ca.getValue()[0]);
				}
				else{
					if(col)
						for(int i = 0; i< ca.getValue().Length; i++)
							gp[egp.getIndex()].setColumnHeader(i,ca.getValue()[i]);
					else
						for(int i = 0; i< ca.getValue().Length; i++)
							gp[egp.getIndex()].setRowHeader(i,ca.getValue()[i]);
				}
			}
			catch(Exception ex){
				MessageBox.Show("error\n\n" + ex.Message);
			}
			return null;
		}

	}//end class
}//end namespace