using System;
using System.Windows.Forms;
using System.Reflection;
using System.IO;

using bre20;
using RomRep;
//css_reference ..\..\bre.exe;
//css_reference ..\..\SimpleGrid.dll;

namespace BensScripts
{
	public class Nitrous
	{
		protected Form1 mainWind;
		protected child romWind;
		protected HondaEcu ecu;
		LabeledText tps;
		LabeledText ron;
		LabeledText roff;
		CheckBox chk;

		private static int tpsloc = 0x4e0b;
		private static int onrpmloc = 0x4e19;
		private static int offrpmloc = 0x4e0f;

		public static object AppliesTo()
		{
			return "pr3";
		}
		
		public static object ScriptTitle()
		{
			return "Nitrous Control by Ben Ogle";
		}

		//args[0] = main window of type bre20.Form1
		//args[1] = child window of type bre20.child
		public object ScriptMain(params object[] args)
		{
			mainWind = (Form1)args[0];
			romWind = (child)args[1];

			ecu = romWind.getRom();
			
			Control[] con;
			con = new Control[4];
			con[0]=chk=romWind.CreateCheckBox(10,24,"Enabled","BensScripts.Nitrous.chk_Click");
			con[1]=ron=romWind.CreateText(20,85,"Min RPM","BensScripts.Nitrous.lowRpm_Change","Change when the solenoid turns ON",70,LabeledText.LTMODEINT);
			con[2]=roff=romWind.CreateText(20,125,"Max RPM","BensScripts.Nitrous.highRpm_Change","Change when the solenoid turns OFF again",70,LabeledText.LTMODEINT);
			con[3]=tps=romWind.CreateText(20,45,"TPS (%)","BensScripts.Nitrous.tps_Change","The output will not turn on until this TPS is reached (in %)",70,LabeledText.LTMODEINT);

			if(ecu.getByte(0x4e00) == 0xc4)
			{
				con[0].Enabled = false;
				((CheckBox)con[0]).Checked = true;
				setBoxes((LabeledText)con[3],(LabeledText)con[1],(LabeledText)con[2]);

				addMaps(false);	//add maps, dont copy contents
			}
			else
			{
				tps.Enabled = false;
				ron.Enabled = false;
				roff.Enabled = false;
			}

			GroupBox grp = romWind.CreateGroupBox(5,10,"Nitrous Control", "BensScripts.Nitrous", 130,170,con);

			romWind.AddControlGroup(grp);

			return null;
		}

		private int addMaps(bool copy)
		{
			return romWind.AddMapPair("Nitrous Fuel", new MapLocs(0x791a, 0,0x3bd6),	//fuel map data
					"Nitrous Ign", new MapLocs(0x76fe,0, 0x3bb6),	//ignition map data
					ecu.getWidth(), ecu.getHeight(),				//x, y
					1,												//copy index: vtec map
					copy);											//copy the contents??
		}

		private void setBoxes(LabeledText tps, LabeledText ron, LabeledText roff)
		{
			if(tps != null)
				tps.setText( ((((int)ecu.getByte(tpsloc) - 25)*100)/205).ToString() );
			if(ron != null)
				ron.setText( ecu.getTwoByteRpm(onrpmloc).ToString() );
			if(roff != null)
				roff.setText( ecu.getTwoByteRpm(offrpmloc).ToString() );
		}

		//the params are:
		//args[0] = main window
		//args[1] = rom window (the child object)
		//args[2] = object that sent the event 
		//args[3] = event object (will be base class of EventArgs)
		public object chk_Click(params object[] args)
		{
			if(chk.Checked && chk.Enabled)
			{
				tps.Enabled = true;
				ron.Enabled = true;
				roff.Enabled = true;
				chk.Enabled = false;

				//add the maps
				addMaps(true);	//add maps, do copy contents

				//get the file & parse the byte changes
				//different for pw0s!!
				Assembly asm = Assembly.GetAssembly(this.GetType());
				romWind.parseByteChanges(Path.GetDirectoryName(asm.Location) + "\\Pr3NitrousChanges.txt", true);

				//then we make our changes
				ecu.setByte(tpsloc, 0xca);
				ecu.setTwoByteRpm(onrpmloc,3000);
				ecu.setTwoByteRpm(offrpmloc,4000);

				//then in here the rom will be passed back to the child window
				setBoxes(tps,ron,roff);
			}
			
			return null;
		}

		public object tps_Change(params object[] args)
		{
			try
			{
				ecu.setByte(tpsloc, (int)(tps.getIntValue() * 205.0)/100 + 25);
				setBoxes(tps,null,null);
			}
			catch{}

			return null;
		}

		public object lowRpm_Change(params object[] args)
		{
			try
			{
				ecu.setTwoByteRpm(onrpmloc,ron.getIntValue());
				setBoxes(null,ron,null);
			}
			catch{}

			return null;
		}
		
		public object highRpm_Change(params object[] args)
		{
			try
			{
				ecu.setTwoByteRpm(offrpmloc,roff.getIntValue());
				setBoxes(null,null,roff);
			}
			catch{}

			return null;
		}

	}//end class
}//end namespace