#pragma once

using namespace System;
using namespace System::ComponentModel;
using namespace System::Collections;
using namespace System::Windows::Forms;
using namespace System::Data;
using namespace System::Drawing;

#include "PortSetupDialog.h"
#include "datalogcom.h"
#include "DataLogger.h"
#include "LogGridPage.h"
#include "Loganalyzer.h"
#include "DatalogOptions.h"
#include "DigiDash.h"
#include "CharWrap.h"
#include "child.h"
#include "common.h"

namespace bre20
{
	/// <summary> 
	/// Summary for DataLogForm
	/// </summary>
	public __gc class DataLogForm : public System::Windows::Forms::Form
	{
    //my data members...
	protected:	
//		bre20::DataLogger * textLog; //textual rep of realtime data
		bre20::LogGridPage * gridLog; //map trace of realtime data
		DigiDash * dash;
		System::Windows::Forms::Timer *  timer1;
		System::Windows::Forms::TabControl *  tabControl1;

		bre20::child * rom;

		DatalogCom* comport;

		int parity;
		bool tofile; // true if logging to file
		bool replay; //true if replaying, false if logging

		int tminterval;
		int maptracetype;
		int o2type;

		int port;
		int baudrate;

		String * header;
		String * fname;
		String * title;

		//for real time editing.
		float smchange, lgchange;
		bool percentchange, curvtec;

		int curcol,currow;

		//realtime auto tune.
		bool autotune;
		float afrmap __gc[,];
		float afr;
		int rowinterp,colinterp;

		float atmin, atmax, atmincell, atminfreq, atmaxrowdev, atmaxcoldev;
		int intervalcount, numintervals;

		int goodsamples __gc[,,];

		float atavg __gc[,,];
			 float atfreq __gc[,,];
		//end realtime editing.

		void setHeaders();

		System::IO::StreamWriter * logout;
		System::IO::StreamReader * replayin;

		int readInterval(System::IO::StreamReader *sr);

		virtual float pickO2(unsigned char cmd);

		virtual void runDL();
		virtual void runReplay() = 0;
		virtual void logStart() = 0;
		virtual void logStop() = 0;
		
		void timer_Tick(Object * myObject, EventArgs * myEventArgs)
		{
			if(replay)
				runReplay();
			else
				runDL();
		}

	protected:
		void Dispose(Boolean disposing)
		{
			if (disposing && components)
			{
				components->Dispose();
			}
			__super::Dispose(disposing);
		}

		void clickStart()
		{
			start_Click(NULL,NULL);
		}

	public: 
		DataLogForm(String * header, bre20::child * rom);

		~DataLogForm()
		{
			timer1->Stop();
			if(comport!= NULL)
				comport->close();
			if(logout != NULL)
				logout->Close();
			if(replayin != NULL)
				replayin->Close();
		}

		bool connectToPort()
		{
			bool ok = comport->connectToEcu(String::Concat(S"COM",port.ToString()),baudrate,parity);

			return ok;
		}

	private: System::Windows::Forms::MenuItem *  loggerAnalyze; 
	private: System::Windows::Forms::MainMenu *  mainMenu1;
	private: System::Windows::Forms::MenuItem *  menuItem1;
	private: System::Windows::Forms::MenuItem *  LoggerOptions;
	private: System::Windows::Forms::MenuItem *  portsetupmenu;
	private: System::Windows::Forms::StatusBarPanel *  rtatstatus;
	private: System::Windows::Forms::GroupBox *  groupBox1;
	private: System::Windows::Forms::RadioButton *  logOption;
	private: System::Windows::Forms::RadioButton *  replayOption;

	private: System::Windows::Forms::Button *  start;
	private: System::Windows::Forms::Button *  portSetup;
	private: System::Windows::Forms::Button *  connect;
	private: System::Windows::Forms::CheckBox *  toFile;
	private: System::Windows::Forms::MenuItem *  menuItem2;
	private: System::Windows::Forms::MenuItem *  incFuel;
	private: System::Windows::Forms::MenuItem *  incFuelLg;
	private: System::Windows::Forms::MenuItem *  decFuel;
	private: System::Windows::Forms::MenuItem *  decFuelLg;
	private: System::Windows::Forms::StatusBar *  statusBar1;
	private: System::Windows::Forms::StatusBarPanel *  connected;
	private: System::Windows::Forms::MenuItem *  menuItem3;
	private: System::Windows::Forms::MenuItem *  menuItem4;
		
		/// <summary>
		/// Required designer variable.
		/// </summary>
		System::ComponentModel::Container* components;

		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		void InitializeComponent(void)
		{
			System::Resources::ResourceManager *  resources = new System::Resources::ResourceManager(__typeof(bre20::DataLogForm));
			this->tabControl1 = new System::Windows::Forms::TabControl();
			this->portSetup = new System::Windows::Forms::Button();
			this->connect = new System::Windows::Forms::Button();
			this->toFile = new System::Windows::Forms::CheckBox();
			this->start = new System::Windows::Forms::Button();
			this->groupBox1 = new System::Windows::Forms::GroupBox();
			this->replayOption = new System::Windows::Forms::RadioButton();
			this->logOption = new System::Windows::Forms::RadioButton();
			this->mainMenu1 = new System::Windows::Forms::MainMenu();
			this->menuItem1 = new System::Windows::Forms::MenuItem();
			this->LoggerOptions = new System::Windows::Forms::MenuItem();
			this->portsetupmenu = new System::Windows::Forms::MenuItem();
			this->loggerAnalyze = new System::Windows::Forms::MenuItem();
			this->menuItem2 = new System::Windows::Forms::MenuItem();
			this->incFuel = new System::Windows::Forms::MenuItem();
			this->incFuelLg = new System::Windows::Forms::MenuItem();
			this->decFuel = new System::Windows::Forms::MenuItem();
			this->decFuelLg = new System::Windows::Forms::MenuItem();
			this->menuItem3 = new System::Windows::Forms::MenuItem();
			this->menuItem4 = new System::Windows::Forms::MenuItem();
			this->statusBar1 = new System::Windows::Forms::StatusBar();
			this->connected = new System::Windows::Forms::StatusBarPanel();
			this->rtatstatus = new System::Windows::Forms::StatusBarPanel();
			this->groupBox1->SuspendLayout();
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->connected))->BeginInit();
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->rtatstatus))->BeginInit();
			this->SuspendLayout();
			// 
			// tabControl1
			// 
			this->tabControl1->Anchor = (System::Windows::Forms::AnchorStyles)(((System::Windows::Forms::AnchorStyles::Top | System::Windows::Forms::AnchorStyles::Bottom) 
				| System::Windows::Forms::AnchorStyles::Left) 
				| System::Windows::Forms::AnchorStyles::Right);
			this->tabControl1->Location = System::Drawing::Point(0, 0);
			this->tabControl1->Name = S"tabControl1";
			this->tabControl1->SelectedIndex = 0;
			this->tabControl1->Size = System::Drawing::Size(640, 272);
			this->tabControl1->TabIndex = 0;
			// 
			// portSetup
			// 
			this->portSetup->Anchor = (System::Windows::Forms::AnchorStyles)(System::Windows::Forms::AnchorStyles::Bottom | System::Windows::Forms::AnchorStyles::Left);
			this->portSetup->Location = System::Drawing::Point(8, 288);
			this->portSetup->Name = S"portSetup";
			this->portSetup->Size = System::Drawing::Size(96, 24);
			this->portSetup->TabIndex = 1;
			this->portSetup->Text = S"Setup Port";
			this->portSetup->Click += new System::EventHandler(this, portSetup_Click);
			// 
			// connect
			// 
			this->connect->Anchor = (System::Windows::Forms::AnchorStyles)(System::Windows::Forms::AnchorStyles::Bottom | System::Windows::Forms::AnchorStyles::Left);
			this->connect->Location = System::Drawing::Point(120, 288);
			this->connect->Name = S"connect";
			this->connect->Size = System::Drawing::Size(88, 24);
			this->connect->TabIndex = 2;
			this->connect->Text = S"Connect";
			this->connect->Click += new System::EventHandler(this, connect_Click);
			// 
			// toFile
			// 
			this->toFile->Anchor = (System::Windows::Forms::AnchorStyles)(System::Windows::Forms::AnchorStyles::Bottom | System::Windows::Forms::AnchorStyles::Left);
			this->toFile->Location = System::Drawing::Point(336, 288);
			this->toFile->Name = S"toFile";
			this->toFile->Size = System::Drawing::Size(80, 24);
			this->toFile->TabIndex = 3;
			this->toFile->Text = S"Log to File";
			this->toFile->CheckedChanged += new System::EventHandler(this, toFile_CheckedChanged);
			// 
			// start
			// 
			this->start->Anchor = (System::Windows::Forms::AnchorStyles)(System::Windows::Forms::AnchorStyles::Bottom | System::Windows::Forms::AnchorStyles::Left);
			this->start->Location = System::Drawing::Point(224, 288);
			this->start->Name = S"start";
			this->start->Size = System::Drawing::Size(88, 24);
			this->start->TabIndex = 5;
			this->start->Text = S"Start";
			this->start->Click += new System::EventHandler(this, start_Click);
			// 
			// groupBox1
			// 
			this->groupBox1->Anchor = (System::Windows::Forms::AnchorStyles)(System::Windows::Forms::AnchorStyles::Bottom | System::Windows::Forms::AnchorStyles::Left);
			this->groupBox1->Controls->Add(this->replayOption);
			this->groupBox1->Controls->Add(this->logOption);
			this->groupBox1->Location = System::Drawing::Point(440, 280);
			this->groupBox1->Name = S"groupBox1";
			this->groupBox1->Size = System::Drawing::Size(168, 40);
			this->groupBox1->TabIndex = 7;
			this->groupBox1->TabStop = false;
			this->groupBox1->Text = S"Mode";
			// 
			// replayOption
			// 
			this->replayOption->Location = System::Drawing::Point(96, 16);
			this->replayOption->Name = S"replayOption";
			this->replayOption->Size = System::Drawing::Size(64, 16);
			this->replayOption->TabIndex = 1;
			this->replayOption->Text = S"Replay";
			this->replayOption->CheckedChanged += new System::EventHandler(this, replayOption_CheckedChanged);
			// 
			// logOption
			// 
			this->logOption->Checked = true;
			this->logOption->Location = System::Drawing::Point(24, 16);
			this->logOption->Name = S"logOption";
			this->logOption->Size = System::Drawing::Size(48, 16);
			this->logOption->TabIndex = 0;
			this->logOption->TabStop = true;
			this->logOption->Text = S"Log";
			this->logOption->CheckedChanged += new System::EventHandler(this, logOption_CheckedChanged);
			// 
			// mainMenu1
			// 
			System::Windows::Forms::MenuItem* __mcTemp__1[] = new System::Windows::Forms::MenuItem*[1];
			__mcTemp__1[0] = this->menuItem1;
			this->mainMenu1->MenuItems->AddRange(__mcTemp__1);
			// 
			// menuItem1
			// 
			this->menuItem1->Index = 0;
			System::Windows::Forms::MenuItem* __mcTemp__2[] = new System::Windows::Forms::MenuItem*[10];
			__mcTemp__2[0] = this->LoggerOptions;
			__mcTemp__2[1] = this->portsetupmenu;
			__mcTemp__2[2] = this->loggerAnalyze;
			__mcTemp__2[3] = this->menuItem2;
			__mcTemp__2[4] = this->incFuel;
			__mcTemp__2[5] = this->incFuelLg;
			__mcTemp__2[6] = this->decFuel;
			__mcTemp__2[7] = this->decFuelLg;
			__mcTemp__2[8] = this->menuItem3;
			__mcTemp__2[9] = this->menuItem4;
			this->menuItem1->MenuItems->AddRange(__mcTemp__2);
			this->menuItem1->MergeOrder = 1;
			this->menuItem1->Text = S"Logger";
			// 
			// LoggerOptions
			// 
			this->LoggerOptions->Index = 0;
			this->LoggerOptions->Text = S"Options...";
			this->LoggerOptions->Click += new System::EventHandler(this, LoggerOptions_Click);
			// 
			// portsetupmenu
			// 
			this->portsetupmenu->Index = 1;
			this->portsetupmenu->Text = S"Port Setup...";
			this->portsetupmenu->Click += new System::EventHandler(this, portSetup_Click);
			// 
			// loggerAnalyze
			// 
			this->loggerAnalyze->Index = 2;
			this->loggerAnalyze->Text = S"Analyze Current Log...";
			this->loggerAnalyze->Click += new System::EventHandler(this, loggerAnalyze_Click);
			// 
			// menuItem2
			// 
			this->menuItem2->Index = 3;
			this->menuItem2->Text = S"-";
			// 
			// incFuel
			// 
			this->incFuel->Index = 4;
			this->incFuel->Shortcut = System::Windows::Forms::Shortcut::CtrlI;
			this->incFuel->Text = S"Increase Fuel";
			this->incFuel->Click += new System::EventHandler(this, incFuel_Click);
			// 
			// incFuelLg
			// 
			this->incFuelLg->Index = 5;
			this->incFuelLg->Shortcut = System::Windows::Forms::Shortcut::CtrlShiftI;
			this->incFuelLg->Text = S"Increase Fuel (large change)";
			this->incFuelLg->Click += new System::EventHandler(this, incFuelLg_Click);
			// 
			// decFuel
			// 
			this->decFuel->Index = 6;
			this->decFuel->Shortcut = System::Windows::Forms::Shortcut::CtrlD;
			this->decFuel->Text = S"Decrease Fuel";
			this->decFuel->Click += new System::EventHandler(this, decFuel_Click);
			// 
			// decFuelLg
			// 
			this->decFuelLg->Index = 7;
			this->decFuelLg->Shortcut = System::Windows::Forms::Shortcut::CtrlShiftD;
			this->decFuelLg->Text = S"Decrease Fuel (large change)";
			this->decFuelLg->Click += new System::EventHandler(this, decFuelLg_Click);
			// 
			// menuItem3
			// 
			this->menuItem3->Index = 8;
			this->menuItem3->Text = S"-";
			// 
			// menuItem4
			// 
			this->menuItem4->Index = 9;
			this->menuItem4->Shortcut = System::Windows::Forms::Shortcut::CtrlShiftR;
			this->menuItem4->Text = S"Start Real-time Autotune";
			this->menuItem4->Click += new System::EventHandler(this, menuItem4_Click);
			// 
			// statusBar1
			// 
			this->statusBar1->Location = System::Drawing::Point(0, 322);
			this->statusBar1->Name = S"statusBar1";
			System::Windows::Forms::StatusBarPanel* __mcTemp__3[] = new System::Windows::Forms::StatusBarPanel*[2];
			__mcTemp__3[0] = this->connected;
			__mcTemp__3[1] = this->rtatstatus;
			this->statusBar1->Panels->AddRange(__mcTemp__3);
			this->statusBar1->ShowPanels = true;
			this->statusBar1->Size = System::Drawing::Size(640, 24);
			this->statusBar1->TabIndex = 8;
			this->statusBar1->Text = S"statusBar1";
			// 
			// connected
			// 
			this->connected->Text = S"Connected to COM10";
			this->connected->Width = 150;
			// 
			// rtatstatus
			// 
			this->rtatstatus->Text = S"test";
			this->rtatstatus->Width = 300;
			// 
			// DataLogForm
			// 
			this->AutoScaleBaseSize = System::Drawing::Size(5, 13);
			this->ClientSize = System::Drawing::Size(640, 346);
			this->Controls->Add(this->statusBar1);
			this->Controls->Add(this->groupBox1);
			this->Controls->Add(this->start);
			this->Controls->Add(this->toFile);
			this->Controls->Add(this->connect);
			this->Controls->Add(this->portSetup);
			this->Controls->Add(this->tabControl1);
			this->Icon = (__try_cast<System::Drawing::Icon *  >(resources->GetObject(S"$this.Icon")));
			this->Menu = this->mainMenu1;
			this->Name = S"DataLogForm";
			this->Text = S"Data Logger";
			this->Resize += new System::EventHandler(this, DataLogForm_Resize);
			this->Closing += new System::ComponentModel::CancelEventHandler(this, DataLogForm_Closing);
			this->groupBox1->ResumeLayout(false);
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->connected))->EndInit();
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->rtatstatus))->EndInit();
			this->ResumeLayout(false);

		}		








//***********************************************************************************
//event handlers

		void connect_Click(System::Object *  sender, System::EventArgs *  e)
		{
			if(!String::Equals(connect->Text, S"Disconnect"))
			{
				bool con = connectToPort();
				if(con)
				{
					connect->Text = "Disconnect";
					start->Enabled = true;
					portSetup->Enabled = false;
					connected->Text = String::Concat(S"Connected to COM",port.ToString());
				}
				else
					connected->Text = "Failed to open port...";
			}
			else //need to disconnect
			{
				comport->close();
				connect->Text = "Connect";
				start->Enabled = false;
				portSetup->Enabled = true;
				connected->Text = "Disconnected";
			}
		}

		void portSetup_Click(System::Object *  sender, System::EventArgs *  e)
		{
			PortSetupDialog * psd = new PortSetupDialog();
			psd->setPortName(Globals::Props->getParam(S"logPort"));
			psd->setBaud(baudrate);
			psd->setParity(parity);

			psd->ShowDialog(this);

			port = psd->getPortName();
			baudrate = psd->getBaud();
			parity = psd->getParity();

			Common::setParam(S"logPort",String::Concat(S"COM",port.ToString()));	
		}
		
		void start_Click(System::Object *  sender, System::EventArgs *  e)
		{
			if(String::Equals(start->Text, S"Start"))
			{	
				logStart();
				timer1->Start();
				start->Text = "Stop";
				if(!replay)
					toFile->Enabled = false;
				this->groupBox1->Enabled = false;

				try
				{
					String* str = Common::getParam(S"emuSmallChange",Common::getNewAppSettings());

					if(str->Chars[str->Length-1] == '%')
						percentchange = true;

					if(percentchange)
						str = str->Substring(0,str->Length-1);

					this->smchange = Convert::ToSingle(str)/100.0F;

					str = Common::getParam(S"emuLargeChange",Common::getNewAppSettings());

					if(percentchange)
						str = str->Substring(0,str->Length-1);

					this->lgchange = Convert::ToSingle(str)/100.0F;

				}
				catch(...){}
			}
			else
 			{	
				logStop();
				timer1->Stop();
				start->Text = "Start";
				if(!replay)
					toFile->Enabled = true;
				this->groupBox1->Enabled = true;
			}
		}

		void toFile_CheckedChanged(System::Object *  sender, System::EventArgs *  e)
		{
			if(!toFile->Checked)
			{
				this->Text = title;
				tofile = false;
				if(logout != NULL)
					logout->Close();
			}

			System::Windows::Forms::SaveFileDialog *  openFile = new SaveFileDialog();

			openFile->Filter = S"CSV files (*.csv)|*.csv|All files (*.*)|*.*";

			if(!fname->Equals(S""))
				openFile->FileName = fname;
			
			if(openFile->ShowDialog() == DialogResult::OK)
			{
				fname = openFile->FileName;
				if(System::IO::File::Exists(fname))
					logout = new System::IO::StreamWriter(fname);
				else
					logout = System::IO::File::CreateText(fname);

				this->Text = String::Concat(title,S" - ", fname);
				tofile = true;
				logout->WriteLine(timer1->Interval.ToString());
				logout->WriteLine(header);
				logout->Flush();
			}
			else
			{	
				toFile->Checked = false;
				tofile = false;
			}
		}

		void logOption_CheckedChanged(System::Object *  sender, System::EventArgs *  e)
		{
			if(logOption->Checked == true){
				replay = false;
				this->Text = title;

				if(replayin != NULL){
					replayin->Close();
					replayin = NULL;
				}
				start->Enabled = false;
				connect->Enabled = true;
				portSetup->Enabled = true;
				toFile->Enabled = true;
				connected->Text = S"Disconnected";
				timer1->Interval = tminterval;
				gridLog->clearGrid();
			}
		}

		void replayOption_CheckedChanged(System::Object *  sender, System::EventArgs *  e)
		{
			if(!replayOption->Checked)
			{
				replayOption->Checked = false;
				logOption->Checked = true;
				replay = false;
				return;
			}

			System::Windows::Forms::OpenFileDialog *  openFile = new OpenFileDialog();

			openFile->Filter = S"CSV files (*.csv)|*.csv|All files (*.*)|*.*";

			if(!fname->Equals(S""))
				openFile->FileName = fname;
			
			if(openFile->ShowDialog() != DialogResult::OK)
				return;

			fname = openFile->FileName;
			replayin = new System::IO::StreamReader(fname);
			replay = true;

			this->Text = String::Concat(title,S" Replay - ", fname);
			
			//if connected then we disconnect
			if(String::Equals(connect->Text, S"Disconnect"))
			{
				comport->close();
				connect->Text = "Connect";
			}

			start->Enabled = true;
			connect->Enabled = false;
			portSetup->Enabled = false;
			toFile->Enabled = false;
			connected->Text = S"Replay Mode";

			int inter = readInterval(replayin);
			if(inter<=0)
			{
				logOption->Checked = true;
				logOption_CheckedChanged(NULL,NULL);
				MessageBox::Show("Something is wrong with the input file","BRE Datalogger");
				return;
			}
			else
				timer1->Interval = inter; 
			replayin->ReadLine(); //maybe someday this line tells me where to put the values
		}

		void LoggerOptions_Click(System::Object *  sender, System::EventArgs *  e)
		{
			DatalogOptions * dlo = new DatalogOptions(NULL);
		
			dlo->ShowDialog(this);

			if(replay)
				tminterval = dlo->getInterval();
			else
			{
				tminterval = dlo->getInterval();
				timer1->Interval = tminterval;
			}

			o2type = dlo->getO2Type();

			maptracetype = dlo->getTraceType();

			if(dash != NULL)
			{
				if(dlo->getUnitsType() == DatalogOptions::SAE)
					dash->convertUnits(true);
				else
					dash->convertUnits(false);
			}
		}

		void loggerAnalyze_Click(System::Object *  sender, System::EventArgs *  e)
		{
			if(start->Text->Equals("Start"))
			{
				this->toFile->Checked = false;

//				LogAnalyzer* la = new LogAnalyzer(this->rom,this->fname);

//				la->Visible = true;
			}
		}

		void DataLogForm_Closing(System::Object *  sender, System::ComponentModel::CancelEventArgs *  e)
		{
			if(comport != NULL)
				comport->close();

			if(logout != NULL)
				logout->Close();

			if(replayin != NULL)
				replayin->Close();
		}

		void DataLogForm_Resize(System::Object *  sender, System::EventArgs *  e){this->connected->Width = this->Size.Width -30;}

		void changeFuel(float change);
		void incFuel_Click(System::Object *  sender, System::EventArgs *  e){changeFuel(smchange);}
		void incFuelLg_Click(System::Object *  sender, System::EventArgs *  e){changeFuel(lgchange);}
		void decFuel_Click(System::Object *  sender, System::EventArgs *  e){changeFuel(smchange*-1);}
		void decFuelLg_Click(System::Object *  sender, System::EventArgs *  e){changeFuel(lgchange*-1);}

		//real time autotune menu item
		void menuItem4_Click(System::Object *  sender, System::EventArgs *  e);
		bool setFuelCell(int col, int row, float perc, float afrmeasure, float afrtarget);
	};
}