#include "stdafx.h"
#include "devUtil.h"
#include "obd0vtececu.h"
#include "obd0vtecmaps.h"
#include "common.h"

//editor must make sure that the pressure value is between the 
//previous and next column's values

void RomRep::DevUtil::recalcPressureHeaders(int mapsensor, HondaEcu* f)
{
	
	int lastNACol = f->getLastNaCol();

	unsigned char file __gc[] = f->getFile();

	int xover = (int)(getXOver(mapsensor)*255.0/5.0);
	int val;
	int x = getRomWidth(file);
	
	int i;
	for(i = 0; i<= lastNACol; i++)
	{
		int sptr = mapscalarloc + 6 + (x-2-i)*3;
		val = (i*xover)/lastNACol;
		if(val >= xover)
			val = xover-3;
		file[sptr] = val;
	}
	for(; i<getRomWidth(file); i++)
	{
		int deltaR = 255-xover;
		int bcolumn = i- (lastNACol+1);
		int numcolumn = getRomWidth(file) - (lastNACol+2);

		int sptr = mapscalarloc + 6 + (x-2-i)*3;
		val = ((deltaR*bcolumn)/numcolumn) + xover;
		if(val <= xover)
			val = xover+1;
		else if(val>255)
			val = 255;
		file[sptr] = val;
	}
	setMapSensor(f,mapsensor);
}

//****************
int RomRep::DevUtil::getRawHeaders(int maxmap, unsigned char thefile __gc[]) __gc[]
{
	int lastNACol = DevUtil::findLastNACol(thefile);

	int x = DevUtil::getRomWidth(thefile);
	int y = DevUtil::getRomHeight(thefile);

	int colheaders __gc[] = new int __gc[x];

	colheaders[colheaders->Length -1] = maxmap;

	for(int i = x-2; i>=0; i--)
	{
		int sptr = mapscalarloc + 6 + (x-2-i)*3;
		int mapval = thefile[sptr];

		colheaders[i] = mapval;
	}
	return colheaders;
}

String* RomRep::DevUtil::calcHeaders(int maxmap, unsigned char thefile __gc[])[]
{
	int lastNACol = DevUtil::findLastNACol(thefile);

	int x = DevUtil::getRomWidth(thefile);
	int y = DevUtil::getRomHeight(thefile);

	String* colheaders[] = new String*[x];

	//column headers...
	int natoboost = (int)(DevUtil::getXOver(DevUtil::getMapSensor(thefile))*255.0/5.0);// = 146; 

	float volts = Engine::rawToVolts(maxmap);
	if(maxmap<natoboost)//calc inches of mercury. vacuum, y0
	{
		int inhg = Engine::voltsToInHg(volts,DevUtil::getMapSensor(thefile));//(int)((volts - xovervolts) / (-.1));
		String * s = inhg.ToString();
		colheaders[x-1] = s;
	}
	else //boost in psi
	{
		//2.85 volts with key on engine off and 4.5xx volts at 10.6 psi
		float psi = Engine::voltsToPsi(volts,DevUtil::getMapSensor(thefile));//(volts - xovervolts)/.1557;
		String * s = psi.ToString("0.00");
		String * p = S"psi";
		s = String::Concat(s,p);
		colheaders[x-1] = s;
	}

	for(int i = x-2; i>=0; i--)
	{
		int sptr = mapscalarloc + 6 + (x-2-i)*3;
		int mapval = thefile[sptr];

		float volts = Engine::rawToVolts(mapval);
		String * s;
		
		if(mapval<natoboost)//calc inches of mercury. vacuum, y0
			s = Engine::voltsToInHg(volts,DevUtil::getMapSensor(thefile)).ToString();
		else //boost in psi
			s = Engine::voltsToPsi(volts,DevUtil::getMapSensor(thefile)).ToString("0.00");

		colheaders[i] = s;
	}
	return colheaders;
}

RomRep::MapHeaderChangeArgs* RomRep::DevUtil::devNotify(RomRep::ByteChange* bc, unsigned char file __gc[], int maxmapadd)
{
	if(bc->getLocation() == DevUtil::mapsensorloc)
		return new RomRep::MapHeaderChangeArgs(Map::HEADER_EVENT_ALL,Map::HEADER_EVENT_NONE,
			DevUtil::calcHeaders(file[maxmapadd],file));

	int x = DevUtil::getRomWidth(file);
	//find out if its ours
	if(bc->getLocation() < mapscalarloc || bc->getLocation() > x*3+mapscalarloc)
		return NULL;

	int col = bc->getLocation() - mapscalarloc - 6;
	if(col%3 != 0)
		return NULL;

	col = (col/(-3))+x-2; //find the column

	if(col > -1 && col<x-1)
	{
		int natoboost = (int)(DevUtil::getXOver(DevUtil::getMapSensor(file))*255.0/5.0);
		int b = (int)bc->getByte();
		float volts = Engine::rawToVolts(b);
		String * s;
		
		if(b<natoboost)//calc inches of mercury. vacuum, y0
			s = Engine::voltsToInHg(volts,DevUtil::getMapSensor(file)).ToString();
		else //boost in psi
			s = Engine::voltsToPsi(volts,DevUtil::getMapSensor(file)).ToString("0.00");

		String* str[] = new String*[1];
		str[0] = s;
		return new RomRep::MapHeaderChangeArgs(col,Map::HEADER_EVENT_NONE,str);
	}

	return NULL;
}

//****************

RomRep::ByteChange* RomRep::DevUtil::mapHeaderChanged(unsigned char file __gc[], MapHeaderChangeArgs* args,
									 String* colHead[], String* RowHead[])[]
{
	//called from some outside source. This function is responsible for figuring out the 
	//event type, converting the values from string to raw.

	if(args->getMarker() != CellChangeArgs::Marker::ARGS_SPECIAL)
		return NULL;
	int x = args->getX();
	int y = args->getY();
	bool col = false;
	bool all = false;

	if(y < 0) //col event
	{
		col = true;
		if(y == RomRep::Map::HEADER_EVENT_ALL) 
			all = true;
	}
	else if(x < 0) //row event
		if(x == RomRep::Map::HEADER_EVENT_ALL) 
			all = true;

	if(!all)
	{
		RomRep::ByteChange* changes[] = new RomRep::ByteChange*[1];
		if(col)
		{
			int lastNACol = DevUtil::findLastNACol(file);
			if(x>lastNACol){
				float val = bre20::Common::extractDouble(args->getValue()[0]);
				changes[0] = setPressureVal(x,val,file);
			}
			else{
				int val = bre20::Common::extractInt(args->getValue()[0]);
				changes[0] = setPressureVal(x,val,file);
			}
		}
		else
		{
		//	int rpm = bre20::Common::extractInt(args->getValue()[0]);
		}
		return changes;
	}

	//handle the all event here. I dont think it will ever happen though.
	return NULL;
}

bool RomRep::DevUtil::canEditHeader(unsigned char file __gc[], bool column, int columnOrRow)
{
	if(!column)
		return false;

	if(columnOrRow > 0 && columnOrRow < RomRep::DevUtil::getRomWidth(file)-1)
		return true;
	return false;
}

RomRep::ByteChange* RomRep::DevUtil::setHeader( bool column, int roworcol, String* val, unsigned char file __gc[])
{
	if(!column)
		return NULL;

	int lastNACol = DevUtil::findLastNACol(file);
	RomRep::ByteChange* bc = NULL;
	if(roworcol > lastNACol && roworcol < getRomWidth(file))
		bc = setPressureVal(roworcol,bre20::Common::extractDouble(val),file);
	else if(roworcol >= 0 && roworcol <= lastNACol)
		bc = setPressureVal(roworcol,bre20::Common::extractInt(val),file);
	
	//protect the user from themselves!! If these map vectors 
	//arent decreasing, bad shit will happen.
	if(bc != NULL && bc->getByte() >= file[bc->getLocation()-3])
		bc = new ByteChange(bc->getLocation(), file[bc->getLocation()-3]-1);
	else if(bc != NULL && bc->getByte() < file[bc->getLocation()+3])
		bc = new ByteChange(bc->getLocation(), file[bc->getLocation()+3]+1);

	return bc;
}

//need to make sure a neg pressure is not entered...
RomRep::ByteChange* RomRep::DevUtil::setPressureVal(int col, int inhg, unsigned char file __gc[])
{
	int x = getRomWidth(file);
	int sptr = mapscalarloc + 6 + (x-2-col)*3;
	int raw = Engine::inHgToRaw(inhg,getMapSensor(file));
	//file[sptr] = raw;
	RomRep::ByteChange* bc = new RomRep::ByteChange(sptr,(unsigned char) raw);
	
	float volts = Engine::rawToVolts(raw);
	int vac = Engine::voltsToInHg(volts,getMapSensor(file));
	//headers[col] = vac.ToString();
	return bc;
}

//need to make sure a pos pressure is not entered...
RomRep::ByteChange* RomRep::DevUtil::setPressureVal(int col, float psi, unsigned char file __gc[])
{
	int x = getRomWidth(file);
	int sptr = mapscalarloc + 6 + (x-2-col)*3;
	int raw = Engine::psiToRaw(psi,getMapSensor(file));
	//file[sptr] = raw;
	RomRep::ByteChange* bc = new RomRep::ByteChange(sptr,(unsigned char) raw);

	float volts = Engine::rawToVolts(raw);
	float pres = Engine::voltsToPsi(volts,getMapSensor(file));
	//headers[col] = pres.ToString("0.00");
	return bc;
}
int RomRep::DevUtil::findLastNACol(unsigned char file __gc[])
{
	int i,x = getRomWidth(file);
	int xover = (int)(getXOver(getMapSensor(file))*255.0/5.0);
	for(i = x-2; i>=0; i--)
	{
		int sptr = mapscalarloc + 6 + (x-2-i)*3;
		int mapval = file[sptr];

		if(mapval<xover)
			break;
		
	}
	return i;
}
int RomRep::DevUtil::getRomWidth(unsigned char file __gc[]){
	return file[0x7ffc];
}
int RomRep::DevUtil::getRomHeight(unsigned char file __gc[]){
	return file[0x7ffd];
}

int RomRep::DevUtil::getMapSensor(unsigned char file __gc[]){
	//0 for stock
	//1 for 2 bar
	//2 for 2.5 bar
	//4 for motorola2.5 bar
	//3 for 3 bar

	int num = file[mapsensorloc];
	if(num>MAXMAPSENSOR)
		num = 0;
	return num;
}
void RomRep::DevUtil::setMapSensor(RomRep::HondaEcu* ecu,int num){
	ecu->setByte(DevUtil::mapsensorloc, num);
}