#pragma once
using namespace System;

#include "Engine.h"
#include "ByteChange.h"
#include "Common.h"

namespace RomRep
{
	///This class is for
	///the Dev roms (RomRep.JdmPr3Dev, RomRep.JdmPw0Dev, RomRep.EuroPw0Dev). 
	///They exend their type
	///(JdmPr3, JdmPw0, EuroPw0) and then use the functionality of this class.

	public __gc class DevUtil
	{
	private:

	public:
		DevUtil(){}

		static bool canEditHeader(unsigned char file __gc[], bool column, int columnOrRow);

		///sets a vacuum column header in file
		static RomRep::ByteChange* setPressureVal(int col, int inhg, unsigned char file __gc[],String* headers[]);
		///sets a boost column header in file
		static RomRep::ByteChange* setPressureVal(int col,float psi,unsigned char file __gc[],String* headers[]);

		static RomRep::ByteChange* setHeader( bool column, int roworcol, String* val, unsigned char file __gc[], String* headers[]);

		///called when the dev roms are notified of a byte change
		static int devNotify(RomRep::ByteChange* bc, unsigned char file __gc[],String* headers[], int maxmapadd);

		///gets the width of the maps stored in ROM
		static int getRomWidth(unsigned char file __gc[]);
		///gets the height of the maps stored in ROM
		static int getRomHeight(unsigned char file __gc[]);
		///gets the type of map sensor stored in ROM
		static int getMapSensor(unsigned char file __gc[]);
		///sets the type of map sensor in ROM
		static void setMapSensor(RomRep::HondaEcu* ecu,int sensor);

		///Recalculates the pressure headers for the given mapsensor type.
		static void recalcPressureHeaders(int mapsensor, HondaEcu* file);
		///Takes the raw values from ROM and assembles the column header string array. 
		static void calcHeaders(int maxmap, unsigned char thefile __gc[], String* colheaders[]);

		///figures out what index the last NA column is 
		static int findLastNACol(unsigned char file __gc[]);

		///gets the NA to boost crossover voltage for a given map sensor
		static float getXOver(int mapsensor){
			if(mapsensor == 1)
				return XOVER2BAR;
			else if(mapsensor == 2)
				return XOVER25BAR;
			else if(mapsensor == 3)
				return XOVER3BAR;
			else if(mapsensor == 4)
				return XOVERMOTOROLA25BAR;
			else
				return XOVERSTOCK;
		};
		//this should have all the common 
		//locations and values as static shit
		//e.g. map loc, scalar loc, etc.

		///location of column header scalars
		static const int mapscalarloc = 0x63d2;
		static const int natoboost = 153;
		static const int mapsensorloc = 0x7ff0;

		///ignition map address
		static const int igmap = 0x6600;
		///VTEC ignition map address
		static const int igmapvtec = 0x6984;
		///fuel map address
		static const int fmap = 0x6D08;
		///VTEC fuel map address
		static const int fmapvtec = 0x70aa;
		
		///Unused address of revscalars
		static const int revscalar_ig = 0x648c;
		///Unused address of revscalars
		static const int revscalar_igv = 0x64e9;
		///Unused address of revscalars
		static const int revscalar_fu = 0x6546;
		///Unused address of revscalars
		static const int revscalar_fuv = 0x65a3;

		static const int MAXMAPSENSOR = 4;

		///2 bar map sensor type
		static const int MAP2BAR = 1;
		///2.5 bar map sensor type
		static const int MAP25BAR = 2;
		///3 bar map sensor type
		static const int MAP3BAR = 3;
		///motorola 2.5 bar map sensor type
		static const int MAPMOTOROLA25BAR = 4;
		///stock map sensor type
		static const int MAPSTOCK = 0;

		///NA to boost voltage for 2 bar map sensor type
		static const float XOVER2BAR = 2.5F;
		///NA to boost voltage for GM 2.5 bar map sensor type
		static const float XOVER25BAR = 2.0F;
		///NA to boost voltage for motorola 2.5 bar map sensor type
		static const float XOVERMOTOROLA25BAR = 1.864F;
		///NA to boost voltage for 3 bar map sensor type
		static const float XOVER3BAR = 1.6F;
		///NA to boost voltage for stock map sensor type
		static const float XOVERSTOCK = 2.8F; //3.0

		///boost calculation constant for 3 bar map sensor type
		static const float PSICONST3BAR = .1156F;
		///boost calculation constant for 2.5 bar map sensor type
		static const float PSICONST25BAR = .13612F;
		///boost calculation constant for motorola 2.5 bar map sensor type
		static const float PSICONSTMOTOROLA25BAR = .14069F;
		///boost calculation constant for 2 bar map sensor type
		static const float PSICONST2BAR = .171F;
		///boost calculation constant for stock map sensor type
		static const float PSICONSTSTOCK = .18707F; //.1557

		///NA calculation constant for 3 bar map sensor type
		static const float INHGCONST3BAR = -.0535F;
		///NA calculation constant for 2.5 bar map sensor type
		static const float INHGCONST25BAR = -.0668F;
		///NA calculation constant for motorola 2.5 bar map sensor type
		static const float INHGCONSTMOTOROLA25BAR = -.0689F;
		///NA calculation constant for 2 bar map sensor type
		static const float INHGCONST2BAR = -.084F;
		///NA calculation constant for stock map sensor type
		static const float INHGCONSTSTOCK = -.094F;
		
	};
}
