#include "stdafx.h"
#include "dynocalculation.h"

bre20::DynoCalculation::DynoCalculation(int inter)
{
	//interval = inter;

	frontArea = Globals::Props->getFloatParam(S"dynFrontArea");
	cd = Globals::Props->getFloatParam(S"dynCd");
	fdr = Globals::Props->getFloatParam(S"dynFDR");
	gear = Globals::Props->getFloatParam(S"dynGearRatio");
	tireh = Globals::Props->getFloatParam(S"dynTireHeight");
	weight = Globals::Props->getFloatParam(S"dynCarWeight");

	mintps = Globals::Props->getIntParam(S"dynMinTps");

	sae = false; 
	String* str = Globals::Props->getParam(S"logUnits")->ToLower();
	if(str->Equals("sae"))
		sae = true;
	
	Globals::Props->add_OnPropertyChanged(new PropertyEventHandler(this,prop_Changed));
}

void bre20::DynoCalculation::recalculate(DynoLib::DynoRun* run)
{
	ArrayList* data = run->get_RunData();
	float newdata __gc[];
	for(int i = 0; i<data->Count; i++)
	{
		DynoLib::GraphItem* item = static_cast<DynoLib::GraphItem*>(data->get_Item(i));
		newdata = calculate(item->Data);
		if(newdata != NULL)
			item->Data = newdata;
	}
}
float bre20::DynoCalculation::calculate(float lastVals __gc[]) __gc[] //recalculate
{
	if(lastVals == NULL || lastVals->Length < DynoLib::DynoGraph::Indices::IND_MAX )
		return NULL;

	int rpm = DynoLib::DynoGraph::Indices::IND_RPM;
	int tq = DynoLib::DynoGraph::Indices::IND_TQ;
	int hp = DynoLib::DynoGraph::Indices::IND_HP;
	int lrpm = DynoLib::DynoGraph::Indices::IND_LASTRPM;

	int spd = DynoLib::DynoGraph::Indices::IND_MAX;

	//do stuff
	//order of GraphItem: rpm,hp,tq,afr,tps,oldrpm
	lastVals[tq] = calcTq(lastVals[rpm], lastVals[lrpm]);
	lastVals[hp] = calcHp(lastVals[tq],lastVals[rpm]); //hp

	return lastVals;
}
//
DynoLib::GraphItem* bre20::DynoCalculation::calculate(__int64 lasttime, 
													  float lastVals __gc[], 
													  __int64 curtime, float curvals __gc[])
{
	if(lastVals == NULL || curvals== NULL || lastVals->Length < 3 || curvals->Length < 3)
		return NULL;

	int time = (int)(curtime - lasttime);
	interval = ((double)time)/10000.0;
	//do stuff
	//order of incoming vals: rpm,afr,tps
	//order of GraphItem: rpm,hp,tq,afr,tps,oldrpm
	float item __gc[] = new float __gc[6];
	item[DynoLib::DynoGraph::Indices::IND_RPM] = curvals[0]; //rpm
	item[2] = calcTq(curvals[0], lastVals[0]); //tq
	//item[2] = calcTq(curvals[3], lastVals[3]); //tq from spd
	item[1] = calcHp(item[2],curvals[0]); //hp
	item[3] = curvals[1]; //afr
	item[4] = curvals[2]; //tps <min tps is enforced in the graph 
	item[5] = lastVals[0]; //oldrpm <for easy recalc

	return new DynoLib::GraphItem(item);
}

float bre20::DynoCalculation::calcTq(float newrpm, float oldrpm)
{
	//well, this still returns an sae value (ft.lb) for now
	float FEET_PER_METER = .22f;//?

	double pi = 3.1415926538f;
	double sec = (interval/1000.0f);
	double rpmdif = double(newrpm-oldrpm);
	//float spddif = newrpm - oldrpm;

	double acc,mass,arm;
//	if(sae)
//	{
		///sae: inches, lbs
		acc = (rpmdif / gear / fdr * pi * tireh / 12.0f / 60.0f)/sec;
	//	acc = spddif * 88.0f / sec;

		mass = weight/32.2f;
		arm = tireh/2.0f/12.0f;
//	}
//	else
//	{
		///metric: meters, kg
//		acc = (rpmdif / gear / fdr * pi * (tireh *FEET_PER_METER) / 60.0f)/sec;
//		mass = weight;
//		arm = (tireh/2.0f)*FEET_PER_METER;
//	}

    return (float)((mass*acc*arm)/(gear*fdr)); //force*arm/all_the_gearing
}
float bre20::DynoCalculation::calcHp(float tq, float rpm)
{
	//sae
	return tq*rpm/5252.0f;
}

void bre20::DynoCalculation::setInterval(int inter)
{
	interval = inter;
}

void bre20::DynoCalculation::prop_Changed(Object* sender, PropertyEventArgs* pe)
{
	String* name = pe->getName();
	String* val = pe->getValue();

	try
	{
		if(name->Equals("dynFrontArea"))
			frontArea = Convert::ToSingle(val);
		else if(name->Equals("dynCd"))
			cd = Convert::ToSingle(val);
		else if(name->Equals("dynFDR"))
			fdr = Convert::ToSingle(val);
		else if(name->Equals("dynGearRatio"))
			gear = Convert::ToSingle(val);
		else if(name->Equals("dynTireHeight"))
			tireh = Convert::ToSingle(val);
		else if(name->Equals("dynCarWeight"))
			weight = Convert::ToSingle(val);
		//else if(name->Equals("dynMinTps"))
		//	mintps = Convert::ToInt32(val);
		else if(name->Equals("logUnits")){
			sae = false; 
			String* str = Globals::Props->getParam(S"logUnits")->ToLower();
			if(str->Equals("sae"))
				sae = true;
		}
	}
	catch(...){}
}