#include "StdAfx.h"
#include "DynoOptions.h"

bre20::DynoOptions::DynoOptions(DynoRule* rule)
{
	InitializeComponent();

	m_dynoparam = new DynoParameters();
	this->tabControl1->TabPages->Add(m_dynoparam);

	m_dyno = rule;

	m_dyno->getDynoPage()->getGraph()->add_RunsChanged(new DynoLib::RunsChangedHandler(this,dyno_RunsChanged));

	getRuns();

	for(int i = 0; i<m_runs->Count; i++)
	{
		DynoLib::DynoRun* run = getRun(m_runs,i);
		runBox->SetItemChecked(i,run->Visible);
	}
}
void bre20::DynoOptions::getRuns()
{
	this->runBox->Items->Clear();
	m_runs = m_dyno->getDynoPage()->getGraph()->Runs; //real copy of the arraylist

	for(int i = 0; i<m_runs->Count; i++){
		DynoLib::DynoRun* run = getRun(m_runs,i);
		this->runBox->Items->Add(run->Name,run->Visible);
	}
}
DynoLib::DynoRun* bre20::DynoOptions::getRun(ArrayList* list, int index)
{
	if(index>=0 && index<list->Count)
		return static_cast<DynoLib::DynoRun*>(list->get_Item(index));
	return NULL;
}

void bre20::DynoOptions::setRunDataEnable(bool enable)
{
	runColor->Enabled = enable;
	colorBox->Enabled = enable;
	mixSlider->Enabled = enable;
	mixChk->Enabled = enable;
	powerSlider->Enabled = enable;
	powerChk->Enabled = enable;
	torqueSlider->Enabled = enable;
	torqueChk->Enabled = enable;

	deleteRun->Enabled = enable;

	colorBox->BackColor = Color::LightGray;
}
void bre20::DynoOptions::getCurrentRunData()
{
	if(runBox->SelectedIndex >= 0 && runBox->SelectedIndex < m_runs->Count)
	{
		setRunDataEnable(true);

		DynoLib::DynoRun* run = getRun(m_runs,runBox->SelectedIndex);
		colorBox->BackColor = run->GraphColor;

		mixSlider->Value = (int)run->GetAlphaChannel((int)DynoGraph::Indices::IND_MIX);
		powerSlider->Value = (int)run->GetAlphaChannel((int)DynoGraph::Indices::IND_HP);
		torqueSlider->Value = (int)run->GetAlphaChannel((int)DynoGraph::Indices::IND_TQ);

		mixChk->Checked = run->GetGraphShowing((int)DynoGraph::Indices::IND_MIX);
		powerChk->Checked = run->GetGraphShowing((int)DynoGraph::Indices::IND_HP);
		torqueChk->Checked = run->GetGraphShowing((int)DynoGraph::Indices::IND_TQ);
	}
	else
		this->setRunDataEnable(false);
}
////********
void bre20::DynoOptions::dyno_RunsChanged(Object* sender, DynoLib::RunsChangedArgs* args)
{
	getRuns();
	getCurrentRunData();
}

void bre20::DynoOptions::applyBtn_Click(System::Object *  sender, System::EventArgs *  e)
{
	//will apply the settings then recalc
	m_dynoparam->persistPreferences();
	m_dyno->RecalculateAllRuns();
	m_dyno->getDynoPage()->Refresh();
}
void bre20::DynoOptions::okBtn_Click(System::Object *  sender, System::EventArgs *  e)
{
	applyBtn_Click(NULL,NULL);
	this->Close();
}
void bre20::DynoOptions::btnCancel_Click(System::Object *  sender, System::EventArgs *  e)
{
	this->Close();
}

void bre20::DynoOptions::addRun_Click(System::Object *  sender, System::EventArgs *  e)
{
	String* strs[] = Common::openFileDialog(true,false,"",Common::csvFilter());
	if(strs != NULL && strs->Length > 0 && strs[0] != NULL)
		m_dyno->addFile(strs[0]);
}
void bre20::DynoOptions::deleteRun_Click(System::Object *  sender, System::EventArgs *  e)
{
	if(runBox->SelectedIndex >0)
	{
		m_dyno->getDynoPage()->getGraph()->removeDynoRun(getRun(m_runs,runBox->SelectedIndex));
		m_dyno->getDynoPage()->getGraph()->Refresh();
	}
	else
		MessageBox::Show("Sorry, you cant delete the main run.", "BRE");
}


void bre20::DynoOptions::runBox_ItemCheck(System::Object *  sender, System::Windows::Forms::ItemCheckEventArgs *  e)
{
	bool c = false;
	if(e->NewValue == CheckState::Checked)
		c = true;
	getRun(m_runs,e->Index)->Visible = c;
	m_dyno->getDynoPage()->getGraph()->Refresh();
}
void bre20::DynoOptions::runBox_Click(System::Object *  sender, System::EventArgs *  e)
{}
void bre20::DynoOptions::runBox_SelectedIndexChanged(System::Object *  sender, System::EventArgs *  e)
{
	this->getCurrentRunData();
}
void bre20::DynoOptions::colorBox_Click(System::Object *  sender, System::EventArgs *  e)
{//dbl click
	ColorDialog* cd = new ColorDialog();
	if(cd->ShowDialog() == DialogResult::OK)
	{
		getRun(m_runs,runBox->SelectedIndex)->GraphColor = cd->Color;
		colorBox->BackColor = cd->Color;
		m_dyno->getDynoPage()->getGraph()->Refresh();
	}
}

void bre20::DynoOptions::changeRunVisibility(int graphIndex, bool show)
{
	//for(int i = 0; i< m_runs->Count; i++)
	getRun(m_runs,runBox->SelectedIndex)->SetGraphShowing(show,graphIndex);
	m_dyno->getDynoPage()->getGraph()->Refresh();
}
void bre20::DynoOptions::powerChk_CheckedChanged(System::Object *  sender, System::EventArgs *  e)
{
	changeRunVisibility((int)DynoLib::DynoGraph::Indices::IND_HP,powerChk->Checked);
}
void bre20::DynoOptions::torqueChk_CheckedChanged(System::Object *  sender, System::EventArgs *  e)
{
	changeRunVisibility((int)DynoLib::DynoGraph::Indices::IND_TQ,torqueChk->Checked);
}
void bre20::DynoOptions::mixChk_CheckedChanged(System::Object *  sender, System::EventArgs *  e)
{
	changeRunVisibility((int)DynoLib::DynoGraph::Indices::IND_MIX,mixChk->Checked);
}

void bre20::DynoOptions::changeRunOpacity(int graphIndex, int opacity)
{
	if(opacity >= 0 && opacity <256)
	{
		//for(int i = 0; i< m_runs->Count; i++)
		getRun(m_runs,runBox->SelectedIndex)->SetAlphaChannel((unsigned char)opacity,graphIndex);
		m_dyno->getDynoPage()->getGraph()->Refresh();
	}
}
void bre20::DynoOptions::powerSlider_Scroll(System::Object *  sender, System::EventArgs *  e)
{
	changeRunOpacity((int)DynoLib::DynoGraph::Indices::IND_HP,powerSlider->Value);
}
void bre20::DynoOptions::torqueSlider_Scroll(System::Object *  sender, System::EventArgs *  e)
{
	changeRunOpacity((int)DynoLib::DynoGraph::Indices::IND_TQ,torqueSlider->Value);
}
void bre20::DynoOptions::mixSlider_Scroll(System::Object *  sender, System::EventArgs *  e)
{
	changeRunOpacity((int)DynoLib::DynoGraph::Indices::IND_MIX,mixSlider->Value);
}
