#include "stdafx.h"
#include "dynorule.h"
#include "common.h"
#include "loganalyzer.h"

bre20::DynoRule::DynoRule()
{
	isSetup = false;
	fromAnalyzer = false;
	graph = new DynoPage();
	Globals::Props->add_OnPropertyChanged(new PropertyEventHandler(this,prop_changed));

	mintps = new DynoLib::IgnoreRule((int)DynoLib::DynoGraph::Indices::IND_TPS,
		Globals::Props->getFloatParam("dynMinTps"));
	//graph->getGraph()->addIgnoreRule(mintps,true);
}

void bre20::DynoRule::prop_changed(Object* sender, PropertyEventArgs* pe)
{
	String* name = pe->getName();
	String* val = pe->getValue();
	float imintps = -1.0f;
	try
	{
		if(name->Equals("dynMinTps"))
			imintps = Convert::ToSingle(val);
	}
	catch(...){}
	if(imintps >= 0){
		mintps->Value = imintps;
		assembleAllRunData();
		graph->getGraph()->Refresh();
	}
}

void bre20::DynoRule::assembleAllRunData()
{
	for(int i = 0; i<graph->getGraph()->Runs->Count; i++)
		assembleRunData(static_cast<DynoRun*>(graph->getGraph()->Runs->get_Item(i)));
}
void bre20::DynoRule::assembleRunData(DynoRun* run)
{
	if(run == NULL)
		return;
	int tps = (int)DynoLib::DynoGraph::Indices::IND_TPS;
	ArrayList* newData = new ArrayList();
	curData = run->AllData;

	for(int i = 0; i< curData->Count; i++)
	{
		GraphItem* item = static_cast<GraphItem*>(curData->get_Item(i));
		if(item->Data[tps] >= mintps->Value)
			newData->Add(item);
	}
	newData->Sort();
	run->RunData = newData;
}

void bre20::DynoRule::RecalculateAllRuns()
{
	for(int i = 0; i<graph->getGraph()->Runs->Count; i++)
		RecalculateRun(i);
	graph->getGraph()->ResetGraphLimits();
}
void bre20::DynoRule::RecalculateRun(int runIndex)
{
	ArrayList* runs = graph->getGraph()->Runs;
	if(runIndex < runs->Count)
	{
		curRun = static_cast<DynoRun*>(runs->get_Item(runIndex));
		calcs = new DynoCalculation(curRun->get_Interval());
		calcs->recalculate(curRun);
	}
}

void bre20::DynoRule::addFile(String* fname)
{
	LogFileReader* rd = LogFileReader::openLogFile(fname);
	if(rd == NULL)
		return;

	setupRule(rd);

	String* str = rd->readLine();
	while(str != NULL)
	{
		executeRule(rd);
		str = rd->readLine();
	}
	finishRule(NULL);
}
void bre20::DynoRule::setupRule(LogFileReader* rd)
{
	lastitem = NULL;
	
	curData = new ArrayList();
	curRun = new DynoRun(getRunName(rd->getFileName()),Color::Empty);
	curRun->Interval = rd->getInterval();
	
	calcs = new DynoCalculation(rd->getInterval());
}
bool bre20::DynoRule::executeRule(LogFileReader* rd)
{
	float rpm, tps, afr, speed;
	__int64 time;

	try
	{
		rpm = Convert::ToSingle(rd->getValue("Rpm"));
		afr = Convert::ToSingle(rd->getValue("Afr"));
		tps = Common::extractDouble(rd->getValue("TPS"));
		speed = Common::extractDouble(rd->getValue("Speed"));
		time = Convert::ToInt64(rd->getValue("#100ns"));
	}
	catch(...)
	{return false;}

	if(lastitem == NULL )//&& tps >= (float)calcs->getMinTps())
	{
		lastitem = new float __gc[4];//rpm,afr,tps
		lasttime = time;
		lastitem[0] = rpm;
		lastitem[1] = afr;
		lastitem[2] = tps;
		lastitem[3] = speed;
		return true;
	}
	else if(lastitem == NULL)
	{
		return true;
	}

	//deceleration, I think we should ignore these, eh?
	if(rpm <= lastitem[0])// || tps < (float)calcs->getMinTps()) 
	//if(speed <= lastitem[3])
	{
		lastitem = NULL;
		return true;
	}

	float curitem __gc[] = new float __gc[4];//rpm,afr,tps
	curitem[0] = rpm;
	curitem[1] = afr;
	curitem[2] = tps;
	curitem[3] = speed;

	GraphItem* item = calcs->calculate(lasttime,lastitem, time, curitem);
	if(item != NULL)
		curData->Add(item);

	lastitem = curitem;
	lasttime = time;

	return true;
}

///******************************
///** overrides
void bre20::DynoRule::setupRule(Form* frm)
{
	//TODO: make it so the we always replace [0] with the analyzer's graph
	//and dont interrupt the addition of new graphs or existing overlays. 
	//Use the new addDynoRun function to do it.
	ana = static_cast<LogAnalyzer*>(frm);
//	if(!isSetup || !getRunName(static_cast<LogAnalyzer*>(ana)->
//		getFileReader()->getFileName())->
//		Equals( static_cast<DynoRun*>(graph->getGraph()->Runs->get_Item(0))->get_Name() ))
//	{
		fromAnalyzer = true;
		setupRule(static_cast<LogAnalyzer*>(ana)->getFileReader());
//	}
///	else
//	{
//		curRun = NULL;
//	}
}

void bre20::DynoRule::finishRule(Form* frm)
{
	if(curRun != NULL)
	{
		curRun->set_AllData( curData );
		assembleRunData(curRun);
		if(fromAnalyzer && graph->getGraph()->Runs->Count > 0)
			graph->getGraph()->addDynoRun(curRun,0,true);
		else
			graph->getGraph()->addDynoRun(curRun);
		graph->getGraph()->Refresh();
	}
	fromAnalyzer = false;

	if(!isSetup)
	{
		static_cast<LogAnalyzer*>(ana)->getTabControl()->TabPages->Add(graph);
		isSetup = true;
	}
}

bool bre20::DynoRule::executeRule(Form* frm)
{
//	if(needToRun)
		return executeRule(static_cast<LogAnalyzer*>(ana)->getFileReader());
//	else
//		return true;
}

bool bre20::DynoRule::updateRule(Form* frm){return true;}
bool bre20::DynoRule::executeReplay(String* keys[], Hashtable* values){return true;}