#include "stdafx.h"
#include "Engine.h"

//#include "JdmPw0Boost.h"
#include "JdmPw0.h"
#include "Pm6Ng.h"
#include "Obd0Pr3.h"
#include "JdmPr3.h"
#include "JdmPr3Dev.h"
#include "JdmPw0Dev.h"
#include "JdmPr3ITB.h"
//#include "EuroPw0Boost.h"
#include "EuroPw0Dev.h"
//#include "JdmPr3Boost.h"
#include "EuroPw0.h"
#include "GhdPm6.h"
#include "Pm6ng.h"
#include "EuroPw0ITB.h"
#include "JdmPw0ITB.h"

RomRep::Engine::Engine(){}

unsigned char RomRep::Engine::getFile(String* fname, int size) __gc[]
{
	BinaryReader * br = new BinaryReader(new FileStream(fname,FileMode::Open));
	unsigned char rom __gc[] = br->ReadBytes(size);
	br->Close();

	return rom; 

}


bool RomRep::Engine::doesFileExist(String* fname) //can call before getRom()
{
	try
	{
		FileStream* fs = new FileStream(fname,FileMode::Open);
		return true;
	}
	catch(...)
	{
		return false;
	}
}

//identifies what kind of ECU the file 'fname' is. 
//Returns an object of this ECU type.
RomRep::HondaEcu* RomRep::Engine::getRom(String* fname, int size)
{
	unsigned char fil __gc[] = Engine::getFile(fname, size);

	if(fil == NULL)
		return NULL;

	HondaEcu* rom = NULL;

	unsigned char head __gc[] = new unsigned char __gc[RomRep::HondaEcu::headlen];

	for(int i = 0; i<HondaEcu::headlen; i++)
		head[i] = fil[i];

	//go through and figure out which kind of ECU the file is from.
	if(JdmPw0Dev::validate(fil))
		rom = new JdmPw0Dev(fil, size);
	else if(JdmPw0ITB::validate(fil))
		rom = new JdmPw0ITB(fil, size);
	else if(JdmPw0::validate(head))
		rom = new JdmPw0(fil, size);
	else if(Obd0Pr3::validate(head))
		rom = new Obd0Pr3(fil, size);
	else if(JdmPr3Dev::validate(fil))
		rom = new JdmPr3Dev(fil, size);
	else if(JdmPr3ITB::validate(fil))
		rom = new JdmPr3ITB(fil, size);
	else if(JdmPr3::validate(head))
		rom = new JdmPr3(fil, size);
	else if(EuroPw0Dev::validate(fil))
		rom = new EuroPw0Dev(fil, size);
	else if(EuroPw0ITB::validate(fil))
		rom = new EuroPw0ITB(fil, size);
	else if(EuroPw0::validate(head))
		rom = new EuroPw0(fil, size);
	else if(Pm6Ng::validate(head))
		rom = new Pm6Ng(fil, size);
//	else if(GhdPm6::validate(head))
//		rom = new GhdPm6(fil, size);
//	else if(Pm6::validate(head))
//		rom = new Pm6(fil, size);


	return rom;
}

bool RomRep::Engine::saveFile(unsigned char rom __gc[], String* fname, int size)
{
	try
	{
		BinaryWriter * br = new BinaryWriter(new FileStream(fname,FileMode::Create));
		
		br->Write(rom);

		br->Close();
	}
	catch(...)
	{
		return false;
	}

	return true;
}


//*********************************************************************************
int RomRep::Engine::psiToRaw(float psi, int sensor)
{
	
	int ret = 0;
	if(sensor == 1)//2 bar
	{
//		float xovervolts = 2.5;
		float volts = (psi*RomRep::DevUtil::PSICONST2BAR)+RomRep::DevUtil::XOVER2BAR;
		ret = (int)(volts*255.0/5.0);
	}
	if(sensor == 2)//2.5 bar
	{
//		float xovervolts = 2;
		float volts = (psi*RomRep::DevUtil::PSICONST25BAR)+RomRep::DevUtil::XOVER25BAR;
		ret = (int)(volts*255.0/5.0);
	}
	if(sensor == 4)//2.5 bar
	{
//		float xovervolts = 2;
		float volts = (psi*RomRep::DevUtil::PSICONSTMOTOROLA25BAR)+RomRep::DevUtil::XOVERMOTOROLA25BAR;
		ret = (int)(volts*255.0/5.0);
	}
	if(sensor == 3)//3 bar
	{
//		float xovervolts = 1.6;
		float volts = (psi*RomRep::DevUtil::PSICONST3BAR)+RomRep::DevUtil::XOVER3BAR;
		ret = (int)(volts*255.0/5.0);
	}
	else
	{
//		float xovervolts = 3.0;
		float volts = (psi*RomRep::DevUtil::PSICONSTSTOCK)+RomRep::DevUtil::XOVERSTOCK;
		ret = (int)(volts*255.0/5.0);
	}

	if(ret >255)
		ret = 255;
	if(ret<0)
		ret = 0;

	return ret;
}
int RomRep::Engine:: inHgToRaw(int inhg, int sensor)
{
		int ret = 0;
	if(sensor == 1)//2 bar
	{
//		float xovervolts = 2.5;
		float volts = (inhg*RomRep::DevUtil::INHGCONST2BAR)+RomRep::DevUtil::XOVER2BAR;
		ret = (int)(volts*255.0/5.0);
	}
	if(sensor == 2)//2.5 bar
	{
//		float xovervolts = 2;
		float volts = (inhg*(RomRep::DevUtil::INHGCONST25BAR))+RomRep::DevUtil::XOVER25BAR;
		ret = (int)(volts*255.0/5.0);
	}
	if(sensor == 4)//2.5 bar
	{
//		float xovervolts = 2;
		float volts = (inhg*(RomRep::DevUtil::INHGCONSTMOTOROLA25BAR))+RomRep::DevUtil::XOVERMOTOROLA25BAR;
		ret = (int)(volts*255.0/5.0);
	}
	if(sensor == 3)//3 bar
	{
//		float xovervolts = 1.6;
		float volts = (inhg*(RomRep::DevUtil::INHGCONST3BAR))+RomRep::DevUtil::XOVER3BAR;
		ret = (int)(volts*255.0/5.0);
	}
	else
	{
//		float xovervolts = 3.0;
		float volts = (inhg*(RomRep::DevUtil::INHGCONSTSTOCK))+RomRep::DevUtil::XOVERSTOCK;
		ret = (int)(volts*255.0/5.0);
	}

	if(ret >255)
		ret = 255;
	if(ret<0)
		ret = 0;

	return ret;
}

float RomRep::Engine::voltsToPsi(float volts, int sensor)
{
	if(sensor == 1)//2 bar
		return (volts - RomRep::DevUtil::XOVER2BAR)/RomRep::DevUtil::PSICONST2BAR;
	if(sensor == 2)//2.5 bar
		return (volts - RomRep::DevUtil::XOVER25BAR)/RomRep::DevUtil::PSICONST25BAR;
	if(sensor == 4)//2.5 bar
		return (volts - RomRep::DevUtil::XOVERMOTOROLA25BAR)/RomRep::DevUtil::PSICONSTMOTOROLA25BAR;
	if(sensor == 3)//3 bar
		return (volts - RomRep::DevUtil::XOVER3BAR)/RomRep::DevUtil::PSICONST3BAR;
	else
		return (volts - RomRep::DevUtil::XOVERSTOCK)/RomRep::DevUtil::PSICONSTSTOCK;
}

int RomRep::Engine::voltsToInHg(float volts, int sensor)
{
	if(sensor == 1)//2 bar
		return (int)((volts - RomRep::DevUtil::XOVER2BAR)/(RomRep::DevUtil::INHGCONST2BAR));
	if(sensor == 2)//2.5 bar
		return (int)((volts - RomRep::DevUtil::XOVER25BAR)/(RomRep::DevUtil::INHGCONST25BAR));
	if(sensor == 4)//2.5 bar
		return (int)((volts - RomRep::DevUtil::XOVERMOTOROLA25BAR)/(RomRep::DevUtil::INHGCONSTMOTOROLA25BAR));
	if(sensor == 3)//3 bar
		return (int)((volts - RomRep::DevUtil::XOVER3BAR)/(RomRep::DevUtil::INHGCONST3BAR));
	else
		return (int)((volts - RomRep::DevUtil::XOVERSTOCK)/(RomRep::DevUtil::INHGCONSTSTOCK));
}
float RomRep::Engine::rawToVolts(int raw)
{
	if(raw<0)
		raw = 0;
	if(raw>255)
		raw = 255;

	return ((float)raw*5.0F)/255.0F;
}

//this is for interp based o2 conversion 
float RomRep::Engine::rawToAfr(int raw, float volts __gc[], float afr __gc[])
{
	float key = Engine::rawToVolts(raw);		
			
			
	//vcal_0
	int i = 0;
	for(i = 0; key<volts[i]; i++);
			
	// now, volts[i-1] > key > volts[i]; so we interpolate between afr[i-1] and afr[i].

	bool negative = false;
	float r0 = key-volts[i];
	float r1 = volts[i-1]-volts[i];
	float A = afr[i]-afr[i-1];
	
	if(A<0)
	{
//		A = A*-1;
		negative = true;
	}

	A=A*r0/r1;

	if(negative)
		A = afr[i]-A;
	else
		A = afr[i]+A;

	return A;
}