#pragma once
using namespace System;
using namespace System::IO;

#include "hondaecu.h"

namespace RomRep
{
	[Serializable] public __gc class Engine
	{
	public:
		Engine();

		///
		///\param fname file name
		///\param size size to read in
		///\returns byte array containing the entire file
		///
		static unsigned char getFile(String* fname, int size) __gc[]; //size in num of bytes
		///
		///\param fname file name
		///\returns bool indicating whether the fiel exists
		///
		static bool doesFileExist(String* fname); //can call before getRom() 
		///
		///Opens a the file then tries to determine if it is a known type of ROM. 
		///If so that type of ROM is instantiated and returned, otherwise a null is
		///returned
		///
		///\param fname file name
		///\param size number of bytes to read
		///\returns HondaEcu object or null
		///
		static HondaEcu* getRom(String* fname, int size); 

		///
		///\param rom byte array containing the whole rom
		///\param fname file name
		///\param size number of bytes to save.
		///\return true if successful, false otherwise
		///
		static bool saveFile(unsigned char rom __gc[] ,String* fname, int size);

		///
		///Converts voltage to PSI
		///
		///\param sensor type of map sensor
		///
		static float voltsToPsi(float volts, int sensor);
		///
		///Converts voltage to InHg
		///
		///\param sensor type of map sensor
		///
		static int voltsToInHg(float volts, int sensor);
		///
		///Converts PSI to raw byte
		///
		///\param sensor type of map sensor
		///
		static int psiToRaw(float psi, int sensor);
		///
		///Converts InHg to raw byte 
		///
		///\param sensor type of map sensor
		///
		static int inHgToRaw(int inhg, int sensor);
		///
		///Converts byte to voltage
		///
		static float rawToVolts(int raw);
		///
		///Converts raw byte to afr. Interpolates the AFR from the 
		///arrays passed in.
		///
		///\param volts float array with voltages
		///\param afr float array with afrs that correspond to the voltages
		///
		static float rawToAfr(int raw, float volts __gc[], float afr __gc[]);
	};
}

//#endif