#include "stdafx.h"
#include "ExtraTableGrid.h"

void bre20::ExtraTableGrid::init(int width, String* titles[])
{
	if(width > -1)
		grid->setAllColumnWidths(width);

	this->grid->Mode = SimpleGrid::Grid::MODE_INT;
	this->grid->PaintMode = SimpleGrid::Grid::M_PAINT_NO_COLORS;
	this->mode = GridPage::MODE_NOGRAPH;
	this->CellChanged += new System::EventHandler(this, grid_Click);
	grid->RowHeaderWidth = 80;

	sae = false;
	if(Globals::Props->getParam(S"logUnits")->Equals("sae"))
		sae = true;

	Globals::Props->add_OnPropertyChanged(new PropertyEventHandler(this,prop_Changed));
	this->rom->add_OnByteChanged(new RomRep::ByteChangeHandler(this, byte_Changed));

	if(!vcal)
		return;
	fillGrid(titles);
}
void bre20::ExtraTableGrid::fillGrid(String* titles[])
{
	int x = this->grid->GridSize.Width;
	int valpos = 1;
	if(!haskey)
	{
		this->grid->GridSize = System::Drawing::Size(x,1);
		valpos = 0;
	}

	for(int i = 0; i<x; i++)
	{
		this->grid->setValue(i,0,calcKey(i));
		this->grid->setValue(i,valpos,calcVal(i));
		
		if(titles == NULL)
			this->grid->setHeader(i,i.ToString());
		else if(i<titles->Length && titles[i] != NULL)
			grid->setHeader(i,titles[i]);
	}

	grid->setRowHeader(0,getUnits(keymode));
	grid->setRowHeader(valpos,getUnits(valmode));
}

String* bre20::ExtraTableGrid::getUnits(int mode)
{
	String* units = "Raw";
	if(mode == MODE_TEMP)
	{
		units = "Temp (C)";
		if(sae)
			units = "Temp (F)";
	}
	else if(mode == MODE_RPM)
		units = "RPM";

	return units;
}
int bre20::ExtraTableGrid::calcKey(int col)
{
	if(!haskey)
		return -1;

	int mult = 2;
	if(addmode == MODE_WORD)
		mult = 3;
	int val = (int)rom->getByte(addy+col*mult);
	if(keymode == MODE_TEMP)
		val = (int)rom->calcTemp(val,sae);
//	else if(keymode == MODE_RPM)
//		val = rom->calc1ByteRpm(vtec,val);
	return val;
}
int bre20::ExtraTableGrid::calcVal(int col)
{
	Addys* ad = calcAddys(col);

	int lowaddy = ad->lowAddy;
	int highaddy = ad->highAddy;

	int val = (int)rom->getByte(lowaddy);
	if(addmode == MODE_WORD)
	{
		if(valmode == MODE_RPM)
			val = rom->getTwoByteRpm(highaddy,lowaddy);
		else		
			val = val+(int)rom->getByte(highaddy)*256;
	}

	return val;
}

bre20::Addys* bre20::ExtraTableGrid::calcAddys(int col)
{
	Addys* ad = new Addys();

	int mult = 2;
	if(addmode == MODE_WORD && haskey)
		mult = 3;
	else if(addmode == MODE_BYTE && !haskey)
		mult = 1;
	ad->highAddy = addy+col*mult+2;
	ad->lowAddy = addy+col*mult+1;

	if(!haskey)
	{
		ad->lowAddy -=1;
		ad->highAddy -=1;
	}

	return ad;

}

int bre20::ExtraTableGrid::getWidth()
{
	return (grid->GridSize.Width * grid->getColumnWidth(0) + grid->RowHeaderWidth); 
}
int bre20::ExtraTableGrid::getHeight()
{
	return (grid->GridSize.Height * (grid->RowHeight+1));
}

void bre20::ExtraTableGrid::grid_Click(System::Object *  sender, System::EventArgs *  e)
{
	SimpleGrid::CellEvent* ce = static_cast<SimpleGrid::CellEvent*>(e);

	Addys* ad = calcAddys(ce->Column);

	int keyrow = 0;
	int valrow = 1;
	if(!haskey)
	{
		keyrow--;
		valrow--;
	}

	if(ce->Row == keyrow)
		this->grid->setValue(ce->Column,0,calcKey(ce->Column));

	else if(ce->Row == valrow)
	{
		if(addmode == MODE_BYTE)
		{
			rom->setByte(ad->lowAddy,(int)this->grid->getValue(ce->Column,valrow));
		}
		else if(addmode == MODE_WORD && valmode == MODE_RPM)
			rom->setTwoByteRpm(ad->highAddy,ad->lowAddy,(int)this->grid->getValue(ce->Column,valrow));
		else 
		{
			int val = (int)this->grid->getValue(ce->Column,valrow);
			int high = val/256;
			int low = val%256;
			rom->setByte(ad->lowAddy,low);
			rom->setByte(ad->highAddy,high);
		}
		this->grid->setValue(ce->Column,valrow,calcVal(ce->Column));
	}
}

void bre20::ExtraTableGrid::prop_Changed(System::Object *  sender, PropertyEventArgs *  e)
{
	if(!e->getName()->Equals("logUnits"))
		return;

	bool tmpsae = false;
	if(e->getValue()->Equals("sae"))
		tmpsae = true;

	if(sae == tmpsae)
		return;

	sae = tmpsae;

	if(!vcal)
		return;
	fillGrid(headers);

}

void bre20::ExtraTableGrid::byte_Changed(Object* sender, RomRep::ByteChangeArgs* e)
{
	if(!vcal)
		return;

	int mult = 2;
	if(addmode == MODE_WORD && haskey)
		mult = 3;
	else if(addmode == MODE_BYTE && !haskey)
		mult = 1;

	int len = grid->GridSize.Width * mult;

	int loc = e->getByteChange()->getLocation();

	if(loc<addy || loc>addy+len-1)
		return;

	int x = (loc-addy)/mult;

	int valpos = 1;
	if(!haskey)
		valpos = 0;

	if(haskey && (loc-addy) % mult == 0)//its a key
		grid->setValue(x,0,calcKey(x));
	else
		grid->setValue(x,valpos,calcVal(x));
}