#include "stdafx.h"
#include "Form1.h"
#include "Engine.h"
#include <windows.h>
#include "splash.h"

using namespace bre20;

int APIENTRY _tWinMain(HINSTANCE hInstance,HINSTANCE hPrevInstance,
                     LPTSTR lpCmdLine,int nCmdShow)
{
	try{
		Splash::ShowSplashScreen();
	}
	catch(...){}

	System::Threading::Thread::CurrentThread->ApartmentState = System::Threading::ApartmentState::STA;

	String* cmd = new String(lpCmdLine);
	Char delim[] = {'"'};
	String* tokens[] = cmd->Split(delim);
	Application::Run(new Form1(tokens));
	return 0;
}

bre20::Form1::Form1(String * fname[])
{
	InitializeComponent();

	Globals::LogFile = new LogFileObj("brelogs.txt");
	Globals::Props = new Properties();
	this->scriptManager = new CSScriptLibrary::ScriptManager(false);
	this->scriptManager->MdiParent = this;

	this->Text = About::BRETITLE;

	openFiles(openStartupFile(fname));
}

///opens the startup file.
String* bre20::Form1::openStartupFile(String* files[])[]
{
	String* stname = Globals::Props->getParam("startupFile");
	if(stname->Equals(""))
		return files;

	if(files == NULL)
	{
		String* noofiles[] = new String*[1];
		noofiles[0] = stname;
		return noofiles;
	}

	String* noofiles[] = new String*[files->Length+1];
	for(int i = 0; i<files->Length; i++)
		noofiles[i] = files[i];

	noofiles[noofiles->Length-1] = stname;
	return noofiles;
}

void bre20::Form1::openFiles(String* files[])
{
	int flen = 0x8000;
	for(int i = 0; i<files->Length; i++)
	{
		files[i] = files[i]->Trim();
		if(!files[i]->Equals(""))
		{
			try
			{
				RomRep::HondaEcu* vtrom = RomRep::Engine::getRom(files[i],flen);
				OnOpenRomArgs* romargs = new OnOpenRomArgs(files[i]);
				romargs->setRom(vtrom);

				//do this here so the script kiddies have access to it.
				ParamPage* ppage = NULL;
				if(vtrom != NULL)
					ppage = getParamPage(vtrom->getType(),vtrom);

				Object* args[] = new Object*[1];
				args[0] = romargs;
				args = this->scriptManager->CallStaticMethod(S"OnOpenRom", args, true);

				if(romargs->isOpenable())// && vtrom == NULL)
				{
					vtrom = romargs->getRom();
					ppage = romargs->getParamPage();
				}

				if(vtrom != NULL && ppage == NULL)
					ppage = getParamPage(vtrom->getType(),vtrom);

				if(vtrom != NULL && ppage != NULL)
				{
					child* box = new child(vtrom, flen, files[i], ppage);
					box->MdiParent = this;
					box->OnFunctionCalled += new ScriptFunctionCallHandler(this,child_function_called);
					this->scriptManager->AddChild(box, vtrom->getType());
					box->add_Closing(new CancelEventHandler(this,child_closing));
					box->add_Activated(new EventHandler(this,child_activated));
					box->Show();
				}
				else
				{
					MessageBox(NULL,"Open a valid .bin file","File not supported",0);
				}
			}
			catch(Exception* ex)
			{
				MessageBox(NULL,"Error while opening","BRE",0);
			//	MessageBox(NULL,String::Concat("Error while opening: ",ex->Message,S"\n\n",ex->StackTrace),"BRE",0);
			}
		}
	}
}//end openFiles


//!Saves the active MDI bre20.child form. Length of the binary file in bytes is len.
void bre20::Form1::saveActive(String * fname, int len)
{
	Form* form = this->ActiveMdiChild;
	save(fname,len,form);
}

//!Saves the file contained in form. Form must be of Type bre20.child
void bre20::Form1::save(String * fname, int len, Form * form)
{
	if(form == NULL)
		return;
	try
	{
		child* rombox = dynamic_cast<child*>(form);
		rombox->setChecksum();
		unsigned char rom __gc[] = rombox->getFile();
		rombox->setRomSaved();
		RomRep::Engine::saveFile(rom, fname, len);
	}
	catch(...){}
}

ParamPage* bre20::Form1::getParamPage(String * type, RomRep::HondaEcu* ecu)
{
	if(type->Equals(S"Ghd Pm6"))
		return new GhdPM6Param(ecu);
	else if(type->IndexOf(S"NG") >= 0)
		return new NGParam(ecu);
	else if(type->Equals(S"90-91 Pm6"))
		return new PM6Param(ecu);
	else if(type->Equals(S"JDM PW0") || type->Equals(S"JDM PW0 ITB") || type->Equals(S"Euro PW0") || type->Equals(S"Euro PW0 ITB"))
		return new VT0Param(ecu,1,4);
	else if(type->Equals(S"JDM PR3-58") || type->Equals(S"PR3-93") || type->Equals(S"PR3 ITB ROM"))
		return new VT0Param(ecu,5,8);
	else if(type->Equals(S"Jdm Pr3 dev") || type->Equals(S"Euro Pw0 dev")|| type->Equals(S"Jdm Pw0 dev"))
		return new DevParam(static_cast<RomRep::VT0Ecu*>(ecu));
	else
		return new ParamPage();
	{}
}

int bre20::Form1::Checksum(unsigned char thefile __gc[])
{
	int i;
	int j;
	long ad = 0;

	for (i = 0;i< 0x8000; i++)//checksumlen
	{
		ad += thefile[i];
	}
	j = ad % 256;

 	return j;
}

//**************************************events
void bre20::Form1::mnuToggleTopMost_Click(System::Object *  sender, System::EventArgs *  e)
{
	Form* form = ActiveMdiChild;
	if(frmTopMost != NULL){
		frmTopMost->TopMost = false;
		//SetParent((HWND)form->Handle.ToInt32(),(HWND)frmTopMostParent);
		frmTopMost->Parent = NULL;
		frmTopMost->MdiParent = this;
	}
	form->TopMost = !form->TopMost;

	if(form == frmTopMost){
		frmTopMost = NULL;
		return;
	}
/*
	HWND hwnd = (HWND)form->Handle.ToInt32();
	if(SetWindowPos(hwnd,HWND_TOPMOST,0,0,0,0,SWP_NOSIZE))
		MessageBox(NULL,"worked","Poo",NULL);
	else
		MessageBox(NULL,"no worky","Poo",NULL);

	form->BringToFront(); this->act*/
	frmTopMost = NULL;
	if(form->TopMost){
		frmTopMost = form;
		frmTopMostParent = (int)GetParent((HWND)form->Handle.ToInt32());
		form->MdiParent = NULL;
		form->Parent = this;
		//SetParent((HWND)form->Handle.ToInt32(),(HWND)this->Handle.ToInt32());
	}
}

void bre20::Form1::child_activated(System::Object *  sender, System::EventArgs *  e)
{
	//if(frmTopMost != NULL && static_cast<Form*>(sender) != frmTopMost){
	//	frmTopMost->Activate();
	//	frmTopMost->Focus();
	//	ActivateMdiChild(frmTopMost);
	//}
}

void bre20::Form1::child_function_called(Object* sender, ScriptFunctionCallArgs * e)
{
	Object * args[] = new Object*[4];

	args[0] = this;
	args[1] = sender;
	args[2] = e->getComponent();
	args[3] = e->getEventArgs();

	CSScriptLibrary::Script * scr = scriptManager->CSScript->getScript(e->getMethod());
	scr->ExecuteMethod(sender,CSScriptLibrary::CSScript::getMethod(e->getMethod()),args);
}

void bre20::Form1::openMenu_Click(System::Object *  sender, System::EventArgs *  e)
{
	openFileDialog1->Filter = S"Bin files (*.bin)|*.bin|All files (*.*)|*.*";
		
	if(openFileDialog1->ShowDialog() == DialogResult::OK)
	{
		String * fname[] = openFileDialog1->FileNames;
		openFiles(fname);
	}//end open

	System::GC::Collect();
}

void bre20::Form1::saveMenu_Click(System::Object *  sender, System::EventArgs *  e)
{
	Form* form = this->ActiveMdiChild;

	if(form == NULL)
		return;

	try
	{
		child* rombox = dynamic_cast<child*>(form);
		
		saveActive(rombox->getFileName(), rombox->getFileLength());
	}
	catch(...)
	{
		MessageBox(NULL,"Save Failed","BRE",0);
	}
}

void bre20::Form1::saveAsMenu_Click(System::Object *  sender, System::EventArgs *  e)
{
	Form* form = this->ActiveMdiChild;

	if(form == NULL)
		return;

	child* rombox = dynamic_cast<child*>(form);
	
	saveFileDialog1->Filter = "Bin files (*.bin)|*.bin|All files (*.*)|*.*";
	if(saveFileDialog1->ShowDialog() == DialogResult::OK)
		saveActive(saveFileDialog1->FileName,0x8000);
	else
		return;

	rombox->setFileName(saveFileDialog1->FileName);
}

void bre20::Form1::Form1_Load(System::Object *  sender, System::EventArgs *  e)
{
	try{
	Splash::CloseForm();
	}catch(...){}
	this->Focus();
	menuItem4_Click(NULL,NULL);
}


//**********************************************
//******* Data Logger stuff

void bre20::Form1::menuItem4_Click(System::Object *  sender, System::EventArgs *  e)
{
	//the about box...
	About* ab = new About();
	ab->StartPosition = System::Windows::Forms::FormStartPosition::CenterScreen;
	ab->Focus();
	ab->ShowDialog();
}

void bre20::Form1::toolBar1_ButtonClick(System::Object *  sender, System::Windows::Forms::ToolBarButtonClickEventArgs *  e)
{
	ToolBarButton* b = e->Button;

	switch (toolBar1->Buttons->IndexOf(e->Button))
	{
		case 1 :
			this->openMenu_Click(b,NULL);
			break;
		case 2 :
			this->saveMenu_Click(b,NULL);
			break;
		case 3 :
			this->saveAsMenu_Click(b,NULL);
			break;
	}
}

void bre20::Form1::startupFile_Click(System::Object *  sender, System::EventArgs *  e)
{
	StartupFile* sf = new StartupFile();
	sf->MdiParent = this;

	sf->Show();
}