#pragma once
using namespace System;
using namespace System::Windows::Forms;
using namespace System::Collections;

#include "FormWithRules.h"
#include "Rule.h"

namespace bre20
{
	///Base class for all classes that need rules
	public __gc class FormWithRules: public Form
	{
	protected:
		///list of rules
		ArrayList* rules;

		///executes all rules
		///\returns false if failed
		virtual bool executeRules();

		///sets up all rules
		///\returns false if failed
		virtual bool setupRules();

		virtual bool finishRules();

		virtual bool executeReplays(String* keys[], Hashtable* values);

		virtual bool updateRules();
	public:
		///basic constructor
		FormWithRules();

		///rule loading constructor 
		///
		///\param rulearray an array of rules to load
		FormWithRules(ArrayList* rulearray);

		//Adds a rule
		//
		//\returns false if failed
	//	virtual bool addRule(Rule* rule);

		///Replaces a rule with your rule
		///
		///\param modified the \ref rulefield "rule field" of the Rule to replace
		///\param all set to true if you want to match all fields in the \ref rulefield "rule field"
		///\param rule the Rule to replace the old one 
		///\returns one of the static ints: ERR_ERROR, ERR_NOT_FOUND, REPLACED
		virtual int replaceRule(String* modified, bool all, Rule* rule);

		///Attempts to find a rule
		///
		///\param modified the \ref rulefield "rule field" of the Rule to replace
		///\param all set to true if you want to match all fields in the \ref rulefield "rule field"
		///\returns true if the rule is found
		virtual bool hasRule(String* modified, bool all);

		///Attempts to find a Rule
		///
		///\param modified the \ref rulefield "rule field" of the Rule to replace
		///\param all set to true if you want to match all fields in the \ref rulefield "rule field"
		///\returns the index of the rule or one of the static ints: ERR_ERROR, ERR_NOT_FOUND
		virtual int findRule(String* modified, bool all);

		///Attempts to find a Rule and remove it
		///
		///\param modified the \ref rulefield "rule field" of the Rule to replace
		///\param all set to true if you want to match all fields in the \ref rulefield "rule field"
		///\returns true if removed
		virtual bool removeRule(String* modified, bool all);

		///Attempts to remove a Rule
		///
		///\param rule the Rule to remove
		///\returns true if removed
		virtual bool removeRule(Rule* rule);

		//Retrieves the Rule array
		//
		//\returns the rules
	//	virtual ArrayList* getRules(){return rules;}

		///if rule is not found
		static const int ERR_NOT_FOUND = -1;
		///if there is a real error
		static const int ERR_ERROR = -2;
		///if the rule was replaced
		static const int REPLACED = 1;

	private:
		/// <summary>
		/// Required designer variable.
		/// </summary>
		System::ComponentModel::Container* components;

		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		void InitializeComponent(void)
		{
		}

	};
}