//This adds a Datlogger definition definition to BRE

using System;
using System.Windows.Forms;
using System.Collections;
using System.Drawing;

using bre20;
//css_reference ..\bre.exe;
//css_reference ..\Gauge.dll;

namespace BensScripts
{
	public class Vt0Logger
	{
		child romWind;
		public static object AppliesTo(){return "pr3:pw0";}
		public static object ScriptTitle(){return "Obd0 Vtec Logger Defn by Ben Ogle";}
		public object ScriptMain(params object[] args)
		{
			romWind = (child)args[1];
			//subscribe to the event
			romWind.subscribeToEvent("OnDataloggerOpened", "BensScripts.Vt0Logger.log_Opened");
			return null;
		}
		public object log_Opened(params object[] args)
		{	//when the logger is opened, then we add our rule
			romWind.addLoggerRule(new Vt0LoggerDef(romWind.getRom()));
			return null;
		}
	}//end class

	public class Vt0LoggerDef: Rule
	{
		Logger log;
		bool dev,sae,oldsae;
		int mapsensor, tpsmin, tpslen, tempmax;
		RomRep.HondaEcu ecu;
		string tempunit;

		//save a reference to our keys
		LoggerKey rpm, fcell, boost, vac, vtec, tps, speed, ect, iat, fuel, ign;
		Gauge.Gauge grpm,gboost,gvac,gtps,gspeed, gect, giat;

		//need type, mapsensor type,
		public Vt0LoggerDef(RomRep.HondaEcu rom)
		{
			ecu = rom;
			dev = Common.isMatch("dev",rom.getType(),false);
			mapsensor = RomRep.DevUtil.getMapSensor(rom.getFile());
		}
		public override void setupRule(Form frm)
		{
			log = ((Logger)frm);

			tpsmin = 50;
			tpslen = 205;

			sae = oldsae = log.isSae();

			log.setParity(1);
			log.setBaud(19200);

			//here we set up the rpm LoggerKey
			rpm = new LoggerKey("Rpm",true,true);//
			fcell = new LoggerKey("Fuel Cell",false,true);//
			boost = new LoggerKey("Boost",true,true);
			vac = new LoggerKey("Vacuum",true,true);
			vtec = new LoggerKey("VTEC",true,true);//
			tps = new LoggerKey("TPS",true,true);//
			speed = new LoggerKey("Speed",true,true);//
			ect = new LoggerKey("Water Temp",true,true);
			iat = new LoggerKey("Air Temp",true,true);
			fuel = new LoggerKey("Inj. (ms)",true,true);
			ign = new LoggerKey("Ignition",true,true);

			int maxboost = 20;
			if(mapsensor > 1)
				maxboost = 30;

			tempunit = "C";
			tempmax = 120;
			string tpsunit = "KPH";
			if(sae)
			{
				tpsunit = "MPH";
				tempunit = "F";
				tempmax = 240;
			}

			gboost = log.addGauge("Boost","psi",120,520,160,0,maxboost,10,0);
			gvac = log.addGauge("Vacuum","InHg",120,72,160,0,30,10,0);
			gtps = log.addGauge("TPS","%",120,296,184,0,100,10,0);
			gspeed = log.addGauge("Speed", tpsunit,200,360,16,0,160,8,0);
			grpm = log.addGauge("Rpm","x1000",200,152,16,0,10,10,0);
			gect = log.addGauge("ECT",tempunit,150,0,24,0,tempmax,12,0);
			giat = log.addGauge("IAT",tempunit,150,560,24,0,tempmax,12,0);

			/*//old code. Just different gauge positions.
			gboost = log.addGauge("Boost","psi",120,276,0,0,maxboost,10,0);
			gvac = log.addGauge("Vacuum","InHg",120,156,0,0,30,10,0);
			gtps = log.addGauge("TPS","%",120,36,0,0,100,10,0);
			gspeed = log.addGauge("Speed", tpsunit,200,220,120,0,160,8,0);
			grpm = log.addGauge("Rpm","x1000",200,12,120,0,10,10,0);
			gect = log.addGauge("ECT",tempunit,150,430,10,0,tempmax,12,0);
			giat = log.addGauge("IAT",tempunit,150,430,160,0,tempmax,12,0);
			//*/

			//add it to the logger
			log.getLoggerData().addLoggerKey(rpm);
			log.getLoggerData().addLoggerKey(fcell);
			log.getLoggerData().addLoggerKey(boost);
			log.getLoggerData().addLoggerKey(vac);
			log.getLoggerData().addLoggerKey(vtec);
			log.getLoggerData().addLoggerKey(tps);
			log.getLoggerData().addLoggerKey(speed);
			log.getLoggerData().addLoggerKey(ect);
			log.getLoggerData().addLoggerKey(iat);
			log.getLoggerData().addLoggerKey(fuel);
			log.getLoggerData().addLoggerKey(ign);
		}
		public override bool executeRule(Form frm)
		{	
			//get the results values from the hash table
			//the names/labels/keys are from loggerCmd.ini
			Hashtable hash = log.getLoggerData().getHashtable();
			float fval;
			string str;
			bool vtech = false;
 
			try
			{
				//RPM
				int val = ((Integer)hash["rpm low"]).Value;
				int val2= ((Integer)hash["rpm high"]).Value;
				val = (val + 256*(val2));
				if(val != 0)
					val = RomRep.VT0Ecu.getRevConstant()/val;
				else 
					val = 12000;
				//set the value for the output
				grpm.CurrentValue = (float)val/1000.0F;
				rpm.Value = val.ToString();
			//	Globals.LogFile.WriteLine(this,"Srpm\t\t: " + val.ToString());

				log.setCurrentRpm(val);

				//TPS
				val = ((Integer)hash["tps"]).Value;
				if(val<tpsmin)
					tpsmin = val;
				if(val>(tpsmin+tpslen))
					tpsmin =val-tpslen;
				val = val-tpsmin;

				fval = ((float)val*100.0F)/(float)tpslen;
				gtps.CurrentValue = fval;
				tps.Value = fval.ToString("0.00")+"%";
		//		Globals.LogFile.WriteLine(this,"Stps\t\t: " + tps.Value);

				log.setCurrentTps((int)fval);

				//TEMPS
				val = ((Integer)hash["water temp"]).Value;
				int airval = ((Integer)hash["air temp"]).Value;
			
				float airfval = calcTemp(sae,airval);
				fval = calcTemp(sae,val);
			
				iat.Value = airfval.ToString("0.00") + tempunit;
				ect.Value = fval.ToString("0.00") + tempunit;

				giat.CurrentValue = airfval;
				gect.CurrentValue = fval;

				//VTEC
				val = ((Integer)hash["vtec"]).Value;
				vtec.Value = "off " + val.ToString();
				val2 = 0;
				if(val >= 128)
				{
					vtech = true;
					vtec.Value = "ON  " + val.ToString();
					val2 = 1;
				}
				log.setMapIndex(val2);

				//SPEED
				val = ((Integer)hash["speed"]).Value;
				fval = 0.6436F; //metric
				str = "KPH";
				if(sae)
				{
					fval = .4F;
					str = "MPH";
				}
				val = (int)((float)val*fval);
				gspeed.CurrentValue = (float)val;
				speed.Value = val.ToString() + str;

			//	Globals.LogFile.WriteLine(this,"Sspd\t\t: " + val.ToString());

				log.setCurrentSpeed(val);

				//FUEL CELL
				val = ((Integer)hash["fuel y"]).Value;
				if(dev)
					val2 = ((Integer)hash["dev x"]).Value;
				else
					val2 = ((Integer)hash["map image"]).Value / 16;
				fcell.Value = val2.ToString() + "," + val.ToString();
				log.setCurrentColumn(val2);
				log.setCurrentFuelRow(val);
				log.setCurrentIgnitionRow(((Integer)hash["ign y"]).Value);

				setWeights(hash, val2, val, vtech);

				//BOOST
				val = ((Integer)hash["map"]).Value;
				fval = RomRep.Engine.voltsToPsi(RomRep.Engine.rawToVolts(val),mapsensor);
				if(fval <0)
					fval = 0.0F;
				gboost.CurrentValue = fval;
				boost.Value = fval.ToString("0.00") + "psi";

				//VACUUM
				val2 = RomRep.Engine.voltsToInHg(RomRep.Engine.rawToVolts(val),mapsensor);
				if(fval>0)
					val2 = 0;
				gvac.CurrentValue = (float)val2;
				vac.Value = val2.ToString() + "inhg";

				//TODO: add the igntion / fuel logging. 
				//fuel formula: ms = val/246.875; (val is time open in number of timer ticks...) 
				//FUEL value
				val = ((Integer)hash["fuel low"]).Value;
				val2= ((Integer)hash["fuel high"]).Value;
				val = (val + 256*(val2));
				fval = ((float)val)/246.875f;
				//set the value for the output
				fuel.Value = fval.ToString("0.00");

				//IGNITION
				fval = (float)((Integer)hash["ign"]).Value;
				fval = 45.0f - (45.0f*fval/256.0f);
				//set the value for the output
				ign.Value = fval.ToString("0.00") + "";
			}
			catch(Exception e)
			{
				MessageBox.Show(e.Message +"\n\n"+ e.StackTrace);
			}

			return true;
		}
		public override bool updateRule(Form frm)
		{
			oldsae = sae;
			sae = log.isSae();
			if(oldsae != sae)
			{
				if(sae) makeSae();
				else makeMetric();
			}
			return true;
		}
		private float calcTemp(bool sae, int raw)
		{
			if(sae && raw>45)
				return -.8334F*(float)raw + 194.25F;
			else if(sae && raw<=45)
				return -2.3529F*(float)raw + 263.0F;
			else if(!sae && raw>66)
				return -.463F*(float)raw + 125.693F;
			else //if(!sae && raw<=45)
				return -1.037F*(float)raw + 163.888F;
		}
		private void makeMetric()
		{
			gspeed.Units = "KPH";
			tempunit = "C";
			tempmax = 120;

			updateUnits();
		}
		private void makeSae()
		{
			gspeed.Units = "MPH";
			tempunit = "F";
			tempmax = 240;

			updateUnits();
		}

		private void updateUnits()
		{
			iat.Value = "";
			ect.Value = "";
			speed.Value = "";
			giat.Units = tempunit;
			gect.Units = tempunit;
			gect.MaxValue = tempmax;
			giat.MaxValue = tempmax;
		}
		//weights for the fuel maps...
		private void setWeights(Hashtable hash, int col, int row, bool vtech)
		{
			int colinterp = ((Integer)hash["map image"]).Value % 16;
			int rowinterp = ((Integer)hash["frow interp"]).Value;

			//compute avg
			//need to take interp into account...
			float curcolpercent = (float)(15-colinterp)/15.0F;
			float nextcolpercent = (float)(colinterp)/15.0F;
		
			int rowsize = ecu.getRpmScalarValue(row,true,vtech)-1;
			if(rowsize<=0)
				rowsize = 1;

			float currowpercent = (float)(rowsize-rowinterp)/(float)rowsize;
			float nextrowpercent = (float)(rowinterp)/(float)rowsize;

			if(currowpercent > 1.0F)
				currowpercent = 1.0F;
			if(nextrowpercent > 1.0F)
				nextrowpercent = 1.0F;

			log.setCellWeight(0,0,curcolpercent * currowpercent);
			log.setCellWeight(1,0,nextcolpercent * currowpercent);
			log.setCellWeight(1,1,nextcolpercent * nextrowpercent);
			log.setCellWeight(0,1,curcolpercent * nextrowpercent);
		}

		public override bool executeReplay(string[] keys, Hashtable values)
		{
			//could update the gauges here as well... Oh well.
			fcell.Value = (string)values["x"] + ", " + (string)values["fuely"];
			return true;
		}
		//this is what we are modifying
		public override string modifiedFields(){return "Obd0 VTEC base";}
	}
}//end namespace