#pragma once

#include "map.h"
#include "romrepeventargs.h"

using namespace System;
using namespace System::ComponentModel;
using namespace System::Collections;
using namespace System::Windows::Forms;
using namespace System::Data;
using namespace System::Drawing;
using namespace System::Text;

namespace bre20
{
	public __gc class GridPageGenericArgs: public EventArgs
	{
	private:
		Object* m_sender;
		EventArgs* m_args;
	public:
		GridPageGenericArgs(Object* sender, EventArgs* args){
			m_sender = sender;
			m_args = args;
		}
		Object* getSender(){return m_sender;}
		EventArgs* getArgs(){return m_args;}
	};

	__delegate void GridPageGenericHandler(Object* page, GridPageGenericArgs* args);

	/// <summary> 
	/// tab page with a grid
	/// </summary>
	public __gc class GridPage : public System::Windows::Forms::TabPage
	{
	protected:
		int index;
		int mode; //0 for fuel, 1 for ign
		int lastnacol;
		SimpleGrid::Grid *  grid;
		BGraphLib::MapGraphControl* graph;

		
		bool IsInputKey(System::Windows::Forms::Keys keyData)
		{
			bool ret = true;

			switch (keyData)
			{
				case System::Windows::Forms::Keys::Left:
					break;
				case System::Windows::Forms::Keys::Right:
					break;
				case System::Windows::Forms::Keys::Up:
					break;
				case System::Windows::Forms::Keys::Down:
					break;
				case System::Windows::Forms::Keys::Tab:
					break;
				case System::Windows::Forms::Keys::Enter:
					break;
				default:
					ret = TabPage::IsInputKey(keyData);
					break;
			}
			return ret;
		}
		


private: 
		System::Windows::Forms::ContextMenu *  contextMenu1;
		System::Windows::Forms::MenuItem *  plusOne;
		System::Windows::Forms::MenuItem *  plusFive;
		System::Windows::Forms::MenuItem *  minusFive;
		System::Windows::Forms::MenuItem *  minusOne;
		System::Windows::Forms::MenuItem *  multiply;
		System::Windows::Forms::MenuItem *  add;
		System::Windows::Forms::MenuItem *  copy;
		System::Windows::Forms::MenuItem *  paste;
		System::Windows::Forms::MenuItem *  insertCol;
		System::Windows::Forms::MenuItem *  removeCol;
		System::Windows::Forms::MenuItem *  insertRow;
		System::Windows::Forms::MenuItem *  removeRow;
		System::Windows::Forms::MenuItem *  ignRetard;
		System::Windows::Forms::MenuItem *  interpX;
		System::Windows::Forms::MenuItem *  interpY;

		System::Void interpX_Click(System::Object *  sender, System::EventArgs *  e);
		System::Void interpY_Click(System::Object *  sender, System::EventArgs *  e);

		System::Void ignRetard_Click(System::Object *  sender, System::EventArgs *  e);
		System::Void copy_Click(System::Object *  sender, System::EventArgs *  e);

		System::Void paste_Click(System::Object *  sender, System::EventArgs *  e);
		System::Void insertCol_Click(System::Object *  sender, System::EventArgs *  e);
		
		System::Void multiply_Click(System::Object *  sender, System::EventArgs *  e);
		System::Void add_Click(System::Object *  sender, System::EventArgs *  e);

		System::Void plus1_Click(System::Object *  sender, System::EventArgs *  e){
			changeRange(1,0.0);
		}
		System::Void plus5_Click(System::Object *  sender, System::EventArgs *  e){
			changeRange(2,0.0);
		}
		System::Void minus1_Click(System::Object *  sender, System::EventArgs *  e){
			changeRange(4,0.0);
		}
		System::Void minus5_Click(System::Object *  sender, System::EventArgs *  e){
			changeRange(3,0.0);
		}
		System::Void mapPlot_NodeMoved(System::Object *  sender, BGraphLib::GraphNodeDragEventArgs *  e)
		{
			SimpleGrid::CellEvent * ce = new SimpleGrid::CellEvent();
			int c = e->X,r = e->Y;
			ce->Row = r;
			ce->Column = c;
		
			grid->setValue(c,r,Convert::ToSingle(e->Value+0.01));

			grid_CellChanged(NULL,ce);

			graph->setValue(c,r,Convert::ToDouble(grid->getValue(c,r)));
		}
		System::Void grid_HeaderChanged(System::Object *  sender, SimpleGrid::CellEvent* ce)
		{
			String* strs[] = new String*[1];
			strs[0] = NULL;
			if(ce->Row < 0 && ce->Column >= 0)
				strs[0] = getColumnHeader(ce->Column);
			else if(ce->Row >= 0 && ce->Column < 0)
				strs[0] = getRowHeader(ce->Row);

			RomRep::MapHeaderChangeArgs *  e = new 
				RomRep::MapHeaderChangeArgs(ce->Column,ce->Row,strs);
			//let us know this event is coming from someone who doesnt want it back...
			e->setMarker(RomRep::CellChangeArgs::Marker::ARGS_SPECIAL); 
			onHeaderChanged(e);
		}
		System::Void grid_CellChanged(System::Object *  sender, SimpleGrid::CellEvent *  e)
		{
			SimpleGrid::CellEvent* noo = new SimpleGrid::CellEvent();
			noo->Column = ((SimpleGrid::CellEvent*)e)->Column;
			noo->Row = ((SimpleGrid::CellEvent*)e)->Row;
			try
			{
				if(mode != MODE_NOGRAPH)
					graph->setValue(noo->Column,noo->Row,grid->getValue(noo->Column,noo->Row));
			}
			catch(...){}

			onCellChanged(noo);
		}

		float plus1(int x, int y){return grid->getValue(x,y) * 1.01;}
		float plus5(int x, int y){return grid->getValue(x,y) * 1.05;}
		float minus5(int x, int y){return grid->getValue(x,y)/1.05;}
		float minus1(int x, int y){return grid->getValue(x,y)/1.01;}

		float multiplyBy(int x, int y, float scalar)
		{return grid->getValue(x,y) * scalar;}
		float addNum(int x, int y, float scalar)
		{return grid->getValue(x,y) + scalar;}

	protected:
		virtual void onCellChanged(SimpleGrid::CellEvent* e){
			if (CellChanged != NULL)
				CellChanged(this,e);
		}

		virtual void onHeaderChanged(int xx, int yy, String* str){
			String* strs[] = new String*[1];
			strs[0] = str;
			RomRep::MapHeaderChangeArgs* e = new RomRep::MapHeaderChangeArgs(xx,yy,strs);
			onHeaderChanged(e);
		}
		virtual void onHeaderChanged(RomRep::MapHeaderChangeArgs* e){
			if (HeaderChanged != NULL)
				HeaderChanged(this,e);
		}

		virtual void onGenericEvent(Object* sender, EventArgs* args){
			if(GenericEvent != NULL)
				GenericEvent(this, new GridPageGenericArgs(sender,args) );
		}

		virtual void changeRange(int mode, float scal);//float (*fp)(int,int));


	public: 
		GridPage(void);
		GridPage(String * tabName, int columns, int rows, int mode);

		void addExtraContext();
		void setLastNaCol(int lnac);

		int getSelectedCells() __gc[];

		virtual void setSelection(int x, int y, int w, int h, bool refresh);

		virtual void setColumnWidth(int col,int width);

		virtual void setCellColor(int column, int row, Color color);
		virtual void setColumnHeader(int column, String * str);
		virtual String* getColumnHeader(int column);
		virtual void setGridColumnHeader(int column, String * str);
		virtual void setRowHeader(int row, String * str);
		virtual String* getRowHeader(int row);

		virtual void setHeaderEditable(bool col, int index, bool isEditable);
		virtual bool getHeaderEditable(bool col, int index);

		///This will set a value in the table but it will NOT trigger any events unless
		///event is set to true.
		virtual void setValue(int column, int row, float value, bool event);

		virtual int getMode(){return mode;}
		virtual void setMode(int mode){this->mode = mode;}
		virtual int getIndex(){return index;}
		virtual void setIndex(int i){index = i;}

		virtual float getColumnHeaderDbl(int column);
		virtual float getValue(int column, int row);

		virtual void setupGraph();
		virtual void changeView(bool table);

		virtual int getColumns(){return grid->GridSize.Width;};
		virtual int getRows(){return grid->GridSize.Height;};
		
		virtual void clearGrid();

		virtual SimpleGrid::Crosshairs* getCrosshairs()
		{
			if(grid != NULL)
				return grid->Crosshairs;
			return NULL;
		}

		virtual void setReadOnly(bool onlyread){
			grid->ReadOnly = onlyread;
		}

		virtual bool getReadOnly(){
			return grid->ReadOnly;
		}

		virtual void setColumns(int width)
		{
			int colwidth = 40;
			grid->GridSize = System::Drawing::Size(width, getRows());
			this->grid->setAllColumnWidths(colwidth);
		};
		virtual void setRows(int height){
			grid->GridSize = System::Drawing::Size(getColumns(), height);
		};

		///Any owner can subscribe stuff to this handler. It will grab the event,
		///encapsulate the event args and original sender into another event args 
		///object, then pitch a GenericEvent with this gridpage as the sender.
		void generic_event(System::Object *  sender, System::EventArgs *  e){
			onGenericEvent(sender,e);
		}

		__event GridPageGenericHandler* GenericEvent;
		
		__event EventHandler* CellChanged;
		__event RomRep::MapHeaderChangeHandler* HeaderChanged;

		virtual void callCopy(){copy_Click(NULL,NULL);};
		virtual void callPaste(){paste_Click(NULL,NULL);};
		
		static const int MODE_FUEL = RomRep::Map::MODE_FUEL;
		static const int MODE_IGN = RomRep::Map::MODE_IGNITION;
		static const int MODE_NOGRAPH = 2;

	public:
		void mapHeaderChanged(Object* sender, RomRep::MapHeaderChangeArgs* args);
		private: System::Void tabPage1_Enter(System::Object *  sender, System::EventArgs *  e){
			 grid->Focus();
		 }
	};
}