#include "stdafx.h"
#include "gridpagearray.h"
#include "adjustfuel.h"
#include "boosteff.h"
#include "romrepeventargs.h"


bre20::GridPageArray::GridPageArray(TabControl* tc, ParamPage* ppage, RomRep::HondaEcu* ecu)
{
	this->tabControl1 = tc;

	paramPage = ppage;
	paramPage->setParent(this);

	pages = NULL;
	adjustFuel = NULL;
	
	vtrom = ecu;
	if(vtrom != NULL)
		setupTabs(vtrom);
	
	paramPage->anchorGrid();

	scriptPages = NULL;
	extraMaps = new ArrayList(6);
}

void bre20::GridPageArray::setupTabs(RomRep::HondaEcu* ecu)
{
	int x = ecu->getWidth();
	int y = ecu->getHeight();

	this->tabControl1->TabStop = false;
	int numtabs = ecu->getNumMaps();

	String * typ = ecu->getType();

	bool wasnull = false;
	if(pages == NULL)
	{
		wasnull = true;
		pages = new GridPage*[numtabs*2];
		adjustFuel = new MenuItem*[numtabs];
	}
	for(int i = 0; i<numtabs; i++)
	{
		if(wasnull)
		{
			//evens are fuel maps
			pages[i*2] = new bre20::GridPage(String::Concat(S"Fuel Map ",(i+1).ToString()), x, y,0); //mode, index
			fillMap(pages[i*2], i, MODEFUEL,true);//fuel map index i
			pages[i*2]->setIndex(i);

			//odds are ign maps
			pages[i*2+1] = new bre20::GridPage(String::Concat(S"Ignition Map ",(i+1).ToString()), x, y,1); //mode, index
			fillMap(pages[i*2+1], i, MODEIGN, true);//ign map index i
			pages[i*2+1]->setIndex(i);

			if(typ->Equals(S"Euro Pw0 dev") || typ->Equals(S"Jdm Pr3 dev") || typ->Equals(S"Jdm Pw0 dev"))
			{
				pages[i*2]->addExtraContext();
				pages[i*2+1]->addExtraContext();
			}

			this->pages[i*2]->CellChanged += new System::EventHandler(this, fuelPage_CellChanged);
			this->pages[i*2+1]->CellChanged += new System::EventHandler(this, ignPage_CellChanged);

			this->pages[i*2]->ColumnHeaderChanged += new System::EventHandler(this, page_ColHeaderChanged);
			this->pages[i*2+1]->ColumnHeaderChanged += new System::EventHandler(this, page_ColHeaderChanged);
			this->pages[i*2]->RowHeaderChanged += new System::EventHandler(this, page_RowHeaderChanged);
			this->pages[i*2+1]->RowHeaderChanged += new System::EventHandler(this, page_RowHeaderChanged);

			ecu->getIgnitionMap(i)->add_OnRealMapChanged(new RomRep::RealMapChangeHandler(this,map_RealChanged));
			ecu->getFuelMap(i)->add_OnRealMapChanged(new RomRep::RealMapChangeHandler(this,map_RealChanged));
		//	ecu->getFuelMap(i)->add_OnMultiplierChanged(new RomRep::MapChangeHandler(this,edit_MapEvent));
			ecu->getFuelMap(i)->add_OnMultiplierChanged(new RomRep::MapChangeHandler(this, map_MultChanged));

			adjustFuel[i] = new MenuItem();
			adjustFuel[i]->Text = S"Fuel Adjust...";
			adjustFuel[i]->Click += new System::EventHandler(this, adjustFuelMain_Click);
			adjustFuel[i]->Enabled = true;

			pages[i*2]->ContextMenu->MenuItems->Add(adjustFuel[i]);
			pages[i*2]->TabStop = false;
			pages[i*2+1]->TabStop = false;
		}

		this->tabControl1->Controls->Add(this->pages[i*2]);
		this->tabControl1->Controls->Add(this->pages[i*2+1]);
	}

	for(int i =0 ; extraMaps != NULL && i<this->extraMaps->Count; i++)
		this->tabControl1->Controls->Add(static_cast<TabPage*>(extraMaps->get_Item(i)));
		
	if(paramPage != NULL)
	{
		this->tabControl1->Controls->Add(this->paramPage);
		if(wasnull)
			this->paramPage->CellChanged += new System::EventHandler(this, paramPage_CellChanged);
	}

	bool devecu = false;
	if (typ->ToLower()->IndexOf("dev") > 0)
		devecu = true;

	if(devecu && wasnull)
	{
		lastnacol = ecu->getLastNaCol();

		boostEff = new MenuItem*[numtabs];
		for(int i = 0; i<numtabs; i++)
		{
			this->boostEff[i] = new MenuItem();
			this->boostEff[i]->Text = S"Boost Efficiency...";
			this->boostEff[i]->Click += new System::EventHandler(this, boostEff_Click);
			this->boostEff[i]->Enabled = true;
			
			this->pages[i*2]->ContextMenu->MenuItems->Add(boostEff[i]);
			pages[i*2]->setLastNaCol(lastnacol);
			pages[i*2+1]->setLastNaCol(lastnacol);	
		}
	}

	for(int i = 0; scriptPages != NULL && i<this->scriptPages->Length; i++)
		this->tabControl1->Controls->Add(scriptPages[i]);

}

bool bre20::GridPageArray::AddControlToScriptPage(Control* con, int index)
{
	try
	{
		if(scriptPages == NULL)
		{
			scriptPages = new TabPage*[1];
			scriptPages[0] = new TabPage(S"Script Functions");
			index = 0;

			this->tabControl1->TabPages->Add(scriptPages[0]);
			this->tabControl1->Refresh();
		}

		scriptPages[index]->Controls->Add(con);
		//!call the script control handler here...

		return true;
	}
	catch(Exception * e)
	{
		MessageBox::Show(String::Concat(e->Message,S"\n\n", e->InnerException->Message));
		return false;
	}
}

void bre20::GridPageArray::setViewMode(bool viewGrid)
{
	for(int i = 0; i<pages->Length; i++)
		if(pages[i] != NULL)
			pages[i]->changeView(viewGrid);
}

//!Sets the value in the fuel map.
//!Maps indexed from left of tab order. base index = 0.
//!index = 0 for non vtec, 1 for vtec
void bre20::GridPageArray::setFuelCell(int x, int y, float value, int index) 
{
	if(x>=0 && y>=0 && x < vtrom->getWidth() && y < vtrom->getHeight())
	{
		GridPage* page;
		try
		{
			page = pages[index*2];
		}
		catch(...)
		{
			return;
		}
	    
		SimpleGrid::CellEvent* ce = new SimpleGrid::CellEvent();
		ce->Column = x;
		ce->Row = y;

		page->setValue(x,y,value,false);
		changeCell(page,index,ce,MODEFUEL);
	}
}

//!Sets the value in the fuel map. 
//!Maps indexed from left of tab order. base index = 0
float bre20::GridPageArray::getFuelCell(int x, int y, int index) //!index = 0 for non vtec, 1 for vtec
{
	if(x>=0 && y>=0 && x < vtrom->getWidth() && y < vtrom->getHeight())
	{
		GridPage* page;
		try
		{
			page = pages[index*2];
		}
		catch(...)
		{
			return -1.0;
		}

		return page->getValue(x,y);
	}
	return -1.0;
}



//***************************************************************************
//uitls

void bre20::GridPageArray::scaleFuelMaps(float multiplier)
{
	SimpleGrid::CellEvent * ce = new SimpleGrid::CellEvent();

	for(int i = 0; i<pages->Length/2; i++)
		for(int c = 0; c<vtrom->getWidth(); c++)
			for(int r = 0; r<vtrom->getHeight(); r++)
			{
				ce->Row = r;
				ce->Column = c;

				pages[i*2]->setValue(c,r,pages[i*2]->getValue(c,r)*multiplier,false);
				changeCell(pages[i*2],i,ce,MODEFUEL);
			}

	this->tabControl1->Refresh();
}

//***************************************************************************

void bre20::GridPageArray::AddExtraMap(GridPage* page)
{
	for(int i = 0; extraMaps != NULL && i<this->extraMaps->Count; i++)
	{
		int index;
		if( (index=this->tabControl1->TabPages->IndexOf(static_cast<TabPage*>(extraMaps->get_Item(i)))) >= 0)
		{
			tabControl1->TabPages->RemoveAt(index);
			i--; 
		}
	}

	this->extraMaps->Add(page);

	tabControl1->TabPages->Clear();
	setupTabs(vtrom);
}
int bre20::GridPageArray::AddMapPair(String* fuelTabName, RomRep::MapLocs* flocs, String* ignTabName, RomRep::MapLocs* ignlocs, int x, int y, int copyIndex, bool copyContents)
{
	try
	{
		int index = vtrom->addMapPair(flocs,ignlocs,fuelTabName,ignTabName,x,y,copyIndex, copyContents);

		this->tabControl1->TabPages->Clear();

		if(paramPage != NULL)
			paramPage->addMultiplierRow(vtrom->getFuelMap(index)->getIntMultipliers());

		setupTabs(vtrom);

		return index;
	}
	catch(...)
	{
		return -1;
	}

}

System::Void bre20::GridPageArray::fuelPage_CellChanged(System::Object *  sender, System::EventArgs *  e)
{
	bre20::GridPage* page = static_cast<GridPage*>(sender);

	for(int i = 0;i<pages->Length/2; i++)
		if(page == pages[i*2])
			changeCell(page,i,e,MODEFUEL);
}
System::Void bre20::GridPageArray::ignPage_CellChanged(System::Object *  sender, System::EventArgs *  e)
{
	bre20::GridPage* page = static_cast<GridPage*>(sender);

	for(int i = 0;i<pages->Length/2; i++)
		if(page == pages[i*2+1])
			changeCell(page,i,e,MODEIGN);
}

void bre20::GridPageArray::changeColHeader(bool vtec, int col, String* str)
{
	for(int i = 0; i<pages->Length; i++)
		pages[i]->setColumnHeader(col,str);
}

void bre20::GridPageArray::fillMap(GridPage* page, int index, int mode, bool reg)
{
	int i,j;
	System::String* str = NULL;
	RomRep::Map* map = NULL;//dont delete! using the rom's map!
	if(mode == MODEFUEL)
		map = vtrom->getFuelMap(index);
	if(mode == MODEIGN)
		map = vtrom->getIgnitionMap(index);

	if(map != NULL)
	{
		String* s = map->getName();
		if(s != NULL && !s->Equals("")) 
			page->Text = s;
		if(mode == MODEFUEL)//fuel map
		{
			for(i = 0; i<vtrom->getWidth(); i++)
				for(j = 0; j<vtrom->getHeight(); j++)
				{
					float val = map->getVal(i,j);
					page->setValue(i,j,val,false);
					setCellColor(page,i,j,val,MODEFUEL);
				}

		}
		if(mode == MODEIGN)//ign map
		{
			for(i = 0; i<vtrom->getWidth(); i++)
				for(j = 0; j<vtrom->getHeight(); j++)
				{
					float val = map->getVal(i,j);
					page->setValue(i,j,val,false);
					setCellColor(page,i,j,val,MODEIGN);
				}
		}

		//set the headers... 
		for(i = 0; i<vtrom->getWidth(); i++)
		{
			//broken
			str = vtrom->getColHeader(RomRep::Map::MODE_FUEL,index,i);
			page->setColumnHeader(i,str);
		}
		if(page == pages[0] || page == pages[1])
				index+=2;

		for(i = 0; i<vtrom->getHeight(); i++)
		{
			str = map->getRowHeader(i);//vtrom->getRowHeader(index,i);
			page->setRowHeader(i,str);
		}

		if(reg)
			map->add_OnMapHeaderChanged(new RomRep::MapHeaderChangeHandler(page,&GridPage::mapHeaderChanged));

		//setup row headers and vaues for the graph
		page->setupGraph();
	}//end map!=null
}

void bre20::GridPageArray::setCellColor(GridPage* page, int x, int y, float val, int mode)
{
	try
	{
		if(mode == MODEFUEL)
		{
			float num = val;
			if((int)num>=16.0F)
				num = 15.9F;
			if(num<0.0F)
				num = 0.0F;
			if(num>5.0F)
				page->setCellColor(x,y,System::Drawing::Color::FromArgb(70,200-((int)((num-6.0F)*12.5F)%256),255));
			else
				page->setCellColor(x,y,System::Drawing::Color::FromArgb(70,200,(int)((num)*42.6F)%256));
		}
		else
		{
			float num = val;
			if(num>43.0F)
				num = 43.0F;
			if(num<0.0F)
				num = 0.0F;
			if(num>33.0F)
				page->setCellColor(x,y,System::Drawing::Color::FromArgb(255,(int)((num-33.0F)* 25.5F),0));
			else
				page->setCellColor(x,y,System::Drawing::Color::FromArgb(255, (int)(255.0-(55.0+(num*5.0))),(int)(255.0-(num*6.0))));
		}
	}
	catch(...)
	{}
}

//event shit...............

System::Void bre20::GridPageArray::paramPage_CellChanged(System::Object *  sender, System::EventArgs *  e)
{
	int x = static_cast<SimpleGrid::CellEvent*>(e)->Column;
	int y = static_cast<SimpleGrid::CellEvent*>(e)->Row;

	int val = paramPage->getGridValue(x,y);
	RomRep::ByteChange* bc = vtrom->setMultiplierEx(y,x,val);

//	child* ch = NULL;

	GridPage* page = NULL;

	try
	{
		page = pages[y*2];
	}
	catch(...)
	{
		return;
	}

	RomRep::Map* map = NULL;//dont delete! using the rom's map!
	map = vtrom->getFuelMap(y);

	for(int j = 0; j<vtrom->getHeight(); j++)
	{
		float val = map->getVal(x,j);
		page->setValue(x,j,val,false);
		setCellColor(page,x,j,val,MODEFUEL);
	}
}

//TODO: why does this get called 3 times?
void bre20::GridPageArray::map_MultChanged(Object * sender, RomRep::MapChangeArgs * args)
{
	int index = static_cast<RomRep::Map*>(sender)->getIndex();
	paramPage->setGridValue(args->getX(),index,args->getByte());
}
void bre20::GridPageArray::map_RealChanged(Object * sender, RomRep::RealMapChangeArgs * args)
{
	float val = args->getValue();
	RomRep::Map* map = static_cast<RomRep::Map*>(sender);
	int index = map->getIndex()*2;
	if(map->getMode() == RomRep::Map::MODE_IGNITION)
		index++;

	GridPage* page = pages[index];
	setCellColor(page,args->getX(),args->getY(),val,map->getMode());
	page->setValue(args->getX(),args->getY(),val,false);
}

void bre20::GridPageArray::changeCell(GridPage* page, int index, EventArgs* e, int mode)//1 for fuel, 2 for ign
{
	//TODO: take out the code that sets the value in the map,
		//subscribe to the RealMapChangedEvent and set the map in there
		//Same goes for the multiplier.

	RomRep::Map* map = NULL;
	int col, row;
	col = static_cast<SimpleGrid::CellEvent*>(e)->Column;
	row = static_cast<SimpleGrid::CellEvent*>(e)->Row;
	float val = page->getValue(col,row);

	RomRep::ByteChange* bc = NULL;
	if(mode == MODEFUEL)//fuel
		map = vtrom->getFuelMap(index);
	else
		map = vtrom->getIgnitionMap(index);
	
	bc = map->setVal(col,row,val);

	//fuel adjustment for multipliers. Maybe this should be taken care of in 
	//the FuelMap class?
	/*
	if(mode == MODEFUEL && bc == NULL)
	{
	//	System::Windows::Forms::MessageBox::Show("bc == null.","BRE");
		//the value was outside the range of the function. if low then 
		//do nothing. if high, change multiplier.
		bool adj = false;
		int mult = -1;

		RomRep::FuelMap* fm = static_cast<RomRep::FuelMap*>(map);

		if(val - map->getVal(col,row)>1.0) //then we need to go up one multiplier
		{
			adj = true;

			mult = vtrom->getFuelMap(index)->getIntMultipliers()[col]+1;

			fm->setMultiplier(col,mult);
		}//end if big
		else if(val-map->getVal(col,row)<1.0) //go down a mult if possible
		{
			mult = vtrom->getFuelMap(index)->getIntMultipliers()[col];

			if(mult > 0)
			{
				mult -= 1;
				fm->setMultiplier(col,mult);
				adj = true;
			}
			else
				map->setVal(col,row,1.08F);
		}

		if(adj && mult > -1)
		{
			
			//paramPage->setGridValue(col,index,mult);

			for(int i = 0; i<vtrom->getHeight(); i++)
			{
				page->setValue(col,i,page->getValue(col,i)+.01F,false);

				SimpleGrid::CellEvent* ce = new SimpleGrid::CellEvent();
				ce->Row = i;
				ce->Column = col;

				changeCell(page, index, ce, mode);
			}//end for
		}

	}//end if null
	else
	{
		//end emulation stuff...

//		val = map->getVal(col,row);
//		setCellColor(page,col,row,val,mode);
//		page->setValue(col,row,val,false);
	}
	*/
	this->OnCellChanged(page,static_cast<SimpleGrid::CellEvent*>(e));
//	this->fireScriptEvent(page,S"OnChangeMapCell",e);
}

void bre20::GridPageArray::page_ColHeaderChanged(System::Object *  sender, System::EventArgs *  e)
{
	this->OnFireScriptEvent(sender,"OnColumnHeaderChanged",e);
}
void bre20::GridPageArray::page_RowHeaderChanged(System::Object *  sender, System::EventArgs *  e)
{
	this->OnFireScriptEvent(sender,"OnRowHeaderChanged",e);
}

void bre20::GridPageArray::adjustFuelMain_Click(System::Object *  sender, System::EventArgs *  e)
{
	AdjustFuel * af = new AdjustFuel();

	af->setInjectors(240);
	af->setEngine(1595);

	if(af->ShowDialog()==DialogResult::OK)
	{
		scaleFuelMaps(af->getFactor());
	}
}

void bre20::GridPageArray::boostEff_Click(System::Object *  sender, System::EventArgs *  e)
{
	BoostEff * beff = new BoostEff();

	SimpleGrid::CellEvent* noo = new SimpleGrid::CellEvent();

	if(beff->ShowDialog() == DialogResult::OK)
	{
		float multiplier = beff->getPercentage()/100.0f;

		for(int i = 0; i<pages->Length/2; i++)
			for(int c = lastnacol+1; c<vtrom->getWidth(); c++)
				for(int r = 0; r<vtrom->getHeight(); r++)
				{
					noo->Column = c;
					noo->Row = r;
					float fin = pages[i*2]->getValue(lastnacol,r) + (0.565f*multiplier* 
						pages[i*2]->getColumnHeaderDbl(c));

					pages[i*2]->setValue(c,r,fin,false);
					changeCell(pages[i*2],i,noo,MODEFUEL);
				}

		this->tabControl1->Refresh();
	}
}//!end boost efficiency