#pragma once

#include "GridPage.h"
#include "ParamPage.h"
#include "HondaEcu.h"
#include "scriptcontrols.h"
#include "map.h"

using namespace System;
using namespace System::Windows::Forms;

namespace bre20
{
	__delegate void FireScriptEventHandler(Object* sender, FireScriptEventArgs* args);

	//! The collection of tabpages for the child class
	public __gc class GridPageArray
	{
	private:
		int lastnacol;

		TabControl* tabControl1;
		RomRep::HondaEcu* vtrom;

		bre20::GridPage * pages[];
		ParamPage* paramPage;
		TabPage* scriptPages[];
		ArrayList* extraMaps;

		ArrayList* xinterpMI;
		ArrayList* yinterpMI;

		MenuItem * boostEff[];
		MenuItem * adjustFuel[];

		void setupTabs(RomRep::HondaEcu* ecu);
		void fillMap(GridPage* page, int index, int mode, bool reg); //!mode = 1 for fuel, 2 for ign
		void changeCell(GridPage* page, int index, EventArgs* e, int mode);

		MenuItem* createInterpMenuItem(GridPage* page, bool xinterp);

		//interpolation (x and y) menu items' handler
		void interp_click(Object* page, GridPageGenericArgs* args);

		void fuelPage_CellChanged(System::Object *  sender, System::EventArgs *  e);
		void ignPage_CellChanged(System::Object *  sender, System::EventArgs *  e);
		void paramPage_CellChanged(System::Object *  sender, System::EventArgs *  e);
		void page_HeaderChanged(System::Object *  sender, RomRep::MapHeaderChangeArgs *  e);
		void adjustFuelMain_Click(System::Object *  sender, System::EventArgs *  e);
		void boostEff_Click(System::Object *  sender, System::EventArgs *  e);

		void map_MultChanged(Object * sender, RomRep::MapChangeArgs * args);
		void map_RealChanged(Object * sender, RomRep::RealMapChangeArgs * args);

		void OnCellChanged(Object* sender, SimpleGrid::CellEvent* ce)
		{
			if(CellChanged != NULL)
				CellChanged(sender,ce);
		}

		void OnFireScriptEvent(Object* sender, String* name, EventArgs* args)
		{
			if(FireScriptEvent != NULL)
				FireScriptEvent(sender,new FireScriptEventArgs(name,args));
		}

	public: 
		GridPageArray(TabControl* tc, ParamPage* ppage, RomRep::HondaEcu* ecu);

		bool AddControlToScriptPage(Control* con, int index);

		//!sets the cell color of a cell in the map sprcified by the grid page 
		void setCellColor(GridPage* page, int x, int y, float val, int mode);

		//!Sets the value in the fuel map.
		//!Maps indexed from left of tab order. base index = 0.
		//!index = 0 for non vtec, 1 for vtec
		void setFuelCell(int x, int y, float value, int index) ;

		//!Sets the value in the fuel map. 
		//!Maps indexed from left of tab order. base index = 0
		float getFuelCell(int x, int y, int index); //!index = 0 for non vtec, 1 for vtec

		void setViewMode(bool viewGrid);

		void scaleFuelMaps(float multiplier);

		void AddExtraMap(GridPage* page);
		int AddMapPair(String* fuelTabName, RomRep::MapLocs* flocs, String* ignTabName, 
			RomRep::MapLocs* ignlocs, int x, int y, int copyIndex, bool copyContents);

		void changeColHeader(bool vtec, int col, String* str);

		void copy()
		{//!copy click
			GridPage* grid = static_cast<GridPage*>(this->tabControl1->SelectedTab);
			grid->callCopy();
		}
		void paste()
		{
			GridPage* grid = static_cast<GridPage*>(this->tabControl1->SelectedTab);
			grid->callPaste();
		}
		void doBoostEfficiency(){ this->boostEff_Click(NULL,NULL); }
		void doFuelAdjust(){ this->adjustFuelMain_Click(NULL,NULL); }

		//!grabs the col headers from the ROM and resets the grid headers
		void setAllColHeaders()
		{
			for(int i = 0; i<vtrom->getWidth(); i++)
				for(int j = 0; j<pages->Length; j++){
					//broken
					String * str = vtrom->getColHeader(RomRep::Map::MODE_FUEL,j%2,i);//mode,map,col
					pages[j]->setColumnHeader(i,str);
				}
		}

		TabPage* getScriptTabPages()[]{return this->scriptPages;}

		__event SimpleGrid::GridChangedHandler* CellChanged;
		__event FireScriptEventHandler* FireScriptEvent;
	};
}