#include "stdafx.h"
#include "HondaEcu.h"
#include "ManagedUtil.h"

RomRep::HondaEcu::HondaEcu(String * typ)
{
	//default: 1 fuel map, 1 ign map, no vtec, obd0, type unknown
	fuelmaps = new FuelMap*[1];
	fuelmaps[0] = NULL;
	ignmaps = new Map*[1];
	ignmaps[0] = NULL;

	bcList = new ByteChangeNotifyList();

    type = typ;
	obd = 0;
	vtec = false;
	nummaps = 1;
}

RomRep::HondaEcu::HondaEcu(unsigned char rom __gc[], int len, int obdd, bool vtech, int nummap, String * typ)
{
	int i;
	fuelmaps = new FuelMap*[nummap];
	ignmaps = new Map*[nummap];

	bcList = new ByteChangeNotifyList();

	flen = len;

	for(i = 0; i < nummap; i++)
	{	
		fuelmaps[i] = NULL;
		ignmaps[i] = NULL;
	}

	thefile = rom;

    type = typ;
	obd = obdd;
	vtec = vtech;
	nummaps = nummap;

	target_afr_loc = new int __gc[nummap];
}

RomRep::HondaEcu::~HondaEcu(){}

//
void RomRep::HondaEcu::setupMaps(MapLocs* mapinfo[])
{
	unsigned char map __gc[,];
	unsigned char mlt __gc[];

	//fuel
	for(int i = 0; i<nummaps; i++)
	{
		//go get the maps from the rom and put the arrays into the map constructors...
		map = HondaEcu::get2DArray(thefile, mapinfo[i]->mapLoc, x,y);
		mlt = HondaEcu::getArray(thefile, mapinfo[i]->mapLoc+(x*y), x);
		
		fuelmaps[i] = createFuelMap(x,y,i, NULL,  mapinfo[i], map,mlt, false);
		fuelmaps[i]->add_OnMapChanged(new MapChangeHandler(this,edit_MapEvent));
		fuelmaps[i]->add_OnMultiplierChanged(new MapChangeHandler(this,edit_MapEvent));
		fuelmaps[i]->add_OnByteChangeRequest(new MapByteChangeRequest(this,maps_ByteChange));

		bcList->add(fuelmaps[i]);
	}

	//ignition
	for(int i = nummaps; i<nummaps*2; i++)
	{
		int index = i-nummaps;
		map = HondaEcu::get2DArray(thefile, mapinfo[i]->mapLoc, x,y);

		ignmaps[index] = createIgnitionMap(x,y,index, NULL,  mapinfo[i], map, false);
		ignmaps[index]->add_OnMapChanged(new MapChangeHandler(this,edit_MapEvent));
		ignmaps[index]->add_OnByteChangeRequest(new MapByteChangeRequest(this,maps_ByteChange));

		bcList->add(ignmaps[index]);
	}
}

void RomRep::HondaEcu::maps_ByteChange(Object* sender, ByteChangeArgs* bca)
{
	ByteChange* bc = bca->getByteChange();
	this->setByte(bc->getLocation(),(int)bc->getByte());
}
int RomRep::HondaEcu::addMapPair(MapLocs* fmap, MapLocs* imap, String* fuelName, String* ignName, int xx, int yy, int copyIndex, bool copyContents)
{
	RomRep::FuelMap* tempf[] = fuelmaps;

	RomRep::Map* tempi[] = ignmaps;

	ignmaps = new RomRep::Map*[ignmaps->Length + 1];
	fuelmaps = new RomRep::FuelMap*[fuelmaps->Length + 1];

	bool vtecmaps = true;
	if(copyIndex > -1 && copyIndex < ignmaps->Length)
		vtecmaps = tempi[copyIndex]->isVtec();

	int i;
	for(i = 0; i< tempf->Length; i++)
		fuelmaps[i] = tempf[i];

	for(i = 0; i< tempi->Length; i++)
		ignmaps[i] = tempi[i];

	unsigned char map __gc[,];
	unsigned char mlt __gc[];

	//go get the maps from the rom and put the arrays into the map constructors...
	map = HondaEcu::get2DArray(thefile, fmap->mapLoc, xx,yy);
	mlt = HondaEcu::getArray(thefile, fmap->mapLoc+(xx*yy), xx);

	int newIndex = fuelmaps->Length - 1;
	fuelmaps[newIndex] = createFuelMap(xx,yy,newIndex, fuelName,  fmap, map,mlt, vtecmaps);
	fuelmaps[newIndex]->add_OnMapChanged(new MapChangeHandler(this,edit_MapEvent));
	fuelmaps[newIndex]->add_OnMultiplierChanged(new MapChangeHandler(this,edit_MapEvent));

	map = HondaEcu::get2DArray(thefile, imap->mapLoc, xx,yy);//0x3DE4

	ignmaps[newIndex] = createIgnitionMap(xx,yy,newIndex, ignName,  imap, map, vtecmaps);
	ignmaps[newIndex]->add_OnMapChanged(new MapChangeHandler(this,edit_MapEvent));

	if(copyIndex > -1 && xx == fuelmaps[copyIndex]->getX() && yy == fuelmaps[copyIndex]->getY() &&
		copyContents)
	{
		for(i = 0; i< xx; i++)
		{
			fuelmaps[newIndex]->setMultiplier(i,fuelmaps[copyIndex]->getIntMultipliers()[i]);

			for(int j = 0; j < yy; j++)
			{
				fuelmaps[newIndex]->setVal(i,j,fuelmaps[copyIndex]->getVal(i,j));
				ignmaps[newIndex]->setVal(i,j,ignmaps[copyIndex]->getVal(i,j));
			}//end inner for
		}//end outer for
	}//end if

	if(fuelmaps->Length == ignmaps->Length)
		nummaps = ignmaps->Length;
	else
		nummaps++;

	return newIndex;
}

//----------------------------------------------------------------------
String* RomRep::HondaEcu::getRowHeader(int mode, int mapindex, int row)
{
	if(mode == MODEIGN)
		return ignmaps[mapindex]->getRowHeader(row);
	else if(mode == MODEFUEL)
		return fuelmaps[mapindex]->getRowHeader(row);
	else
		return NULL;
}
String* RomRep::HondaEcu::getColHeader(int mode, int mapindex, int col)
{
	if(mode == MODEIGN)
		return ignmaps[mapindex]->getColumnHeader(col);
	else if(mode == MODEFUEL)
		return fuelmaps[mapindex]->getColumnHeader(col);
	else
		return NULL;
}

unsigned char RomRep::HondaEcu::get2DArray(unsigned char array __gc[], int loc, int x, int y) __gc[,]
{
	int i,j;

	unsigned char map __gc[,] = new unsigned char __gc[x,y];

	int k = loc;

    for(i=0;i<y;i++)
        for(j=0;j<x;j++)
		{
            map[j,i] = array[k];
			k++;
		}

	return map;
}

unsigned char RomRep::HondaEcu::getArray(unsigned char array __gc[], int loc, int len) __gc[]
{
	int i, j;
	unsigned char map __gc[]= new unsigned char __gc[len];

	j = loc;

	for(i = 0; i<len; i++)
		map[i] = array[loc+i];

	return map;
}

void RomRep::HondaEcu::setArray(unsigned char array __gc[], int loc, int len)
{
	for(int i = 0; i<len; i++)
	{
		setByte(loc+i,array[i]);
	}
}


void RomRep::HondaEcu::setByte(int loc, int val)//unsigned char val)
{
	if(val >=0 && val < 256)
	{
		//add undo shit here...
		//push a ByteHistory onto a stack (ByteChange array with a len)
		thefile[loc] = (unsigned char)val;
		RomRep::ByteChange* bc = new ByteChange(loc,(char)val);
		bcList->notify(bc);
		this->ByteChanged(bc);
	}
}
RomRep::ByteChange* RomRep::HondaEcu::setByteEx(int loc, int val)//unsigned char val)
{
	if(val >=0 && val < 256)
	{
		//add undo shit here...
		//push a ByteHistory onto a stack (ByteChange array with a len)
		thefile[loc] = (unsigned char)val;

		RomRep::ByteChange* bc = new ByteChange(loc,(char)val);
		bcList->notify(bc);
		this->ByteChanged(bc);
		return bc;
	}
	else
		return NULL;
}

RomRep::FuelMap* RomRep::HondaEcu::getFuelMap(int index)
{
	if(index>=0 && index<nummaps)
		return fuelmaps[index];
	else 
		return NULL;
}

RomRep::Map* RomRep::HondaEcu::getIgnitionMap(int index)
{
	if(index>=0 && index<nummaps)
		return ignmaps[index];
	else 
		return NULL;
}

float RomRep::HondaEcu::getLambdaMap(int index) __gc[,]
{
	if(index >= nummaps)
		return NULL;

	unsigned char afrmap __gc[,]= get2DArray(thefile,getAfrMapLoc(index),getWidth(),getHeight());
	float finalmap __gc[,] = new float __gc[x,y];

	for(int i = 0; i<x; i++)
		for(int j = 0; j<y; j++)
			finalmap[i,j] = calculateAfr(afrmap[i,j]);

	return finalmap;

}

float RomRep::HondaEcu::getAfrCell(int xx, int yy, int index)
{
	if(index>=0 && index<nummaps)
	{
		try
		{
			return calculateAfr(thefile[Map::getCellLocation(xx,yy,x,this->target_afr_loc[index])]);
		}
		catch(...)
		{return -1.0F;}
	}
	else
		return -1.0F;
}

RomRep::ByteChange* RomRep::HondaEcu::setAfrCellEx(int xx, int yy, float val, int index)
{
	if(index>=0 && index<nummaps)
	{
		int byte = (int)calculateAfr(val);
		ByteChange* bc = setByteEx(Map::getCellLocation(xx,yy,x,this->target_afr_loc[index]), byte);
		this->AfrChanged(xx,yy,getAfrCell(xx,yy,index),index);
		return bc;
	}
	else
		return NULL;
}

void RomRep::HondaEcu::edit_MapEvent(Object* sender, MapChangeArgs* args)
{
	setByte(args->getLocation(),args->getByte());
}


//void RomRep::HondaEcu::setAfrCell(int xx, int yy, int val, int index)
//{
//	if(index>=0 && index<nummaps)
//	{
//		setByte(Map::getCellLocation(xx,yy,x,this->target_afr_loc[index]), val);
//	}
//}
//RomRep::ByteChange* RomRep::HondaEcu::setAfrCellEx(int xx, int yy, int val, int index)
//{
//	if(index>=0 && index<nummaps)
//	{
//		ByteChange* bc = setByteEx(Map::getCellLocation(xx,yy,x,this->target_afr_loc[index]), val);
//
//		return bc;
//	}
//	else
//		return NULL;
//}

void RomRep::HondaEcu::setMultiplier(int mapnum, int column, int val)
{

	if(column>=0 && column<=x)
	{
//		setByteEx((*fuelmaps[mapnum]).getMultiplierLocation(column),val);
		(*fuelmaps[mapnum]).setMultiplier(column, val);
	}
}

RomRep::ByteChange* RomRep::HondaEcu::setMultiplierEx(int mapnum, int column, int val)
{

	if(column>=0 && column<=x)
	{
//		ByteChange* bc = setByteEx((*fuelmaps[mapnum]).getMultiplierLocation(column),val);
		(*fuelmaps[mapnum]).setMultiplier(column, val);

		return NULL;
	}
	else
		return NULL;
}

int RomRep::HondaEcu::Checksum()
{
	int i;
	int j;
 	long ad = 0;
 
 	for (i = 0;i< checksumlen; i++)//checksumlen
 	{
 		ad += thefile[i];
 	}
 	j = ad % 256;
 
 	return j;
}

int RomRep::HondaEcu::setChecksum()
{
	int cs;
	thefile[checksumaddy]=0;
	cs = this->Checksum();
	thefile[checksumaddy] = (256 - cs);
	
	return this->Checksum();
}

//both of these need to fix the column headers too...
//void RomRep::HondaEcu::insertColumn(int column)
//{
//	//for(int i = 0; i<nummaps; i++)
//	//{
//	//	fuelmaps[i]->insertColumn(column);
//	//	ignmaps[i]->insertColumn(column);
//	//}
//
//	//x++;
//}
//void RomRep::HondaEcu::insertRow(int row)
//{
//	//for(int i = 0; i<nummaps; i++)
//	//{
//	//	fuelmaps[i]->insertRow(row);
//	//	ignmaps[i]->insertRow(row);
//	//	
//	//}
//
//	//y++;
//}