
#pragma once
using namespace System;

#define MODEFUEL 0
#define MODEIGN 1

#include "map.h"
#include "FuelMap.h"
#include "bytechange.h"

namespace RomRep
{
	public __delegate void ByteChangeHandler(Object *, ByteChangeArgs *);
	public __delegate void TargetAfrChangeHandler(Object *, AfrChangeArgs *);

	///
	///Class for passing around associated map locations
	///
	public __gc class MapLocs: public MarshalByRefObject
	{
	public:
		MapLocs(int loc,  int colscalarloc, int revscalarloc){
			mapLoc = loc;
			mapRevScalarLoc = revscalarloc;
			mapColScalarLoc = colscalarloc;
		}

		///location of the map. Generally the FIRST byte in the map
		int mapLoc;
		///location of the rev scalars
		int mapRevScalarLoc;
		///location of the column/pressure scalars. 
		///NOT used by the obd0 VTEC ecus.
		int mapColScalarLoc;
	};


	//!
	//!Base class for ALL ROMs will be derived from this
	//!
	[Serializable] public __gc class HondaEcu: public MarshalByRefObject
	{

	protected:
		unsigned char thefile __gc[]; //muy importante!!
		int flen; //file length
		int x,y;

		FuelMap* fuelmaps[];
		Map* ignmaps[];

		ByteChangeNotifyList* bcList;

		int target_afr_loc __gc[];

		String * type;
		int obd, nummaps; //nummaps will be a 1 if there is 1 ignition and 1 fuel map 
		bool vtec;
		int checksumaddy;
		int checksumlen;

		///the column and row header strings
		///x = nummaps;
		///x == 0 is the nonvtec maps, x == 1 is the vtec maps
		//String* colheaders[,];//colheaders[x][y] <-- x is which map, y is col#

		///The address of the max map voltage for the dev roms
//		int maxmapadd;
		///The inde of the last NA column for the dev roms
		int lastNACol;

		///sets an array of bytes
		void setArray(unsigned char array __gc[], int loc, int len);

		///sets up the heders
//		virtual void setupHeaders() = 0;//called in this class' constructor
		///returns the checksum
		virtual int Checksum();

		///called when there is a MapEvent (user edits a map) 
		virtual void edit_MapEvent(Object* sender, MapChangeArgs* args);

		/*!
		You must override this method so this base class can properly
		setup the maps. 
		
		If mapname == null, the function must give the map a name.
		
		C++ Example from the VT0Ecu class:
		
		\code
		RomRep::FuelMap* RomRep::VT0Ecu::createFuelMap(int xx, int yy, int index, String* mapname,
							RomRep::MapLocs* maploc,unsigned char map __gc[,],
							unsigned char mlt __gc[], bool vtec)
		{
			if(index == 0)
				vtec = false;
			else if(index == 1)
				vtec = true;

			//now call the constructor to the map you need to create
			FuelMap* fm = new VT0FuelMap(xx, yy, maploc, map, maploc->mapLoc+(xx*yy), 
				mlt,thefile,0, maploc->mapRevScalarLoc,vtec);
			
			if(mapname != NULL)
				fm->setName(mapname);
			else if(index == 0)
				fm->setName("Low Fuel");
			else if(index == 1)
				fm->setName("VTEC Fuel");
			else //if(index >1)
				fm->setName(String::Concat("Fuel: ",index.ToString()));

			return fm;
		}
		\endcode
		*/
		virtual FuelMap* createFuelMap(int xx, int yy, int index, String* mapname,
			RomRep::MapLocs* maploc,unsigned char map __gc[,], unsigned char mlt __gc[], bool vtec) = 0;

		///must override this method so this base class can properly
		///setup the maps.
		///
		///If mapname == null, the function must give the map a name.
		///
		///see createFuelMap() for an example
		virtual Map* createIgnitionMap(int xx, int yy, int index, String* mapname,
			RomRep::MapLocs* maploc,unsigned char map __gc[,], bool vtec) = 0;

		///Sets up the fuel and ignition maps for the ROM. This MUST be called by
		///any class that directly extends HondaEcu or the maps will not get set up.
		///
		///\param mapinfo array of objects which store the information
		///about the maps. It is assumed that the first half of the objects
		///are the info for the fuel maps, and the last half for the ignition
		///maps. So nummaps will reflect this by being mapinfo.Length/2.
		virtual void RomRep::HondaEcu::setupMaps(MapLocs* mapinfo[]);

		void ByteChanged(RomRep::ByteChange* bc)
		{
			if (OnByteChanged != NULL)
				OnByteChanged(this, new ByteChangeArgs(bc));
		}

		void AfrChanged(int xx, int yy, float afr,int index)
		{
			if (OnTargetAfrChanged != NULL)
				OnTargetAfrChanged(this, new AfrChangeArgs(xx,yy,afr,index));
		}

	private:
		void maps_ByteChange(Object* sender, ByteChangeArgs* bca);
	public:
		HondaEcu(String * type);

		///Child classes will deal with setting up the maps in their constructor 
		///because we dont know what type of maps must be set up.
		///
		///Basic Constructor
		HondaEcu(unsigned char rom __gc[], int length, int obd, bool vtec, int nummaps, String * type);

		virtual ~HondaEcu();

		[NonSerialized] __event ByteChangeHandler * OnByteChanged;
		[NonSerialized] __event TargetAfrChangeHandler * OnTargetAfrChanged;

		virtual int addMapPair(MapLocs* fmap, MapLocs* imap, String* fuelName, String* ignName,int x, int y, int copyIndex, bool copyContents);

		///loc is the location of the FIRST (low) byte in the value.
		///Sets the RPM into ROM
		virtual void setTwoByteRpm(int highloc, int lowloc, int rpm) = 0;
		///Gets the RPM that adheres to revcount/dec
		virtual int getTwoByteRpm(int highloc, int lowloc) = 0; //return rpm

		///Sets the byte
		virtual void setByte(int loc, int val);
		///Sets the byte and returns the RomRep.ByteChange. 
		virtual ByteChange* setByteEx(int loc, int val);
		///Gets the byte at the address
		virtual unsigned char getByte(int loc)
		{return thefile[loc];};

		virtual int setChecksum();
		
		///Returns the width of the maps
		virtual int getWidth(){	return x;};
		///Returns the height of the maps
		virtual int getHeight(){return y;};
		///Returns the number of maps. 1 means there is 1 fuel and 1 ignition map.
		virtual int getNumMaps(){return nummaps;};

		///Returns the RomRep.FuelMap of your choosing. Indexed from 0.
		///Number of elements is getNumMaps();
		///Index of maps can be found by looking at the tabs in the main
		///rom window. Left most map is index 0 and so on.
		///
		virtual FuelMap* getFuelMap(int index);
		///
		///Returns the ignition map: RomRep.Map of your choosing. Indexed from 0.
		///Number of elements is getNumMaps();
		///Index of maps can be found by looking at the tabs in the main
		///rom window. Left most map is index 0 and so on.
		virtual Map* getIgnitionMap(int index);

		///Sets a multiplier in the specified mapnum (index)
		virtual ByteChange* setMultiplierEx(int mapnum, int column, int val);
		///Sets a multiplier in the specified mapnum (index)
		virtual void setMultiplier(int mapnum, int column, int val);

		///Gets a row header. 
		///map == 0 for non vtec, 1 for vtec
		virtual String * getRowHeader(int mode, int mapindex, int row);
		///Gets a column header. 
		///map == 0 for non vtec, 1 for vtec
		virtual String * getColHeader(int mode, int map, int column);

		///Gets CheckSum Address
		virtual int getChecksumAddy(){return checksumaddy;}
		virtual int getChecksumLen(){return checksumlen;}

		///Gets 2D array from the ROM file. 
		///loc is the location of the first byte in the array
		unsigned char get2DArray(unsigned char array __gc[], int loc, int x, int y) __gc[,];
		///Gets a 1D array
		///loc is the location of the first byte in the array
		unsigned char getArray(unsigned char array __gc[], int loc, int len) __gc[];

		///Returns value in the rpm scalar (usually contained in each map)
		virtual int getRpmScalarValue(int row, bool fuel, bool vtec){return -1;}; 

		///Gets the name of the ROM file (i.e. JDM Pr3-58)
		virtual String * getType()
		{return type;}

		virtual unsigned char getFile() __gc[]
		{
			return thefile;//getArray(thefile,0,flen);
		}

		///gets the length of the file
		virtual int getLength()
		{
			return flen;
		}

		static int headlen = 10;//int RomRep::HondaEcu::headlen = 10;

		///gets the length of the last non-boost column in the maps. 
		///Only really functional on dev roms
		virtual int getLastNaCol(){return lastNACol;};
		///Gets the max value (0-255) from ROM that the map sensor can see.
//		virtual unsigned char getMaxMapValue(){return thefile[maxmapadd];};
		///Gets the array of column geader names.
		//virtual String* getColHeaders()[,]{ return colheaders;};
		
		
		///Returns the float (afr) representation of the target afr map.
		virtual float getLambdaMap(int index) __gc[,];
		///Sets the target afr
		virtual RomRep::ByteChange* setAfrCellEx(int x, int y, float val, int index);
		///Sets the target afr
//		virtual ByteChange* setAfrCellEx(int x, int y, int val, int index);
		
		///Sets the target afr
	//	virtual void setAfrCell(int x, int y, int val, int index);
		///Gets the target afr
		virtual float getAfrCell(int x, int y, int index);
		///gets the afr map's location
		int getAfrMapLoc(int index)
		{ 
			if(index<nummaps)
				return target_afr_loc[index];
			else return -1;
		};

		//*******************************
		//** Calc functions all of the base classes should implement...
		///\returns Percentage
		virtual float calcTPS(int raw) = 0;
		///\returns raw value
		virtual int calcTPS(float percentage)=0;

		///\param raw the raw value
		///\param sae true if sae, false if metric
		///\returns the real temp.  
		virtual float calcTemp(int raw, bool sae)=0;
		///\param temp the real world temp value
		///\param sae true if sae, false if metric
		///\returns the raw temp.  
		virtual int calcTemp(float temp, bool sae)=0;

		///Calculates the RPM. If there are more than one format 
		///of rpm value, assume it is the most common format (i.e.
		///the 2 byte value)
		virtual float calcRpm(int raw)=0;
		///Calculates the raw value fromf RPM. If there are more than one format 
		///of rpm value, assume it is the most common format (i.e.
		///the 2 byte value)
		virtual int calcRpm(float rpm)=0;

	protected:
		virtual float calculateAfr(unsigned char raw) = 0;
		virtual unsigned char calculateAfr(float afr) = 0;

	};
}

//#endif