#pragma once

using namespace System;
using namespace System::Windows::Forms;
using namespace System::Text::RegularExpressions;

#include "common.h"

 
namespace bre20
{
	/// <summary> 
	/// Summary for LabeledText
	/// </summary>
	public __delegate void TextChangedEventHandler(Object *, EventArgs *);

	__gc public class LabeledText :  public System::Windows::Forms::Panel
	{
	public:
		__event TextChangedEventHandler * OnTextChanged;

		static const int LTMODEINT = 0;
		static const int LTMODEDOUBLE = 1;
		static const int LTMODETEXT = 2;


		LabeledText() : Panel()
		{
			this->mode = LTMODEINT;
			setup(S"",S"",80);
		}
		LabeledText(String * caption, String* tooltip, int mode) : Panel()
		{
			this->mode = mode;
			setup(caption,tooltip,80);
		}
		LabeledText(String * caption,String* tooltip, int width, int mode) : Panel()
		{
			this->mode = mode;
			setup(caption,tooltip,width);
		}

		void setWidth(int width)
		{
			
			this->Size.Width = width+1;
			if(label != NULL)
			{
				box->Size.Width = width-8;
				this->label->Size.Width = width;
			}
			else
				box->Size.Width = width;

		}

		void setText(String * txt)
		{
			box->Text = txt;
		}

		void setCaption(String * str)
		{
			if(label != NULL)
			{
				label->Text = str;
			}
			else
			{
				setup(str,S"", box->Size.Width);
			}
		}
		String* getCaption()
		{
			return label->Text;
		}
		void setToolTip(String * str)
		{
			tt->SetToolTip(box,str);
		}

		void setMode(int m)
		{
			mode = m;
		}

		int getMode()
		{
			return mode;
		}

		TextBox* getBox()
		{
			return box;
		}

		Label* getLabel()
		{
			return label;
		}

		float getDblValue()
		{
			return Convert::ToSingle(box->Text);

		}
		int getIntValue()
		{
			return Convert::ToInt32(box->Text);
		}
		
	protected: 
		int mode;

		TextBox* box;
		Label* label;
		ToolTip* tt;

		void setup(String *str, String* tooltip, int width)
		{
			this->Size = System::Drawing::Size(width+1, 40);

			box = new TextBox();
			box->Name = S"box";
			box->Text = S"";
			box->Location = System::Drawing::Point(8, 16);
			box->TabIndex = 0;
			box->Size = System::Drawing::Size(width-8, 20);
			box->KeyPress += new System::Windows::Forms::KeyPressEventHandler(this, box_KeyPress);
			box->Leave += new EventHandler(this, box_Leave);

			this->Controls->Add(box);

			if(str->Equals(S""))
			{
				box->Location = System::Drawing::Point(0, 0);
				this->Size = System::Drawing::Size(width+1, box->Size.Height+1);
			}
			else
			{
				label = new Label();
				label->Location = System::Drawing::Point(0, 0);
				this->label->Name = S"label";
				this->label->Size = System::Drawing::Size(width, 16);
				this->label->TabIndex = 1;
				this->label->Text = str;
				this->Controls->Add(label);
			}

			// Create the ToolTip and associate with the Form container.
			if(!tooltip->Equals(S""))
			{
				tt = new ToolTip();

				// Set up the delays for the ToolTip.
				tt->AutoPopDelay = 5000;
				tt->InitialDelay = 1000;
				tt->ReshowDelay = 500;
				// Force the ToolTip text to be displayed whether or not the form is active.
				tt->ShowAlways = true;
				      
				// Set up the ToolTip text for the Button and Checkbox.
				tt->SetToolTip(this->box, tooltip);
			}

		}
	private:

		//events
		void box_Leave(System::Object *  sender, EventArgs *  e)
		{
			if (OnTextChanged != NULL)
			{
				OnTextChanged(this, new EventArgs());
			}
		}//end event

		void box_KeyPress(System::Object *  sender, System::Windows::Forms::KeyPressEventArgs *  e)
		{
			if(mode == LTMODEINT)
			{
				Common::handleIntKeyPress(box,e,false);
			}
			if(mode == LTMODEDOUBLE)
			{
				Common::handleDoubleKeyPress(box,e,false);
			}

			if(e->KeyChar == 13)
				box_Leave(NULL,NULL);

		}//end event
	};
}