#include "StdAfx.h"
#include "LogAnalyzer.h"
#include "HondaEcu.h"
#include "obd0vtececu.h"
#include "dynorule.h"

bre20::LogAnalyzer::LogAnalyzer(ArrayList* rules, child * romm, String * fname, AutoEdit* autoEdit): FormWithRules(rules)
{
	InitializeComponent();

	System::Resources::ResourceManager *  resources = new System::Resources::ResourceManager(__typeof(bre20::LogAnalyzer));
	this->Icon = (__try_cast<System::Drawing::Icon *  >(resources->GetObject(S"$this.Icon")));

	reader = openLog(fname);
	if(reader == NULL)
	{
		this->Close();
		this->Dispose(true);
		return;
	}

	rom = romm;
	romm->getRom()->add_OnTargetAfrChanged(new RomRep::TargetAfrChangeHandler(this,targetTabs_AfrChanged));
	Globals::Props->add_OnPropertyChanged(new PropertyEventHandler(this,props_AfrToLambda));
	String* units = Globals::Props->getParam("logO2Units");
	if(units->Equals("afr"))
		afr = true;
	else
		afr = false;

	if(this->rules == NULL)
		this->rules = new ArrayList();

	//dynoRule = new DynoRule();
	//this->rules->Add(dynoRule);

	autotuned = false;

	this->minDisplayFactor = 2.0F;

	this->estimate = true;

	ae = autoEdit;

	try
	{
		fillAnalyzer();
	}
	catch(Exception* ex)
	{
		MessageBox::Show(ex->get_StackTrace());
		//MessageBox::Show("An error has occurred while trying to analyze the file. Please make sure this file is a BRE generated log file.","BRE");
		this->Close();
	}
}

bre20::LogFileReader* bre20::LogAnalyzer::openLog(String* fname)
{
	return LogFileReader::openLogFile(fname);
}

void bre20::LogAnalyzer::fillAnalyzer()
{
	reader->closeFile();
	reader = new LogFileReader(reader->getFileName());
	this->Text = String::Concat(S"Log Analyzer - ",reader->getFileName());

		RomRep::HondaEcu* ecu = rom->getRom();
		x = ecu->getWidth();
		y = ecu->getHeight();
		nummaps = ecu->getNumMaps();

		setupRules();

		createGraphs();
		float meas __gc[,,] = readLog();
		float targ __gc[,,] = fillExtraGraphs(meas);

		fillFinalArray(meas,targ);

		finishRules();
}

void bre20::LogAnalyzer::createGraphs()
{
	bool newtabs = false;
	if(measuredTabs == NULL)
	{
		measuredTabs = new LogGridPage*[nummaps];
		targetTabs = new GridPage*[nummaps];
		differenceTabs = new LogGridPage*[nummaps];

		newtabs = true;
	}


	int i = 0;
	RomRep::HondaEcu* ecu = rom->getRom();
	String * str;

	for(i = 0; i<measuredTabs->Length; i++)
	{
		if(newtabs)
		{
			measuredTabs[i] = new LogGridPage(x,y,String::Concat(S"Measured AFR ",(i+1).ToString()));
			targetTabs[i] = new GridPage(String::Concat(S"Target AFR ",(i+1).ToString()),x,y,0);
			differenceTabs[i] = new LogGridPage(x,y,String::Concat(S"% Difference ",(i+1).ToString()));
			differenceTabs[i]->setPaintMode(true);
			measuredTabs[i]->setPaintMode(true);
		}
		
		measuredTabs[i]->setReadOnly(true);
		differenceTabs[i]->setReadOnly(true);
		measuredTabs[i]->clearGrid();
		targetTabs[i]->clearGrid();
		differenceTabs[i]->clearGrid();

		if(newtabs)
		{
			targetTabs[i]->CellChanged += new System::EventHandler(this, targetTabs_CellChanged);

			this->tabControl1->Controls->Add(measuredTabs[i]);
		
			int j;
			for(j = 0; j< ecu->getWidth(); j++)
			{
				//broken
				str = ecu->getColHeader(0,i,j);
				measuredTabs[i]->setColumnHeader(j,str);
				targetTabs[i]->setColumnHeader(j,str);
				differenceTabs[i]->setColumnHeader(j,str);
			}

			int index = i+2;

			for(j = 0; j<ecu->getHeight(); j++)
			{
				str = ecu->getRowHeader(MODEFUEL,i,j);
				measuredTabs[i]->setRowHeader(j,str);
				targetTabs[i]->setRowHeader(j,str);
				differenceTabs[i]->setRowHeader(j,str);
			}//end inner for
		}//end new tabs
	}//end for

	if(newtabs)
	{
		for(int i = 0; i<targetTabs->Length;i++)
			this->tabControl1->Controls->Add(targetTabs[i]);

		for(int i = 0; i<differenceTabs->Length;i++)
			this->tabControl1->Controls->Add(differenceTabs[i]);
	}//end if

	
}

float bre20::LogAnalyzer::parseWeights(String* w) __gc[,]
{
	String* delim = "|";
	String* strs[] = w->Split(delim->ToCharArray());

	float weights __gc[,] = new float __gc[2,2];
	weights[0,0] = Convert::ToSingle(strs[0]);
	weights[1,0] = Convert::ToSingle(strs[1]);
	weights[1,1] = Convert::ToSingle(strs[2]);
	weights[0,1] = Convert::ToSingle(strs[3]);

	return weights;
}

float bre20::LogAnalyzer::readLog() __gc[,,]
{
	float nums __gc[,,] = new float __gc[nummaps,x,y];
	float avg __gc[,,] = new float __gc[nummaps,x,y];
	float result __gc[,,] = new float __gc[nummaps,x,y]; 
	String * line = reader->readLine();
	int iterations = 2;

	float mincellweight = Globals::Props->getFloatParam(S"atMinCellWeight");

    while(line != NULL)
	{	
		int tblindex = Convert::ToInt32(reader->getValue("index"));

		int col = Convert::ToInt32(reader->getValue("x"));
		int row = Convert::ToInt32(reader->getValue("fuely"));

		float afr = Convert::ToSingle(reader->getValue(S"Afr"));

		float weights __gc[,] = parseWeights(reader->getValue(S"weights"));

		if(col>=0 && row+1<y && col+1<x)
		{
			float interim;
			for(int i = 0; i<2; i++)
				for(int j = 0; j<2; j++)
				{
					if(weights[i,j]>mincellweight)
					{
						interim = (nums[tblindex,col+i,row+j] * avg[tblindex,col+i,row+j]) + (afr*weights[i,j]);
						nums[tblindex,col+i,row+j]+= weights[i,j];
						avg[tblindex,col+i,row+j] = interim/nums[tblindex,col+i,row+j];
					}
				}
		}

		executeRules();

		line = reader->readLine();
		iterations++;
	}

	float value;
	for(int tbl = 0; tbl < nummaps; tbl++)
	{
		for(int col = 0; col<x; col++)
		{
			for(int row = 0; row < y; row++)
			{
				this->measuredTabs[tbl]->setCellColor(col,row,Color::White);
				if(nums[tbl,col,row] >= this->minDisplayFactor)
				{
					value = avg[tbl,col,row];
					if(!afr)
						value = Common::calcLambda(value);
					this->measuredTabs[tbl]->setValue(col,row,value, false);

					result[tbl,col,row] = avg[tbl,col,row];

					if(avg[tbl,col,row]>=10.0)
						rom->setCellColor(measuredTabs[tbl],col,row,(avg[tbl,col,row]-10.0F)*3.0F,1);
				}
			}
		}

		this->measuredTabs[tbl]->Refresh();
	}

	reader->closeFile();
	return result;
}


float bre20::LogAnalyzer::fillExtraGraphs(float meas __gc[,,]) __gc[,,]
{
	
	RomRep::HondaEcu* hecu = rom->getRom();
	float result __gc[,,] = new float __gc[nummaps,x,y];

	for(int tbl = 0; tbl<targetTabs->Length;tbl++)
	{
		float afrmap __gc[,]= hecu->getLambdaMap(tbl);

		for(int col = 0; col< x; col++)
		{
			for(int row = 0; row < y; row++)
			{
				result[tbl,col,row] = afrmap[col,row];
				float value = afrmap[col,row];//calcAfr(afrmap[col][row]);
				if(!afr)
					value = Common::calcLambda(value);
				targetTabs[tbl]->setValue(col,row,value,false);

				rom->setCellColor(targetTabs[tbl],col,row,(afrmap[col,row]-10.0F)*3.0F,0);

				if(meas[tbl,col,row] >1.0F)
				{
					
					float percent = (100.0F*meas[tbl,col,row]/afrmap[col,row])-100.0F;//measuredTabs[tbl]->getValue(col,row)

					differenceTabs[tbl]->setValue(col,row,percent, false);
					rom->setCellColor(differenceTabs[tbl],col,row,percent+30.0F,2);
				}
				else
					differenceTabs[tbl]->setCellColor(col,row,Color::White);
				
				
			}
		}

		differenceTabs[tbl]->Refresh();
		targetTabs[tbl]->Refresh();
	}
	return result;
}

//************************************
//for estimation
void bre20::LogAnalyzer::fillFinalArray(float meas __gc[,,],float targ __gc[,,])
{
	finalafr = new float __gc[nummaps,x,y];
	for(int tbl = 0; tbl<nummaps; tbl++)
	{
		for(int i = 0; i<x; i++)
		{
			for(int j = 0; j<y; j++)
			{
				float mafr = meas[tbl,i,j];
				if(mafr > 1.0)
				{
					finalafr[tbl,i,j] = mafr;
				}

			}//end y loop
		}//end x loop
	}//end tbl loop

	for(int tbl = 0; tbl<nummaps; tbl++)
	{
		for(int i = 0; i<x; i++)
		{
			for(int j = 0; j<y; j++)
			{
				float mafr = -1.0;
				if(finalafr[tbl,i,j] < 1.0 && estimate)
				{
					mafr = estimateCell(tbl,i,j);
				}//end if
				
				if(mafr > 1.0)
				{
					finalafr[tbl,i,j] = mafr;

					if(!afr)
						mafr = Common::calcLambda(mafr);
					measuredTabs[tbl]->setValue(i,j,mafr,false);

					float percent = (100.0F*finalafr[tbl,i,j]/targ[tbl,i,j])-100.0F;

					differenceTabs[tbl]->setValue(i,j,percent,false);
				}

			}//end y loop
		}//end x loop
		measuredTabs[tbl]->Refresh();
		differenceTabs[tbl]->Refresh();
	}//end tbl loop
}


//dir: 0 right, 1 down, 2 left, 3 up, if negative grab all cells. 
//count: number of blanks so far in this dir.
bre20::AfrEstimation * bre20::LogAnalyzer::estimateCell(int tbl, int col, int row, int count, int dir)
{

	int numsteps = 3;
	if(dir == 0) //right
	{
		for(int i = col; i<x && i-col<=numsteps; i++)
			if(finalafr[tbl,i,row] > 1.0)
				return new bre20::AfrEstimation(finalafr[tbl,i,row],i-col,0);
	}

	else if(dir == 1) //down
	{
		for(int i = row; i<y && i-row<=numsteps; i++)
			if(finalafr[tbl,col,i] > 1.0)
				return new bre20::AfrEstimation(finalafr[tbl,col,i],i-row,1);
	}
	
	else if(dir == 2) //left
	{
		for(int i = col; i>=0 && col-i<=numsteps; i--)
			if(finalafr[tbl,i,row] > 1.0)
				return new bre20::AfrEstimation(finalafr[tbl,i,row],col-i,2);
	}
	
	else if(dir == 3) //up
	{
		for(int i = row; i>=0 && row-i<=numsteps; i--)
			if(finalafr[tbl,col,i] > 1.0)
				return new bre20::AfrEstimation(finalafr[tbl,col,i],row-i,3);
	}

	return NULL;

}

float bre20::LogAnalyzer::estimateCell(int tbl, int col, int row)
{

	try
	{
		float test = finalafr[tbl,col,row];
	}
	catch(...)
	{
		return NULL;
	}

	if(finalafr[tbl,col,row] > 1.0)
		return finalafr[tbl,col,row];

	bre20::AfrEstimation* left;
	bre20::AfrEstimation* right;
	bre20::AfrEstimation* top;
	bre20::AfrEstimation* bottom;
	//here if we need to estimate the cell...

	right = estimateCell(tbl, col, row, 0, 0);
	bottom = estimateCell(tbl, col, row, 0, 1);
	left = estimateCell(tbl, col, row, 0, 2);
	top = estimateCell(tbl, col, row, 0, 3);


	float rowavg = -1.0, colavg = -1.0;
	int countsum;
	if(left != NULL && right != NULL)
	{
		countsum = left->getSteps() + right->getSteps();

		float step = (right->getAfr() - left->getAfr())/(float)countsum;

		colavg = left->getAfr()+(step*(float)left->getSteps());

	}
	if(top != NULL && bottom != NULL)
	{
		countsum = top->getSteps() + bottom->getSteps();

		float step = (bottom->getAfr() - top->getAfr())/(float)countsum;

		rowavg = top->getAfr()+(step*(float)top->getSteps());
	}	

	if(rowavg <0.0 && colavg < 0.0F)
		return -1.0F;

	if(rowavg < 0.0F)
		return colavg;

	if(colavg < 0.0F)
		return rowavg;

	return (colavg+rowavg)/2.0F;
}
//************************************
void bre20::LogAnalyzer::targetTabs_AfrChanged(Object* sender, RomRep::AfrChangeArgs* e)
{
	float tafr = e->getAfr();
	int i = e->getIndex();
	int col = e->getX();
	int row = e->getY();
	rom->setCellColor(targetTabs[i],col,row,(tafr-10.0F)*3.0F,0);

	if(measuredTabs[i]->getValue(col,row) >1.0F)
	{
		float val = measuredTabs[i]->getValue(col,row);
		if(!afr)
			val = Common::calcAfr(val);
		float percent = Math::Abs((100.0F*val/tafr)-100.0F);

		differenceTabs[i]->setValue(col,row,percent,false);
		rom->setCellColor(differenceTabs[i],col,row,percent,2);
	}

	if(!afr)
		tafr = Common::calcLambda(tafr);
	targetTabs[i]->setValue(col,row,tafr,false);

//	targetTabs[i]->Refresh();
}
void bre20::LogAnalyzer::targetTabs_CellChanged(System::Object *  sender, System::EventArgs *  e)
{
	GridPage* page = static_cast<GridPage*>(sender);

	SimpleGrid::CellEvent* ce = static_cast<SimpleGrid::CellEvent*>(e);

	int i;
	for( i = 0; i<targetTabs->Length; i++ )
		if(targetTabs[i] == page)
			break;

	if(i>=targetTabs->Length)
		return;

	int col = ce->Column;
	int row = ce->Row;

	float tafr = page->getValue(col,row);
	if(!afr)
		tafr = Common::calcAfr(tafr);
	rom->getRom()->setAfrCellEx(col,row,tafr,i);

//	rom->addByteChange(bc);
}

void bre20::LogAnalyzer::props_AfrToLambda(Object* sender, bre20::PropertyEventArgs* e)
{
	if(!e->getName()->Equals("logO2Units"))
		return;

//	MessageBox::Show(e->getValue());

	String* tmp = e->getValue();
	bool tmpafr = false;
	if(tmp->Equals("afr"))
		tmpafr = true;

	//if we convert back and forth we lose accuracy, so
	//when the user changes from lambda to afr or vise versa
	//then we recalculate everything. Not likely to happen 
	//very often though.
	if((tmpafr && !afr) || (!tmpafr && afr))
	{
		afr = tmpafr;
		autotuned = false;
		this->autoTune->Enabled = true;
		fillAnalyzer();
	}
}

