#include "StdAfx.h"
#include "Logger.h"
#include "Engine.h"
#include "primitivewrap.h"
#include "Global.h"
#include "Common.h"

bre20::Logger::Logger(child * romm, AutoEdit* autotune, ArrayList* rules): FormWithRules(rules)
{
	InitializeComponent();

	//-------
	rom = romm;
	logdata = new LoggerData(rom->getRom()->getType());
	logwriter = new DatalogFileWriter();
	logwriter->add_OnFileNameChanged(new EventHandler(this,file_changed));
	autoEdit = autotune;

	initMisc();

	readProperties();
	
	//-------
			
	this->setupRules();

	setLocale();

	//get the loggerKey from autoEdit object for the realtime updates...
	millisKey = new LoggerKey("#100ns",true,false);
	logdata->addLoggerKey(millisKey);
	logdata->addLoggerKey(autoEdit->getLoggerKey());
	loggrid->InitGrid(logdata->getLoggerKeys());
	initUserOutput();
}

void bre20::Logger::readProperties()
{
	try
	{
		//get all the options from the config file...
		this->timer1 = new System::Windows::Forms::Timer();
		timer1->Interval = tminterval = Globals::Props->getIntParam(S"logInterval");
		this->timer1->Tick += new System::EventHandler(this, timer_Tick);
		
		//-------
		port = PortSetupDialog::getPortName(Globals::Props->getParam(S"logPort"));

		this->o2type = Globals::Props->getIntParam(S"logSensor");

		String* str = Globals::Props->getParam(S"logTrace")->ToLower();
		if(str->Equals(S"afr"))
			this->maptracetype = DatalogOptions::AFRTRACE;
		else if(str->Equals(S"knock"))
			this->maptracetype = DatalogOptions::KNOCKTRACE;
		else
			this->maptracetype = DatalogOptions::NOTRACE;

		str = Globals::Props->getParam(S"logUnits")->ToLower();
		if(str->Equals(S"metric"))
			this->units = DatalogOptions::METRIC;
		else
			this->units = DatalogOptions::SAE;

		maptrace = Globals::Props->getBoolParam("logTraceEnabled");
		romtrace = Globals::Props->getBoolParam("logRomTraceEnabled");
		interptrace = Globals::Props->getBoolParam("logInterpTraceEnabled");

		str = Globals::Props->getParam(S"logO2Units")->ToLower();
		setLambdaMode(!str->Equals("afr"));

		compMult = Globals::Props->getFloatParam(S"logO2MultiplyAdjust");
		compAdd = Globals::Props->getFloatParam(S"logO2AddAdjust");

		TESTING = Globals::Props->getBoolParam("LOG_TESTMODE");
	}
	catch(Exception* ex)
	{
		MessageBox::Show("Please fix the bre.exe.config file. One of the parameters is not in the correct format.");
		Globals::LogFile->WriteLine(this,String::Concat(S"Logger error: ", ex->Message));
		this->Close();
	}
}
void bre20::Logger::initMisc()
{
	this->title = "Data Logger";

	System::Resources::ResourceManager *  resources = new System::Resources::ResourceManager(__typeof(bre20::Logger));
	this->Icon = (__try_cast<System::Drawing::Icon *  >(resources->GetObject(S"$this.Icon")));

	tester = 0;
	sendmode = 0;

	loggrid = new LoggerGrid(160, 275);
	loggrid->Location = System::Drawing::Point(608, 20);
	loggrid->Anchor = (System::Windows::Forms::AnchorStyles)((System::Windows::Forms::AnchorStyles::Top | System::Windows::Forms::AnchorStyles::Bottom) 
		| System::Windows::Forms::AnchorStyles::Right);
	loggrid->Name = S"loggrid";
	this->Controls->Add(loggrid);

	replayTrack = new TrackBar();
	replayTrack->Location = System::Drawing::Point(608, 308);
	replayTrack->Size = System::Drawing::Size(160, 45);
	replayTrack->Anchor = (System::Windows::Forms::AnchorStyles)((System::Windows::Forms::AnchorStyles::Bottom) 
		| System::Windows::Forms::AnchorStyles::Right);
	replayTrack->Enabled = false;
	replayTrack->Minimum = 0;
	replayTrack->TickFrequency = 0;
	replayTrack->Visible = false;
	this->Controls->Add(replayTrack);

	//-------
	//setup the port:
	comport = new DatalogCom();
	//-------

	dash = new TabPage();
	dash->Text = S"Digital Dash";
	setupTraceGrids();
	this->tabControl1->Controls->Add(dash);

	start->Enabled = false;

//	fname = new String(S"");
	replay = false;

	rtatstatus = new StatusBarPanel();
	rtatstatus->Text = "";
	rtatstatus->Width = 300;
	this->statusBar1->Panels->Add(rtatstatus);

	if(rom == NULL)
	{
		this->menuItem4->Enabled = false;
		this->loggerAnalyze->Enabled = false;
		this->incFuel->Enabled = false;
		this->incFuelLg->Enabled = false;
		this->decFuel->Enabled = false;
		this->decFuelLg->Enabled = false;

	}
}

bre20::Logger::~Logger()
{
	timer1->Stop();
	if(comport!= NULL)
		comport->close();
	if(logwriter != NULL)
		logwriter->close();
	if(replayin != NULL)
		replayin->Close();
}

bool bre20::Logger::connectToPort()
{
	return comport->connectToEcu(String::Concat(S"COM",port.ToString()),baudrate,parity);
}

void bre20::Logger::setAfr(float afr)
{
	autoEdit->setAfr(afr);
	if(lambdamode)
		afr = afr/14.7F;
	logdata->getAfrKey()->Value = afr.ToString("0.00");
}

void bre20::Logger::setLambdaMode(bool lambda)
{
	lambdamode = lambda;
	if(lambda)
		logdata->getAfrKey()->Label = "Lambda";
	else
		logdata->getAfrKey()->Label = "Afr";
	loggrid->setKeyValue(0,logdata->getAfrKey()->Label,"");
	loggrid->Refresh();
}

void bre20::Logger::setLocale()
{
	if(!Common::getNumberDivider()->Equals("."))
		SEP = ";";
	else
		SEP = ",";
}

void bre20::Logger::setupTraceGrids()
{
	tracegrids = new LogGridPage*[rom->getRom()->getNumMaps()*2];
	int height = rom->getRom()->getHeight();
	int width = rom->getRom()->getWidth();
	for(int num = 0; num < rom->getRom()->getNumMaps(); num++)
	{
		//fuel
		GridPage* gp = static_cast<GridPage*>(rom->getTabConrol()->TabPages->get_Item(num*2));
		tracegrids[num*2] = new LogGridPage(width,height,gp->Text);
		gp = static_cast<GridPage*>(rom->getTabConrol()->TabPages->get_Item(num*2+1));
		tracegrids[num*2+1] = new LogGridPage(width,height,gp->Text);
		for(int i=0; i<width; i++){
			tracegrids[num*2]->setGridColumnHeader(i,i.ToString());
			tracegrids[num*2 +1]->setGridColumnHeader(i,i.ToString());
		}
		for(int i=0; i<height; i++){
			tracegrids[num*2]->setRowHeader(i,i.ToString());
			tracegrids[num*2+1]->setRowHeader(i,i.ToString());
		}

		this->tabControl1->Controls->Add(tracegrids[num*2]);
		this->tabControl1->Controls->Add(tracegrids[num*2+1]);
		
		tracegrids[num*2]->setIndex(num*2);
		tracegrids[num*2+1]->setIndex(num*2+1);
	}
}

int bre20::Logger::readInterval(System::IO::StreamReader *sr)
{
	String* s = replayin->ReadLine();
	Char delim[] = SEP->ToCharArray();
	try
	{
		return System::Convert::ToInt32(s->Split(delim)[0]);
	}
	catch(...){return -1;}
}

void bre20::Logger::toggleInterpolationTrace(bool setbool, bool enable)
{
	if(setbool)
		interptrace = enable;
	
	for(int i = 0; i<tracegrids->Length; i++){
		if(i%2 == 0 || !enable){ //only enable fuel for now...
			GridPage* gp = static_cast<GridPage*>(rom->getTabConrol()->TabPages->get_Item(i));
			gp->getCrosshairs()->Enabled = enable;
			gp->Refresh();
			tracegrids[i]->getCrosshairs()->Enabled = enable;
			tracegrids[i]->Refresh();
		}
	}
}

//realtime auto tune menu item
void bre20::Logger::menuItem4_Click(System::Object *  sender, System::EventArgs *  e)
{
	if(menuItem4->Text->IndexOf(S"Start") >= 0)
	{
		menuItem4->Text = S"Stop Real-time Autotune";
		this->rtatstatus->Text = "Real-Time AutoTune started";
		autoEdit->RtAutoTuneInit();
	}
	else
	{
		menuItem4->Text = S"Start Real-time Autotune";
		this->rtatstatus->Text = "Real-Time AutoTune stopped";
		autoEdit->RtAutoTuneStop();
	}
}
//_____________________________________________________________________________________

int bre20::Logger::getReplayHashInt(String* key)
{
	Object* o = replayHash->get_Item(key);
	return Convert::ToInt32(static_cast<String*>(o));
}
float bre20::Logger::getReplayHashFloat(String* key)
{
	Object* o = replayHash->get_Item(key);
	return Convert::ToSingle(static_cast<String*>(o));
}
String* bre20::Logger::getReplayHashStr(String* key)
{
	Object* o = replayHash->get_Item(key);
	return static_cast<String*>(o);
}
void bre20::Logger::runReplay()
{
	//when run, split values, get label/value, put in hash 
	//go through keys, get label from key, get value from hash...
	String* delim = SEP;
	try{
		replayValues = replayin->ReadLine()->Split(delim->ToCharArray());
		replayTrack->Value += 1;
	}catch(...){
	//	MessageBox::Show("Bad input!","BRE");
		clickStart();
		reopenReplayFile();
	}

	int max = Math::Min(replayLabels->Length,replayValues->Length);
	for(int i = 0; i<max; i++)
		replayHash->set_Item(replayLabels[i], replayValues[i]);

	//order: index,x,fuely,igny,afr,weights,
	try{
		int num = getReplayHashInt("index");
		setMapIndex(num);

		num = getReplayHashInt("x");
		setCurrentColumn(num);

		num = getReplayHashInt("fuely");
		setCurrentFuelRow(num);

		num = getReplayHashInt("igny");
		setCurrentIgnitionRow(num);

		float fnum = getReplayHashFloat("Afr");
		setAfr(fnum);

		float fweight __gc[,] = AutoEdit::parseWeights(this->getReplayHashStr("weights"));
		autoEdit->setCellWeight(0,0,fweight[0,0]);
		autoEdit->setCellWeight(0,1,fweight[0,1]);
		autoEdit->setCellWeight(1,0,fweight[1,0]);
		autoEdit->setCellWeight(1,1,fweight[1,1]);
	}catch(...){}

	try
	{
		for(int i = 0; i<logdata->getNumLoggerKeys(); i++){
			LoggerKey* key = logdata->getLoggerKey(i);
			if(key->ToGrid && key->Label != NULL)
				key->Value = static_cast<String*>(replayHash->get_Item(key->Label));
		}
	}catch(...){}

	executeReplays(replayLabels, replayHash);

	//output the data
	outputUserData();
}

void bre20::Logger::runDL()
{
	unsigned char cmd __gc[] = logdata->getCommands();
	
	if(!TESTING)
	{
		if(sendmode == 0)
			cmd = comport->sendBlock1b(cmd);
		else if(sendmode == 1)
		{
			bool wasnull = false;
			unsigned char cmd2 __gc[] = new unsigned char __gc[cmd->Length];
			unsigned char res __gc[] = NULL;
			for(int i = 0; i<cmd->Length; i++)
			{
				if(res == NULL)
					res = new unsigned char __gc[1];
				res[0] = cmd[i];
				res = comport->sendBlock1b(res);
				if(res != NULL)
					cmd2[i] = res[0];
				else
					cmd2[i] = 0;
			}
			cmd = cmd2;
		}
	}
	else
	{
		tester++;
		for(int i = 0; i<cmd->Length; i++)
			cmd[i] = tester;
	}

	if(cmd!=NULL)
	{
		millisKey->Value = System::DateTime::Now.Ticks.ToString();
		try
		{
			logdata->setResultsArray(cmd);
		}
		catch(Exception* ex)
		{
			Globals::LogFile->WriteLine(this,String::Concat("setResultsArray Exception: ", ex->Message, "\n\t", ex->StackTrace));
		}
		float fafr = Common::getAfr((unsigned char)static_cast<Integer*>(logdata->getHashtable()->get_Item(S"o2afr"))->Value,this->getO2Type());
		setAfr(fafr);

		//execute rules here...
		executeRules();

		setAfr(this->autoEdit->getAfr()*this->compMult + compAdd);

		logwriter->makeDecision();

		//output the data
		outputUserData();

		//call event
		DatalogRun();

		//call auto tune stuff last...
		autoEdit->RtAutoTuneRun();
	}
	else
	{
		clickStart();
		MessageBox::Show("The Logger timed out.", "BRE");
	}
}

void bre20::Logger::initMainOutput()
{
	logwriter->setFileHeader(String::Concat(S"index",SEP,S"x",SEP,S"fuely",SEP,
							S"igny",SEP,S"Afr",SEP,S"weights",SEP));
	//logout->Write(S"index,x,fuely,igny,Afr,weights,");

	//weights start from current cell and go clockwise
	//so .1|.2|.3|.6 is:
	//   .1 .2
	//   .6 .3
}
void bre20::Logger::setMapTrace(TabControl* tc, String* type)
{
	int ind = autoEdit->getIndex()*2;
	try{

		TabPage* p = tc->SelectedTab;
		if(p->GetType()->Name->Equals(type))
		{
			int selind = tc->SelectedIndex;
			if(selind%2 == 0 && selind != ind) //fuel and current index not selected
				tc->SelectedIndex = ind;
			else if(selind %2 == 1 && selind != ind+1)
				tc->SelectedIndex = ind+1;

			GridPage* lgp = static_cast<GridPage*>(p);
			if(selind == ind){
				if(interptrace){
					lgp->getCrosshairs()->Interpolation = PointF(
						autoEdit->getCellWeight(0,0) + autoEdit->getCellWeight(0,1),
						autoEdit->getCellWeight(0,0) + autoEdit->getCellWeight(1,0));
					lgp->getCrosshairs()->Enabled = true;
				}
				lgp->setSelection(autoEdit->getColumn(),autoEdit->getFuelRow(),2,2,true);
			}
			else if(selind == ind+1){
				lgp->getCrosshairs()->Enabled = false;
				lgp->setSelection(autoEdit->getColumn(),autoEdit->getIgnitionRow(),2,2,true);
			}
			else
				lgp->setSelection(-1,-1,1,1,true);
		}

	}catch(Exception* e){}
}
void bre20::Logger::outputMainData()
{
	float fafr;
	if(romtrace)
	{
		TabControl* tc = rom->getTabConrol();
		setMapTrace(tc, "GridPage");
	}
	if(maptrace)
	{
		int ind = autoEdit->getIndex()*2;
	//	try
	//	{
			if(maptracetype == DatalogOptions::AFRTRACE)
			{
				fafr = autoEdit->getAfr();
				if(lambdamode)
					fafr = fafr/14.7F;
				static_cast<LogGridPage*>(this->tabControl1->TabPages->get_Item(ind))->
					setValue(autoEdit->getColumn(),autoEdit->getFuelRow(),fafr,false);
			}

			setMapTrace(tabControl1,"LogGridPage");
	//	}
	//	catch(Exception* e){}
	}
	
	if(replay)
		return;

	logwriter->Write(String::Concat(autoEdit->getIndex().ToString(),SEP, //index
		autoEdit->getColumn().ToString(),SEP, //x
		autoEdit->getFuelRow().ToString(),SEP, //fuely
		autoEdit->getIgnitionRow().ToString(),SEP, //igny
		autoEdit->getAfr().ToString(),SEP, //afr
		autoEdit->getCellWeight(0,0).ToString(),S"|", //current cell
		autoEdit->getCellWeight(1,0).ToString(),S"|", //east cell
		autoEdit->getCellWeight(1,1).ToString(),S"|", //south east cell
		autoEdit->getCellWeight(0,1).ToString(),SEP)); //south cell

}

void bre20::Logger::initUserOutput()
{
	initMainOutput();

	LoggerKey* key;
	int count = 0;
	for(int i = 0; i<logdata->getNumLoggerKeys(); i++){
		key = logdata->getLoggerKey(i);
		if(key->Label != NULL && key->ToCsv)
			logwriter->appendFileHeader(String::Concat(key->Label,SEP));
		if(key->ToGrid && key->Label != NULL)
			loggrid->setKeyValue(count++,key->Label, "");
	}
//	logwriter->WriteLineNow("");
}

//writes the header to the csv
void bre20::Logger::outputUserData()
{
	outputMainData();

	LoggerKey* key;
	int count = 0;
	for(int i = 0; i<logdata->getNumLoggerKeys(); i++){
		key = logdata->getLoggerKey(i);
		if(key->ToCsv && !replay){
			if(key->Value != NULL)
				logwriter->Write(String::Concat(key->Value,SEP));
			else
				logwriter->Write(String::Concat("",SEP));
		}
		if(key->ToGrid){
			if(key->Value != NULL)
				loggrid->setKeyValue(count,key->Value);
			count++;
		}
	}
	logwriter->WriteLine("");

	loggrid->Refresh();
}

void bre20::Logger::addGauge(Gauge::Gauge* gauge)
{
	dash->Controls->Add(gauge);
}
Gauge::Gauge* bre20::Logger::addGauge(String* title, String* unitname, int size,int x, int y,
	int minval,int maxval,int divisions, int numdecimals)
{
	Gauge::Gauge* g = new Gauge::Gauge();
	dash->Controls->Add(g);
    
	g->CurrentValue = 0;
	g->BackColor = System::Drawing::Color::Transparent;
	g->GaugeSize = System::Drawing::Size(size, size);
	g->Location = System::Drawing::Point(x,y);
	g->MaxValue = (float)maxval;
	g->MinValue = (float)minval;
	g->NumberDivisions = divisions;
	g->NumberIntermediateLines = 5;
	g->NumberOfDecimals = numdecimals;
	g->Size = System::Drawing::Size(size, size);
	g->Title = title;
	g->Units = unitname;

	return g;
}

int bre20::Logger::addO2Type(AfrRule* afrrule)
{
	if(afrrule == NULL || afrrule->getName()->Length < 2 || logging)
		return -1;

	if(extrao2 == NULL)
		extrao2 = new String*[1];
	else
	{
		String* tmp[] = extrao2;
		extrao2 = new String*[tmp->Length+1];
		for(int i = 0; i<tmp->Length; i++)
			extrao2[i] = tmp[i];
	}

	int newindex = extrao2->Length-1;
	extrao2[newindex] = afrrule->getName();
	newindex = newindex+DatalogOptions::WBMAX+1;

	afrrule->setIndex(newindex);
	afrrule->setAfrKey(logdata->getLoggerKey(0));

	this->rules->Add(afrrule);

	return newindex;
}