#pragma once

using namespace System;
using namespace System::ComponentModel;
using namespace System::Collections;
using namespace System::Windows::Forms;
using namespace System::Data;
using namespace System::Drawing;

#include "PortSetupDialog.h"
#include "datalogcom.h"
#include "LogGridPage.h"
#include "Loganalyzer.h"
#include "DatalogOptions.h"
#include "DigiDash.h"
#include "CharWrap.h"
#include "child.h"
#include "common.h"
#include "autoedit.h"
#include "LoggerGrid.h"
#include "LoggerData.h"
#include "FormWithRules.h"
#include "DatalogFileWriter.h"

namespace bre20
{
	/// <summary> 
	/// The main form of the DataLogger. See \ref loggerusage "Datalogger Usage and Extension".
	/// </summary>
	public __gc class Logger : public FormWithRules
	{
	private:
		static bool TESTING = false;
		static String * SEP = ",";
    //my data members...
	protected:	
		bre20::LogGridPage * tracegrids[]; //map trace of realtime data
		TabPage * dash;
		System::Windows::Forms::Timer *  timer1;
		System::Windows::Forms::TabControl *  tabControl1;
		LoggerGrid* loggrid;

		AutoEdit* autoEdit;

		bre20::child * rom;

		DatalogCom* comport;
		LoggerData* logdata;

		LoggerKey* millisKey;

		String* extrao2[];

		int parity; 
		///true if replaying, false if logging
		bool replay;
		///true if actually logging
		bool logging; 
		///true if map trace is enabled
		bool maptrace;
		bool romtrace;
		bool interptrace;
		///
		int sendmode;

		//replay
		String* replayLabels[];
		String* replayValues[];
		Hashtable* replayHash;
		//end replay

		int tminterval;
		int errorcount;
		int errorcountMax;
		int maptracetype;
		int o2type;
		int units;
		bool lambdamode;
		int mixtureunit;

		float compMult;
		float compAdd;

		unsigned char tester;

		int port;
		int baudrate;

		String * fname;
		String * title;

		void setupTraceGrids();

		bre20::DatalogFileWriter* logwriter;
		System::IO::StreamReader * replayin;

		int readInterval(System::IO::StreamReader *sr);

		virtual void runDL();
		virtual void runReplay();
		virtual int getReplayHashInt(String* key);
		virtual float getReplayHashFloat(String* key);
		virtual String* getReplayHashStr(String* key);
		
		void timer_Tick(Object * myObject, EventArgs * myEventArgs)
		{
			if(replay)
				runReplay();
			else
				runDL();
		}

	protected:
		void Dispose(Boolean disposing)
		{
			if (disposing && components)
			{
				components->Dispose();
			}
			__super::Dispose(disposing);
		}

		void clickStart(){
			start_Click(NULL,NULL);
		}
		void setMapTrace(TabControl* tc, String* type);

	public: 
		Logger(bre20::child * rom, AutoEdit* autotune, ArrayList* rules);
		~Logger();

		bool connectToPort();

		///\returns the LoggerData object which is used to pass
		///the command list and data results between the logger and 
		///Rules
		LoggerData* getLoggerData(){return logdata;}
		///\returns the DatalogCom object the logger is using  
		DatalogCom* getSerialPort(){return comport;}
		///\returns the LoggerGrid control. Its the control on the right 
		///side of the logger 
		LoggerGrid* getLoggerGrid(){return loggrid;}

		//\returns the DatalogFileWriter object
//		DatalogFileWriter* getDatalogFileWriter(){return logwriter;} //we dont want outside access

		///Use this to set the AFR if necessary. Do NOT handle lambda,
		///as conversion from afr to lambda will be done by BRE if necessary.
		///This must be set for auto tune to work.
		///
		///This also updates the afr LoggerKey.
		void setAfr(float afr);
		///This must be set for auto tune to work.
		void setMapIndex(int index){autoEdit->setIndex(index);}
		///This must be set for auto tune to work.
		void setCurrentColumn(int x){autoEdit->setCurrentColumn(x);}
		///This must be set for auto tune to work.
		void setCurrentFuelRow(int fy){autoEdit->setCurrentFuelRow(fy);}
		///This must be set for auto tune to work.
		void setCurrentIgnitionRow(int iy){autoEdit->setCurrentIgnitionRow(iy);}
		///This must be set for auto tune to work.
		///It sets a cells weight based on the interpolation.
		///
		/*!
		index [0,0] is the current cell, index [1,0]
		is the east cell, [0,1] is the south cell, etc.
		The weights should always add up to 1.
		*/
		void setCellWeight(int col, int row, float value){autoEdit->setCellWeight(col,row,value);}

		///Sets the current speed for the logger's AutoStart feature 
		void setCurrentSpeed(int speed){ logwriter->setSpeed(speed); }
		///Sets the current rpm for the logger's AutoStart feature 
		void setCurrentRpm(int rpm){ logwriter->setRpm(rpm); }
		///Sets the current tps for the logger's AutoStart feature 
		void setCurrentTps(int tps){ logwriter->setTps(tps); }

		///Adds a new o2 type to the o2 type options. Match this number
		///with the value returned from getO2Type() to find out if your
		///new o2 type is being used.
		///\returns the index in the o2 type combobox of the new o2 type.
		///-1 in error.
		int addO2Type(AfrRule* newo2);
		///\returns the combobox index of the o2 type being currently used.
		int getO2Type(){return o2type;};

		bool isSae(){if(units == DatalogOptions::SAE) return true; else return false;}

		///adds a gauge to the digital dash pane. You must create the gauge first.
		void addGauge(Gauge::Gauge* gauge);
		///Creates and adds a Gauge to the digital dash pane.
		Gauge::Gauge* addGauge(String* title, String* units, int size, int x, int y,
			int minval,int maxval,int divisions, int numdecimals);

		__event EventHandler * OnDataLoggerStart;
		__event EventHandler * OnDataLoggerRun;
		__event EventHandler * OnDataLoggerStop;
		///sender is the newly created DatalogOptions dialog
		__event EventHandler * OnDatalogOptionsCreated;
		///sender is the newly created PortSetupDialog object
		__event EventHandler * OnPortSetupCreated;

		void setParity(int p){this->parity = p;}
		void setBaud(int baud){this->baudrate = baud;}

		///send a whole string for each send to the ecu
		static const int SENDMODE_STRING = 0;
		///bounce bytes back an forth.
		static const int SENDMODE_BYTE = 1;
		void setSendMode(int smode){sendmode = smode;}
		int getSendMode(){return sendmode;}

	private: System::Windows::Forms::TrackBar *  replayTrack; 
	private: System::Windows::Forms::MenuItem *  loggerAnalyze; 
	private: System::Windows::Forms::MainMenu *  mainMenu1;
	private: System::Windows::Forms::MenuItem *  menuItem1;
	private: System::Windows::Forms::MenuItem *  LoggerOptions;
	private: System::Windows::Forms::MenuItem *  portsetupmenu;
	private: System::Windows::Forms::GroupBox *  groupBox1;
	private: System::Windows::Forms::RadioButton *  logOption;
	private: System::Windows::Forms::RadioButton *  replayOption;
	private: System::Windows::Forms::Button *  start;
	private: System::Windows::Forms::Button *  portSetup;
	private: System::Windows::Forms::Button *  connect;
	private: System::Windows::Forms::CheckBox *  toFile;
	private: System::Windows::Forms::MenuItem *  menuItem2;
	private: System::Windows::Forms::MenuItem *  incFuel;
	private: System::Windows::Forms::MenuItem *  incFuelLg;
	private: System::Windows::Forms::MenuItem *  decFuel;
	private: System::Windows::Forms::MenuItem *  decFuelLg;
	private: System::Windows::Forms::StatusBar *  statusBar1;
	private: System::Windows::Forms::StatusBarPanel *  connected;
	private: System::Windows::Forms::StatusBarPanel *  rtatstatus;
	private: System::Windows::Forms::MenuItem *  menuItem3;
	private: System::Windows::Forms::MenuItem *  menuItem4;
		
		/// <summary>
		/// Required designer variable.
		/// </summary>
		System::ComponentModel::Container* components;

		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		void InitializeComponent(void)
		{
			this->tabControl1 = new System::Windows::Forms::TabControl();
			this->portSetup = new System::Windows::Forms::Button();
			this->connect = new System::Windows::Forms::Button();
			this->toFile = new System::Windows::Forms::CheckBox();
			this->start = new System::Windows::Forms::Button();
			this->groupBox1 = new System::Windows::Forms::GroupBox();
			this->replayOption = new System::Windows::Forms::RadioButton();
			this->logOption = new System::Windows::Forms::RadioButton();
			this->mainMenu1 = new System::Windows::Forms::MainMenu();
			this->menuItem1 = new System::Windows::Forms::MenuItem();
			this->LoggerOptions = new System::Windows::Forms::MenuItem();
			this->portsetupmenu = new System::Windows::Forms::MenuItem();
			this->loggerAnalyze = new System::Windows::Forms::MenuItem();
			this->menuItem2 = new System::Windows::Forms::MenuItem();
			this->incFuel = new System::Windows::Forms::MenuItem();
			this->incFuelLg = new System::Windows::Forms::MenuItem();
			this->decFuel = new System::Windows::Forms::MenuItem();
			this->decFuelLg = new System::Windows::Forms::MenuItem();
			this->menuItem3 = new System::Windows::Forms::MenuItem();
			this->menuItem4 = new System::Windows::Forms::MenuItem();
			this->statusBar1 = new System::Windows::Forms::StatusBar();
			this->connected = new System::Windows::Forms::StatusBarPanel();
			this->groupBox1->SuspendLayout();
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->connected))->BeginInit();
			this->SuspendLayout();
			// 
			// tabControl1
			// 
			this->tabControl1->Anchor = (System::Windows::Forms::AnchorStyles)(((System::Windows::Forms::AnchorStyles::Top | System::Windows::Forms::AnchorStyles::Bottom) 
				| System::Windows::Forms::AnchorStyles::Left) 
				| System::Windows::Forms::AnchorStyles::Right);
			this->tabControl1->Location = System::Drawing::Point(0, 0);
			this->tabControl1->Name = S"tabControl1";
			this->tabControl1->SelectedIndex = 0;
			this->tabControl1->Size = System::Drawing::Size(600, 296);
			this->tabControl1->TabIndex = 0;
			// 
			// portSetup
			// 
			this->portSetup->Anchor = (System::Windows::Forms::AnchorStyles)(System::Windows::Forms::AnchorStyles::Bottom | System::Windows::Forms::AnchorStyles::Left);
			this->portSetup->Location = System::Drawing::Point(8, 310);
			this->portSetup->Name = S"portSetup";
			this->portSetup->Size = System::Drawing::Size(96, 24);
			this->portSetup->TabIndex = 1;
			this->portSetup->Text = S"Setup Port";
			this->portSetup->Click += new System::EventHandler(this, portSetup_Click);
			portSetup->FlatStyle = FlatStyle::System;
			// 
			// connect
			// 
			this->connect->Anchor = (System::Windows::Forms::AnchorStyles)(System::Windows::Forms::AnchorStyles::Bottom | System::Windows::Forms::AnchorStyles::Left);
			this->connect->Location = System::Drawing::Point(120, 310);
			this->connect->Name = S"connect";
			this->connect->Size = System::Drawing::Size(88, 24);
			this->connect->TabIndex = 2;
			this->connect->Text = S"Connect";
			this->connect->Click += new System::EventHandler(this, connect_Click);
			connect->FlatStyle = FlatStyle::System;
			// 
			// toFile
			// 
			this->toFile->Anchor = (System::Windows::Forms::AnchorStyles)(System::Windows::Forms::AnchorStyles::Bottom | System::Windows::Forms::AnchorStyles::Right);
			this->toFile->Location = System::Drawing::Point(344, 312);
			this->toFile->Name = S"toFile";
			this->toFile->Size = System::Drawing::Size(80, 24);
			this->toFile->TabIndex = 3;
			this->toFile->Text = S"Log to File";
			this->toFile->CheckedChanged += new System::EventHandler(this, toFile_CheckedChanged);
			toFile->FlatStyle = FlatStyle::System;
			// 
			// start
			// 
			this->start->Anchor = (System::Windows::Forms::AnchorStyles)(System::Windows::Forms::AnchorStyles::Bottom | System::Windows::Forms::AnchorStyles::Left);
			this->start->Location = System::Drawing::Point(224, 310);
			this->start->Name = S"start";
			this->start->Size = System::Drawing::Size(88, 24);
			this->start->TabIndex = 5;
			this->start->Text = S"Start";
			this->start->Click += new System::EventHandler(this, start_Click);
			start->FlatStyle = FlatStyle::System;
			// 
			// groupBox1
			// 
			this->groupBox1->Anchor = (System::Windows::Forms::AnchorStyles)(System::Windows::Forms::AnchorStyles::Bottom | System::Windows::Forms::AnchorStyles::Right);
			this->groupBox1->Controls->Add(this->replayOption);
			this->groupBox1->Controls->Add(this->logOption);
			this->groupBox1->Location = System::Drawing::Point(432, 302);
			this->groupBox1->Name = S"groupBox1";
			this->groupBox1->Size = System::Drawing::Size(168, 36);
			this->groupBox1->TabIndex = 7;
			this->groupBox1->TabStop = false;
			this->groupBox1->Text = S"Mode";
			groupBox1->FlatStyle = FlatStyle::System;
			// 
			// replayOption
			// 
			this->replayOption->Location = System::Drawing::Point(96, 14);
			this->replayOption->Name = S"replayOption";
			this->replayOption->Size = System::Drawing::Size(64, 16);
			this->replayOption->TabIndex = 1;
			this->replayOption->Text = S"Replay";
			this->replayOption->CheckedChanged += new System::EventHandler(this, replayOption_CheckedChanged);
			replayOption->FlatStyle = FlatStyle::System;
			// 
			// logOption
			// 
			this->logOption->Checked = true;
			this->logOption->Location = System::Drawing::Point(24, 14);
			this->logOption->Name = S"logOption";
			this->logOption->Size = System::Drawing::Size(48, 16);
			this->logOption->TabIndex = 0;
			this->logOption->TabStop = true;
			this->logOption->Text = S"Log";
			this->logOption->CheckedChanged += new System::EventHandler(this, logOption_CheckedChanged);
			logOption->FlatStyle = FlatStyle::System;
			// 
			// mainMenu1
			// 
			System::Windows::Forms::MenuItem* __mcTemp__1[] = new System::Windows::Forms::MenuItem*[1];
			__mcTemp__1[0] = this->menuItem1;
			this->mainMenu1->MenuItems->AddRange(__mcTemp__1);
			// 
			// menuItem1
			// 
			this->menuItem1->Index = 0;
			System::Windows::Forms::MenuItem* __mcTemp__2[] = new System::Windows::Forms::MenuItem*[10];
			__mcTemp__2[0] = this->LoggerOptions;
			__mcTemp__2[1] = this->portsetupmenu;
			__mcTemp__2[2] = this->loggerAnalyze;
			__mcTemp__2[3] = this->menuItem2;
			__mcTemp__2[4] = this->incFuel;
			__mcTemp__2[5] = this->incFuelLg;
			__mcTemp__2[6] = this->decFuel;
			__mcTemp__2[7] = this->decFuelLg;
			__mcTemp__2[8] = this->menuItem3;
			__mcTemp__2[9] = this->menuItem4;
			this->menuItem1->MenuItems->AddRange(__mcTemp__2);
			this->menuItem1->MergeOrder = 1;
			this->menuItem1->Text = S"Logger";
			// 
			// LoggerOptions
			// 
			this->LoggerOptions->Index = 0;
			this->LoggerOptions->Text = S"Options...";
			this->LoggerOptions->Shortcut = Shortcut::F9;
			this->LoggerOptions->Click += new System::EventHandler(this, LoggerOptions_Click);
			// 
			// portsetupmenu
			// 
			this->portsetupmenu->Index = 1;
			this->portsetupmenu->Text = S"Port Setup...";
			this->portsetupmenu->Shortcut = Shortcut::CtrlShiftP;
			this->portsetupmenu->Click += new System::EventHandler(this, portSetup_Click);
			// 
			// loggerAnalyze
			// 
			this->loggerAnalyze->Index = 2;
			this->loggerAnalyze->Text = S"Analyze Current Log...";
			this->loggerAnalyze->Shortcut = Shortcut::CtrlShiftO;
			this->loggerAnalyze->Click += new System::EventHandler(this, loggerAnalyze_Click);
			// 
			// menuItem2
			// 
			this->menuItem2->Index = 3;
			this->menuItem2->Text = S"-";
			// 
			// incFuel
			// 
			this->incFuel->Index = 4;
			this->incFuel->Shortcut = System::Windows::Forms::Shortcut::CtrlI;
			this->incFuel->Text = S"Increase Fuel";
			this->incFuel->Click += new System::EventHandler(this, incFuel_Click);
			// 
			// incFuelLg
			// 
			this->incFuelLg->Index = 5;
			this->incFuelLg->Shortcut = System::Windows::Forms::Shortcut::CtrlShiftI;
			this->incFuelLg->Text = S"Increase Fuel (large change)";
			this->incFuelLg->Click += new System::EventHandler(this, incFuelLg_Click);
			// 
			// decFuel
			// 
			this->decFuel->Index = 6;
			this->decFuel->Shortcut = System::Windows::Forms::Shortcut::CtrlD;
			this->decFuel->Text = S"Decrease Fuel";
			this->decFuel->Click += new System::EventHandler(this, decFuel_Click);
			// 
			// decFuelLg
			// 
			this->decFuelLg->Index = 7;
			this->decFuelLg->Shortcut = System::Windows::Forms::Shortcut::CtrlShiftD;
			this->decFuelLg->Text = S"Decrease Fuel (large change)";
			this->decFuelLg->Click += new System::EventHandler(this, decFuelLg_Click);
			// 
			// menuItem3
			// 
			this->menuItem3->Index = 8;
			this->menuItem3->Text = S"-";
			// 
			// menuItem4
			// 
			this->menuItem4->Index = 9;
			this->menuItem4->Shortcut = System::Windows::Forms::Shortcut::CtrlShiftR;
			this->menuItem4->Text = S"Start Real-time Autotune";
			this->menuItem4->Click += new System::EventHandler(this, menuItem4_Click);
			// 
			// statusBar1
			// 
			this->statusBar1->Location = System::Drawing::Point(0, 344);
			this->statusBar1->Name = S"statusBar1";
			System::Windows::Forms::StatusBarPanel* __mcTemp__3[] = new System::Windows::Forms::StatusBarPanel*[1];
			__mcTemp__3[0] = this->connected;
			this->statusBar1->Panels->AddRange(__mcTemp__3);
			this->statusBar1->ShowPanels = true;
			this->statusBar1->Size = System::Drawing::Size(776, 24);
			this->statusBar1->TabIndex = 8;
			this->statusBar1->Text = S"statusBar1";
			// 
			// connected
			// 
			this->connected->Text = S"Disconnected";
			this->connected->Width = 300;
			// 
			// Logger
			// 
			this->AutoScaleBaseSize = System::Drawing::Size(5, 13);
			this->ClientSize = System::Drawing::Size(776, 368);
			this->Controls->Add(this->statusBar1);
			this->Controls->Add(this->groupBox1);
			this->Controls->Add(this->start);
			this->Controls->Add(this->toFile);
			this->Controls->Add(this->connect);
			this->Controls->Add(this->portSetup);
			this->Controls->Add(this->tabControl1);
			this->Menu = this->mainMenu1;
			this->Name = S"Logger";
			this->Text = S"Data Logger";
			this->Resize += new System::EventHandler(this, Logger_Resize);
			this->Closing += new System::ComponentModel::CancelEventHandler(this, Logger_Closing);
			this->groupBox1->ResumeLayout(false);
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->connected))->EndInit();
			this->ResumeLayout(false);

		}		

		//-------------------------------------------------------------------------
		//events
		void DatalogStart(){
			if (OnDataLoggerStart != NULL)
				OnDataLoggerStart(this, new EventArgs());
		}
		void DatalogStop(){
			if (OnDataLoggerStop != NULL)
				OnDataLoggerStop(this, new EventArgs());
		}
		void DatalogRun(){
			if (OnDataLoggerRun != NULL)
				OnDataLoggerRun(this, new EventArgs());
		}

		void DatalogOptionsCreated(DatalogOptions* dlo){
			if (OnDatalogOptionsCreated != NULL)
				OnDatalogOptionsCreated(dlo, new EventArgs());
		}

		void PortSetupCreated(PortSetupDialog* psd){
			if (OnPortSetupCreated != NULL)
				OnPortSetupCreated(psd, new EventArgs());
		}

		void initMainOutput();
		void outputMainData();
		//outputs the user header
		void initUserOutput();
		//outputs the user data 
		void outputUserData();

		void setLambdaMode(bool lambda);

		void readProperties();
		void initMisc();
		void setLocale();

		void toggleInterpolationTrace(bool setbool, bool enable);
//***********************************************************************************
//event handlers
		void file_changed(System::Object *  sender, System::EventArgs *  e)
		{
			this->Text = String::Concat(title, " - ", logwriter->getCurrentFile());
		}
		void connect_Click(System::Object *  sender, System::EventArgs *  e)
		{
			if(!String::Equals(connect->Text, S"Disconnect"))
			{
				bool con = true;
				if(!TESTING)
					con = connectToPort();
				if(con)
				{
					connect->Text = "Disconnect";
					start->Enabled = true;
					portSetup->Enabled = false;
					connected->Text = String::Concat(S"Connected to COM",port.ToString());
				}
				else
					connected->Text = "Failed to open port...";
			}
			else //need to disconnect
			{
				if(!String::Equals(start->Text, S"Start"))
					clickStart();

				comport->close();
				connect->Text = "Connect";
				start->Enabled = false;
				portSetup->Enabled = true;
				connected->Text = "Disconnected";
			}
		}

		void portSetup_Click(System::Object *  sender, System::EventArgs *  e)
		{
			PortSetupDialog * psd = new PortSetupDialog();
			psd->setPortName(Globals::Props->getParam(S"logPort"));
			psd->setBaud(baudrate);
			psd->setParity(parity);

			psd->ShowDialog(this);

			port = psd->getPortName();
			baudrate = psd->getBaud();
			parity = psd->getParity();

			Globals::Props->setParam(this,S"logPort",String::Concat(S"COM",port.ToString()));

//			delete psd;
		}
		
		void start_Click(System::Object *  sender, System::EventArgs *  e)
		{
			if(String::Equals(start->Text, S"Start"))
			{	
				autoEdit->DlEditInit();

				logging = true;
				errorcount = 0;
				errorcountMax = Globals::Props->getIntParam("logNumErrorTries");
				logdata->setLocked(true);

				DatalogStart();

				start->Text = "Stop";
				if(!replay)
					toFile->Enabled = false;
				this->groupBox1->Enabled = false;

			//	logStart();
				timer1->Start();
			}
			else
 			{	
			//	logStop();
				timer1->Stop();
				start->Text = "Start";
				if(!replay)
					toFile->Enabled = true;
				this->groupBox1->Enabled = true;

				logging = false;
				logdata->setLocked(false);

				DatalogStop();
			}
		}

		void toFile_CheckedChanged(System::Object *  sender, System::EventArgs *  e)
		{
			if(!toFile->Checked)
			{
				this->Text = title;
				logwriter->setFile(NULL);
				logwriter->setFileWriteEnable(false);
				logwriter->close();
				return;
			}

			String* fname[] = Common::openFileDialog(false,false,logwriter->getFile(),S"CSV files (*.csv)|*.csv|All files (*.*)|*.*");
			
			if(fname != NULL)
			{
				logwriter->setFile(fname[0]);

				//this->Text = String::Concat(title,S" - ", fname[0]);
				logwriter->setFileWriteEnable(true);
				initUserOutput();
				logwriter->Flush();
			}
			else
			{	
				toFile->Checked = false;
				logwriter->setFileWriteEnable(false);
			}
		}

		void logOption_CheckedChanged(System::Object *  sender, System::EventArgs *  e)
		{
			if(logOption->Checked){
				replay = false;
				this->Text = title;

				if(replayin != NULL){
					replayin->Close();
					replayin = NULL;
				}
				start->Enabled = false;
				connect->Enabled = true;
				portSetup->Enabled = true;
				toFile->Enabled = true;
				connected->Text = S"Disconnected";
				timer1->Interval = tminterval;
				for(int i = 0; i<tracegrids->Length; i++)
					this->tracegrids[i]->clearGrid();
			}
		}

		void replayOption_CheckedChanged(System::Object *  sender, System::EventArgs *  e)
		{
			if(!replayOption->Checked)
			{
				replayTrack->Visible = false;
				replayHash = NULL;
				replayLabels = NULL;
				replayValues = NULL;
				replayOption->Checked = false;
				logOption->Checked = true;
				replay = false;
				System::GC::Collect();
				return;
			}

			if(this->toFile->Checked)
			{
				toFile->Checked = false;
				toFile_CheckedChanged(NULL,NULL);
			}

			String* fnames[] = Common::openFileDialog(true,false,logwriter->getFile(),S"CSV files (*.csv)|*.csv|All files (*.*)|*.*");

			if(fnames == NULL)
			{
				this->logOption->Checked = true;
				return;
			}

			fname = fnames[0];
			try
			{
				replayin = new System::IO::StreamReader(fname);
			}
			catch(...){
				LogFileReader::displayError(fname);
				logOption->Checked = true;
				return;
			}
			replay = true;

			this->Text = String::Concat(title,S" Replay - ", fname);
			
			//if connected then we disconnect
			if(String::Equals(connect->Text, S"Disconnect"))
			{
				comport->close();
				connect->Text = "Connect";
			}

			start->Enabled = true;
			connect->Enabled = false;
			portSetup->Enabled = false;
			toFile->Enabled = false;
			connected->Text = S"Replay Mode";

			int inter = readInterval(replayin);
			//split labels, put in new hash table
			//when run, split values, get label/value, put in hash 
			//go through keys, et label from key, get value from hash...
			try
			{
				String* delim = SEP;
				replayLabels = replayin->ReadLine()->Split(delim->ToCharArray());
				if(inter<=0)
					throw new Exception("");
				else
					timer1->Interval = inter;
				String* line = replayin->ReadLine();
				int max = 1;
				while(line != NULL)
				{
					max++;
					line = replayin->ReadLine();
				}
				replayTrack->Maximum = max;
				replayTrack->Visible = true;
				reopenReplayFile();
			}catch(...){
				logOption->Checked = true;
				logOption_CheckedChanged(NULL,NULL);
				MessageBox::Show("Something is wrong with the input file","BRE Datalogger");
				return;
			}

			replayHash = new Hashtable();
			for(int i = 0; replayLabels != NULL && i<replayLabels->Length; i++)
				replayHash->Add(replayLabels[i],NULL);
		}
		void reopenReplayFile()
		{
			replayin->Close();
			replayin = new System::IO::StreamReader(fname);
			replayin->ReadLine();replayin->ReadLine();
			replayTrack->Value = 0;
		}

		void LoggerOptions_Click(System::Object *  sender, System::EventArgs *  e)
		{
			DatalogOptions * dlo = new DatalogOptions(extrao2);

			//fire event
			DatalogOptionsCreated(dlo);
		
			dlo->ShowDialog(this);

			if(replay)
				tminterval = dlo->getInterval();
			else
			{
				tminterval = dlo->getInterval();
				timer1->Interval = tminterval;
			}

			o2type = dlo->getO2Type();

			maptracetype = dlo->getTraceType();

			units = dlo->getUnitsType();

			toggleInterpolationTrace(true, dlo->isIntepTraceEnabled());

			maptrace = dlo->isMapTraceEnabled();
			romtrace = dlo->isRomTraceEnabled();

			setLambdaMode(dlo->getMixtureUnits() == DatalogOptions::MIXLAMBDA);
				
			this->compAdd = dlo->getAddAdjust();
			this->compMult = dlo->getMultiplyAdjust();

//			delete dlo;

			updateRules();
		}

		void loggerAnalyze_Click(System::Object *  sender, System::EventArgs *  e)
		{
			if(start->Text->Equals("Start"))
			{
				this->toFile->Checked = false;

				rom->openLogAnalyzer(this->fname);
			}
		}

		void Logger_Closing(System::Object *  sender, System::ComponentModel::CancelEventArgs *  e)
		{
			if(comport != NULL)
				comport->close();

			if(logwriter != NULL)
				logwriter->close();

			if(replayin != NULL)
				replayin->Close();

			toggleInterpolationTrace(false,false);
		}

		void Logger_Resize(System::Object *  sender, System::EventArgs *  e){
			int sz = this->Size.Width -30;
			this->connected->Width = sz/3;
			this->rtatstatus->Width = sz*2/3;
		}

		
		void incFuel_Click(System::Object *  sender, System::EventArgs *  e){autoEdit->changeFuel(autoEdit->getSmallChange());}
		void incFuelLg_Click(System::Object *  sender, System::EventArgs *  e){autoEdit->changeFuel(autoEdit->getLargeChange());}
		void decFuel_Click(System::Object *  sender, System::EventArgs *  e){autoEdit->changeFuel(autoEdit->getSmallChange()*-1);}
		void decFuelLg_Click(System::Object *  sender, System::EventArgs *  e){autoEdit->changeFuel(autoEdit->getLargeChange()*-1);}

		//real time autotune menu item
		void menuItem4_Click(System::Object *  sender, System::EventArgs *  e);
	};
}