#pragma once
using namespace System;
using namespace System::Collections;

namespace bre20
{
	///A label/key class for the datalogger system.
	///This key will be read by the gui and file output 
	///methods. When you create a LoggerKey in a Rule,
	///keep a reference to it!! If you are 
	///searching or recreating it, you will slow the 
	///logger down.
	public __gc class LoggerKey
	{
	public:
		LoggerKey(String* label, bool tocsv, bool togrid)
		{
			Label = label;
			if(label == NULL)
				throw new Exception("The label CANNOT be null");
			ToCsv = tocsv;
			ToGrid = togrid;
			Value = NULL;
		}

		///The label/key 
		String* Label;
		///True if this label is output to the csv file
		bool ToCsv;
		///True if this label is output to the grid for viewing
		bool ToGrid;
		///the value that is written/displayed 
		String* Value;
	};

	///Contains the data for the Logger such as the command list,
	///label list for the hash table, etc.
	public __gc class LoggerData
	{
	private:
		bool locked;

		static String* fname = "loggerCmd.ini"; 
		
		///The list of commands
		unsigned char commands __gc[];
		///The list of labels corresponding to the commands
		String* labels[];

		///The hash table for ease of value retrieval 
		Hashtable* hash;

		///The list of output values
		ArrayList* outLoggerKeys;

	//	static const int HIGHEST_SPECIAL = 0;

	public:

		///Reads the ini file and assembles the 
		///command and label arrays.
		///
		///\param ecuType the type of the ecu
		LoggerData(String* ecuType);

		///Adds a new command to the respective arrays 
		///and to the ini file.
		void addCommand(int cmd, String* label);
		///Adds a new key to the output list
		void addLoggerKey(LoggerKey* key);
		///returns a LoggerKey 
		LoggerKey* getLoggerKey(int index);
		///returns the number of logger keys
		int getNumLoggerKeys();

		//this clears the LoggerKeys except for the important ones. 
	//	void clearLoggerKeys();
	
		///Removes a key from the list
		void removeLoggerKey(LoggerKey* key);
		//Method the logger uses to set the results array in the hash table
		void setResultsArray(unsigned char results __gc[]);

		unsigned char getCommands() __gc[] {return commands;}
		String* getLabels()[]{return labels;}

		void setLocked(bool l){locked = l;} 

		/*!
		When a rule is called, this hash table contains the results
		of a successful ECU send and recieve cycle.
		The keys/labels to each value in the hash table can be viewed by looking at the
		loggerCmd.ini file in the bre20.exe directory.
		*/
		Hashtable* getHashtable(){return hash;}
		ArrayList* getLoggerKeys(){if(!locked) return outLoggerKeys; else return NULL;}

		LoggerKey* getAfrKey(){return getLoggerKey(0);};
	};
}

/*
Maybe some day if its necessary, I could make the ini support
multiple byte commands and mutiple byte responses...

# When its used in the command list (the keys below) it 
# indicates a multiple byte command.
# BRE will assume that the right most byte is the most 
# significant. Any bumber of bytes can be concatinated 
# to indicate multiple byte commands.
#
# The colon after the name/label of the command tells BRE 
# how many bytes to expect and what to do with the results. 
# The commands recognized are:
#
# $#_byte	//# of bytes, keep the bytes separate. eg. $2_byte
# $#_combine	//# of bytes, combine the bytes into one number. eg. $2_combine
#
# If this is left out it is assumed that only one byte will 
# be returned.
#
# 0x21|0x22=Fuel:$2_byte IMPLEMENT LATER
*/
