#include "StdAfx.h"
#include "NGParam.h"
#include "addcode.h"


bre20::NGParam::NGParam():ParamPage()
{
	setupPage();
}
bre20::NGParam::NGParam(RomRep::HondaEcu* ecu): ParamPage(ecu)
{
	rom = ecu;
	init();
	setupPage();
	this->Refresh();
}

void bre20::NGParam::init()
{
	RomRep::Pm6Ng* ecu = static_cast<RomRep::Pm6Ng*>(rom);
	launchbox = new LabeledText("RPM","Change the Launch RPM",90,LabeledText::LTMODEINT);
	launchspeed = new LabeledText("Speed (KPH)","Change the Launch speed",90,LabeledText::LTMODEINT);
	revbox = new LabeledText("Warm Limit","Change the Revlimit",90,LabeledText::LTMODEINT);
	coldrevbox = new LabeledText("Cold Limit","Change the Cold Revlimit",90,LabeledText::LTMODEINT);

//	this->boostEnabled = new System::Windows::Forms::CheckBox();
	o2On = new System::Windows::Forms::RadioButton();
	o2OffBoost = new System::Windows::Forms::RadioButton();
	o2Off = new System::Windows::Forms::RadioButton();
	launchEnabled = new System::Windows::Forms::CheckBox();

	mapGrp = new GroupBox();
	discodesgroup = new GroupBox();
	discodesbox = new ComboBox();
	discodesadd = new Button();
	discodesrem = new Button();

	populateCodes();

	int limit = ecu->getRevlimit();
	int launch = ecu->getLaunch();
	revbox->setText(limit.ToString());
	launchbox->setText(launch.ToString());
	
	limit = ecu->getColdRevlimit();
	launch = ecu->getLaunchSpeed();
	coldrevbox->setText(limit.ToString());
	launchspeed->setText(launch.ToString());

	launchEnabled->Checked = ecu->isLaunchEnabled();

	int o2mode = ecu->getO2Mode();
	if(o2mode == RomRep::Pm6Ng::O2OFF)
		o2Off->Checked = true;
	else if(o2mode == RomRep::Pm6Ng::O2OFFBOOST)
		o2OffBoost->Checked = true;
	else
		o2On->Checked = true;

	txtVtEngage = new System::Windows::Forms::TextBox();
	txtMinTps = new System::Windows::Forms::TextBox();
	txtVtDis = new System::Windows::Forms::TextBox();
	chkVtecEnable = new System::Windows::Forms::CheckBox();

	txtVtEngage->Text = ecu->getVtecEngageRpm().ToString();
	txtVtDis->Text = ecu->getVtecDisengageRpm().ToString();
	txtMinTps->Text = ecu->getVtecTps().ToString();
	chkVtecEnable->Checked = ecu->isVtecEnabled();
}

void bre20::NGParam::populateCodes()
{
	RomRep::Pm6Ng* ecu = static_cast<RomRep::Pm6Ng*>(rom);
	ArrayList* list = ecu->getDisabledCodes();
	list->Sort();

	discodesbox->Items->Clear();
	for(int i = 0; i< list->Count; i++)
		discodesbox->Items->Add( list->get_Item(i) );

	if(list->Count >0)
		discodesbox->SelectedIndex = 0;
}

void bre20::NGParam::addExtraFeatures()
{
	this->Controls->Add(grpVtec);
	this->Controls->Add(discodesgroup);
	this->Controls->Add(mapGrp);
}

void bre20::NGParam::setupPage()
{
	grpVtec = new System::Windows::Forms::GroupBox();
	labVtEn = new System::Windows::Forms::Label();
	labMinTps = new System::Windows::Forms::Label();
	labVtDis = new System::Windows::Forms::Label();
	labPerc = new System::Windows::Forms::Label();

	namelabel = new Label();
	this->revGrp = new System::Windows::Forms::GroupBox();
	this->launchGrp = new System::Windows::Forms::GroupBox();
	this->genGrp = new System::Windows::Forms::GroupBox();

	this->revGrp->SuspendLayout();
	this->launchGrp->SuspendLayout();
	this->genGrp->SuspendLayout();
	this->discodesgroup->SuspendLayout();
	this->grpVtec->SuspendLayout();

	addExtraFeatures();
	this->Controls->Add(this->revGrp);
	this->Controls->Add(this->launchGrp);
	this->Controls->Add(this->genGrp);

	this->Text = S"NG Parameters";
	// 
	// revGrp
	// 
	this->revGrp->Controls->Add(this->revbox);
	this->revGrp->Controls->Add(this->coldrevbox);
	this->revGrp->Location = System::Drawing::Point(304, 8);
	this->revGrp->Name = S"revGrp";
	this->revGrp->Size = System::Drawing::Size(136, 144);
	this->revGrp->TabIndex = 0;
	this->revGrp->TabStop = false;
	this->revGrp->Text = S"Revlimits";
	this->revGrp->FlatStyle = FlatStyle::System;
	// 
	// launchGrp
	// 
	this->launchGrp->Controls->Add(this->launchspeed);
	this->launchGrp->Controls->Add(this->launchbox);
	this->launchGrp->Controls->Add(this->launchEnabled);
	this->launchGrp->Location = System::Drawing::Point(160, 8);
	this->launchGrp->Name = S"launchGrp";
	this->launchGrp->Size = System::Drawing::Size(136, 144);
	this->launchGrp->TabIndex = 1;
	this->launchGrp->TabStop = false;
	this->launchGrp->Text = S"Launch";
	this->launchGrp->FlatStyle = FlatStyle::System;
	// 
	// genGrp
	// 
	this->genGrp->Controls->Add(this->o2Off);
	this->genGrp->Controls->Add(this->o2OffBoost);
	this->genGrp->Controls->Add(this->o2On);
//	this->genGrp->Controls->Add(this->boostEnabled);
	this->genGrp->Location = System::Drawing::Point(16, 8);
	this->genGrp->Name = S"genGrp";
	this->genGrp->Size = System::Drawing::Size(136, 144);
	this->genGrp->TabIndex = 2;
	this->genGrp->TabStop = false;
	this->genGrp->Text = S"General";
	this->genGrp->FlatStyle = FlatStyle::System;
	// 
	// boostEnabled
	// 
//	this->boostEnabled->Location = System::Drawing::Point(16, 24);
//	this->boostEnabled->Name = S"boostEnabled";
//	this->boostEnabled->Size = System::Drawing::Size(104, 16);
//	this->boostEnabled->TabIndex = 0;
//	this->boostEnabled->Text = S"Boost Enabled";
//	this->boostEnabled->CheckedChanged += new System::EventHandler(this, boostEnabled_CheckedChanged);
	//
	// o2Off
	// 
	this->o2Off->Location = System::Drawing::Point(16, 88);
	this->o2Off->Name = S"o2Off";
	this->o2Off->Size = System::Drawing::Size(72, 16);
	this->o2Off->TabIndex = 3;
	this->o2Off->Text = S"O2 OFF";
	this->o2Off->CheckedChanged += new System::EventHandler(this, o2Off_CheckedChanged);
	this->o2Off->FlatStyle = FlatStyle::System;
	// 
	// o2OffBoost
	// 
	this->o2OffBoost->Location = System::Drawing::Point(16, 64);
	this->o2OffBoost->Name = S"o2OffBoost";
	this->o2OffBoost->Size = System::Drawing::Size(112, 16);
	this->o2OffBoost->TabIndex = 2;
	this->o2OffBoost->Text = S"O2 OFF in boost";
	this->o2OffBoost->FlatStyle = FlatStyle::System;
	this->o2OffBoost->CheckedChanged += new System::EventHandler(this, o2OffBoost_CheckedChanged);
	// 
	// o2On
	// 
	this->o2On->Location = System::Drawing::Point(16, 40);
	this->o2On->Name = S"o2On";
	this->o2On->Size = System::Drawing::Size(72, 16);
	this->o2On->TabIndex = 1;
	this->o2On->Text = S"O2 ON";
	this->o2On->FlatStyle = FlatStyle::System;
	this->o2On->CheckedChanged += new System::EventHandler(this, o2On_CheckedChanged);
	// 
	// launchEnabled
	// 
	this->launchEnabled->Location = System::Drawing::Point(16, 24);
	this->launchEnabled->Name = S"launchEnabled";
	this->launchEnabled->Size = System::Drawing::Size(80, 16);
	this->launchEnabled->TabIndex = 0;
	this->launchEnabled->Text = S"Enabled";
	this->launchEnabled->FlatStyle = FlatStyle::System;
	this->launchEnabled->CheckedChanged += new System::EventHandler(this, launchEnabled_CheckedChanged);
	// 
	// revbox
	// 
	this->revbox->Location = System::Drawing::Point(20, 24);
	this->revbox->Name = S"revbox";
	this->revbox->add_OnTextChanged(new TextChangedEventHandler(this,revbox_Changed));
	// 
	// coldrevbox
	// 
	this->coldrevbox->Location = System::Drawing::Point(20, 88);
	this->coldrevbox->Name = S"coldrevbox";
	this->coldrevbox->add_OnTextChanged(new TextChangedEventHandler(this,coldrevbox_Changed));
	// 
	// launchbox
	// 
	this->launchbox->Location = System::Drawing::Point(20, 48);
	this->launchbox->Name = S"launchbox";
	this->launchbox->add_OnTextChanged(new TextChangedEventHandler(this,launchbox_Changed));
	// 
	// launchspeed
	// 
	this->launchspeed->Location = System::Drawing::Point(20, 96);
	this->launchspeed->Name = S"launchspeed";
	this->launchspeed->add_OnTextChanged(new TextChangedEventHandler(this,launchspeed_Changed));
	// 
	// label3
	// 
	this->namelabel->Location = System::Drawing::Point(376, 8);
	this->namelabel->Name = S"namelabel";
	this->namelabel->Size = System::Drawing::Size(136, 16);
	this->namelabel->TabIndex = 4;
	this->namelabel->TextAlign = System::Drawing::ContentAlignment::MiddleRight;
	this->namelabel->Text = rom->getType();

	// 
	// vtecGroup
	// 
	this->discodesgroup->Controls->Add(this->discodesbox);
	this->discodesgroup->Controls->Add(this->discodesadd);
	this->discodesgroup->Controls->Add(this->discodesrem);
	this->discodesgroup->Location = System::Drawing::Point(16, 164);
	this->discodesgroup->Name = S"discodesgroup";
	this->discodesgroup->Size = System::Drawing::Size(136, 70);
	this->discodesgroup->TabIndex = 3;
	this->discodesgroup->TabStop = false;
	this->discodesgroup->Text = S"Disable CEL Codes";
	this->discodesgroup->FlatStyle = FlatStyle::System;

	this->revGrp->ResumeLayout(false);
	this->launchGrp->ResumeLayout(false);
	this->genGrp->ResumeLayout(false);
	this->discodesgroup->ResumeLayout(false);

	// 
	// combobox
	// 
	this->discodesbox->Location = System::Drawing::Point(8, 30);
	this->discodesbox->Name = S"discodesbox";
	this->discodesbox->Size = System::Drawing::Size(55, 16);
	this->discodesbox->TabIndex = 1;
	this->discodesbox->Text = S"";
	this->discodesbox->DropDownStyle = ComboBoxStyle::DropDownList;

	
		// 
	// add button
	// 
	this->discodesadd->Location = System::Drawing::Point(69, 16);
	this->discodesadd->Name = S"discodesadd";
	this->discodesadd->Size = System::Drawing::Size(58, 20);
	this->discodesadd->TabIndex = 2;
	this->discodesadd->Text = S"Add...";
	this->discodesadd->FlatStyle = FlatStyle::System;
	discodesadd->add_Click(new System::EventHandler(this, discodeadd_Click));


		// 
	// remove button
	// 
	this->discodesrem->Location = System::Drawing::Point(69, 42);
	this->discodesrem->Name = S"discodesrem";
	this->discodesrem->Size = System::Drawing::Size(58, 20);
	this->discodesrem->TabIndex = 3;
	this->discodesrem->Text = S"Remove";
	this->discodesrem->FlatStyle = FlatStyle::System;
	discodesrem->add_Click(new System::EventHandler(this, discoderemove_Click));

	mapsensor = new ComboBox();
	maplabel = new Label();
	this->mapGrp->Controls->Add(mapsensor);
	this->mapGrp->Controls->Add(maplabel);

	this->mapGrp->Location = System::Drawing::Point(160, 164);
	this->mapGrp->Name = S"mapGrp";
	this->mapGrp->Size = System::Drawing::Size(136, 70);
	this->mapGrp->Text = S"Map Sensor";
	this->mapGrp->FlatStyle = FlatStyle::System;

	// 
	// label
	// 
	this->maplabel->Location = System::Drawing::Point(16, 20);
	this->maplabel->Name = S"maplabel";
	this->maplabel->Size = System::Drawing::Size(106, 16);
//	this->maplabel->TabIndex = 7;
	this->maplabel->Text = S"Map Sensor Type:";
	// 
	// mapsensor box
	// 
	this->mapsensor->Location = System::Drawing::Point(16, 40);
	this->mapsensor->Name = S"mapsensor";
	this->mapsensor->Size = System::Drawing::Size(106, 20);
	this->mapsensor->DropDownStyle = ComboBoxStyle::DropDownList;
//	this->mapsensor->TabIndex = 2;
	/*
	MAPSTOCK = 0;
	MAP2BAR = 1;
	MAP25BAR = 2;
	MAP3BAR = 3;
	MAPMOTOROLA25BAR = 4;
	*/
	this->mapsensor->Items->Add(S"Stock");
	this->mapsensor->Items->Add(S"GM 2 Bar");
	this->mapsensor->Items->Add(S"2.5 Bar");
	this->mapsensor->Items->Add(S"GM 3 Bar");
	this->mapsensor->Items->Add(S"Motorola 2.5 Bar");

	mapsensor->SelectedIndex = static_cast<RomRep::Pm6Ng*>(rom)->getMapSensor();

	this->mapsensor->add_SelectedIndexChanged(new EventHandler(this,combo_SelectedIndexChanged));


	// 
	// grpVtec
	// 
	grpVtec->Controls->Add(labPerc);
	grpVtec->Controls->Add(chkVtecEnable);
	grpVtec->Controls->Add(labVtDis);
	grpVtec->Controls->Add(txtVtDis);
	grpVtec->Controls->Add(labMinTps);
	grpVtec->Controls->Add(labVtEn);
	grpVtec->Controls->Add(txtMinTps);
	grpVtec->Controls->Add(txtVtEngage);
	grpVtec->FlatStyle = System::Windows::Forms::FlatStyle::System;
	grpVtec->Location = System::Drawing::Point(448, 8);
	grpVtec->Name = "grpVtec";
	grpVtec->Size = System::Drawing::Size(152, 144);
	grpVtec->TabIndex = 1;
	grpVtec->TabStop = false;
	grpVtec->Text = "VTEC";
	// 
	// txtVtEngage
	// 
	txtVtEngage->Location = System::Drawing::Point(80, 80);
	txtVtEngage->Name = "txtVtEngage";
	txtVtEngage->Size = System::Drawing::Size(48, 20);
	txtVtEngage->TabIndex = 0;
	txtVtEngage->KeyPress += new System::Windows::Forms::KeyPressEventHandler(this,intBox_KeyPress);
	txtVtEngage->KeyPress += new System::Windows::Forms::KeyPressEventHandler(this, txtVtEngage_KeyPress);
	txtVtEngage->Leave += new System::EventHandler(this,txtVtEngage_Leave);
	// 
	// txtMinTps
	// 
	txtMinTps->Location = System::Drawing::Point(80, 48);
	txtMinTps->Name = "txtMinTps";
	txtMinTps->Size = System::Drawing::Size(48, 20);
	txtMinTps->TabIndex = 1;
	txtMinTps->KeyPress += new System::Windows::Forms::KeyPressEventHandler(this,intBox_KeyPress);
	txtMinTps->KeyPress += new System::Windows::Forms::KeyPressEventHandler(this, txtMinTps_KeyPress);
	txtMinTps->Leave += new System::EventHandler(this,txtMinTps_Leave);
	// 
	// labVtEn
	// 
	labVtEn->FlatStyle = System::Windows::Forms::FlatStyle::System;
	labVtEn->Location = System::Drawing::Point(16, 80);
	labVtEn->Name = "labVtEn";
	labVtEn->Size = System::Drawing::Size(48, 16);
	labVtEn->TabIndex = 3;
	labVtEn->Text = "Engage:";
	labVtEn->TextAlign = System::Drawing::ContentAlignment::MiddleLeft;
	// 
	// labMinTps
	// 
	labMinTps->FlatStyle = System::Windows::Forms::FlatStyle::System;
	labMinTps->Location = System::Drawing::Point(16, 48);
	labMinTps->Name = "labMinTps";
	labMinTps->Size = System::Drawing::Size(56, 16);
	labMinTps->TabIndex = 4;
	labMinTps->Text = "Min TPS:";
	labMinTps->TextAlign = System::Drawing::ContentAlignment::MiddleLeft;
	// 
	// labVtDis
	// 
	labVtDis->FlatStyle = System::Windows::Forms::FlatStyle::System;
	labVtDis->Location = System::Drawing::Point(16, 112);
	labVtDis->Name = "labVtDis";
	labVtDis->Size = System::Drawing::Size(64, 16);
	labVtDis->TabIndex = 7;
	labVtDis->Text = "Disengage:";
	labVtDis->TextAlign = System::Drawing::ContentAlignment::MiddleLeft;
	// 
	// txtVtDis
	// 
	txtVtDis->Location = System::Drawing::Point(80, 112);
	txtVtDis->Name = "txtVtDis";
	txtVtDis->Size = System::Drawing::Size(48, 20);
	txtVtDis->TabIndex = 6;
	txtVtDis->KeyPress += new System::Windows::Forms::KeyPressEventHandler(this,intBox_KeyPress);
	txtVtDis->KeyPress += new System::Windows::Forms::KeyPressEventHandler(this, txtVtDis_KeyPress);
	txtVtDis->Leave += new System::EventHandler(this,txtVtDis_Leave);
	// 
	// chkVtecEnable
	// 
	chkVtecEnable->FlatStyle = System::Windows::Forms::FlatStyle::System;
	chkVtecEnable->Location = System::Drawing::Point(16, 24);
	chkVtecEnable->Name = "chkVtecEnable";
	chkVtecEnable->Size = System::Drawing::Size(104, 16);
	chkVtecEnable->TabIndex = 8;
	chkVtecEnable->Text = "Enabled";
	chkVtecEnable->CheckedChanged += new System::EventHandler(this,chkVtecEnable_CheckedChanged);

	// 
	// labPerc
	// 
	labPerc->FlatStyle = FlatStyle::System;
	labPerc->Location = System::Drawing::Point(128, 48);
	labPerc->Name = "labPerc";
	labPerc->Size = System::Drawing::Size(16, 16);
	labPerc->TabIndex = 9;
	labPerc->Text = "%";
	labPerc->TextAlign = System::Drawing::ContentAlignment::BottomLeft;
	
}


//events...
void bre20::NGParam::txtVtEngage_Leave(System::Object *  sender, System::EventArgs *  e){
	RomRep::Pm6Ng* ecu = (static_cast<RomRep::Pm6Ng*>(rom));
	setBoxVal(txtVtEngage, 
		new bre20::intGetFunction(ecu,&RomRep::Pm6Ng::getVtecEngageRpm), 
		new bre20::intSetFunction(ecu,&RomRep::Pm6Ng::setVtecEngageRpm),
		1000, 12000);
}
void bre20::NGParam::txtMinTps_Leave(System::Object *  sender, System::EventArgs *  e){
	RomRep::Pm6Ng* ecu = (static_cast<RomRep::Pm6Ng*>(rom));
	setBoxVal(txtMinTps, 
		new bre20::intGetFunction(ecu,&RomRep::Pm6Ng::getVtecTps), 
		new bre20::intSetFunction(ecu,&RomRep::Pm6Ng::setVtecTps),
		0, 100);
}
void bre20::NGParam::txtVtDis_Leave(System::Object *  sender, System::EventArgs *  e){
	RomRep::Pm6Ng* ecu = (static_cast<RomRep::Pm6Ng*>(rom));
	setBoxVal(txtVtDis, 
		new bre20::intGetFunction(ecu,&RomRep::Pm6Ng::getVtecDisengageRpm), 
		new bre20::intSetFunction(ecu,&RomRep::Pm6Ng::setVtecDisengageRpm),
		1000, 12000);
}
System::Void bre20::NGParam::coldrevbox_Changed(System::Object *  sender, System::EventArgs *  e){
	RomRep::Pm6Ng* ecu = (static_cast<RomRep::Pm6Ng*>(rom));
	setBoxVal(coldrevbox, 
		new bre20::intGetFunction(ecu,&RomRep::Pm6Ng::getColdRevlimit), 
		new bre20::intSetFunction(ecu,&RomRep::Pm6Ng::setColdRevlimit),
		1000, 12000);
}
System::Void bre20::NGParam::launchspeed_Changed(System::Object *  sender, System::EventArgs *  e)
{
	RomRep::Pm6Ng* ecu = (static_cast<RomRep::Pm6Ng*>(rom));
	setBoxVal(launchspeed, 
		new bre20::intGetFunction(ecu,&RomRep::Pm6Ng::getLaunchSpeed), 
		new bre20::intSetFunction(ecu,&RomRep::Pm6Ng::setLaunchSpeed),
		0, 260);
}
System::Void bre20::NGParam::revbox_Changed(System::Object *  sender, System::EventArgs *  e){
	RomRep::Pm6Ng* ecu = (static_cast<RomRep::Pm6Ng*>(rom));
	setBoxVal(revbox, 
		new bre20::intGetFunction(ecu,&RomRep::Pm6Ng::getRevlimit), 
		new bre20::intSetFunction(ecu,&RomRep::Pm6Ng::setRevlimit),
		1000, 12000);
}
System::Void bre20::NGParam::launchbox_Changed(System::Object *  sender, System::EventArgs *  e)
{
	RomRep::Pm6Ng* ecu = (static_cast<RomRep::Pm6Ng*>(rom));
	setBoxVal(launchbox, 
		new bre20::intGetFunction(ecu,&RomRep::Pm6Ng::getLaunch), 
		new bre20::intSetFunction(ecu,&RomRep::Pm6Ng::setLaunch),
		1000, 12000);
}

System::Void bre20::NGParam::boostEnabled_CheckedChanged(System::Object *  sender, System::EventArgs *  e){}
System::Void bre20::NGParam::o2On_CheckedChanged(System::Object *  sender, System::EventArgs *  e){
	if(o2On->Checked)
		(static_cast<RomRep::Pm6Ng*>(rom))->setO2Mode(RomRep::Pm6Ng::O2ON);
}
System::Void bre20::NGParam::o2OffBoost_CheckedChanged(System::Object *  sender, System::EventArgs *  e){
	if(o2OffBoost->Checked)
		(static_cast<RomRep::Pm6Ng*>(rom))->setO2Mode(RomRep::Pm6Ng::O2OFFBOOST);
}
System::Void bre20::NGParam::o2Off_CheckedChanged(System::Object *  sender, System::EventArgs *  e){
	if(o2Off->Checked)
		(static_cast<RomRep::Pm6Ng*>(rom))->setO2Mode(RomRep::Pm6Ng::O2OFF);
}
System::Void bre20::NGParam::launchEnabled_CheckedChanged(System::Object *  sender, System::EventArgs *  e){
	(static_cast<RomRep::Pm6Ng*>(rom))->setLaunchEnabled(launchEnabled->Checked);
}


System::Void bre20::NGParam::discodeadd_Click(System::Object *  sender, System::EventArgs *  e)
{
	AddCode* ac = new AddCode();
	RomRep::Pm6Ng* vte = (static_cast<RomRep::Pm6Ng*>(rom));

	DialogResult res = ac->ShowDialog();
	int code = ac->getCode();
	if(res == DialogResult::OK && code > 0 && code <= 24)
	{
		bool good = true;
		for(int i = 0; i< discodesbox->Items->Count; i++)
		{
			String* str = static_cast<String*>(discodesbox->Items->get_Item(i));
			good = good && Convert::ToInt32(str) != code;
		}
		//tell me if its able to disable the bit and it is NOT disabled already then we add it.
		if(good && vte->toggleCodeDisable(code,true)){
			discodesbox->Items->Add(code.ToString());
			MessageBox::Show("CEL code disabled successfully", "BRE");
		}
		else
			MessageBox::Show("An Error occurred while trying to disable the CEL code", "BRE");
	}
	else if(code < 1 || code > 24)
		MessageBox::Show("You entered an invalid number", "BRE");

	if(discodesbox->Items->Count>0)
		discodesbox->SelectedIndex = 0;
}
System::Void bre20::NGParam::discoderemove_Click(System::Object *  sender, System::EventArgs *  e)
{
	int sel = discodesbox->SelectedIndex;
	String* str = static_cast<String*>(discodesbox->Items->Item[sel]);
	RomRep::Pm6Ng* vte = (static_cast<RomRep::Pm6Ng*>(rom));
	int code = Convert::ToInt32(str);

	if(vte->toggleCodeDisable(code,false)){
		discodesbox->Items->RemoveAt(sel);
		MessageBox::Show("CEL code enabled successfully", "BRE");
	}
	else
		MessageBox::Show("An Error occurred while trying to enable the CEL code", "BRE");

	if(discodesbox->Items->Count>0)
		discodesbox->SelectedIndex = 0;
}

void bre20::NGParam::combo_SelectedIndexChanged(Object * sender, EventArgs* args)
{
	RomRep::Pm6Ng* ecu = static_cast<RomRep::Pm6Ng*>(rom);

	if(!ecu->setMapSensor(mapsensor->SelectedIndex)){
		MessageBox::Show("That sensor is not supported!","BRE");
		mapsensor->SelectedIndex = ecu->getMapSensor();
	}
}