

#pragma once
using namespace System;

#include "HondaEcu.h"
#include "RpmValue.h"
#include "Obd0VTECMaps.h"
#include "FuelMap.h"

namespace RomRep
{

	///
	///Base class for all OBD0 VTEC ECUs
	///
	///
	public __gc class VT0Ecu: public HondaEcu
	{
	public:
		static int revconst = 1852000;
	protected:
		RpmTwoByteValue* revlimit;
		RpmTwoByteValue* revrestart;
		RpmByteValue* vtec[];
		int numvtec;
		int celvecloc;

		///overridden to properly
		///create a fuel map.
		///
		///If mapname == null, this function will give the map a name.
		///
		///colscalarloc parameter is ignored.\n
		///vtec parameter ignored if index <= 1 
		virtual FuelMap* createFuelMap(int xx, int yy, int index, String* mapname,
			RomRep::MapLocs* maploc,unsigned char map __gc[,],unsigned char mlt __gc[], bool vtec);

		///overridden to properly
		///create an ignition map.
		///
		///If mapname == null, this function will give the map a name.
		///
		///colscalarloc parameter is ignored.\n
		///vtec parameter ignored if index <= 1
		virtual Map* createIgnitionMap(int xx, int yy, int index, String* mapname,
			RomRep::MapLocs* maploc,unsigned char map __gc[,],bool vtec);

		//these return ints because bools didnt work for some reason.
		///\cond TEST

		virtual void aDisableOxygenSensors(int loc, bool disable);
		virtual void aDisable2ndOxygenSensor(int loc, bool disable);

		virtual void aDisableChecksum(int loc, bool disable); 
		virtual int aisChecksumDisabled(int loc);
		virtual int aisOxygenSensorsDisabled(int loc);
		virtual int ais2ndOxygenSensorDisabled(int loc);
		virtual int aisKnockSensorDisabled(int loc);
		virtual void adisableKnockSensor(int loc1,int loc2, int loc3, bool disable);
		virtual void asetLaunchRpm(int loc, int val);
		virtual void asetLaunchSpeed(int loc, int val);
		virtual void adisableCelCheck(int loc, bool disable);

		virtual int aisSpeedLimiterDisabled(int loc);
		virtual int aisVssCheckDisabled(int loc);
		virtual int aisTempCheckDisabled(int loc);
		virtual int aisCelCheckDisabled(int loc);
		virtual int aisPsCheckDisabled(int loc);
		virtual int aisLaunchDisabled(int loc);

		virtual int agetLaunchRpm(int loc);
		virtual int agetLaunchSpeed(int loc);

		virtual void aDisableCodes(int loc, bool disable);
		virtual int aisCodesDisabled(int loc);

		///\endcond


	public:
		VT0Ecu();

		//the file, location, location, location of first one, number of vtec values.
		VT0Ecu(unsigned char rom __gc[], int flen, int revloc, int restartloc, int vtecloc, int vtecnum, int ignmapad1, int ignmapad2, int fuelmapad1, int fuelmapad2, String *  type, int width);

		//the file, location, location, location of first one, number of vtec values.
		VT0Ecu(unsigned char rom __gc[], int flen, int revloc, int restartloc, int vtecloc, int vtecnum, int ignmapad1, int ignmapad2, int fuelmapad1, int fuelmapad2, int revscalars __gc[], String *  type, int width);

		virtual void setup(unsigned char rom __gc[], int flen, int revloc, int restartloc, int vtecloc, int vtecnum, int ignmapad1, int ignmapad2, int fuelmapad1, int fuelmapad2, int revscalars __gc[], String *  type, int width);

		virtual ~VT0Ecu();

		virtual void setTwoByteRpm(int highloc, int lowloc, int rpm);
		virtual int getTwoByteRpm(int highloc, int lowloc); //returns rpm

		//for RTP thes return bytechange ptr
		//come in and leave as real world values
		virtual void setVtecEngage(int loc, int val);
		virtual void setVtecDisengage(int loc, int val);
		virtual void setLaunchRpm(int val) = 0;
		virtual void setRevlimit(int val);
		virtual void setRevRestart(int val);
		virtual void setRpm(RpmTwoByteValue * stor, int val);
		virtual void disableOxygenSensors(bool disable) = 0;
		virtual void disable2ndOxygenSensor(bool disable) = 0;
		virtual void setLaunchSpeed(int val) = 0;
		virtual void enableLaunch(bool enable) = 0;
		virtual void disableChecksum(bool disable) = 0; 
		virtual void disableSpeedLimiter(bool disable) = 0; 
		virtual void disableVssCheck(bool disable) = 0; 
		virtual void disableTempCheck(bool disable) = 0; 
		virtual void disablePsCheck(bool disable) = 0; 
		virtual void disableCelCheck(bool disable) = 0; 
		virtual void disableKnockSensor(bool disable) = 0;
		virtual void disableCodes(bool disable) = 0;

		///\returns rpm
		virtual int getVtecEngage(int loc);
		///\returns rpm
		virtual int getVtecDisengage(int loc);
		///\returns rpm
		virtual int getRevlimit();
		///\returns rpm
		virtual int getRevRestart();
		///\returns rpm
		virtual int getLaunchRpm() = 0;
		///\returns speed in KPH
		virtual int getLaunchSpeed() = 0;
		///\returns the number of vtec values in the rom
		virtual int getNumVtec(){return numvtec;};

		//bools dont propigate well, so 1==true 0==false

		///\returns 1 for true, 0 for false (issue with managed + unmanaged code and bools)
		virtual int isKnockSensorDisabled() = 0;
		///\returns 1 for true, 0 for false (issue with managed + unmanaged code and bools)
		virtual int isChecksumDisabled() = 0;
		///\returns 1 for true, 0 for false (issue with managed + unmanaged code and bools)
		virtual int isOxygenSensorsDisabled() = 0;
		///\returns 1 for true, 0 for false (issue with managed + unmanaged code and bools)
		virtual int is2ndOxygenSensorDisabled() = 0;
		///\returns 1 for true, 0 for false (issue with managed + unmanaged code and bools)
		virtual int isSpeedLimiterDisabled() = 0;
		///\returns 1 for true, 0 for false (issue with managed + unmanaged code and bools)
		virtual int isVssCheckDisabled() = 0;
		///\returns 1 for true, 0 for false (issue with managed + unmanaged code and bools)
		virtual int isTempCheckDisabled() = 0;
		///\returns 1 for true, 0 for false (issue with managed + unmanaged code and bools)
		virtual int isCelCheckDisabled() = 0;
		///\returns 1 for true, 0 for false (issue with managed + unmanaged code and bools)
		virtual int isPsCheckDisabled() = 0;
		///\returns 1 for true, 0 for false (issue with managed + unmanaged code and bools)
		virtual int isLaunchDisabled() = 0;

		///\returns value in the rpm scalar
		virtual int getRpmScalarValue(int row, bool fuel, bool vtec); 

		///\returns 1 if the code for vector disables is in the rom
		virtual int isCodesDisabled() = 0;

		///\returns negative for fail, 1 for success
		virtual int addCelDisable(int code); //return negative for fail, 1 for success
//		virtual int* getCelDisableVec();

		///\returns negative for fail, 1 for success
		virtual int removeCelDisable(int code); //return negative for fail, 1 for success

		///\returns the location of the disabled vector
		virtual int getCelVectorLocation()
		{return celvecloc;};

		///\returns is the input code a bit disable? If not then its a vector disable
		virtual int isCodeBitDisable(int code) = 0;

		///disable the damn code via bit disable. true disables the code
		virtual int codeBitDisable(int code, bool disable) = 0;

		///is this code bit disabled? 
		///\returns 1 if yes, 0 if no, -1 if code is vec disable 
		virtual int isCodeBitDisabled(int code) = 0;


		/////\returns float array with target lambda
		//virtual float getLambdaMap(int index) __gc[,];
		/////sets the target afr at (x,y) in the map (index)
		/////\returns byte change object
		//virtual ByteChange* setAfrCellEx(int x, int y, float val, int index);
		/////\returns the target AFR at (x,y) in map (index)
		//virtual float getAfrCell(int x, int y, int index);

		static int getRevConstant(){return revconst;}

		virtual float calcTPS(int raw);
		///\returns raw value
		virtual int calcTPS(float percentage);

		///\param raw the raw value
		///\param sae true if sae, false if metric
		///\returns the real temp.  
		virtual float calcTemp(int raw, bool sae);
		///\param temp the real world temp value
		///\param sae true if sae, false if metric
		///\returns the raw temp.  
		virtual int calcTemp(float temp, bool sae);

		///Calculates the RPM. If there are more than one format 
		///of rpm value, assume it is the most common format (i.e.
		///the 2 byte value)
		virtual float calcRpm(int raw);
		///Calculates the raw value fromf RPM. If there are more than one format 
		///of rpm value, assume it is the most common format (i.e.
		///the 2 byte value)
		virtual int calcRpm(float rpm);

	protected:
			///Sets the target afr
		virtual float calculateAfr(unsigned char raw)
		{
			return ((float)raw/25.0F)+10.0F;
		}
		virtual unsigned char calculateAfr(float afr)
		{
			return (unsigned char)((afr-10.0F)*25.0F);
		}

	};
}