#include "stdafx.h"
#include "Obd0VtecMaps.h"
#include "obd0vtececu.h"

RomRep::VT0IgnMap::VT0IgnMap(int x1, int y1, unsigned int loc, unsigned char themap __gc[,], 
							 unsigned char fil __gc[], int cscalarloc, int rscalarloc, int ind, 
							 int mod):Obd0IgnMap(x1,y1,loc,themap,fil,ind,mod)
{
	setup(true, themap, fil, cscalarloc, rscalarloc, ind);
}

RomRep::VT0IgnMap::VT0IgnMap(bool calchead, int x1, int y1, unsigned int loc, unsigned char themap __gc[,], 
			unsigned char fil __gc[], int cscalarloc, int rscalarloc, int ind, 
			int mod):Obd0IgnMap(x1,y1,loc,themap,fil,ind,mod)
{
	setup(calchead, themap, fil, cscalarloc, rscalarloc, ind);
}
void RomRep::VT0IgnMap::setup(bool calchead, unsigned char themap __gc[,], 
			unsigned char fil __gc[], int cscalarloc, int rscalarloc, int ind)
{
	populate(themap);

	this->vtec = false;
	if( ind > 0 )
		this->vtec = true;

	this->revscalarloc = rscalarloc;
	this->pressscalarloc = cscalarloc;
	if(calchead){
		RomRep::VT0MapHelper::setRpmScalars(rscalarloc,rpm,fil,vtec);
		RomRep::VT0MapHelper::setPressScalars(cscalarloc,vac,fil);
	}
}

void RomRep::VT0IgnMap::recalcRpmScalars()
{
	RomRep::VT0MapHelper::setRpmScalars(this->revscalarloc,rpm,this->thefile,this->vtec);
}

////*********************************************
////*********************************************
////*********************************************
////*********************************************

RomRep::VT0FuelMap::VT0FuelMap(int x1, int y1,unsigned int loc, unsigned char themap __gc[,], 
							   unsigned int multloc1, unsigned char mltarray __gc[], 
							   unsigned char fil __gc[], int cscalarloc, int rscalarloc, 
							   int ind, int mod):Obd0FuelMap(x1,y1,loc,themap,multloc1,mltarray,fil,ind,mod)
{
	setup(true, themap, mltarray, fil, cscalarloc, rscalarloc, ind);
}
RomRep::VT0FuelMap::VT0FuelMap(bool calchead, int x1, int y1,unsigned int loc, unsigned char themap __gc[,], 
							   unsigned int multloc1, unsigned char mltarray __gc[], 
							   unsigned char fil __gc[], int cscalarloc, int rscalarloc, 
							   int ind, int mod):Obd0FuelMap(x1,y1,loc,themap,multloc1,mltarray,fil,ind,mod)
{
	setup(calchead, themap, mltarray, fil, cscalarloc, rscalarloc, ind);
}
void RomRep::VT0FuelMap::setup(bool calchead, unsigned char themap __gc[,], 
							   unsigned char mltarray __gc[], unsigned char fil __gc[], 
							   int cscalarloc, int rscalarloc, int ind)
{
	this->populate(themap,mltarray);

	this->vtec = false;
	if( ind > 0 )
		this->vtec = true;
	revscalarloc = rscalarloc;
	pressscalarloc = cscalarloc;

	if(calchead){
		RomRep::VT0MapHelper::setRpmScalars(rscalarloc,rpm,fil,vtec);
		RomRep::VT0MapHelper::setPressScalars(cscalarloc,vac,fil);
	}
}

void RomRep::VT0FuelMap::recalcRpmScalars()
{
	RomRep::VT0MapHelper::setRpmScalars(this->revscalarloc,rpm,this->thefile,this->vtec);
}

float RomRep::VT0FuelMap::calculateInjMs(int value)
{
	/*PW0=PR3 : Avance = ValRom / 4
    : ValInj = ( ValRom * CoefRom ) / 4
    
    I think its 
    msInj = ( ValInj*4 - 1ah ) /246.875
	*/
	
	//obd1-like values...
	//return (float)(value/4.0f);

	float temp = (float)( value /*- 0x1a*/ ) / 246.875F;
	if(temp < 0.0f) temp = 0.0f;

	return temp;
}

int RomRep::VT0FuelMap::calculateInjByte(float injMs, int m)
{
	//obd1-like values...
	//return (int)(injMs*4.0f/(float)m);

	if(injMs < 0.0f) injMs = 0.0f;
	int temp = (int)(((injMs * 246.875F) /*+ 26.0F*/))/m;
	return temp;
}

float RomRep::VT0FuelMap::calculate(int x1, int y1, int intval)
{
	int m = (int)pow(2,(int)mult[x1]);
	return calculateInjMs(intval * m);
}

int RomRep::VT0FuelMap::calculate(int x1, int y1, float mapval)
{
	int m = (int)pow(2,(int)mult[x1]);
	return calculateInjByte(mapval,m);
}

//***************************
//***************************
//** helper class
String *  RomRep::VT0MapHelper::calcNonVtecRow(int sum)
{
	int er0, er3;

	if(sum >= 0xcf)
	{
		er0 = 0x8000;
		er3 = 0x80;
	}
	else if(sum >= 0x8f)
	{
		er0 = 0x10000;
		er3 = 0x40;
	}
	else if(sum >= 0x4f)
	{
		er0 = 0x20000;
		er3 = 0;
	}
	else
	{
		er0 = 0x40000;
		er3 = -64;
	}
					
	return (((sum - er3)*VT0Ecu::revconst)/er0).ToString();
}

String * RomRep::VT0MapHelper::calcVtecRow(int sum)
{
	int rpm = 0;
	if(sum>0)
	{
		rpm = 0xd000/sum;
		rpm = VT0Ecu::revconst/rpm;
	}
	return rpm.ToString();
}

void RomRep::VT0MapHelper::setRpmScalars(int rpmscalarloc, String* header[], unsigned char thefile __gc[], bool vtec)
{
	int sum = 256;
	int yy = header->Length;

	if(vtec)
	{	
		header[yy-1] = VT0MapHelper::calcVtecRow(sum);

		//row headers...
		for(int i = yy-2; i>=0; i--)
		{
			//vtec headers
			int sptr = rpmscalarloc + (i);
			sum -= thefile[sptr];
				
			header[i] = VT0MapHelper::calcVtecRow(sum);
		}//end row header for loop
	}//
	else
	{
		header[header->Length-1] = VT0MapHelper::calcNonVtecRow(sum);

		//row headers...
		for(int i = yy-2; i>=0; i--)
		{
			int sptr = rpmscalarloc + (i);
			sum -= thefile[sptr];
					
			header[i] = VT0MapHelper::calcNonVtecRow(sum);
		}
	}//end if
}

void RomRep::VT0MapHelper::setPressScalars(int presscalarloc, String* header[], unsigned char fil __gc[])
{
	if(header->Length == 15)
	{
		header[0] = "27 In";
		header[1] = "23";
		header[2] = "21";
		header[3] = "19";
		header[4] = "18";
		header[5] = "16";
		header[6] = "14";
		header[7] = "13";
		header[8] = "11";
		header[9] = "9";
		header[10] = "8";
		header[11] = "6";
		header[12] = "3";
		header[13] = "0";
		header[14] = "1.3psi";
	}
}

String* RomRep::VT0MapHelper::getMapName(String* mapname,int index, int mode)
{

	String* type = "Fuel";
	if(mode == Map::MODE_IGNITION)
		type = "Ignition";

	if(mapname != NULL)
		return mapname;
	else if(index == 0)
		return type;
	else if(index == 1)
		return String::Concat("VTEC ",type);
	else //if(index >1)
		return String::Concat(type, ": ", index.ToString());
}