#include "stdafx.h"
#include "Obd0VtecMaps.h"
#include "obd0vtececu.h"
#include "hondaEcu.h"

RomRep::VT0IgnMap::VT0IgnMap(int x1, int y1, unsigned int loc, unsigned char themap __gc[,], 
							 unsigned char fil __gc[], int cscalarloc, int rscalarloc, int ind, 
							 int mod):Obd0IgnMap(x1,y1,loc,themap,fil,ind,mod)
{
	setup(true, themap, fil, cscalarloc, rscalarloc, ind);
}

RomRep::VT0IgnMap::VT0IgnMap(bool calchead, int x1, int y1, unsigned int loc, unsigned char themap __gc[,], 
			unsigned char fil __gc[], int cscalarloc, int rscalarloc, int ind, 
			int mod):Obd0IgnMap(x1,y1,loc,themap,fil,ind,mod)
{
	setup(calchead, themap, fil, cscalarloc, rscalarloc, ind);
}
void RomRep::VT0IgnMap::setup(bool calchead, unsigned char themap __gc[,], 
			unsigned char fil __gc[], int cscalarloc, int rscalarloc, int ind)
{
	populate(themap);

	this->vtec = false;
	if( ind > 0 )
		this->vtec = true;

	this->revscalarloc = rscalarloc;
	this->pressscalarloc = cscalarloc;
}

////*********************************************
////*********************************************
////*********************************************
////*********************************************

RomRep::VT0FuelMap::VT0FuelMap(int x1, int y1,unsigned int loc, unsigned char themap __gc[,], 
							   unsigned int multloc1, unsigned char mltarray __gc[], 
							   unsigned char fil __gc[], int cscalarloc, int rscalarloc, 
							   int ind, int mod):Obd0FuelMap(x1,y1,loc,themap,multloc1,mltarray,fil,ind,mod)
{
	setup(true, themap, mltarray, fil, cscalarloc, rscalarloc, ind);
}
RomRep::VT0FuelMap::VT0FuelMap(bool calchead, int x1, int y1,unsigned int loc, unsigned char themap __gc[,], 
							   unsigned int multloc1, unsigned char mltarray __gc[], 
							   unsigned char fil __gc[], int cscalarloc, int rscalarloc, 
							   int ind, int mod):Obd0FuelMap(x1,y1,loc,themap,multloc1,mltarray,fil,ind,mod)
{
	setup(calchead, themap, mltarray, fil, cscalarloc, rscalarloc, ind);
}
void RomRep::VT0FuelMap::setup(bool calchead, unsigned char themap __gc[,], 
							   unsigned char mltarray __gc[], unsigned char fil __gc[], 
							   int cscalarloc, int rscalarloc, int ind)
{
	this->populate(themap,mltarray);

	this->vtec = false;
	if( ind > 0 )
		this->vtec = true;
	revscalarloc = rscalarloc;
	pressscalarloc = cscalarloc;
}

float RomRep::VT0FuelMap::calculateInjMs(int value)
{
	/*PW0=PR3 : Avance = ValRom / 4
    : ValInj = ( ValRom * CoefRom ) / 4
    
    I think its 
    msInj = ( ValInj*4 - 1ah ) /246.875
	*/
	
	//obd1-like values...
	//return (float)(value/4.0f);

	float temp = (float)( value /*- 0x1a*/ ) / 246.875F;
	if(temp < 0.0f) temp = 0.0f;

	return temp;
}

int RomRep::VT0FuelMap::calculateInjByte(float injMs, int m)
{
	//obd1-like values...
	//return (int)(injMs*4.0f/(float)m);

	if(injMs < 0.0f) injMs = 0.0f;
	int temp = (int)(((injMs * 246.875F) /*+ 26.0F*/))/m;
	return temp;
}

float RomRep::VT0FuelMap::calculate(int x1, int y1, int intval)
{
	int m = (int)pow(2,(int)mult[x1]);
	return calculateInjMs(intval * m);
}

int RomRep::VT0FuelMap::calculate(int x1, int y1, float mapval)
{
	int m = (int)pow(2,(int)mult[x1]);
	return calculateInjByte(mapval,m);
}

//***************************
//***************************
//** helper classes
RomRep::VT0ColumnHeaderCalculator::VT0ColumnHeaderCalculator(Object* map, Object* hondaecu):HeaderCalculator(map,hondaecu){}
RomRep::MapHeaderChangeArgs* RomRep::VT0ColumnHeaderCalculator::calculateHeaders(bool col, int columnOrRow)
{
	if(!col) return NULL;
	unsigned char fil __gc[] = static_cast<RomRep::HondaEcu*>(m_hondaecu)->getFile();
	RomRep::Map* map = static_cast<RomRep::Map*>(m_map);

	String* headers[];

	//we are going to recalculate All the headers all the time.
	if(col){
		int loc = map->getColumnScalarLoc();
		headers = VT0MapHelper::setPressScalars(loc,fil);
	}

	return new MapHeaderChangeArgs(RomRep::Map::HEADER_EVENT_ALL,RomRep::Map::HEADER_EVENT_NONE,headers);
}
float RomRep::VT0ColumnHeaderCalculator::getReferenceArray() __gc[]
{
	float arr __gc[] = {0.0f,32.0f,40.0f,48.0f,56.0f,64.0f,72.0f,80.0f,88.0f,96.0f,104.0f,120.0f,136.0f,144.0f,160.0f};
	return arr;
	//{0a0h,090h,080h,070h,068h,060h,058h,050h,048h,040h,038h,030h,028h,020h,000h};
}

RomRep::VT0RowHeaderCalculator::VT0RowHeaderCalculator(Object* map, Object* hondaecu):HeaderCalculator(map,hondaecu){}
RomRep::MapHeaderChangeArgs* RomRep::VT0RowHeaderCalculator::calculateHeaders(bool col, int columnOrRow)
{
	if(col) return NULL;
	unsigned char fil __gc[] = static_cast<RomRep::HondaEcu*>(m_hondaecu)->getFile();
	RomRep::Map* map = static_cast<RomRep::Map*>(m_map);

	String* headers[];

	if(!col){
		int loc = map->getRowScalarLoc();
		bool vtec = map->getIndex() > 0 ? true : false;
		headers = VT0MapHelper::setRpmScalarsStr(loc,fil,vtec);
	}

	return new MapHeaderChangeArgs(RomRep::Map::HEADER_EVENT_NONE,RomRep::Map::HEADER_EVENT_ALL,headers);
}
float RomRep::VT0RowHeaderCalculator::getReferenceArray() __gc[]
{
	unsigned char fil __gc[] = static_cast<RomRep::HondaEcu*>(m_hondaecu)->getFile();
	RomRep::Map* map = static_cast<RomRep::Map*>(m_map);
	int loc = map->getRowScalarLoc();
	bool vtec = map->getIndex() > 0 ? true : false;
	int inth __gc[] = VT0MapHelper::setRpmScalars(loc,fil,vtec);
	int len = inth->Length;
	float headers __gc[] = new float __gc[len];

	for(int i = 0; i<len; i++)
		headers[i] = (float)inth[i];
	return headers;
}

int RomRep::VT0MapHelper::calcRow(int sum, bool vtec)
{
	if(vtec){
		int rpm = 0;
		if(sum>0){
			rpm = 0xd000/sum;
			rpm = VT0Ecu::revconst/rpm;
		}
		return rpm;//.ToString();
	}

	int er0, er3;
	if(sum >= 0xcf){
		er0 = 0x8000;
		er3 = 0x80;
	}
	else if(sum >= 0x8f){
		er0 = 0x10000;
		er3 = 0x40;
	}
	else if(sum >= 0x4f){
		er0 = 0x20000;
		er3 = 0;
	}
	else{
		er0 = 0x40000;
		er3 = -64;
	}
	return (((sum - er3)*VT0Ecu::revconst)/er0);
}

int RomRep::VT0MapHelper::setRpmScalars(int rpmscalarloc, unsigned char thefile __gc[], bool vtec) __gc[]
{
	int header __gc[] = new int __gc[17];
	int sum = 256;
	int yy = header->Length;

	header[yy-1] = VT0MapHelper::calcRow(sum,vtec);

	//row headers...
	for(int i = yy-2; i>=0; i--){
		int sptr = rpmscalarloc + (i);
		sum -= thefile[sptr];
			
		header[i] = VT0MapHelper::calcRow(sum,vtec);
	}//end row header for loop
	return header;
}
String* RomRep::VT0MapHelper::setRpmScalarsStr(int rpmscalarloc, unsigned char thefile __gc[], bool vtec)[]
{
	int inth __gc[] = setRpmScalars(rpmscalarloc,thefile,vtec);
	String* header[] = new String*[17];
    for(int i = 0; i< header->Length; i++)
		header[i] = inth[i].ToString();
	return header;
}

String* RomRep::VT0MapHelper::setPressScalars(int presscalarloc, unsigned char fil __gc[])[]
{
	String* header[] = new String*[15];
	header[0] = "29 In";
	header[1] = "23";
	header[2] = "21";
	header[3] = "19";
	header[4] = "18";
	header[5] = "16";
	header[6] = "14";
	header[7] = "13";
	header[8] = "11";
	header[9] = "9";
	header[10] = "8";
	header[11] = "6";
	header[12] = "3";
	header[13] = "0";
	header[14] = "1.3psi";
	return header;
}

String* RomRep::VT0MapHelper::getMapName(String* mapname,int index, int mode)
{

	String* type = "Fuel";
	if(mode == Map::MODE_IGNITION)
		type = "Ignition";

	if(mapname != NULL)
		return mapname;
	else if(index == 0)
		return type;
	else if(index == 1)
		return String::Concat("VTEC ",type);
	else //if(index >1)
		return String::Concat(type, ": ", index.ToString());
}