#pragma once

using namespace System;
using namespace System::ComponentModel;
using namespace System::Collections;
using namespace System::Windows::Forms;
using namespace System::Data;
using namespace System::Drawing;

#include "HondaEcu.h"
#include "LabeledText.h"

namespace bre20
{
	__delegate int intGetFunction();
	__delegate void intSetFunction(int val);

	/// <summary> 
	/// Base class for parameter pages. The main function is for
	/// all pages to have the multiplier editing capability with little work.
	/// </summary>
	public __gc class ParamPage : public System::Windows::Forms::TabPage
	{
	public:
		///basic. Try not to use this.
		ParamPage();
		///\param ecu Takes the HondaEcu object so it can deal with the fuel multipliers
		///and all other ROM changes the page is responsible for.
		ParamPage(RomRep::HondaEcu* ecu);

		///
		Object* getParent(){return parent;};
		void setParent(Object* p){parent = p;};

		///Calculates and sets the physical size of the grid
		void setGridSize()
		{
		//	this->grid->GridSize = System::Drawing::Size(rom->getWidth(), rom->getNumMaps());

			int wid = grid->RowHeaderWidth + (grid->getColumnWidth(0)*rom->getWidth())+2;
			int height = grid->RowHeight * (rom->getNumMaps()+1)+1;

			this->grid->Size = System::Drawing::Size(wid,height);
		}

		///Adds a row to the multiplier grid. 
		///It is used when a new fuel map is added to the ROM.
		bool addMultiplierRow(unsigned char values __gc[])
		{
			try
			{
				this->grid->GridSize = System::Drawing::Size(rom->getWidth(), rom->getNumMaps());

				int max = Math::Min(rom->getWidth(),values->Length);

				for( int i = 0; i<max; i++ )
					grid->setValue(i,rom->getNumMaps()-1,values[i]);

				setGridSize();

				this->grid->Location = System::Drawing::Point(16, this->grid->Location.Y-grid->RowHeight);
					
				return true;
			}
			catch(...)
			{
				return false;
			}
		}

		///\returns a value from the multiplier grid
		int getGridValue(int col, int row)
		{
			return grid->getIntValue(col,row);
		}

		///sets a value in the multiplier grid
		///\returns false if fails
		bool setGridValue(int col, int row, int val)
		{
			try
			{
				grid->setValue(col,row,val);
				return true;
			}
			catch(...)
			{
				return false;
			}
		}

		void anchorGrid()
		{
			this->grid->Anchor = (System::Windows::Forms::AnchorStyles)((System::Windows::Forms::AnchorStyles::Bottom) 
				| System::Windows::Forms::AnchorStyles::Left);
		}

		///happens when the user changes a value in the multiplier grid
		__event EventHandler* CellChanged;
		///Makes sure the text box value is an integer.
		///\returns an integer value from the text box
		static int getBoxVal(TextBox* box);
		static int getBoxVal(LabeledText* box);
		static int getBoxVal(TextBox* box, int oldval, int min, int max);

		static void setBoxVal(bre20::LabeledText* box, bre20::intGetFunction* getFn, 
						bre20::intSetFunction* setFn, int min, int max);
		static void setBoxVal(TextBox* box, bre20::intGetFunction* getFn, 
						bre20::intSetFunction* setFn, int min, int max);
	protected:

		Object* parent;
		SimpleGrid::Grid* grid;
		RomRep::HondaEcu* rom;

		///fills the multiplier grid
		void fillGrid();

		///called when the user changes a value in the multiplier grid
		virtual void onCellChanged(SimpleGrid::CellEvent* e)
		{
			if (CellChanged != NULL)
				CellChanged(this,e);
		}

	private:
		System::Void grid_CellChanged(System::Object *  sender, SimpleGrid::CellEvent *  e);
	};

}