#include "stdafx.h"
#include "Pm6Ng.h"
#include "Engine.h"
#include "NGMaps.h"
#include "DevUtil.h"
#include "PrimitiveWrap.h"

RomRep::Pm6Ng::Pm6Ng(unsigned char rom __gc[], int flen): HondaEcu(rom, flen, 0,false,2,"NG60")
{
	x = 15;
	y = 17;

	target_afr_loc[0] = 0x6100;
	target_afr_loc[1] = 0x61ff;

	checksumaddy = 0x3c00;
	checksumlen = 0x8000;
	int launchaddy = 0x7ffa;
	int idleaddy = 0x399F;
	int revloc = 0x7fe4;
	int coldrevloc = 0x7fe6;
	//									high						low
	launchlimit = new RpmTwoByteValue(launchaddy, rom[launchaddy],launchaddy+1,rom[launchaddy+1]);
	revlimit = new RpmTwoByteValue(revloc, rom[revloc], revloc+1, rom[revloc+1]);
	coldrevlimit = new RpmTwoByteValue(coldrevloc, rom[coldrevloc], coldrevloc+1, rom[coldrevloc+1]);
	launchspeedloc = 0x7ff9;

	MapLocs* mapinfo[] = new MapLocs*[4];

	mapinfo[0] = new MapLocs(0x5200,0x4600,0); //fuel
	mapinfo[1] = new MapLocs(0x530e,0x4600,0); //fuel
	mapinfo[2] = new MapLocs(0x5001,0x4600,0);
	mapinfo[3] = new MapLocs(0x5101,0x4600,0);

	try
	{
		HondaEcu::setupMaps(mapinfo);
	}catch(...){}
}


RomRep::FuelMap* RomRep::Pm6Ng::createFuelMap(int xx, int yy, int index, String* mapname,
	RomRep::MapLocs* maploc,unsigned char map __gc[,], unsigned char mlt __gc[], bool vtec)
{
	RomRep::FuelMap* im = new NGFuelMap(xx,yy,maploc->mapLoc,map,maploc->mapLoc+xx*yy,mlt,thefile,
		maploc->mapColScalarLoc,maploc->mapRevScalarLoc,index,Map::MODE_FUEL);
	
	if(mapname != NULL)
		im->setName(mapname);
	else if(index == 0)
		im->setName("NA Fuel");
	else if(index == 1)
		im->setName("Boost Fuel");
	else //if(index >1)
		im->setName(String::Concat("Fuel: ",index.ToString()));
	return im;
}

RomRep::Map* RomRep::Pm6Ng::createIgnitionMap(int xx, int yy, int index, String* mapname,
	RomRep::MapLocs* maploc,unsigned char map __gc[,], bool vtec)
{
	Map* im = new NGIgnMap(xx,yy,maploc->mapLoc,map,thefile,maploc->mapColScalarLoc,
		maploc->mapRevScalarLoc,index, Map::MODE_IGNITION);
	
	if(mapname != NULL)
		im->setName(mapname);
	else if(index == 0)
		im->setName("NA Ignition");
	else if(index == 1)
		im->setName("Boost Ignition");
	else //if(index >1)
		im->setName(String::Concat("Ignition: ",index.ToString()));
	return im;
}

bool RomRep::Pm6Ng::validate(unsigned char h __gc[])
{
//	0x02,0x16,0xBA,0xC0,0xE0,0xC0,0x83,0xC0,0x82,0x61
	int head[10] = {0x02,0x16,0xBA,0xC0,0xE0,0xC0,0x83,0xC0,0x82,0x61}; //pm6

	for(int i = 0; i<HondaEcu::headlen; i++)
		if(head[i] != h[i])
			return false;

	return true;
}

//*********USE HondaEcu::setByte! it will set the byte in the rom
//setByte eventually will contain the undo stuff

void RomRep::Pm6Ng::setTwoByteRpm(int highloc, int lowloc, int rpm){
	int val = revconst/rpm;
	setByte(lowloc, val%256);
	setByte(highloc,val/256);
}

int RomRep::Pm6Ng::getTwoByteRpm(int highloc, int lowloc){
	int denom = (int)thefile[lowloc] + (int)thefile[highloc]*256;
	if(denom != 0)
		return revconst/(denom);
	return revconst;
}

float RomRep::Pm6Ng::calcTps(int raw){
	raw = raw-50;
	if(raw<0) raw = 0;
	float fval = ((float)raw/2.0F);
	return fval;
}
int RomRep::Pm6Ng::calcTps(float tps){
	tps = tps*2.0f +50.0f;
	return (int)tps;
}

//**************************************
bool RomRep::Pm6Ng::isLaunchEnabled(){
	return (int)thefile[0x7ff8] == 1;
}
int RomRep::Pm6Ng::getO2Mode(){
	return (int)thefile[0x7fc0];
}

void RomRep::Pm6Ng::setLaunchEnabled(bool enabled)
{
	int loc = 0x7ff8;
	if(enabled)
		setByte(loc,1);
	else
		setByte(loc,0);
}
void RomRep::Pm6Ng::setO2Mode(int mode){
	setByte(0x7fc0, mode);
}
//**************************************
void RomRep::Pm6Ng::setRevlimit(int val){
	setRpmValue(revlimit, val);
}
int RomRep::Pm6Ng::getRevlimit(){
	return getRpmValue(revlimit);
}

void RomRep::Pm6Ng::setColdRevlimit(int val){
	setRpmValue(coldrevlimit, val);
}
int RomRep::Pm6Ng::getColdRevlimit(){
	return getRpmValue(coldrevlimit);
}

void RomRep::Pm6Ng::setLaunch(int rpm){
	setRpmValue(launchlimit, rpm);
}
int RomRep::Pm6Ng::getLaunch(){//returns RPM
	return getRpmValue(launchlimit);
}

void RomRep::Pm6Ng::setLaunchSpeed(int speed){
	setByte(launchspeedloc,speed);
}
int RomRep::Pm6Ng::getLaunchSpeed(){//returns speed
	return (int)thefile[launchspeedloc];
}

void RomRep::Pm6Ng::setRpmValue(RomRep::RpmTwoByteValue* limit, int rpm)
{
	int bytes;
	if(rpm>0)
		bytes = revconst/rpm;
	else
		return;

	limit->setByte1(getHighByte(bytes));
	limit->setByte2(getLowByte(bytes));
	if(limit->getLocation1() != 0)
	{
		RomRep::HondaEcu::setByte(limit->getLocation1(), getHighByte(bytes));
		RomRep::HondaEcu::setByte(limit->getLocation2(), getLowByte(bytes));
	}
}

int RomRep::Pm6Ng::getRpmValue(RomRep::RpmTwoByteValue* limit)
{
	int bytes = limit->getTwoByte();

	if(bytes >0)
		return revconst/bytes;
	else
		return -1;
}

String * RomRep::Pm6Ng::getColHeader(int map, int column)
{
	return this->fuelmaps[map]->getColumnHeader(column);
}
String* RomRep::Pm6Ng::getColHeaders()[,]
{
	String* heads[,] = new String*[nummaps,this->x];
	for(int i = 0; i< nummaps; i++)
		for(int j = 0; j<this->x; j++)
			heads[i,j] = this->fuelmaps[i]->getColumnHeader(j);

	return heads;
}

RomRep::HeaderCalculator* RomRep::Pm6Ng::GetRowHeaderCalculator(RomRep::Map* map){
	return new NGRowHeaderCalculator(map,this);
}
RomRep::HeaderCalculator* RomRep::Pm6Ng::GetColumnHeaderCalculator(RomRep::Map* map){
	return new NGHeaderCalculator(map,this);
}

unsigned char RomRep::Pm6Ng::getStockRemap() __gc[]
{
	unsigned char remap __gc[] = {0x00,0x00,0x01,0x02,0x03,0x03,0x04,0x05,0x06,0x07,0x07,
		0x08,0x09,0x0A,0x0B,0x0B,0x0C,0x0D,0x0E,0x0F,0x10,0x11,0x12,0x13,0x14,0x15,0x16,
		0x18,0x19,0x1A,0x1B,0x1C,0x1D,0x1F,0x20,0x21,0x22,0x23,0x24,0x25,0x26,0x28,0x29,
		0x2A,0x2B,0x2C,0x2D,0x2F,0x30,0x31,0x32,0x33,0x35,0x36,0x37,0x38,0x3A,0x3B,0x3C,
		0x3D,0x3F,0x40,0x41,0x43,0x45,0x47,0x49,0x4B,0x4D,0x4F,0x50,0x51,0x53,0x55,0x56,
		0x58,0x5A,0x5B,0x5D,0x5F,0x60,0x62,0x65,0x67,0x6A,0x6C,0x6F,0x70,0x72,0x75,0x77,
		0x7A,0x7C,0x7F,0x80,0x82,0x85,0x87,0x8A,0x8C,0x8F,0x90,0x92,0x94,0x96,0x98,0x9A,
		0x9C,0x9F,0xA0,0xA2,0xA5,0xA7,0xAA,0xAC,0xAF,0xB0,0xB2,0xB5,0xB7,0xBA,0xBC,0xBF,
		0xC0,0xC2,0xC5,0xC7,0xCA,0xCC,0xCF,0xD0,0xD2,0xD5,0xD7,0xDA,0xDC,0xDF,0xE0,0xE2,
		0xE4,0xE6,0xE8,0xEA,0xEC,0xEF,0x00,0x03,0x07,0x0B,0x0F,0x10,0x12,0x14,0x16,0x18,
		0x1A,0x1C,0x1F,0x20,0x22,0x24,0x26,0x28,0x2A,0x2C,0x2F,0x30,0x32,0x34,0x36,0x38,
		0x3A,0x3C,0x3F,0x40,0x42,0x45,0x47,0x4A,0x4C,0x4F,0x50,0x52,0x54,0x56,0x58,0x5A,
		0x5C,0x5F,0x60,0x62,0x64,0x66,0x68,0x6A,0x6C,0x6F,0x70,0x72,0x74,0x76,0x78,0x7A,
		0x7C,0x7F,0x80,0x82,0x84,0x86,0x88,0x8A,0x8C,0x8F,0x90,0x92,0x94,0x96,0x98,0x9A,
		0x9C,0x9F,0xA0,0xA2,0xA4,0xA6,0xA8,0xAA,0xAC,0xAF,0xB0,0xB2,0xB4,0xB6,0xB8,0xBA,
		0xBC,0xBF,0xC0,0xC2,0xC5,0xC7,0xCA,0xCC,0xCF,0xD0,0xD1,0xD3,0xD5,0xD6,0xD8,0xDA,
		0xDB,0xDD,0xDF,0xE0,0xEF};
	return remap;
}
unsigned char RomRep::Pm6Ng::get2BarRemap() __gc[]
{
	return NULL;
}
unsigned char RomRep::Pm6Ng::get2_5BarRemap() __gc[]
{
	unsigned char remap __gc[] = {0x00,0x01,0x02,0x03,0x04,0x05,0x06,0x08,0x09,0x0A,0x0B,
		0x0C,0x0D,0x0F,0x10,0x11,0x13,0x14,0x16,0x17,0x19,0x1A,0x1C,0x1D,0x1F,0x21,
		0x23,0x25,0x26,0x28,0x2A,0x2B,0x2D,0x2F,0x30,0x33,0x35,0x37,0x39,0x3B,0x3D,
		0x3F,0x40,0x42,0x45,0x47,0x4A,0x4C,0x4F,0x50,0x52,0x55,0x57,0x5A,0x5C,0x5F,0x60,
		0x63,0x67,0x6B,0x6F,0x70,0x73,0x77,0x7B,0x7F,0x80,0x83,0x87,0x8B,0x8F,0x90,0x93,
		0x97,0x9B,0x9F,0xA0,0xA3,0xA6,0xA9,0xAC,0xAF,0xB0,0xB3,0xB7,0xBB,0xBF,0xC0,0xC3,
		0xC7,0xCF,0xD0,0xD2,0xD5,0xD7, 0xd9,0xDB, 0xdf, 0xE0 ,0xE3,0xE7,0xEB,0xEF,0x00,0x01,0x03,
		0x05,0x07,0x09,0x0B,0x0D,0x0F,0x10,0x11,0x13,0x14,0x16,0x17,0x19,0x1A,0x1C,0x1D,
		0x1F,0x20,0x21,0x23,0x25,0x26,0x28,0x2A,0x2B,0x2D,0x2F,0x30,0x31,0x33,0x34,0x36,
		0x37,0x39,0x3A,0x3C,0x3D,0x3F,0x40,0x41,0x43,0x45,0x46,0x48,0x4A,0x4B,0x4D,0x4F,
		0x50,0x51,0x53,0x54,0x56,0x57,0x59,0x5A,0x5C,0x5D,0x5F,0x60,0x61, 0x62,0x63,0x65,0x66,
		0x68,0x6A,0x6B,0x6D,0x6F,0x70,0x71,0x73,0x74,0x76,0x77,0x79,0x7A,0x7C,0x7D,0x7F,
		0x80,0x81,0x83,0x85,0x86,0x88,0x8A,0x8B,0x8D,0x8F,0x90,0x91,0x93,0x94,0x96,0x97,
		0x99,0x9A,0x9C,0x9D,0x9F,0xA0,0xA1,0xA3,0xA4,0xA6,0xA7,0xA9,0xAA,0xAC,0xAD,0xAF,
		0xB0,0xB1,0xB3,0xB5,0xB6,0xB8,0xBA,0xBB,0xBD,0xBF,0xC0,0xC1,0xC3,0xC4,0xC6,0xC7,
		0xC9,0xCA,0xCC,0xCD,0xCF,0xD0,0xD1,0xD2,0xD3,0xD4,0xD5,0xD6,0xD8,0xD9,0xDA,0xDB,
		0xDC,0xDD,0xDF,0xE0,0xEF};
	return remap;
}
unsigned char RomRep::Pm6Ng::get3BarRemap() __gc[]
{
	unsigned char remap __gc[] = {0x00,0x01,0x02,0x04,0x05,0x06,0x08,0x09,0x0A,0x0C,0x0D,
		0x0F,0x10,0x11,0x13,0x15,0x17,0x19,0x1B,0x1D,0x1F,0x20,0x22,0x24,0x26,0x28,0x2A,
		0x2C,0x2F,0x30,0x32,0x34,0x36,0x38,0x3A,0x3C,0x3F,0x40,0x43,0x47,0x4B,0x4F,0x50,
		0x53,0x56,0x59,0x5C,0x5F,0x60,0x65,0x6A,0x6F,0x70,0x73,0x77,0x7B,0x7F,0x80,0x85,
		0x8A,0x8F,0x90,0x95,0x9A,0x9F,0xA0,0xA3,0xA7,0xAB,0xAF,0xB0,0xB5,0xBA,0xBF,0xC0,
		0xC5,0xCA,0xCF,0xD0,0xD3,0xD7,0xDB,0xDF,0xE0,0xE5,0xEA,0xEF,0x00,0x01,0x03,0x05,
		0x06,0x08,0x0A,0x0B,0x0D,0x0F,0x10,0x11,0x12,0x14,0x15,0x16,0x18,0x19,0x1A,0x1C,
		0x1D,0x1F,0x20,0x21,0x22,0x24,0x25,0x26,0x28,0x29,0x2A,0x2C,0x2D,0x2F,0x30,0x31,
		0x32,0x34,0x35,0x36,0x38,0x39,0x3A,0x3C,0x3D,0x3F,0x40,0x41,0x42,0x44,0x45,0x46,
		0x48,0x49,0x4A,0x4C,0x4D,0x4F,0x50,0x51,0x52,0x54,0x55,0x56,0x58,0x59,0x5A,0x5C,
		0x5D,0x5F,0x60,0x61,0x62,0x64,0x65,0x66,0x68,0x69,0x6A,0x6C,0x6D,0x6F,0x70,0x71,
		0x72,0x74,0x75,0x76,0x78,0x79,0x7A,0x7C,0x7D,0x7F,0x80,0x81,0x82,0x84,0x85,0x86,
		0x88,0x89,0x8A,0x8C,0x8D,0x8F,0x90,0x91,0x92,0x94,0x95,0x96,0x98,0x99,0x9A,0x9C,
		0x9D,0x9F,0xA0,0xA1,0xA2,0xA4,0xA5,0xA6,0xA8,0xA9,0xAA,0xAC,0xAD,0xAF,0xB0,0xB1,
		0xB2,0xB4,0xB5,0xB6,0xB8,0xB9,0xBA,0xBC,0xBD,0xBF,0xC0,0xC1,0xC2,0xC4,0xC5,0xC6,
		0xC8,0xC9,0xCA,0xCC,0xCD,0xCF,0xD0,0xD1,0xD2,0xD3,0xD5,0xD6,0xD7,0xD8,0xDA,0xDB,
		0xDC,0xDD,0xDF,0xE0,0xEF};
	return remap;
}

bool RomRep::Pm6Ng::setMapSensor(int sensor)
{
	switch(sensor)
	{
	case DevUtil::MAPSTOCK:
		NGRemap::remapColumns(this,getStockRemap(),0x1D,0x90);
		return true;

	case DevUtil::MAPMOTOROLA25BAR:
	case DevUtil::MAP25BAR:
		NGRemap::remapColumns(this,get2_5BarRemap(),0x28,0x67);
		return true;

	case DevUtil::MAP3BAR:
		NGRemap::remapColumns(this,get3BarRemap(),0x30,0x56);
		return true;

	default:
		return false;
	}
}


int RomRep::Pm6Ng::getMapSensor(unsigned char file __gc[])
{
	//90h = 1.8
	//81h = 2.0
	//67h = 2.5
	//55-56h = 3.0
	int atmosphere = (int) file[NGMapGlobal::getMaxXOverLocEd()];

	if(atmosphere > 0x8a)
		return DevUtil::MAPSTOCK;
	else if(atmosphere <= 0x8a && atmosphere > 0x7a)
		return DevUtil::MAP2BAR;
	else if(atmosphere <= 0x7a && atmosphere > 0x5f)
		return DevUtil::MAPMOTOROLA25BAR;
	else //if(atmosphere <= 0x5f)
		return DevUtil::MAP3BAR;
}
String* RomRep::Pm6Ng::calcPressureHeaders(int loc, unsigned char file __gc[], bool boost)[]
{
	int raw __gc[] = Pm6Ng::rawPressureHeaders(loc,file,boost);
	int len = raw->Length;
	String* headers[] = new String*[len];
	int mapsensor = getMapSensor(file);
				
	for(int i = 0; i<len; i++){
		if(!boost)
			headers[i] = Engine::voltsToInHg(Engine::rawToVolts(raw[i]),mapsensor).ToString();
		else
			headers[i] = Engine::voltsToPsi(Engine::rawToVolts(raw[i]),mapsensor).ToString("0.00");
	}
	return headers;
}
int RomRep::Pm6Ng::rawPressureHeaders(int loc, unsigned char file __gc[], bool boost) __gc[]
{
	int headers __gc[] = new int __gc[15];
	int ptr = 0;
	if(boost)
		ptr = (int)file[NGMapGlobal::getMaxXOverLocEd()];

	int col = 0;
	int mapsensor = getMapSensor(file);
				
	for(int i = ptr; i<256; i++){
		if(file[loc+i]/16 == col){
			headers[col] = i;
			col++;
		}
	}
	return headers;
}

String* RomRep::Pm6Ng::calcRpmHeaders(int loc, unsigned char file __gc[], bool boost)[]
{
	String* rowheaders[] = new String*[17];
	rowheaders[0] = "500";
	rowheaders[1] = "600";
	rowheaders[2] = "700";
	rowheaders[3] = "800";
	rowheaders[4] = "900";
	rowheaders[5] = "1100";
	rowheaders[6] = "1300";
	rowheaders[7] = "1600";
	rowheaders[8] = "1900";
	rowheaders[9] = "2200";
	rowheaders[10] = "2600";
	rowheaders[11] = "3100";
	rowheaders[12] = "3700";
	rowheaders[13] = "4400";
	rowheaders[14] = "5300";
	rowheaders[15] = "6300";
	rowheaders[16] = "7400";	
	return rowheaders;
}
float RomRep::Pm6Ng::getRpmHeaders() __gc[]
{
	float rowheaders __gc[] = {500.0f,600.0f,700.0f,800.0f,900.0f,1100.0f,1300.0f,
		1600.0f,1900.0f,2200.0f,2600.0f,3100.0f,3700.0f,4400.0f,5300.0f,6300.0f,7400.0f};	
	return rowheaders;
}

ArrayList * RomRep::Pm6Ng::getDisabledCodes()
{
	ArrayList* list = new ArrayList();
	int bytes = 3;
	int loc = 0x7fec;

	for(int i = 0; i< bytes; i++)
	{
		unsigned char mask = (unsigned char)getByte(loc+i);
		for(int shft = 8; shft>0; shft--)
		{
			if(mask%2 == 0)
				list->Add( ((8*i)+shft).ToString() );
			mask/=2;
		}
	}
	
	return list;
}
bool RomRep::Pm6Ng::toggleCodeDisable(int code, bool disable)
{
	code-=1;
	int loc = 0x7fec + code/8;
	unsigned char mask = (unsigned char)(1 << (7-((code)%8)));
	
	unsigned char byte = (unsigned char)getByte(loc);
	if(disable)
	{
		mask = mask ^ 0xff; //invert
		byte &= mask;
	}
	else
		byte |= mask;

	return setByteEx(loc,(int)byte) != NULL;
}


//VTEC**********************************

bool RomRep::Pm6Ng::isVtecEnabled(){
	return getByte(0x7FF1) == 1 ;
}
int RomRep::Pm6Ng::getVtecOutput(){
	return (int)getByte(0x7FF2);
}
int RomRep::Pm6Ng::getVtecTps(){//real tps val
	return (int)calcTps((int)getByte(0x7FF3));
}
int RomRep::Pm6Ng::getVtecEngageRpm(){
	return getTwoByteRpm(0x7ff4,0x7ff5);
}
int RomRep::Pm6Ng::getVtecDisengageRpm(){
	return getTwoByteRpm(0x7ff6,0x7ff7);
}

//SETS.........

void RomRep::Pm6Ng::setVtecEnabled(bool enabled){
	int val = 0;
	if(enabled) val = 1;
	setByte(0x7FF1,val);
}
void RomRep::Pm6Ng::setVtecOutput(int output){
	setByte(0x7FF2,output);
}
void RomRep::Pm6Ng::setVtecTps(int tps){//real tps val
	setByte(0x7FF3, calcTps((float)tps));
}
void RomRep::Pm6Ng::setVtecEngageRpm(int rpm){
	return setTwoByteRpm(0x7ff4,0x7ff5,rpm);
}
void RomRep::Pm6Ng::setVtecDisengageRpm(int rpm){
	return setTwoByteRpm(0x7ff6,0x7ff7,rpm);
}

//*****************************************************
//*** calc functions

float RomRep::Pm6Ng::calcTPS(int raw)
{
	float tmp = (float)raw - 50.0f;
	if(tmp < 0.0f) tmp = 0.0f;
	if(tmp > 205.0f) tmp = 205.0f;

	return tmp / 2.0f;
}
int RomRep::Pm6Ng::calcTPS(float percentage)
{
	if(percentage < 0.0f) percentage = 0.0f;
	if(percentage > 100.0f) percentage = 100.0f;

	return (int)(percentage * 2.0f + 50.0f);
}
 
float RomRep::Pm6Ng::calcTemp(int raw, bool sae)
{
	if(sae && raw>45)
		return -.8334F*(float)raw + 194.25F;
	else if(sae && raw<=45)
		return -2.3529F*(float)raw + 263.0F;
	else if(!sae && raw>66)
		return -.463F*(float)raw + 125.693F;
	else //if(!sae && raw<=45)
		return -1.037F*(float)raw + 163.888F;
}
int RomRep::Pm6Ng::calcTemp(float temp, bool sae)
{
	if(sae && temp<156.8f)
		return (int)(temp - 194.25F / -.8334F);
	else if(sae && temp>=156.8f)
		return (int)(temp - 263.0F / -2.3529F);
	else if(!sae && temp<95.1f)
		return (int)(temp - 125.693F / -.463F);
	else 
		return (int)(temp - 163.888F / -1.037F);
}

float RomRep::Pm6Ng::calcRpm(int raw)
{
	if(raw > 0)
		return (float)revconst/(float)raw;
	else
		return (float)revconst;
}
int RomRep::Pm6Ng::calcRpm(float rpm)
{
	if(rpm > 0)
		return revconst/(int)rpm;
	else
		return revconst;
}