//#pragma unmanaged
//
//#if !defined BENS_Pm6Ng_H
//#define BENS_Pm6Ng_H

#pragma once
using namespace System;

#include "HondaEcu.h"
#include "RpmValue.h"
#include "Obd0Maps.h"

namespace RomRep
{
	[Serializable] public __gc class Pm6Ng: public HondaEcu
	{
	protected:
		//revlimit, restart, idle

		static int revconst = 1920000;

		RpmTwoByteValue* revlimit;
		RpmTwoByteValue* coldrevlimit;
		RpmTwoByteValue* launchlimit;
		int launchspeedloc;

		virtual int getHighByte(int bytes)
		{return bytes/256;}
		virtual int getLowByte(int bytes)
		{return bytes%256;}

		virtual void setupHeaders(){};

		virtual void setRpmValue(RpmTwoByteValue* limit, int rpm);
		virtual int getRpmValue(RpmTwoByteValue* limit);

		virtual RomRep::HeaderCalculator* GetRowHeaderCalculator(RomRep::Map* map);
		virtual RomRep::HeaderCalculator* GetColumnHeaderCalculator(RomRep::Map* map);

		/*!
		You must override this method so this base class can properly
		setup the maps. 
		
		If mapname == null, the function must give the map a name.
		*/
		virtual FuelMap* createFuelMap(int xx, int yy, int index, String* mapname,
			RomRep::MapLocs* maploc,unsigned char map __gc[,], unsigned char mlt __gc[], bool vtec);

		///must override this method so this base class can properly
		///setup the maps.
		///
		///If mapname == null, the function must give the map a name.
		///
		///see createFuelMap() for an example
		virtual Map* createIgnitionMap(int xx, int yy, int index, String* mapname,
			RomRep::MapLocs* maploc,unsigned char map __gc[,], bool vtec);

		///Sets the target afr
		virtual float calculateAfr(unsigned char raw){
			return ((float)raw/25.0F)+10.0F;
		}
		virtual unsigned char calculateAfr(float afr){
			return (unsigned char)((afr-10.0F)*25.0F);
		}

	public:
		Pm6Ng(unsigned char rom __gc[], int flen);

		static float calcTps(int raw);
		static int calcTps(float tps);

		//values will come in and go out as real world values
		virtual void setRevlimit(int rpm);
		virtual void setLaunch(int rpm);
		virtual void setLaunchSpeed(int speed);
		virtual void setColdRevlimit(int rpm);
		//return rpm
		virtual int getRevlimit();
		virtual int getColdRevlimit();
		virtual int getLaunch();
		virtual int getLaunchSpeed();

		virtual bool isLaunchEnabled();
		virtual int getO2Mode();

		virtual void setLaunchEnabled(bool enabled);
		virtual void setO2Mode(int mode);
		
		virtual void setTwoByteRpm(int highloc, int lowloc, int rpm);
		virtual int getTwoByteRpm(int highloc, int lowloc); //returns rpm

		virtual ArrayList* getDisabledCodes();
		virtual bool toggleCodeDisable(int code, bool disable);

		static String* calcPressureHeaders(int loc, unsigned char file __gc[], bool boost)[];
		static String* calcRpmHeaders(int loc, unsigned char file __gc[], bool boost)[];
		static float getRpmHeaders() __gc[];
		static int rawPressureHeaders(int loc, unsigned char file __gc[], bool boost) __gc[];
		static int getMapSensor(unsigned char file __gc[]);
		virtual int getMapSensor(){return getMapSensor(thefile);};
		virtual bool setMapSensor(int senosor);

		static unsigned char getStockRemap() __gc[];
		static unsigned char get2BarRemap() __gc[];
		static unsigned char get2_5BarRemap() __gc[];
		static unsigned char get3BarRemap() __gc[];

		//VTEC stuff...
		virtual bool isVtecEnabled();
		virtual int getVtecTps(); //real tps val
		virtual int getVtecEngageRpm();
		virtual int getVtecDisengageRpm();
		virtual int getVtecOutput();

		virtual void setVtecEnabled(bool);
		virtual void setVtecTps(int tps); //real tps val
		virtual void setVtecEngageRpm(int rpm);
		virtual void setVtecDisengageRpm(int rpm);
		virtual void setVtecOutput(int output);

		static int getRevConstant(){return revconst;}

		static const int O2OFF = 0;
		static const int O2OFFBOOST = 1;
		static const int O2ON = 2;
		//OVERRIDES

		static bool validate(unsigned char h __gc[]);

		///Gets a column header. 
		///map == 0 for na, 1 for boost
		virtual String * getColHeader(int map, int column);
		virtual String* getColHeaders()[,];

		virtual float calcTPS(int raw);
		///\returns raw value
		virtual int calcTPS(float percentage);

		///\param raw the raw value
		///\param sae true if sae, false if metric
		///\returns the real temp.  
		virtual float calcTemp(int raw, bool sae);
		///\param temp the real world temp value
		///\param sae true if sae, false if metric
		///\returns the raw temp.  
		virtual int calcTemp(float temp, bool sae);

		///Calculates the RPM. If there are more than one format 
		///of rpm value, assume it is the most common format (i.e.
		///the 2 byte value)
		virtual float calcRpm(int raw);
		///Calculates the raw value fromf RPM. If there are more than one format 
		///of rpm value, assume it is the most common format (i.e.
		///the 2 byte value)
		virtual int calcRpm(float rpm);
	};
}
