#pragma once

using namespace System;

#include "AppSettings.h"

namespace bre20
{
	/// <summary> 
	/// Handles setting and getting the parameters from the *.exe.config file. 
	/// </summary>
	public __gc class PropertyEventArgs
	{
	private:
		int type;
		String* name;
		String* value;
		Form* dialog;

	public:
		///indicates this object is from a property changed event
		static const int PROPERTY_CHANGED = 0;
		///indicates this object is from a property dialog opened event
		static const int PROPERTY_DIALOG_OPENED = 1;

		PropertyEventArgs(String* propName, String* propVal);
		PropertyEventArgs(Form* propDialog, String* dialogName);

        int getMode()
		{return type;};

		String* getName()
		{return name;};

		///this will return null if in PROPERTY_DIALOG_OPENED mode
		String* getValue()
		{return value;};

		///this will return null if in PROPERTY_CHANGED mode
		Form* getDialog()
		{return dialog;};
	};

	public __delegate void PropertyEventHandler(Object *, PropertyEventArgs *);

	/// <summary> 
	/// Handles setting and getting the parameters from the *.exe.config file. 
	/// </summary>
	public __gc class Properties
	{
	private:
		AppSettings* as;

	protected:
		void PropertyChanged(Object* sender, String* name, String* val)
		{
			if (OnPropertyChanged != NULL)
			{
				OnPropertyChanged(sender, new bre20::PropertyEventArgs(name,val));
			}
		}

		void DialogOpened(Form* dialog, String* name)
		{
			if (OnPropertyDialogOpened != NULL)
			{
				OnPropertyDialogOpened(dialog, new bre20::PropertyEventArgs(dialog,name));
			}
		}

	public:
		Properties()
		{as = new AppSettings();}

		__event PropertyEventHandler * OnPropertyChanged;
		__event PropertyEventHandler * OnPropertyDialogOpened;

		String * getParam(String * name);
		bool getBoolParam(String * name);
		int getIntParam(String * name);
		float getFloatParam(String * name);
		void setParam(Object* sender,String * name, String* value);

		void NotifyDialogOpened(Form* dialog, String* dialogType);
	};
}