#pragma once

using namespace System;

#include "bytechange.h"

namespace RomRep
{
	public __gc class CellChangeArgs: public EventArgs
	{
	protected:
		int x, y, marker;

	public:
		__value enum Marker{ARGS_NORMAL, ARGS_SPECIAL};
		CellChangeArgs(int xx, int yy){
			x = xx;
			y = yy;
			marker = ARGS_NORMAL;
		}
		virtual int getX(){return x;}
		virtual int getY(){return y;}
		virtual int getMarker(){return marker;}
		virtual void setMarker(int m){marker = m;}
	};
//------------------------------------------------------------------------------------------
	public __gc class AfrChangeArgs: public CellChangeArgs
	{
	protected:
		int index;
		float afr;

	public:
		AfrChangeArgs(int xx, int yy, float afrr, int ind):
			CellChangeArgs(xx,yy){
			afr = afrr;
			index = ind;
		}
		float getAfr(){return afr;}
		int getIndex(){return index;}

	};
//------------------------------------------------------------------------------------------
	public __gc class ByteChangeArgs: public EventArgs
	{
	protected:
		RomRep::ByteChange* bc;

	public:
		ByteChangeArgs(RomRep::ByteChange* b){bc = b;}
		RomRep::ByteChange* getByteChange(){return bc;}
	};
//------------------------------------------------------------------------------------------
	public __gc class MapEnableChangeArgs: public CellChangeArgs
	{
	protected:
		bool enabled; 

	public:
		MapEnableChangeArgs(int xx, int yy, bool enable):CellChangeArgs(xx,yy){
			enabled = enable;
		}
		bool isEnabled(){ return enabled; }
	};
//------------------------------------------------------------------------------------------
	public __gc class MapHeaderChangeArgs: public CellChangeArgs
	{
	protected:
		String* newval[]; //this is an array for the HEADER_EVENT_ALL situations...

	public:
		MapHeaderChangeArgs(int xx, int yy, String* newvalues[]):CellChangeArgs(xx,yy){
			newval = newvalues;
		}
		String* getValue()[]{ return newval; }
	};
//------------------------------------------------------------------------------------------
	public __gc class RealMapChangeArgs: public CellChangeArgs
	{
	protected:
		float newval;

	public:
		RealMapChangeArgs(int xx, int yy, float newvalue): CellChangeArgs(xx,yy){
			newval = newvalue;
		}
		float getValue(){ return newval; }
	};
//------------------------------------------------------------------------------------------
	public __gc class MapChangeArgs: public CellChangeArgs
	{
	protected:
		int loc;
		int byte;

	public:
		MapChangeArgs(int loc, int xx, int yy, int byte): CellChangeArgs(xx,yy){
			this->loc = loc;
			this->byte = byte;
		};
		int getLocation(){return loc;}
		int getByte(){return byte;}
	};

	///Handler for when a mapChange event is fired by one of the maps 
	public __delegate void MapChangeHandler(Object *, MapChangeArgs *);
	public __delegate void MapByteChangeRequest(Object*, ByteChangeArgs *);
	public __delegate void RealMapChangeHandler(Object *, RealMapChangeArgs *);
	public __delegate void MapHeaderChangeHandler(Object *, MapHeaderChangeArgs *);
	public __delegate void MapHeaderEnableChangeHandler(Object *, MapEnableChangeArgs *);
}