#include "StdAfx.h"
#include "child.h"

#include "Logger.h"
#include "LogAnalyzer.h"
#include "ProgressDialog.h"
#include "RtpOptions.h"

#include "EuroPw0Dev.h"
#include "JdmPw0Dev.h"
#include "JdmPr3Dev.h"
#include "FuelMap.h"

#include "Form1.h"
#include "hondaecu.h"
#include "Global.h"



//////////////////////////////////////////////////////////////////////
//script public interface

bre20::LabeledText* bre20::child::CreateText(int x, int y, String* caption, String* handler, String* tooltip, int width, int mode)
{
	CSScriptLibrary::ScriptManager* sm = static_cast<Form1*>(this->MdiParent)->getScriptManager();

	if(!ManagedUtil::checkHandlerName(sm,handler) || caption == NULL || tooltip == NULL)
		return NULL;

	ScriptLabeledText* con = new ScriptLabeledText(handler,caption,tooltip,width,mode);
	con->Location = System::Drawing::Point(x, y);
	con->add_OnTextChanged(new TextChangedEventHandler(this,script_Call));
	sm->CSScript->getScript(handler)->addControl(con);

	return con;
}

Button * bre20::child::CreateButton(int x, int y, int width, int height, String* caption, String* handler)
{
	CSScriptLibrary::ScriptManager* sm = static_cast<Form1*>(this->MdiParent)->getScriptManager();

	if(!ManagedUtil::checkHandlerName(sm,handler) || caption == NULL)
		return NULL;

	ScriptButton* con = new ScriptButton(handler);

	con->Text = caption;
	con->Location = System::Drawing::Point(x, y);
	con->Size = System::Drawing::Size(width, height);
	con->FlatStyle = FlatStyle::System;

	sm->CSScript->getScript(handler)->addControl(con);

	con->add_Click(new EventHandler(this,script_Call));
	sm->CSScript->getScript(handler)->addControl(con);

	return con;
}

CheckBox * bre20::child::CreateCheckBox(int x, int y, String* caption, String* handler)
{
	CSScriptLibrary::ScriptManager* sm = static_cast<Form1*>(this->MdiParent)->getScriptManager();

	if(!ManagedUtil::checkHandlerName(sm,handler) || caption == NULL)
		return NULL;

	ScriptCheckBox* con = new ScriptCheckBox(handler);

	con->Text = caption;
	con->Location = System::Drawing::Point(x, y);
	con->Size = System::Drawing::Size(80, 16);
	con->FlatStyle = FlatStyle::System;

	sm->CSScript->getScript(handler)->addControl(con);

	con->add_CheckedChanged(new EventHandler(this,script_Call));
	sm->CSScript->getScript(handler)->addControl(con);

	return con;
	
}

GroupBox* bre20::child::CreateGroupBox(int x, int y, String* caption, String* handler, int width, int height, Control* controls[])
{
	CSScriptLibrary::ScriptManager* sm = static_cast<Form1*>(this->MdiParent)->getScriptManager();

	if(!ManagedUtil::checkHandlerType(sm,handler) || caption == NULL)
		return NULL;

	GroupBox* con= new GroupBox();

	con->Location = System::Drawing::Point(x,y);
	con->Text = caption;
	con->Size = System::Drawing::Size(width,height);
	con->FlatStyle = FlatStyle::System;

	if(controls != NULL)
		for(int i = 0; i<controls->Length; i++)
			con->Controls->Add(controls[i]);

	sm->CSScript->getScript(handler)->addControl(con);

	return con;
}

bre20::GridPage* bre20::child::AddMap(String* tabName, int columns, int rows, String* handler)
{

	CSScriptLibrary::ScriptManager* sm = static_cast<Form1*>(this->MdiParent)->getScriptManager();

	if(!ManagedUtil::checkHandlerType(sm,handler) || tabName == NULL)
		return NULL;

	ScriptGridPage* page = new ScriptGridPage(handler,tabName,columns,rows,MODEFUEL); //fuel b/c then it doesnt have 'add extra context' for ign stuff

	this->pages->AddExtraMap(page);

	page->add_CellChanged(new EventHandler(this,script_Call));

	sm->CSScript->getScript(handler)->addControl(page);

	return page;
}

SimpleGrid::Grid * bre20::child::CreateGrid(int x, int y, String* handler, int columns, int rows)
{
	CSScriptLibrary::ScriptManager* sm = static_cast<Form1*>(this->MdiParent)->getScriptManager();

	if(!ManagedUtil::checkHandlerName(sm,handler) || columns <= 0 || rows <= 0)
		return NULL;

	ScriptGrid* gr = new ScriptGrid(handler);

	gr->GridSize = System::Drawing::Size(columns, rows);
	gr->LastSelectedColumn = -1;
	gr->LastSelectedRow = -1;
	gr->Location = System::Drawing::Point(x, y);
	gr->RowHeaderWidth = 100;
	gr->RowHeight = 17;
	gr->SelectedColumn = -1;
	gr->SelectedForeColor = System::Drawing::Color::White;
	gr->SelectedRow = -1;
	gr->SelectedRowColor = System::Drawing::Color::Blue;
	gr->TabStop = false;
	gr->setAllColumnWidths(25);
	gr->Mode = 1;

	int wid = gr->RowHeaderWidth + (gr->getColumnWidth(0)*x)+2;
	int hei = gr->RowHeight*(rows+1)+5;
	
	gr->Size = System::Drawing::Size(wid,hei);

	gr->add_CellChanged(new SimpleGrid::GridChangedHandler(this,script_Call));

	sm->CSScript->getScript(handler)->addControl(gr);

	return gr;
}

MenuItem* bre20::child::AddMenuItem(String* mainMenu, String* caption, String* handler)
{
	CSScriptLibrary::ScriptManager* sm = static_cast<Form1*>(this->MdiParent)->getScriptManager();

	if(!ManagedUtil::checkHandlerType(sm,handler) || caption == NULL || caption->Equals(""))
		return NULL;

	if(mainMenu == NULL || mainMenu->Equals(""))
	{
		//make top menu...
		ScriptMenuItem* newItem = new ScriptMenuItem("");
		newItem->Text = caption;
		sm->CSScript->getScript(handler)->addMenuItem(newItem);

		this->mainMenu1->MenuItems->Add(newItem);

		return newItem;

	}//end top menu if
	else if(mainMenu != NULL)
	{
		//look for menu, etc
		MenuItem* mi = ManagedUtil::findMenuItem(mainMenu,this->mainMenu1);

		if(mi == NULL)
			return false;

		//the parent has been found
		ScriptMenuItem* newItem = new ScriptMenuItem("");
		newItem->Text = caption;
		sm->CSScript->getScript(handler)->addMenuItem(newItem);

		mi->MenuItems->Add(newItem);

		if(!caption->Equals("-") && ManagedUtil::checkHandlerName(sm,handler))
		{
			newItem->setMethodName(handler);
			newItem->add_Click(new EventHandler(this,script_Call));
		}//end handler if

		return newItem;
	}//end else if

	return NULL;
}

//this will allow you to subscribe to an event
//events to subscribe to are (the event param string):
//"OnAddControl" - happens when a script adds a control
//"OnChangeMapCell" - when one of the maps has a cell changed
//"OnChangeByte" - when any byte in the rom is changed
//"OnChildClosing" - when the child object is closing
bool bre20::child::subscribeToEvent(String * event, String* handler)
{
	CSScriptLibrary::ScriptManager* sm = static_cast<Form1*>(this->MdiParent)->getScriptManager();

	if(!ManagedUtil::checkHandlerName(sm,handler) || event == NULL)
		return false;

	this->eventSubscribe->Add(new EventSubscription(event,handler));
	return true;
}

bool bre20::child::parseByteChanges(String* fname, bool makeChangeOnRight)
{
	ArrayList* bca = ManagedUtil::parseByteChanges(vtrom,fname,makeChangeOnRight);

	if(bca == NULL)
		return false;
	return true;
}
//////////////////////////////////////////////////////////////////////
void bre20::child::verifyItm_Click(System::Object *  sender, System::EventArgs *  e)
{
	if(emulating)
	{
		unsigned char file __gc[]= vtrom->getFile();
		int len = vtrom->getLength();

		if(counter != NULL)
		{
			counter[0] = 0;
			counter[1] = len/0xff;
		}

		//signal for status to stay open
		senddone = false;

		emulationLabel = S"Reading from Emulator...";
		System::Threading::Thread * th = new System::Threading::Thread(new System::Threading::ThreadStart(this, displayEmulationStatus));
		th->Start();

		unsigned char arr __gc[] = emulator->readBlock(0,len,counter);

		//signal to close the status dialog
		senddone = true;

		bool crap = false;
		int lastaddy = 0;
		for(int i = 0; i<len; i++)
		{
			if( file[i] != arr[i])
			{
				lastaddy = i;
				crap = true;
			}
		}
		
		th->Abort();

		if( crap )
			System::Windows::Forms::MessageBox::Show(S"The Emulator ROM is NOT the same.",lastaddy.ToString());
		else
			System::Windows::Forms::MessageBox::Show(S"The ROMs are the same",S"BRE");
	}
}
bool bre20::child::putRom(String * lbl)
{
	unsigned char thefile __gc[]= vtrom->getFile();
	int len = vtrom->getLength();

	if(counter == NULL || counter->Length < 2)
		counter = new int __gc[2]; //[0] has count val, [1] has max
	else
		counter[0] = 0;
		
	counter[1] = emulator->getNumIterations(thefile);

	emulationLabel = lbl;

	senddone = false;
	System::Threading::Thread * th = new System::Threading::Thread(new System::Threading::ThreadStart(this, displayEmulationStatus));
	th->Start();

	bool good = emulator->writeInitialBlock(thefile,0,counter);

    senddone = true;

	if(good)
		emuqueue->Clear();
	else
		System::Windows::Forms::MessageBox::Show(S"An error occured while writing data to the emulator",S"BRE");

	return good;

}
void bre20::child::putItm_Click(System::Object *  sender, System::EventArgs *  e)
{
	putRom(S"Writing ROM to Emulator...");
}

void bre20::child::emuOptions_Click(System::Object *  sender, System::EventArgs *  e)
{
	RtpOptions* rtp = new RtpOptions();
	EmulationOptionsCreated(rtp);
	rtp->ShowDialog();

	liveedit = rtp->isLiveEdit();

	resetOstrichLabel();
}

void bre20::child::dwnldChangesItm_Click(System::Object *  sender, System::EventArgs *  e)
{
		//so if the thread is not already there, create it.
		if(!emusending )//&& !liveedit)
		{
			emusending = true;
			Thread* th = new Thread(new ThreadStart(this,writeEmuQueue));
			th->Start();
		}
}

bool bre20::child::loadEmulator()
{
	bool hisp = Globals::Props->getBoolParam(S"emuHighSpeed");

	this->ostrichStatus->Text = "Looking for Emulator...";
	
	String* ver = emulator->connectToEmulator(Globals::Props->getParam("emuPort"),hisp);

	if(Globals::Props->getParam("emuLiveEdit")->ToLower()->Equals(S"true"))
		liveedit = true;
	else
		liveedit = false;


	if(ver == NULL)
	{
		this->ostrichStatus->Text = "Emulator not found!";
		System::Windows::Forms::MessageBox::Show(S"No Emulator detected", S"BRE");
		return false;
	}
	this->ostrichStatus->Text = "Emulator Found!";

	emulating = true;
	if(Globals::Props->getBoolParam("emuAutoPut"))
		emulating = putRom(String::Concat(S"Emulator ",ver,S" found. Sending ROM to device..."));
	
	return emulating;
}

//emu threads...
void bre20::child::displayEmulationStatus()
{
	ProgressDialog * pd = new ProgressDialog(emulationLabel,counter[1]);
	pd->Visible = true;


	while(!senddone)
	{
		if(counter[1] > 1)
			pd->setMax(counter[1]);
		else
			counter[1] = 2;

		pd->setValue(counter[0]);
		
		if(pd->getPercent() >=100)
			break;

		System::Threading::Thread::Sleep(20);
	}

	pd->Visible = false;
	pd = NULL;

}

void bre20::child::writeEmuQueue()
{
	emusending = true;

	int errors = 0;
	int numwritten = 0;

	if(!liveedit)
		emulationLabel = "Downloading changes...";

	if(counter != NULL)
	{
		counter[0] = 0;
		counter[1] = emuqueue->Count;

		senddone = false;
		if(!liveedit && counter[1] > 1)
		{
			Thread * th = new Thread(new ThreadStart(this,displayEmulationStatus));
			th->Start();
		}

		while(emuqueue->Count > 0)
		{
			extraStatus->Text = emuqueue->Count.ToString();

			if(emuqueue->Count>counter[1])
				counter[1] = emuqueue->Count;

			//You're looking at this code and going "WTF? why does he check the 
			//queue count twice?" Well,I got a couple of exceptions saying that 
			//the queue was empty. Pretty stupid, eh? So now I check 2x. Werd.
			if(emuqueue->Count>0)
			{
				CharWrapArray* array = static_cast<CharWrapArray*>(emuqueue->Dequeue());
				unsigned char ch __gc[] = array->getChars();

				if(!emulator->writeHiddenBlock(ch,array->getLocation(),counter))
					errors++;

				numwritten++;
			}

			//Thread::Sleep(20);

			//lets wait for more queue entries...
			for(int i = 0; emuqueue->Count == 0 && i<5; i++)
			{
				//extraStatus->Text = "Waiting for more...";
				if(emuqueue->Count == 0)
					Thread::Sleep(5);
				else 
					break;
			}
		}
		senddone = true;
	}//end if counter null
	emusending = false;
	extraStatus->Text = "";

	if(!liveedit && errors>0)
		MessageBox::Show(String::Concat(S"There were ", errors.ToString(), S" errors during the write!"));
	else if(errors >0)
		extraStatus->Text = String::Concat(S"There were ",errors.ToString(),S" errors during the write!");
	else if(errors<=0){
		extraStatus->Text = "Finished Sending";
		Thread::Sleep(1000);
		extraStatus->Text = "";
	}
}

//emu events****************
void bre20::child::emu_disconnect(System::Object *  sender, System::EventArgs *  e)
{
	if(emulating)
	{
		menuItem3->Text = S"end";
		menuItem3_Click(NULL,NULL);

		ostrichStatus->Text = S"Emulator Disconnected: Searching...";
		findcon = true;
	}
}

void bre20::child::emu_reconnect(System::Object *  sender, System::EventArgs *  e)
{
	if(findcon)
	{
		ostrichStatus->Text = S"Emulator Found!";

		menuItem3->Text = S"";
		menuItem3_Click(NULL,NULL);

		findcon = false;
	}
}

//void bre20::child::insertColumn(int column)
//{
//	//((RomRep::EuroPw0Dev*)vtrom)->insertColumn(column);
//
//	//page1->setColumns(vtrom->getWidth());
//	//page2->setColumns(vtrom->getWidth());
//	//page3->setColumns(vtrom->getWidth());
//	//page4->setColumns(vtrom->getWidth());
//
//	//fillMap(page1, 0, 0);//fuel map index 0
//	//fillMap(page2, 1, 0);//fuel map index 1
//	//fillMap(page3, 0, 1);//ign map index 0
//	//fillMap(page4, 1, 1);//ign map index 1
//}
//void bre20::child::insertRow(int row)
//{
//
//}



//**************************************************************
//logging stuff
void bre20::child::showLogger(Object * logger)
{
	logItem->Enabled = false;
	
	childforms->Enqueue(logger);

	static_cast<Form*>(logger)->MdiParent = this->MdiParent;

	static_cast<Form*>(logger)->Closed += new System::EventHandler(this, logger_Close);
	static_cast<Form*>(logger)->Show();
}


void bre20::child::logger_Close(System::Object *  sender, System::EventArgs *  e)
{
	loggerrules = NULL;
	logItem->Enabled = true;
}

void bre20::child::logItem_Click(System::Object *  sender, System::EventArgs *  e)
{
	this->fireScriptEvent(this,"OnDataloggerOpened",new EventArgs());
	LoggerOpened(this);
	if(loggerrules == NULL)
		loggerrules = new ArrayList();
	Logger* log = new Logger(this, getAutoEdit(), loggerrules);
	this->fireScriptEvent(log,"OnDataloggerCreated",new EventArgs());
	LoggerCreated(log);
	showLogger(log);
}

void bre20::child::openLogAnalyzer(String* fname)
{
	this->fireScriptEvent(this,"OnLogAnalyzerOpened",new EventArgs());
	LogAnalyzerOpened(this);
	if(fname == NULL){
		OpenFileDialog* fd = new OpenFileDialog();
		fd->Filter = S"CSV files (*.csv)|*.csv|All files (*.*)|*.*";
		if(fd->ShowDialog() == DialogResult::OK)
			fname = fd->FileName;
	}
	if(fname != NULL && fname->Length>0)
	{
		LogAnalyzer * la = new LogAnalyzer(analyzerules,this,fname,getAutoEdit());
		this->fireScriptEvent(la,"OnLogAnalyzerCreated",new EventArgs());
		LogAnalyzerCreated(la);
		try
		{
			childforms->Enqueue(la);
			la->MdiParent = this->MdiParent;
			la->Visible = true;
		}
		catch(...){}
	}
}
void bre20::child::analyzeItem_Click(System::Object *  sender, System::EventArgs *  e)
{
	openLogAnalyzer(NULL);
}
