#pragma once

using namespace System;
using namespace System::ComponentModel;
using namespace System::Threading;
using namespace System::Collections;
using namespace System::Windows::Forms;
using namespace System::Drawing;

#include "string.h"
#include "GridPage.h"
#include "ParamPage.h"
#include "HondaEcu.h"

#include "gridpagearray.h"

#include "PortSetupDialog.h"
#include "EmulatorUtil.h"
#include "CharWrapArray.h"
#include "AutotuneOptions.h"
#include "adjustfuel.h"
#include "tabcontrolnew.h"
#include "boosteff.h"
#include "romrepeventargs.h"
#include "Extratables.h"

#include "ManagedUtil.h"
#include "ScriptControls.h"
#include "LabeledText.h"

#include "AutoEdit.h"
#include "Rule.h"


namespace bre20
{
	//! 
	//!
	//! This is the window class that each rom is contained in.
	//! 
	//! 

	public __delegate void ScriptFunctionCallHandler(Object *, ScriptFunctionCallArgs *);

	public __gc class child : public System::Windows::Forms::Form
	{
	public:
		//!event fired when a script function is called
		__event ScriptFunctionCallHandler * OnFunctionCalled;
		///event fired when the AutotuneOptions dialog is created.
		///The sender object is the AutotuneOptions object 
		__event System::EventHandler * OnAutotuneOptionsCreated;
		///event fired when the RtpOptions dialog is created.
		///The sender object is the RtpOptions object 
		__event System::EventHandler * OnEmulationOptionsCreated;

		///event fired when the Logger object is actually created.
		///The sender object is the Logger object 
		__event System::EventHandler * OnLoggerCreated;
		///event fired when the Logger object is opened, before being created.
		///The sender object is the child object 
		__event System::EventHandler * OnLoggerOpened;

		///event fired when the LogAnalyzer object is actually created
		///The sender object is the LogAnalyzer object 
		__event System::EventHandler * OnLogAnalyzerCreated;
		///event fired when the LogAnalyzer object is opened, before being created.
		///The sender object is the child object 
		__event System::EventHandler * OnLogAnalyzerOpened;

		///event fired when the ExtraTables object is actually created.
		///The sender object is the child object 
		__event System::EventHandler * OnExtraTablesCreated;

		//!fuel mode
		static int FUELMODE = 0;

		//!ignition mode
		static int IGNMODE = 1;

		//!Constructor
		child(RomRep::HondaEcu* ecu, int filelen, String * filename, ParamPage* ppage)
		{
			InitializeComponent();
			fname = filename;
			flen = filelen;

			liveedit = true;
			emulating = false;
			emusending = false;
			findcon = false;
			emuqueue = new Queue();
			childforms = new Queue();

			emulator = new EmulatorUtil();
			emulator->add_Disconnect( new System::EventHandler(this, emu_disconnect));
			emulator->add_Reconnect( new System::EventHandler(this, emu_reconnect));

			this->Text = filename;

			vtrom = ecu;
			vtrom->add_OnByteChanged(new RomRep::ByteChangeHandler(this, rom_ByteChange));
			
			if(vtrom != NULL)
			{			
				x = vtrom->getWidth();
				y = vtrom->getHeight();
			}

			this->Size = System::Drawing::Size(70 + (x*40), 100+(y*17));

			this->ostrichStatus->Text = S"Emulator session not started...";
			//this->ostrichStatus->Width = 40+(x*40);
			this->extraStatus->Text = "";
			this->child_Resize(NULL,NULL);

			pages = new GridPageArray(this->tabControl1,ppage,vtrom);
			pages->add_CellChanged(new SimpleGrid::GridChangedHandler(this,pages_CellChanged));
			pages->add_FireScriptEvent(new FireScriptEventHandler(this,pages_fireScript));

			eventSubscribe = new ArrayList(10);

			edited = false;
		}

		//!Destructor
		~child()
		{
		}

		//
		//scripting private stuff
		//
	private:
		GridPageArray* pages;
	//	TabPage* scriptPages[];
	//	ArrayList* extraMaps;
        ArrayList* eventSubscribe;

		/*
		bool AddControlToScriptPage(Control* con, int index)
		{
			try
			{
				if(scriptPages == NULL)
				{
					scriptPages = new TabPage*[1];
					scriptPages[0] = new TabPage(S"Script Functions");
					index = 0;

					this->tabControl1->TabPages->Add(scriptPages[0]);
					this->tabControl1->Refresh();
				}

				scriptPages[index]->Controls->Add(con);
				//!call the script control handler here...

				return true;
			}
			catch(Exception * e)
			{
				MessageBox::Show(String::Concat(e->Message,S"\n\n", e->InnerException->Message));
				return false;
			}
		}
		*/

		void ScriptFunctionCall(Object * sender, String* method, EventArgs* args)
		{
			if (OnFunctionCalled != NULL)
			{
				OnFunctionCalled(this, new ScriptFunctionCallArgs(method,sender,args));
			}
		}

		void script_Call(Object* sender, EventArgs* args)
		{
			script_Call(sender, ((ScriptControl*)sender)->getMethodName(),args);
		}

		void script_Call(Object* sender, SimpleGrid::CellEvent* args)
		{
			script_Call(sender, ((ScriptControl*)sender)->getMethodName(),args);
		}

		void script_Call(Object* sender, String* method, EventArgs* args)
		{
			try
			{
				ScriptFunctionCall(sender, method ,args);
			}
			catch(Exception * ex)
			{
				if(ex->InnerException != NULL)
					MessageBox::Show(String::Concat("Script function not called...\n\n", ex->Message, S"\n\n", ex->InnerException->Message));
				else
					MessageBox::Show(String::Concat("Script function not called...\n\n", ex->Message));
			}
		}

		void fireScriptEvent(Object* sender, String* eventname, EventArgs* args)
		{
			for(int i = 0; i<eventSubscribe->Count; i++)
			{
				EventSubscription* es = (EventSubscription*)eventSubscribe->get_Item(i);
				if(es->getEventName()->Equals(eventname))
					script_Call(sender,es->getMethodName(),args);
			}
		}

	public:
		//Public interface
		
		//!Returns a checkbox. 
		//!The handler is called when the check box is clicked.
		//!
		//!Please see \ref createControl "Control Creation" notes.
		CheckBox * CreateCheckBox(int x, int y, String* caption, String* handler);

		//!Returns a button. 
		//!The handler is called when the button is clicked.
		//!
		//!Please see \ref createControl "Control Creation" notes.
		Button * CreateButton(int x, int y, int width, int height, String* caption, String* handler);

		//!Returns a LabeletText, which is a textbox with a caption. Its really a Panel object.
		//!The handler happens when the user changes the text (either presses enter or leaves focus)
		//!
		//!Specify the proper mode, it has dumb-user handling. 
		//!
		//!TextBox modes (these ensure the user enters proper input):\n
		//!LabeledText.LTMODEINT    <- nothing other than ints\n
		//!LabeledText.LTMODEDOUBLE <- nothing other than decimal values\n
		//!LabeledText.LTMODETEXT   <- anything\n
		//!
		//!Please see \ref createControl "Control Creation" notes.
		bre20::LabeledText* CreateText(int x, int y, String* caption, String* handler, String* tooltip, int width, int mode);

		//!This creates a group box. USE THEM. Its the only object that should be added to BRE
		//!directly. These ae to be an alternative to the dialog boxes, which you can create
		//!but this is easier.
		//!This function adds the controls you have created to the group box then returns the group box 
		//!to you.
		//!
		//!Please see \ref createControl "Control Creation" notes.
		//!
		//!\param fullyQualClass is in the format "Namespace.Class" 
		GroupBox* CreateGroupBox(int x, int y, String* caption, String* fullyQualClass, int width, int height, Control* controls[]);

		//!This creates a grid object.
		//!The handler is called when a user changes a value in the grid.
		//!Handler event object is SimpleGrid.CellEvent
		//!
		//!Please see \ref createControl "Control Creation" notes.
		SimpleGrid::Grid * CreateGrid(int x, int y, String* handler, int columns, int rows);

		//!Adds a menu item to the menu item of choice. You can effectively add a 
		//!menu under ANY menu item in BRE (including functional items) by using
		//!its name in the mainMenu variable. BRE searches left to right and top to
		//!bottom on each menu item. So if there are duplicate names, the name to
		//!the left or higher at the top of some menu will be picked as the parent.
		//!
		//!The handler cannot be null even if the caption is "-" (for a separator) or 
		//!the menu being created is a top level menu. If this is the case you must 
		//!pass the namespace and classname in.
		//!
		//!Separators cannot be top level menus.
		//!
		//!If mainMenu == "" or null, the menu item will be added as a top level menu
		//!If the menu item you want to be under has text with an underlined letter
		//!then the mainMenu string must have a '&' in front of that letter: &Edit 
		//!for the edit menu. 
		//!
		//!Please see \ref createControl "Control Creation" notes.
		MenuItem* AddMenuItem(String* mainMenu, String* caption, String* handler);

		//!Adds a groupbox to the scripts tab.
		//!
		//!
		//!Please see \ref createControl "Control Creation" notes.
		bool AddControlGroup(GroupBox* con)
		{
			if(pages->AddControlToScriptPage(con,0))
			{
				this->fireScriptEvent(con,S"OnAddControl",new EventArgs());
				return true;
			}
			else
				return false;
		}

		//!
		//!This will allow you to subscribe to an event in BRE.
		//!
		//!Events to subscribe to are (the event param string):
		//!
		//!- \b "OnAddControl" - happens when a script adds a control
		//!returns new EventArgs
		//!
		//!- \b "OnChangeMapCell" - when one of the maps has a cell changed
		//!called AFTER BRE handles the event.
		//!returns SimpleGrid.CellEvent
		//!
		//!- \b "OnChangeByte" - when any byte in the rom is changed
		//!called AFTER BRE handles the event (after the byte
		//!goes to the ostrich)
		//!returns RomRep.ByteChangeArgs
		//!
		//!- \b "OnChildClosing" - when the child object is closing
		//!returns System.ComponentModel.CancelEventArgs
		//!
		//!- \b "OnColumnHeaderChanged" - When a grid page's (i.e. fuel map)
		//!column header is changed this is fired. The sender object is the 
		//!grid page which fired the event, and the EventArgs object is
		//!a SimpleGrid.CellEvent object with the column index in the CellEvent.Column property.
		//!
		//!- \b "OnRowHeaderChanged" - When a grid page's (i.e. fuel map)
		//!row header is changed this is fired. The sender object is the 
		//!grid page which fired the event, and the EventArgs object is
		//!a SimpleGrid.CellEvent object with the row index in the CellEvent.Row property.
		//!
		//!- \b "OnDataloggerCreated" - when the Logger form is actually CREATED.
		//!Returns a System.EventArgs and the Logger object as the sender.
		//!This event is for any extra processing that needs a valid logger
		//!(such as adding a new o2 type).
		//!
		//!Please see \ref createControl "Control Creation" notes.
		//!
		//!\param event the string name of the event
		//!\param handler the handler function in your script
		//!
		bool subscribeToEvent(String * event, String* handler);

		///add a Rule to the Logger. I recommend using subscribeToEvent() 
		///to subscribe to the OnDataloggerOpened event then call this in 
		///the handler function. After the Logger form is opened and closed,
		///the Rule list is deleted, so if you do not use the event, your rule
		///wont work properly.
		///
		///See \ref loggerusage "Datalogger Extension".
		void addLoggerRule(bre20::Rule* r){
			if(loggerrules == NULL)
				loggerrules = new ArrayList();
			loggerrules->Add(r);
		}

		//!returns the array of script tab pages.
		TabPage* getScriptTabPages()[]{return pages->getScriptTabPages();}

		//\returns the Logger object. If the logger has not been opened, the object 
		//will be created when this is called. 
		/*Logger* getDatalogger(){
			if(logger == NULL)
				logger = new Logger(this, getAutoEdit());
			return logger;
		}*/

		///\returns the AutoEdit object. If the logger or autotune options has not 
		///been opened the object will be created when this is called. 
		AutoEdit* getAutoEdit(){
			if(autotune == NULL)
				autotune = new AutoEdit(this);
			return autotune;
		}
		void openAutoTuneOptions()
		{
			autoTuneOp_Click(NULL,NULL);
		}
		void openLogAnalyzer(String* fname);

		void addLogAnalyzerRule(bre20::Rule* r){
			if(analyzerules == NULL)
				analyzerules = new ArrayList();
			analyzerules->Add(r);
		}
		
		//!gets the rom windows tabcontrol
		TabControl * getTabConrol(){return this->tabControl1;}

		//!Adds a map pair to the romWindow
		//!
		//!Please see \ref createControl "Control Creation" notes.
		//!
		//!\param fuelTabName name of the fuel tab
		//!\param fuelloc ROM location of the fuel map
		//!\param fRevScalarLoc ROM location of the fuel rev scalar (1 past the LAST byte in the scalar array for OBD0 VTEC)
		//!
		//!\param ignTabName name of the ignition tab
		//!\param ignloc ROM location of the ign map
		//!\param iRevScalarLoc ROM location of the ign rev scalar (1 past the LAST byte in the scalar array for OBD0 VTEC)
		//!
		//!\param x width of the maps
		//!\param y height of the maps
		//!\param copyIndex the index of the map to copy (really just for the rpm calc)
		//!\param copyContents boolean indicating whether BRE should copy the contents of the map
		int AddMapPair(String* fuelTabName,RomRep::MapLocs* flocs, String* ignTabName, 
			RomRep::MapLocs* ignlocs, int x, int y, int copyIndex, bool copyContents){
			return pages->AddMapPair(fuelTabName, flocs, ignTabName, ignlocs, x, y, copyIndex, 
				copyContents);
		}

		//!Creates a vanilla map with nothing in the headers.
		//!Handler happens when user changes a cell in the map
		//!
		//!Please see \ref createControl "Control Creation" notes.
		GridPage* AddMap(String* tabName, int columns, int rows, String* handler);

		//!this opens and parses a 'byte changes' type text file
		//!these can be generated by winhex. BRE will parse generic winhex comparison files.
		//!comments can be used in the files where ever the user pleases using the '//!' at 
		//!the beginning of the comment. 
		//!
		//!All bytes changed from this function will be sent to the ostrich.
		//!
		//!\param fname the file name
		//!\param makeChangeOnRight specifies which byte to change to: true is right byte, false is left byte
		bool parseByteChanges(String* fname, bool makeChangeOnRight);

		//!changes a column header for the specified map. 0 for non vtec, 1 for vtec, etc... 
		void changeColHeader(bool vtec, int col, String* header){
			pages->changeColHeader(vtec,col,header);}
		//	void insertColumn(int column);
		//	void insertRow(int row);

		//!scales the entire fuel maps with the specified multiplier
		void scaleFuelMaps(float multiplier){
			pages->scaleFuelMaps(multiplier);
		}

		//another multiplier for one cell at a time
//		void changeFuel(int col, int row, bool plus, bool small);

		//!grabs the col headers from the ROM and resets the grid headers
		void setAllColHeaders()
		{
			pages->setAllColHeaders();
		}

        //!sets the cell color of a cell in the map sprcified by the grid page 
//		void setCellColor(GridPage* page, int x, int y, float val, int mode);

		//!sets and returns the checksum 
		int setChecksum(){return vtrom->setChecksum();}

		//!gets the filename
		String * getFileName()
		{
			return fname;
		}

		//!gets the ROM type string
		String* getRomType()
		{
			return vtrom->getType();
		}
		
		//!returns a COPY of the file's byte array
		unsigned char getFile() __gc[] //!gets clean checksummed file
		{
			unsigned char rom __gc[];
			vtrom->setChecksum();
			rom = vtrom->getArray(vtrom->getFile(),0,flen);
			return rom;
		}

		//!byte length of the file
		int getFileLength()
		{
			return flen;
		}

		//!set the file name
		void setFileName(String * filename)
		{
			fname = filename;
			this->Text = fname;
		}

		//
		//external rom stuff for the logging, auto-tune, etc

		//!gets the ROM object. holy power.
		RomRep::HondaEcu * getRom()
		{
			return vtrom;
		}

		void setRom(RomRep::HondaEcu * romm)
		{
			vtrom = romm;
		}

		
		//!Sets the value in the fuel map.
		//!Maps indexed from left of tab order. base index = 0.
		//!index = 0 for non vtec, 1 for vtec
		void setFuelCell(int x, int y, float value, int index) 
		{
			pages->setFuelCell(x,y,value,index);
		}

		//!Sets the value in the fuel map. 
		//!Maps indexed from left of tab order. base index = 0
		float getFuelCell(int x, int y, int index) //!index = 0 for non vtec, 1 for vtec
		{
			return pages->getFuelCell(x,y,index);
		}
		

		///Adds a RomRep.ByteChange object to the emulation queue.
		///Downloads if live edit and enqueues if manual 
		void addByteChange(RomRep::ByteChange* bc)
		{
			unsigned char ch __gc[] = new unsigned char __gc[1];

			if(emulating && bc != NULL && emulator != NULL)
			{
				ch[0] = bc->getByte();
				CharWrapArray* chw = new CharWrapArray(ch,bc->getLocation());
				emuqueue->Enqueue(chw);
			}
			if(emulating && liveedit && emulator != NULL)
				dwnldChangesItm_Click(NULL,NULL);
			/*
			if(liveedit && emulating && bc != NULL && emulator != NULL)
			{
				ch[0] = bc->getByte();

				//emulator->writeHiddenBlock(ch,bc->getLocation(),NULL);
				CharWrapArray* chw = new CharWrapArray(ch,bc->getLocation());
				emuqueue->Enqueue(chw);
				dwnldChangesItm_Click(NULL,NULL);
			}
			else if(!liveedit && emulating && bc != NULL && emulator != NULL)
			{
				ch[0] = bc->getByte();
				CharWrapArray* chw = new CharWrapArray(ch,bc->getLocation());

				emuqueue->Enqueue(chw);
			}
			*/
		}

		void setRomSaved()
		{
			setRomEdited(false);
		}
		bool getWasEdited()
		{
			return edited;
		}

		void setCellColor(GridPage* page, int x, int y, float val, int mode)
		{ pages->setCellColor(page,x,y,val,mode); }

		//***********************************************************************
        
	protected: 
		void Dispose(Boolean disposing)
		{
			if (disposing && components)
			{
				components->Dispose();
			}
			__super::Dispose(disposing);
		}
        
	private:

		int tabs;
		int x, y;
		String* fname;
		int flen;
		Queue * childforms;
	//!	CSerialPort * comport;

		bool edited;

		//!for emulation...
		Queue* emuqueue;
		EmulatorUtil * emulator;
		int counter __gc[]; //![0] has count val, [1] has max
		String * emulationLabel;
		bool senddone, emulating, emusending, findcon;
		MenuItem *  emuOptions;

		bool liveedit;

		void displayEmulationStatus();
		void writeEmuQueue();
		//!end emulation stuff

		//datalogging...
	//	Logger* logger;
		ArrayList* loggerrules;
		ArrayList* analyzerules;
		AutoEdit* autotune;
		//end datalogging
		
		RomRep::HondaEcu * vtrom;

//		void setupTabs(RomRep::HondaEcu* ecu);
//		void fillMap(GridPage* page, int index, int mode); //!mode = 1 for fuel, 2 for ign

//		bre20::GridPage * pages[];
//		ParamPage* paramPage;

		bool loadEmulator();

private: System::Windows::Forms::MainMenu *  mainMenu1;
private: System::Windows::Forms::MenuItem *  menuItem1;
private: System::Windows::Forms::MenuItem *  copyItem;
private: System::Windows::Forms::MenuItem *  pasteItem;
private: System::Windows::Forms::MenuItem *  menuItem2;
private: System::Windows::Forms::MenuItem *  menuItem3;
private: System::Windows::Forms::MenuItem *  menuItem4;

private: System::Windows::Forms::MenuItem *  menuItem5;
private: System::Windows::Forms::MenuItem *  viewGraph;
private: System::Windows::Forms::MenuItem *  viewTable;
private: System::Windows::Forms::MenuItem *  menuItem6;
private: System::Windows::Forms::MenuItem *  logItem;
private: System::Windows::Forms::MenuItem *  analyzeItem;
private: System::Windows::Forms::MenuItem *  menuItem7;
private: System::Windows::Forms::MenuItem *  verifyItm;
private: System::Windows::Forms::MenuItem *  putItm;
private: System::Windows::Forms::MenuItem *  dwnldChangesItm;
private: System::Windows::Forms::Timer *  timer1;
private: System::Windows::Forms::StatusBar *  status;
private: System::Windows::Forms::StatusBarPanel *  ostrichStatus;
private: System::Windows::Forms::StatusBarPanel *  extraStatus;
private: System::Windows::Forms::MenuItem *  autoTuneOp;
private: System::Windows::Forms::MenuItem *  adjustFuelMain;
private: System::Windows::Forms::TabControl *  tabControl1;
private: System::ComponentModel::IContainer *  components;
private: System::Windows::Forms::MenuItem *  extraTables;

		//!/ <summary>
		//!/ Required method for Designer support - do not modify
		//!/ the contents of this method with the code editor.
		//!/ </summary>
		void InitializeComponent(void)
		{
			this->components = new System::ComponentModel::Container();
			System::Resources::ResourceManager *  resources = new System::Resources::ResourceManager(__typeof(bre20::child));
			this->mainMenu1 = new System::Windows::Forms::MainMenu();
			this->menuItem1 = new System::Windows::Forms::MenuItem();
			this->adjustFuelMain = new System::Windows::Forms::MenuItem();
			this->copyItem = new System::Windows::Forms::MenuItem();
			this->pasteItem = new System::Windows::Forms::MenuItem();
			this->menuItem4 = new System::Windows::Forms::MenuItem();
			this->autoTuneOp = new System::Windows::Forms::MenuItem();
			this->menuItem5 = new System::Windows::Forms::MenuItem();
			this->viewGraph = new System::Windows::Forms::MenuItem();
			this->viewTable = new System::Windows::Forms::MenuItem();
			this->menuItem6 = new System::Windows::Forms::MenuItem();
			this->logItem = new System::Windows::Forms::MenuItem();
			this->analyzeItem = new System::Windows::Forms::MenuItem();
			this->extraTables = new System::Windows::Forms::MenuItem();
			this->menuItem2 = new System::Windows::Forms::MenuItem();
			this->menuItem3 = new System::Windows::Forms::MenuItem();
			this->emuOptions = new System::Windows::Forms::MenuItem();
			this->menuItem7 = new System::Windows::Forms::MenuItem();
			this->verifyItm = new System::Windows::Forms::MenuItem();
			this->putItm = new System::Windows::Forms::MenuItem();
			this->dwnldChangesItm = new System::Windows::Forms::MenuItem();
			this->timer1 = new System::Windows::Forms::Timer(this->components);
			this->status = new System::Windows::Forms::StatusBar();
			this->ostrichStatus = new System::Windows::Forms::StatusBarPanel();
			this->extraStatus = new System::Windows::Forms::StatusBarPanel();
			this->tabControl1 = new System::Windows::Forms::TabControl();
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->ostrichStatus))->BeginInit();
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->extraStatus))->BeginInit();
			this->SuspendLayout();
			// 
			// mainMenu1
			// 
			System::Windows::Forms::MenuItem* __mcTemp__1[] = new System::Windows::Forms::MenuItem*[4];
			__mcTemp__1[0] = this->menuItem1;
			__mcTemp__1[1] = this->menuItem5;
			__mcTemp__1[2] = this->menuItem6;
			__mcTemp__1[3] = this->menuItem2;
			this->mainMenu1->MenuItems->AddRange(__mcTemp__1);
			// 
			// menuItem1
			// 
			this->menuItem1->Index = 0;
			System::Windows::Forms::MenuItem* __mcTemp__2[] = new System::Windows::Forms::MenuItem*[5];
			__mcTemp__2[0] = this->adjustFuelMain;
			__mcTemp__2[1] = this->copyItem;
			__mcTemp__2[2] = this->pasteItem;
			__mcTemp__2[3] = this->menuItem4;
			__mcTemp__2[4] = this->autoTuneOp;
			this->menuItem1->MenuItems->AddRange(__mcTemp__2);
			this->menuItem1->MergeOrder = 1;
			this->menuItem1->Text = S"&Edit";
			// 
			// adjustFuelMain
			// 
			this->adjustFuelMain->Index = 0;
			this->adjustFuelMain->Shortcut = System::Windows::Forms::Shortcut::CtrlShiftF;
			this->adjustFuelMain->Text = S"Adjust Fuel... ";
			this->adjustFuelMain->Click += new System::EventHandler(this, adjustFuelMain_Click);
			// 
			// copyItem
			// 
			this->copyItem->Index = 1;
			this->copyItem->Text = S"Copy";
			this->copyItem->Click += new System::EventHandler(this, menuItem2_Click);
			// 
			// pasteItem
			// 
			this->pasteItem->Index = 2;
			this->pasteItem->Text = S"Paste";
			this->pasteItem->Click += new System::EventHandler(this, pasteItem_Click);
			// 
			// menuItem4
			// 
			this->menuItem4->Index = 3;
			this->menuItem4->Text = S"-";
			// 
			// autoTuneOp
			// 
			this->autoTuneOp->Index = 4;
			this->autoTuneOp->Shortcut = System::Windows::Forms::Shortcut::CtrlShiftA;
			this->autoTuneOp->Text = S"Autotune Options...";
			this->autoTuneOp->Click += new System::EventHandler(this, autoTuneOp_Click);
			// 
			// menuItem5
			// 
			this->menuItem5->Index = 1;
			System::Windows::Forms::MenuItem* __mcTemp__3[] = new System::Windows::Forms::MenuItem*[2];
			__mcTemp__3[0] = this->viewGraph;
			__mcTemp__3[1] = this->viewTable;
			this->menuItem5->MenuItems->AddRange(__mcTemp__3);
			this->menuItem5->MergeOrder = 2;
			this->menuItem5->Text = S"&View";
			// 
			// viewGraph
			// 
			this->viewGraph->Index = 0;
			this->viewGraph->Shortcut = System::Windows::Forms::Shortcut::F2;
			this->viewGraph->Text = S"Graph";
			this->viewGraph->Click += new System::EventHandler(this, viewGraph_Click);
			// 
			// viewTable
			// 
			this->viewTable->Checked = true;
			this->viewTable->Index = 1;
			this->viewTable->Text = S"Table";
			this->viewTable->Click += new System::EventHandler(this, viewTable_Click);
			// 
			// menuItem6
			// 
			this->menuItem6->Index = 2;
			System::Windows::Forms::MenuItem* __mcTemp__4[] = new System::Windows::Forms::MenuItem*[3];
			__mcTemp__4[0] = this->logItem;
			__mcTemp__4[1] = this->analyzeItem;
			__mcTemp__4[2] = this->extraTables;
			this->menuItem6->MenuItems->AddRange(__mcTemp__4);
			this->menuItem6->MergeOrder = 3;
			this->menuItem6->Text = S"Tools";
			// 
			// logItem
			// 
			this->logItem->Index = 0;
			this->logItem->Shortcut = System::Windows::Forms::Shortcut::CtrlL;
			this->logItem->Text = S"Datalogger...";
			this->logItem->Click += new System::EventHandler(this, logItem_Click);
			// 
			// analyzeItem
			// 
			this->analyzeItem->Index = 1;
			this->analyzeItem->Shortcut = System::Windows::Forms::Shortcut::CtrlShiftO;
			this->analyzeItem->Text = S"Analyze Log...";
			this->analyzeItem->Click += new System::EventHandler(this, analyzeItem_Click);
			// 
			// extraTables
			// 
			this->extraTables->Index = 2;
			this->extraTables->Shortcut = System::Windows::Forms::Shortcut::CtrlShiftE;
			this->extraTables->Text = S"Extra Tables...";
			this->extraTables->Click += new System::EventHandler(this, extraTables_Click);
			// 
			// menuItem2
			// 
			this->menuItem2->Index = 3;
			System::Windows::Forms::MenuItem* __mcTemp__5[] = new System::Windows::Forms::MenuItem*[6];
			__mcTemp__5[0] = this->menuItem3;
			__mcTemp__5[1] = this->emuOptions;
			__mcTemp__5[2] = this->menuItem7;
			__mcTemp__5[3] = this->verifyItm;
			__mcTemp__5[4] = this->putItm;
			__mcTemp__5[5] = this->dwnldChangesItm;
			this->menuItem2->MenuItems->AddRange(__mcTemp__5);
			this->menuItem2->MergeOrder = 4;
			this->menuItem2->Text = S"Emulation";
			// 
			// menuItem3
			// 
			this->menuItem3->Index = 0;
			this->menuItem3->Shortcut = System::Windows::Forms::Shortcut::CtrlR;
			this->menuItem3->Text = S"Begin Emulator Session...";
			this->menuItem3->Click += new System::EventHandler(this, menuItem3_Click);
			// 
			// emuOptions
			// 
			this->emuOptions->Index = 1;
			this->emuOptions->Shortcut = System::Windows::Forms::Shortcut::CtrlShiftR;
			this->emuOptions->Text = S"Emulation Options...";
			this->emuOptions->Click += new System::EventHandler(this, emuOptions_Click);
			// 
			// menuItem7
			// 
			this->menuItem7->Index = 2;
			this->menuItem7->Text = S"-";
			// 
			// verifyItm
			// 
			this->verifyItm->Enabled = false;
			this->verifyItm->Index = 3;
			this->verifyItm->Shortcut = System::Windows::Forms::Shortcut::CtrlShiftV;
			this->verifyItm->Text = S"Verify Rom";
			this->verifyItm->Click += new System::EventHandler(this, verifyItm_Click);
			// 
			// putItm
			// 
			this->putItm->Enabled = false;
			this->putItm->Index = 4;
			this->putItm->Shortcut = System::Windows::Forms::Shortcut::CtrlShiftP;
			this->putItm->Text = S"Put Rom";
			this->putItm->Click += new System::EventHandler(this, putItm_Click);
			// 
			// dwnldChangesItm
			// 
			this->dwnldChangesItm->Enabled = false;
			this->dwnldChangesItm->Index = 5;
			this->dwnldChangesItm->Shortcut = System::Windows::Forms::Shortcut::F4;
			this->dwnldChangesItm->Text = S"Download Changes";
			this->dwnldChangesItm->Click += new System::EventHandler(this, dwnldChangesItm_Click);
			// 
			// timer1
			// 
			this->timer1->Interval = 10000;
			this->timer1->Tick += new System::EventHandler(this, timer1_Tick);
			// 
			// status
			// 
			this->status->Location = System::Drawing::Point(0, 304);
			this->status->Name = S"status";
			System::Windows::Forms::StatusBarPanel* __mcTemp__6[] = new System::Windows::Forms::StatusBarPanel*[2];
			__mcTemp__6[0] = this->ostrichStatus;
			__mcTemp__6[1] = this->extraStatus;
			this->status->Panels->AddRange(__mcTemp__6);
			this->status->ShowPanels = true;
			this->status->Size = System::Drawing::Size(600, 24);
			this->status->TabIndex = 1;
			// 
			// ostrichStatus
			// 
			this->ostrichStatus->Text = S"Emulator Status";
			this->ostrichStatus->Width = 500;
			// 
			// extraStatus
			// 
			this->extraStatus->Text = S"extraStatus";
			// 
			// tabControl1
			// 
			this->tabControl1->Anchor = (System::Windows::Forms::AnchorStyles)(((System::Windows::Forms::AnchorStyles::Top | System::Windows::Forms::AnchorStyles::Bottom) 
				| System::Windows::Forms::AnchorStyles::Left) 
				| System::Windows::Forms::AnchorStyles::Right);
			this->tabControl1->Location = System::Drawing::Point(0, 0);
			this->tabControl1->Name = S"tabControl1";
			this->tabControl1->SelectedIndex = 0;
			this->tabControl1->Size = System::Drawing::Size(600, 304);
			this->tabControl1->TabIndex = 2;
			this->tabControl1->TabStop = false;
			// 
			// child
			// 
			this->AutoScaleBaseSize = System::Drawing::Size(5, 13);
			this->ClientSize = System::Drawing::Size(600, 328);
			this->Controls->Add(this->tabControl1);
			this->Controls->Add(this->status);
			this->Icon = (__try_cast<System::Drawing::Icon *  >(resources->GetObject(S"$this.Icon")));
			this->Menu = this->mainMenu1;
			this->Name = S"child";
			this->Text = S"child";
			this->Resize += new System::EventHandler(this, child_Resize);
			this->Closing += new System::ComponentModel::CancelEventHandler(this, child_Closing);
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->ostrichStatus))->EndInit();
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->extraStatus))->EndInit();
			this->ResumeLayout(false);

		}		
private: 
	
		void AutotuneOptionsCreated(Object* sender)
		{
			if (OnAutotuneOptionsCreated != NULL)
				OnAutotuneOptionsCreated(sender, new EventArgs());
		}

		void EmulationOptionsCreated(Object* sender)
		{
			if (OnEmulationOptionsCreated != NULL)
				OnEmulationOptionsCreated(sender, new EventArgs());
		}

		void LoggerOpened(Object* sender)
		{
			if (OnLoggerOpened != NULL)
				OnLoggerOpened(sender, new EventArgs());
		}
		void LoggerCreated(Object* sender)
		{
			if (OnLoggerCreated != NULL)
				OnLoggerCreated(sender, new EventArgs());
		}

		void LogAnalyzerOpened(Object* sender)
		{
			if (OnLogAnalyzerOpened != NULL)
				OnLogAnalyzerOpened(sender, new EventArgs());
		}
		void LogAnalyzerCreated(Object* sender)
		{
			if (OnLogAnalyzerCreated != NULL)
				OnLogAnalyzerCreated(sender, new EventArgs());
		}

		void ExtraTablesCreated(Object* sender)
		{
			if (OnExtraTablesCreated != NULL)
				OnExtraTablesCreated(sender, new EventArgs());
		}



		/*
		void fuelPage_CellChanged(System::Object *  sender, System::EventArgs *  e);
		void ignPage_CellChanged(System::Object *  sender, System::EventArgs *  e);
		void paramPage_CellChanged(System::Object *  sender, System::EventArgs *  e);
		void page_ColHeaderChanged(System::Object *  sender, System::EventArgs *  e);
		void page_RowHeaderChanged(System::Object *  sender, System::EventArgs *  e);
		void changeCell(GridPage* page, int index, EventArgs* e, int mode);//!1 for fuel, 2 for ign
		void tabControl1_Resize(System::Object *  sender, System::EventArgs *  e){}
		void tabControl1_MouseUp(System::Object *  sender, System::Windows::Forms::MouseEventArgs *  e){}
		*/
		
		void textBox1_Leave(System::Object *  sender, System::EventArgs *  e){}

		void menuItem2_Click(System::Object *  sender, System::EventArgs *  e)
		{//!copy click
			pages->copy();
		}

		void pasteItem_Click(System::Object *  sender, System::EventArgs *  e)
		{
			pages->paste();
		}

		//!ostrich is clicked...
		void menuItem3_Click(System::Object *  sender, System::EventArgs *  e)
		{
			if(this->menuItem3->Text->ToLower()->IndexOf("end") < 0)
			{
				if(loadEmulator())
				{
					this->menuItem3->Text = S"End Emulator Session";

					verifyItm->Enabled = true;
					dwnldChangesItm->Enabled = true;
					putItm->Enabled = true;

					resetOstrichLabel();
				}
			}
			else
			{
				this->menuItem3->Text = S"Begin Emulator Session...";
				emulating = false;
				
				resetOstrichLabel();

				verifyItm->Enabled = false;
				dwnldChangesItm->Enabled = false;
				putItm->Enabled = false;
				
				if(emulator != NULL)
				{
					emulator->closeAll();
				}
			}
		}

		void setRomEdited(bool wasEdited)
		{
			if(wasEdited == edited)
				return;

			edited = wasEdited;

			if(edited){
				if(!this->Text->EndsWith(S" *"))
					this->Text = String::Concat(this->Text, S" *");
			}else
				this->Text = this->Text->Replace(S" *","");
		}
		
		//!when a byte is changed
		void rom_ByteChange(Object* sender, RomRep::ByteChangeArgs * bce)
		{
			setRomEdited(true);
			this->addByteChange(bce->getByteChange());
			this->fireScriptEvent(sender,S"OnChangeByte",bce);
		}

		void resetOstrichLabel()
		{
			
			this->Text = this->Text->Replace(S"LIVE EDITING - ",S"")->Replace(S"EMULATOR - ",S"");
			ostrichStatus->Text = S"Emulator session ended";
			
			if(liveedit && emulating)
			{
				this->Text = String::Concat(S"LIVE EDITING - ",this->Text);
				ostrichStatus->Text = S"Emulator connected: Live Editing";
			}
			else if(!liveedit && emulating)
			{
				this->Text = String::Concat(S"EMULATOR - ",this->Text);
				ostrichStatus->Text = S"Emulator connected: Manual Download";

			}

		}
		bool putRom(String *lbl);

		void viewGraph_Click(System::Object *  sender, System::EventArgs *  e)
		{
			viewTable->Checked = !viewTable->Checked;
			viewGraph->Checked = viewTable->Checked;
			pages->setViewMode(viewTable->Checked);
		}

		void viewTable_Click(System::Object *  sender, System::EventArgs *  e)
		{
			viewGraph_Click(NULL,NULL);
		}

		
		void child_Closing(System::Object *  sender, System::ComponentModel::CancelEventArgs *  e)
		{

			if(emulator != NULL)
				emulator->closeAll();
			emulator = NULL;

			//!close all child forms;
			while(childforms->Count>0)
			{
				Form* frm = static_cast<Form*>(childforms->Dequeue());
				frm->Close();

			}

			this->fireScriptEvent(sender,S"OnChildClosing",new EventArgs());
			System::GC::Collect();
		}

		void emu_disconnect(System::Object *  sender, System::EventArgs *  e);
		void emu_reconnect(System::Object *  sender, System::EventArgs *  e);


//!************************************************************************************
//!logging stuff
private:
		void showLogger(Object * logger);
		void logger_Close(System::Object *  sender, System::EventArgs *  e);
		void logItem_Click(System::Object *  sender, System::EventArgs *  e);
		void analyzeItem_Click(System::Object *  sender, System::EventArgs *  e);
		void verifyItm_Click(System::Object *  sender, System::EventArgs *  e);
		void putItm_Click(System::Object *  sender, System::EventArgs *  e);
		void dwnldChangesItm_Click(System::Object *  sender, System::EventArgs *  e);
		void emuOptions_Click(System::Object *  sender, System::EventArgs *  e);
		void timer1_Tick(System::Object *  sender, System::EventArgs *  e){}

		void pages_CellChanged(System::Object *  sender, SimpleGrid::CellEvent *  e)
		{
			this->fireScriptEvent(sender,S"OnChangeMapCell",e);
		}

		void pages_fireScript(System::Object *  sender, FireScriptEventArgs *  e)
		{
			this->fireScriptEvent(sender,e->getName(),e->getArgs());
		}
		
		void child_Resize(System::Object *  sender, System::EventArgs *  e)
		{
			int wd = this->Size.Width-30;
			ostrichStatus->Width = wd*2/3;
			extraStatus->Width = wd-ostrichStatus->Width;
		}

		void autoTuneOp_Click(System::Object *  sender, System::EventArgs *  e)
		{
			AutotuneOptions* aop = new AutotuneOptions(getAutoEdit(), this);

			//do stuff with the blockout...

			AutotuneOptionsCreated(aop);
			aop->ShowDialog();
		}

		void adjustFuelMain_Click(System::Object *  sender, System::EventArgs *  e)
		{
			pages->doFuelAdjust();
		}

		void boostEff_Click(System::Object *  sender, System::EventArgs *  e)
		{
			pages->doBoostEfficiency();
		}//!end boost efficiency

		void extraTables_Click(System::Object *  sender, System::EventArgs *  e)
		{
			ExtraTables* et = new ExtraTables(this->getRom());
			childforms->Enqueue(et);
			this->ExtraTablesCreated(et);
			et->MdiParent = this->MdiParent;
			et->Show();
		}

};
}


/*! \page createControl Notes on Creating Controls
	
	Controls can be created by BRE for you to use in your scripts. The 
	functions provided in the bre20.child class
	make script writing simple. Instead of writing a 5-10 lines to create 
	a control, all you need is one line (2 if you count the child assignment). Example:

	\code
	child romWind = (child)args[1];

	//make CheckBox
	CheckBox chk = romWind.CreateCheckBox(10,24,"Hello!!","Namespace.ClassName.chk_Click");
	
	//make LabeledText box
	LabeledText txt = romWind.CreateText(20,85,"Type Below",
		"Namespace.ClassName.txt_Change","ToolTip",70,LabeledText.LTMODEINT);
	\endcode
	
	You must 
	specify a handler function for most controls  
	BRE creates for you. These handler fuction names are passed into the child (rom)
	window on object creation. Notice the \c Namespace.ClassName.txt_Change in the 
	last example. It is the handler function for the LabeledText box.
	
	Handlers are (instance, NOT static) funtions that are mapped to the most
	used event on that control. i.e. a check box handler are invoked when 
	the checkbox changes from checked to unchecked or vise versa, a text box
	handler is fired when the text is changed (the user presses enter or leaves 
	the text box (focus wise)), a Button clicked, etc. 

	The handler function definition from the last code example would then look like:

	\code
	//the params are:
	//args[0] = main window
	//args[1] = rom window (the child object)
	//args[2] = object that sent the event 
	//args[3] = event object (will be base class of EventArgs)
	public object txt_Change(params object[] args)
	{
		//do stuff here

		//you must return something.
		return null;
	}
	\endcode
	
	Case IS important: \c NameSpace.Class.callme is NOT the same as \c namespace.cLass.calLme
	
	Each time a control is created, the handler format is checked and it is made sure the
	function is really there. If either is bad, you will get a null instead of 
	the created object. 
	
	Some of the objects created do not require a handler function. In this case 
	you must pass in the \c namespace.class name so BRE knows who is creating the control. 
	The \c namespace.class string is checked for 
	validity and if the class doesnt exist, you will get a null object. GroupBoxes and top 
	level MenuItems fall in this category.
	
	None of the passed in string variables can be null except in the case of MenuItems. If any of 
	them are, you will be returned a null object instead of your control. Also, if, 
	for some reason, the creation process fails, you will get a null.

	If you are looking for functionality the child class doesn't have, 
	you can make your own dialog boxes or make your own GUI objects and have all the handler info
	in your script.
	
	
*/