#include "stdafx.h"
#include "map.h"

RomRep::HeaderCalculator::HeaderCalculator(Object* map, Object* hondaecu)
{
	//this will throw an invalid cast exception
	m_map = static_cast<RomRep::Map*>(map);
	m_hondaecu = static_cast<RomRep::Map*>(hondaecu);
};
///Splits up the MapHeaderChangeArgs calls into setRowHeader and
///setColumnHeader calls. This does not need to be overriden
void RomRep::HeaderCalculator::mapHeaderChanged(MapHeaderChangeArgs* args)
{
	//called from some outside source. This function is responsible for figuring out the 
	//event type, converting the values from string to raw, then sending byte change requests.
	//Special args indicate we should actually parse the headers. 
	if(args->getMarker() != CellChangeArgs::Marker::ARGS_SPECIAL)
		return;

	Map* m = static_cast<RomRep::Map*>(m_map);

	int x = args->getX();
	int y = args->getY();
	bool col = false;
	bool all = false;

	if(y < 0){ //col event
		col = true;
		if(y == RomRep::Map::HEADER_EVENT_ALL) 
			all = true;
	}
	else if(x < 0) //row event
		if(x == RomRep::Map::HEADER_EVENT_ALL) 
			all = true;

	if(!all){
		if(col)
			m->SendByteChangeRequest(setColumnHeader(x,args->getValue()[0]));
		else
			m->SendByteChangeRequest(setRowHeader(y,args->getValue()[0]));
	}
	else{
		if(col)
			for(int i = 0; i< args->getValue()->Length; i++)
				m->SendByteChangeRequest(setColumnHeader(i,args->getValue()[i]));
		else
			for(int i = 0; i< args->getValue()->Length; i++)
				m->SendByteChangeRequest(setRowHeader(i,args->getValue()[i]));
	}
}

//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
/// The Map class

RomRep::Map::Map()
{
	thefile = NULL;
	map = new unsigned char __gc[1,1];

	vac = new String * [1];
	rpm = new String * [1];

	x = 1;
	y = 1;
	location = 0;
	name = "";
	headerCalculators = new HeaderCalculatorPair();
}
RomRep::Map::Map(int x1, int y1, unsigned int loc, int ind, int mod)
{
	mode = mod;
	index = ind;
	thefile = NULL;
	location = loc;

	map = new unsigned char __gc[x1,y1];

	vac = new String * [x1];
	rpm = new String * [y1];

	x = x1;
	y = y1;
	name = "";
	headerCalculators = new HeaderCalculatorPair();
}
RomRep::Map::Map(int x1, int y1, unsigned int loc,unsigned char fil __gc[], int ind, int mod)
{
	mode = mod;
	index = ind;
	thefile = fil; //shallow copy for a reason...
	location = loc;

	map = new unsigned char __gc[x1,y1];

	vac = new String * [x1];
	rpm = new String * [y1];

	x = x1;
	y = y1;

	name = "";
	headerCalculators = new HeaderCalculatorPair();
}
RomRep::Map::~Map(){}//destruction!... Muahahahahahahahahaaaahahaha!!11!!

//---------------------------end constructors---------------------

float RomRep::Map::getMap() __gc[,]
{
	int i, j;
	float mapp __gc[,]= new float __gc[x,y];

	for(i = 0; i<x; i++)
		for(j = 0; j<y; j++)
			mapp[i,j] = calculate(i,j,map[i,j]);
	return mapp;
}

float RomRep::Map::getVal(int x1, int y1)
{
	if(x1<x && x1>(-1) && y1<y && y1>(-1)) //if its inside the map's bounds
		return calculate(x1,y1,map[x1,y1]);
	else
		return -1;
}

RomRep::ByteChange* RomRep::Map::setVal(int x1, int y1, float val)
{
	int noo = calculate(x1,y1,val);
	if(x1<x && x1>(-1) && y1<y && y1>(-1) && noo>=0 && noo<256) //if its inside the map's bounds
	{
		map[x1,y1] = noo;
		MapChanged(x1, y1, noo);
		return new ByteChange(this->getCellLocation(x1,y1),(char)noo);
	}
	else
		return NULL;
}

void RomRep::Map::populate(unsigned char mapp __gc[,])
{
	int i, j;

	for(i = 0; i<x; i++)
		for(j = 0; j<y; j++)
			map[i,j] = mapp[i,j];
}

//gets the location of the cell (x1,y1) in the rom file
unsigned int RomRep::Map::getCellLocation(int x1, int y1){
	return (unsigned int)((x * y1) + x1 + location); 
}

//gets the location of the cell (x1,y1) in the rom file
unsigned int RomRep::Map::getCellLocation(int x1, int y1, int cols, int loc){
	return (unsigned int)((cols * y1) + x1 + loc); 
}

void RomRep::Map::byteChangeNotify(ByteChange* bc)
{
	//see if the location is between location and location+x*y;
	//if so, update the map and send an event.
	if(bc->getLocation() >= (int)location && bc->getLocation() < (int)location+x*y)
	{
		int off = bc->getLocation() - (int)location;
		//((x * y1) + x1
		int x1 = off%x;
		int y1 = off/x;

		map[x1,y1] = bc->getByte();

		this->RealMapChanged(x1,y1,bc->getByte());
	}

	setHeaders(headerCalculators->GetRowCalculator()->byteChangeNotify(bc));
	setHeaders(headerCalculators->GetColumnCalculator()->byteChangeNotify(bc));
}

///this will blindly place the headers into our vectors and generate events.
void RomRep::Map::setHeaders(RomRep::MapHeaderChangeArgs* args)
{
	if( args == NULL )
		return;

    int xx = args->getX();
	int yy = args->getY();
	bool col = false;
	bool all = false;

	if(xx == RomRep::Map::HEADER_EVENT_ALL){
		col = true;
		all = true;
	}
	else if(yy == RomRep::Map::HEADER_EVENT_ALL)
		all = true;
	else if(xx > -1)
		col = true;

	if(!all){
		if(col)
			vac[xx] = args->getValue()[0];
		else
			rpm[yy] = args->getValue()[0];
	}
	else{
		if(col)
			for(int i = 0; i<x && i<args->getValue()->Length; i++)
				vac[i] = args->getValue()[i];
		else
			for(int i = 0; i<y && i<args->getValue()->Length; i++)
				rpm[i] = args->getValue()[i];
	}
	MapHeaderChanged(args);
}
void RomRep::Map::SendByteChangeRequest(ByteChange* changes[])
{
	if(changes != NULL)
		for(int i = 0; i< changes->Length; i++)
			if(changes[i] != NULL)
				SendByteChangeRequest(changes[i]);
}
//********************************************************
//*** Header editing

void RomRep::Map::setRowHeaderCalculator(HeaderCalculator* he){
	if( he != NULL ){
		headerCalculators->SetRowCalculator(he);
		setHeaders(he->calculateHeaders(false, Map::HEADER_EVENT_ALL));
	}
}
void RomRep::Map::setColumnHeaderCalculator(HeaderCalculator* he){
	if( he != NULL ){
		headerCalculators->SetColumnCalculator(he);
		setHeaders(he->calculateHeaders(true, Map::HEADER_EVENT_ALL));
	}
}

bool RomRep::Map::canEditHeader(bool column, int columnOrRow)
{ 
	HeaderCalculator* he;
	if(column)
		he = headerCalculators->GetColumnCalculator();
	else
		he = headerCalculators->GetRowCalculator();
	return he->canEditHeader(column,columnOrRow);
}

//These only send byte changes because the byte changes will generate events which the 
//the map will send to the HeaderEditors and eventually get a string back to dump in the 
//proper header array.
void RomRep::Map::setRowHeader(int row, String* val){
	SendByteChangeRequest(headerCalculators->GetRowCalculator()->setRowHeader(row,val));
}
void RomRep::Map::setColumnHeader(int col, String* val){
	SendByteChangeRequest(headerCalculators->GetColumnCalculator()->setColumnHeader(col,val));
}

void RomRep::Map::mapHeaderChanged(Object* sender, MapHeaderChangeArgs* args){
	if(args->getX() != Map::HEADER_EVENT_NONE)
		headerCalculators->GetColumnCalculator()->mapHeaderChanged(args);
	if(args->getY() != Map::HEADER_EVENT_NONE)
		headerCalculators->GetRowCalculator()->mapHeaderChanged(args);
}

//void RomRep::Map::insertColumn(int column) //column inserted AFTER 'column'
//{
////	if(column > x-1 || column < 0)
////		return;
////
////	unsigned char tmpmap __gc[,]= map;
////
////	//create larger map
////	map = new unsigned char __gc[x+1,y];
////	//for(int i = 0; i < (x+1); i++)
////	//	map[i] = new int[y];
////
////	x++; //new size
////
////	//populate up to the new column, which gets 1s right now
////	for(int row = 0; row<y; row++)
////		for(int col = 0; col<=column; col++)
////		{
////			int val = tmpmap[col,row];
////
////			map[col,row] = val;
////			thefile[this->getCellLocation(col,row)] = val;
////		}
////
////	for(int row = 0; row<y; row++)
////	{
////		//interpolation goes here...
////		int val = 1;
//////		if(column+2 < x)
//////		{
//////			float left = this->calculate(column,row,map[column,row]); //left val
//////			float right = this->calculate(column+1,row,map[column+2,row]); //right val
//////
//////			val = this->calculate(column,row,(left+right)/2.0);
//////		
//////		}
//////		else
//////		{
////			val = map[column,row];
//////		}
////
////		map[column+1,row] = val;
////		thefile[this->getCellLocation(column+1,row)] = val;
////	}
////
////	//fill after the new column
////	for(int row = 0; row<y; row++)
////		for(int col = column+1; col<(x-1); col++)
////		{
////			map[col+1,row] = tmpmap[col,row];
////			thefile[this->getCellLocation(col+1,row)] = tmpmap[col,row];
////
////		}
////
////	//delete old map
////	//for(int i = 0; i<(x-1); i++)
////	//	delete[] tmpmap[i];
////	//delete[] tmpmap;
//}
//
//void RomRep::Map::insertRow(int row) //column inserted AFTER 'row'
//{
//	//if(row > y-1 || row < 0)
//	//	return;
//
//	//unsigned char tmpmap __gc[,]= map;
//
//	////create larger map
//	//map = new unsigned char __gc[x,y+1];
//	////for(int i = 0; i < x; i++)
//	////	map[i] = new int[y+1];
//
//	//y++; //new size
//
//	////populate up to the new row, which gets 1s right now
//	//for(int r = 0; r<=row; r++)
//	//	for(int col = 0; col<x; col++)
//	//	{
//	//		int val = tmpmap[col,r];
//
//	//		map[col,r] = val;
//	//		thefile[this->getCellLocation(col,r)] = val;
//	//	}
//
//	//for(int col = 0; col<x; col++)
//	//{
//	//	//interpolation goes here...
//	//	int val = 1;
//	//	if(row+2 < y)
//	//	{
//	//		val = (map[col,row] + map[col,row+2])/2; //avg
//	//	
//	//	}
//	//	else
//	//	{
//	//		val = map[col,row];
//	//	}
//
//	//	map[col,row+1] = val;
//	//	thefile[this->getCellLocation(col,row+1)] = val;
//	//}
//
//	////fill after the new row
//	//for(int r = row+1; r<(y-1); r++)
//	//	for(int col = 0; col<x; col++)
//	//	{
//	//		map[col,r+1] = tmpmap[col,r];
//	//		thefile[this->getCellLocation(col,r+1)] = tmpmap[col,r];
//
//	//	}
//
//	////delete old map
//	////for(int i = 0; i<x; i++)
//	////	delete[] tmpmap[i];
//	////delete[] tmpmap;
//}
