#include "stdafx.h"
#include "Engine.h"

RomRep::Engine::Engine(){}

int* RomRep::Engine::getFile(const char* fname, int size)
{
	FILE* bin;
	int* temp = new int[size];
	int i, j;
	i = 0;

	cout<<size<<endl;

	bin = fopen(fname,"rb");
	if (!(bin))
		return NULL;

	while (!feof(bin) && i < size)
	{
		j = fgetc(bin);
		temp[i] = j;
		i++;
	}

	fclose(bin);

	return temp; //returns pointer to array of size 'size' 
}


bool RomRep::Engine::doesFileExist(const char* fname) //can call before getRom()
{
	ifstream fin(fname);
	
	if(fin.is_open())
		return true;
	else
		return false;

	fin.close();
}

//identifies what kind of ECU the file 'fname' is. 
//Returns an object of this ECU type.
RomRep::HondaEcu* RomRep::Engine::getRom(const char* fname, int size)
{
	int* fil = Engine::getFile(fname, size);

	if(fil == NULL)
		return NULL;

	HondaEcu* rom = NULL;

	int* head = new int[HondaEcu::headlen];

	for(int i = 0; i<HondaEcu::headlen; i++)
		head[i] = fil[i];

	//go through and figure out which kind of ECU the file is from.
	if(JdmPw0Dev::validate(fil))
		rom = new JdmPw0Dev(fil, size);
	else if(JdmPw0::validate(head))
		rom = new JdmPw0(fil, size);
	else if(Obd0Pr3::validate(head))
		rom = new Obd0Pr3(fil, size);
	else if(JdmPr3Dev::validate(fil))
		rom = new JdmPr3Dev(fil, size);
	else if(JdmPr3ITB::validate(fil))
		rom = new JdmPr3ITB(fil, size);
	else if(JdmPr3::validate(head))
		rom = new JdmPr3(fil, size);
	else if(EuroPw0Dev::validate(fil))
		rom = new EuroPw0Dev(fil, size);
	else if(EuroPw0::validate(head))
		rom = new EuroPw0(fil, size);
	else if(GhdPm6::validate(head))
		rom = new GhdPm6(fil, size);
	else if(Pm6::validate(head))
		rom = new Pm6(fil, size);
//	else if(JdmPw0Boost::validate(head))
//		rom = new JdmPw0Boost(fil, size);

	//only delete head cause the file gets used by the rom and 
	//the rom pointer gets returned
	delete[] head;

//	if(rom == NULL)
//		cout<< "File Not Recognized"<<endl;

	return rom;
}

bool RomRep::Engine::saveFile(int* rom, const char* fname, int size)
{
	int i;
	FILE* bin;
	bin = fopen(fname,"wb+");
	if (!(bin))
		return false;

	for(i=0;i<size;i++)
		fputc(rom[i],bin);
	fclose(bin);

	/*
	ofstream fout(fname);
	char c;

	if(!fout.is_open())
		return false;

	for(int i = 0; i<size; i++)
	{
		c = (char)rom[i];
		fout.write(&c,1);
	}

	fout.flush();
	fout.close();
*/
	return true;
}


//utils
void RomRep::Engine::appendStr(string *s, string s2)
{
	const char* c = s2.c_str();
	for(int i = 0; i<s2.length(); i++)
		s->insert(s->end(),c[i]);
}

string RomRep::Engine::toStr(int val)
{
	int num = val;
	if(num<0)
		num *= -1;

	string s = "";

	for(int i = 10; ;i*=10)
	{
		int fig = num%10;
		s.insert(s.begin(),fig+48);

		if(val< i)
			break;

		num/=10;
	}

//	num/=10;
//	fig = num%10;
//	s.insert(s.begin(),fig+48);

	if(val<0)
		s.insert(s.begin(),'-');

	return s;
}

string RomRep::Engine::toStr(double val)
{
	int num = (int)val;

	string s = toStr(num);

	s.insert(s.end(),'.');

	int dec = (int)((val-(double)num)*100.0);
	string ext = Engine::toStr((int)dec);
//	s.insert(s.end(),ext.c_str());
	s.append(ext);


	return s;
}

char * RomRep::Engine::intToBin(int num)
{
	int val = num;
	string result = "";

	if(num <= 0 || num>255)
	{
		result = "00000000";
		return (char*)result.c_str();
	}

	while(val > 0)
	{
		int mod = val%2;
		if(mod == 0)
			result.insert(result.begin(),'0');
		if(mod ==1)
			result.insert(result.begin(),'1');
		val/=2;
	}

	val = result.length();
	val = 8 - val; //num of zeros

	for(int i = 0; i<val; i++)
		result.insert(result.begin(),'0');

	return (char*)result.c_str();
}

//*********************************************************************************
int RomRep::Engine::psiToRaw(double psi, int sensor)
{
	
	int ret = 0;
	if(sensor == 1)//2 bar
	{
//		double xovervolts = 2.5;
		double volts = (psi*PSICONST2BAR)+XOVER2BAR;
		ret = (int)(volts*255.0/5.0);
	}
	if(sensor == 2)//2.5 bar
	{
//		double xovervolts = 2;
		double volts = (psi*PSICONST25BAR)+XOVER25BAR;
		ret = (int)(volts*255.0/5.0);
	}
	if(sensor == 4)//2.5 bar
	{
//		double xovervolts = 2;
		double volts = (psi*PSICONSTMOTOROLA25BAR)+XOVERMOTOROLA25BAR;
		ret = (int)(volts*255.0/5.0);
	}
	if(sensor == 3)//3 bar
	{
//		double xovervolts = 1.6;
		double volts = (psi*PSICONST3BAR)+XOVER3BAR;
		ret = (int)(volts*255.0/5.0);
	}
	else
	{
//		double xovervolts = 3.0;
		double volts = (psi*PSICONSTSTOCK)+XOVERSTOCK;
		ret = (int)(volts*255.0/5.0);
	}

	if(ret >255)
		ret = 255;
	if(ret<0)
		ret = 0;

	return ret;
}
int RomRep::Engine:: inHgToRaw(int inhg, int sensor)
{
		int ret = 0;
	if(sensor == 1)//2 bar
	{
//		double xovervolts = 2.5;
		double volts = (inhg*INHGCONST2BAR)+XOVER2BAR;
		ret = (int)(volts*255.0/5.0);
	}
	if(sensor == 2)//2.5 bar
	{
//		double xovervolts = 2;
		double volts = (inhg*(INHGCONST25BAR))+XOVER25BAR;
		ret = (int)(volts*255.0/5.0);
	}
	if(sensor == 4)//2.5 bar
	{
//		double xovervolts = 2;
		double volts = (inhg*(INHGCONSTMOTOROLA25BAR))+XOVERMOTOROLA25BAR;
		ret = (int)(volts*255.0/5.0);
	}
	if(sensor == 3)//3 bar
	{
//		double xovervolts = 1.6;
		double volts = (inhg*(INHGCONST3BAR))+XOVER3BAR;
		ret = (int)(volts*255.0/5.0);
	}
	else
	{
//		double xovervolts = 3.0;
		double volts = (inhg*(INHGCONSTSTOCK))+XOVERSTOCK;
		ret = (int)(volts*255.0/5.0);
	}

	if(ret >255)
		ret = 255;
	if(ret<0)
		ret = 0;

	return ret;
}

double RomRep::Engine::voltsToPsi(double volts, int sensor)
{
	if(sensor == 1)//2 bar
	{
//		double xovervolts = 2.5;
		return (volts - XOVER2BAR)/PSICONST2BAR;
	}
	if(sensor == 2)//2.5 bar
	{
//		double xovervolts = 2;
		return (volts - XOVER25BAR)/PSICONST25BAR;
	}
	if(sensor == 4)//2.5 bar
	{
//		double xovervolts = 2;
		return (volts - XOVERMOTOROLA25BAR)/PSICONSTMOTOROLA25BAR;
	}
	if(sensor == 3)//3 bar
	{
//		double xovervolts = 1.6;
		return (volts - XOVER3BAR)/PSICONST3BAR;
	}
	else
	{
//		double xovervolts = 3.0;
		return (volts - XOVERSTOCK)/PSICONSTSTOCK;
	}
	
}

int RomRep::Engine::voltsToInHg(double volts, int sensor)
{
	if(sensor == 1)//2 bar
	{
//		double xovervolts = 2.5;
		return (int)((volts - XOVER2BAR)/(INHGCONST2BAR));
	}
	if(sensor == 2)//2.5 bar
	{
		double xovervolts = 2;
		return (int)((volts - XOVER25BAR)/(INHGCONST25BAR));
	}
	if(sensor == 4)//2.5 bar
	{
		double xovervolts = 2;
		return (int)((volts - XOVERMOTOROLA25BAR)/(INHGCONSTMOTOROLA25BAR));
	}
	if(sensor == 3)//3 bar
	{
		double xovervolts = 1.6;
		return (int)((volts - XOVER3BAR)/(INHGCONST3BAR));
	}
	else
	{
		double xovervolts = 3.0;
		return (int)((volts - XOVERSTOCK)/(INHGCONSTSTOCK));
	}
}
double RomRep::Engine::rawToVolts(int raw)
{
	if(raw<0)
		raw = 0;
	if(raw>255)
		raw = 255;

	return ((double)raw*5.0)/255.0;
}

//this is for interp based o2 conversion 
double RomRep::Engine::rawToAfr(int raw, double volts[], double afr[])
{
	double key = Engine::rawToVolts(raw);		
			
			
	//vcal_0
	int i = 0;
	for(i = 0; key<volts[i]; i++);
			
	// now, volts[i-1] > key > volts[i]; so we interpolate between afr[i-1] and afr[i].

	bool negative = false;
	double r0 = key-volts[i];
	double r1 = volts[i-1]-volts[i];
	double A = afr[i]-afr[i-1];
	
	if(A<0)
	{
//		A = A*-1;
		negative = true;
	}

	A=A*r0/r1;

	if(negative)
		A = afr[i]-A;
	else
		A = afr[i]+A;

	return A;
}