#include "stdafx.h"
#include "EuroPw0Dev.h"

RomRep::EuroPw0Dev::EuroPw0Dev(): EuroPw0(),DevUtil(){}

RomRep::EuroPw0Dev::EuroPw0Dev(int* rom, int flen): EuroPw0(rom, flen, 0x3928, 0x3930, 0x3AEC, 4,
		DevUtil::igmap, DevUtil::igmapvtec,DevUtil::fmap, DevUtil::fmapvtec, "Euro Pw0 dev",DevUtil::getRomWidth(rom)),
		DevUtil(rom,0x4d5,colheaders,rowheaders)
{

	y = DevUtil::getRomHeight(rom);
	mapscalarloc = 0x63d2;
	widthloc = 0x7ffc;
	heightloc = 0x7ffd;
	tablesizeloc = 0x7ffe;

	//DevUtil::revscalar_ig = 0x648c;
	//DevUtil::revscalar_igv = 0x64e9;
	//DevUtil::revscalar_fu = 0x6546;
	//DevUtil::revscalar_fuv = 0x65a3;

	calcHeaders(getMaxMapVal());
	
}

void RomRep::EuroPw0Dev::calcHeaders(int maxmap)
{
	//inhg = (y - 2.85v) / (-.1)

	//column headers...
	int natoboost = (int)(DevUtil::getXOver(DevUtil::getMapSensor(thefile))*255.0/5.0);// = 146; 
	double xovervolts = 3.0; // = 2.85;

	double volts = Engine::rawToVolts(maxmap);
	if(maxmap<natoboost)//calc inches of mercury. vacuum, y0
	{
		
		int inhg = Engine::voltsToInHg(volts,DevUtil::getMapSensor(thefile));//(int)((volts - xovervolts) / (-.1));
		string s = Engine::toStr(inhg);
		colheaders[0][x-1] = colheaders[1][x-1] = s;
	}
	else //boost in psi
	{
		//2.85 volts with key on engine off and 4.5xx volts at 10.6 psi
		//psi = (v - 2.85)/.1557;
		double psi = Engine::voltsToPsi(volts,DevUtil::getMapSensor(thefile));//(volts - xovervolts)/.1557;
		string s = Engine::toStr(psi);
		string p("psi");
		Engine::appendStr(&s,p);
//		s.insert(s.end(),"psi");
		colheaders[0][x-1] = colheaders[1][x-1] = s;
	}

	for(int i = x-2; i>=0; i--)
	{
		int sptr = mapscalarloc + 6 + (x-2-i)*3;
		int mapval = thefile[sptr];

		double volts = Engine::rawToVolts(mapval);
		if(mapval<natoboost)//calc inches of mercury. vacuum, y0
		{
			int inhg = Engine::voltsToInHg(volts,DevUtil::getMapSensor(thefile));//(int)((volts - xovervolts) / (-.1));
			string s = Engine::toStr(inhg);
			colheaders[0][i] = colheaders[1][i] = s;
		}
		else //boost in psi
		{
			//2.85 volts with key on engine off and 4.5xx volts at 10.6 psi
			//psi = (v - 2.85)/.1557;
			double psi = Engine::voltsToPsi(volts,DevUtil::getMapSensor(thefile));//(volts - xovervolts)/.1557;
			string s = Engine::toStr(psi);
//			s.insert(s.end(),"psi");
			colheaders[0][i] = colheaders[1][i] = s;
		}
	}


	//row headers...
	for(int i = y-1; i>=0; i--)
	{

		int sptr = DevUtil::revscalar_ig + (y-1-i)*2;
		int sum = thefile[sptr];
				
		rowheaders[0][i] = calcNonVtecRow(sum);

		sptr = DevUtil::revscalar_fu + (y-1-i)*2;
		sum = thefile[sptr];
				
		rowheaders[2][i] = calcNonVtecRow(sum);

		//vtec headers
		sptr = DevUtil::revscalar_igv + (y-1-i)*2;
		sum = thefile[sptr];
			
		rowheaders[1][i] = calcVtecRow(sum);

		sptr = DevUtil::revscalar_fuv + (y-1-i)*2;
		sum = thefile[sptr];
			
		rowheaders[3][i] = calcVtecRow(sum);
	}//end row header for loop

}


void RomRep::EuroPw0Dev::insertColumn(int column)
{
	HondaEcu::insertColumn(column); //x has been incremented...

	this->setByte(widthloc,x);
	this->setByte(tablesizeloc,(x*y)%256);
	this->setByte(tablesizeloc+1,(x*y)/256);

	//fix the column scalars
	

	int* oldscalars = this->getArray(thefile,mapscalarloc,x*3);

    int scalarptr = ((x-1)*3)-(column*3);//this is pointing at the map value of the column in the array

	//adjusts the scalars for the columns above the inserted column 
	//and sets the new column's ivalue
	for(int i = 0; i<=scalarptr; i+=3)
	{
		int ivalue = oldscalars[i+1] + oldscalars[i+2]*256;

		ivalue+=0x10;

		this->setByte(mapscalarloc+i+1,ivalue%256);
		this->setByte(mapscalarloc+i+2,ivalue/256);

	}

	//set the new column's map value
	int ivalue = (oldscalars[scalarptr] + oldscalars[scalarptr-3])/2;

	this->setByte(mapscalarloc+scalarptr,ivalue);

	//sets the rest of the values...
	for(int i = scalarptr; i<x*3; i++)
	{
		this->setByte(mapscalarloc+i+3,oldscalars[i]);
	}

	//setup the new headers...
	string ** oldheaders = colheaders;
	colheaders = new string*[2];

	colheaders[0] = new string[x];
	colheaders[1] = new string[x];

	calcHeaders(getMaxMapVal());

	delete[] oldscalars;
	
}

///static stuff........
bool RomRep::EuroPw0Dev::validate(int* h)
{
	bool good = true;
	int head[10] = {0x73,0x16,0x9A,0x16,0x96,0x16,0x8F,0x00,0x1F,0x15};

	for(int i = 0; i<HondaEcu::headlen; i++)
		if(head[i] != h[i])
		{
			good = false;
		}

	int boost[5] = {0xCF,0x02,0x77,0xDF,0x01};

	for(int i = 0; i<4; i++)
		if(boost[i] != h[0x33d0 + i])
			good = false;

	return good;
}

//int RomRep::EuroPw0Dev::getRomWidth(int* file)
//{
//	return file[0x7ffc];
//
//}
//int RomRep::EuroPw0Dev::getRomHeight(int* file)
//{
//	return file[0x7ffd];
//}
//int RomRep::EuroPw0Dev::getMaxMapVal(int* file)
//{
//	return file[0x4d5];
//}