#pragma unmanaged

#if !defined BENS_OBD0VTECECU_H
#define BENS_OBD0VTECECU_H

#include "HondaEcu.h"
#include "RpmValue.h"
#include "Obd0IgnMap.h"
#include "Obd0FuelMap.h"

namespace RomRep
{
	class VT0Ecu: public HondaEcu
	{
	protected:
		//revlimit, restart, idle, vtec engage, vtec disengage 

		int revconst;
		RpmByteValue revlimit, revrestart;
		RpmByteValue* vtec;
		int numvtec;
		int celvecloc;

		int revscalar_ig;
		int revscalar_igv;
		int revscalar_fu;
		int revscalar_fuv;


		//need an idle array too
		int idlescalar_loc;		

		virtual string calcNonVtecRow(int val);
		virtual string calcVtecRow(int val);

		virtual void setupMaps(int* ignmapad, int* fuelmapad);
		
		virtual void setupHeaders();

		
//		these return ints because bools didnt work for some reason.
		//for RTP thsese need to return a ByteChange ptr
		virtual void aDisableOxygenSensors(int loc, bool disable);
		virtual void aDisable2ndOxygenSensor(int loc, bool disable);

		virtual void aDisableChecksum(int loc, bool disable); 
		virtual int aisChecksumDisabled(int loc);
		virtual int aisOxygenSensorsDisabled(int loc);
		virtual int ais2ndOxygenSensorDisabled(int loc);
		virtual int aisKnockSensorDisabled(int loc);
		virtual void adisableKnockSensor(int loc1,int loc2, int loc3, bool disable);
		virtual void asetLaunchRpm(int loc, int val);
		virtual void asetLaunchSpeed(int loc, int val);
		virtual void adisableCelCheck(int loc, bool disable);

		virtual int aisSpeedLimiterDisabled(int loc);
		virtual int aisVssCheckDisabled(int loc);
		virtual int aisTempCheckDisabled(int loc);
		virtual int aisCelCheckDisabled(int loc);
		virtual int aisPsCheckDisabled(int loc);
		virtual int aisLaunchDisabled(int loc);

		virtual int agetLaunchRpm(int loc);
		virtual int agetLaunchSpeed(int loc);

		virtual void aDisableCodes(int loc, bool disable);
		virtual int aisCodesDisabled(int loc);

	public:
		VT0Ecu();

		//the file, location, location, location of first one, number of vtec values.
		VT0Ecu(int* rom, int flen, int revloc, int restartloc, int vtecloc, int vtecnum, int ignmapad1, int ignmapad2, int fuelmapad1, int fuelmapad2, string type, int width);

		virtual ~VT0Ecu();

		//for RTP thes return bytechange ptr
		//come in and leave as real world values
		virtual void setVtecEngage(int loc, int val);
		virtual void setVtecDisengage(int loc, int val);
		virtual void setLaunchRpm(int val) = 0;
		virtual void setRevlimit(int val);
		virtual void setRevRestart(int val);
		virtual void disableOxygenSensors(bool disable) = 0;
		virtual void disable2ndOxygenSensor(bool disable) = 0;
		virtual void setLaunchSpeed(int val) = 0;
		virtual void enableLaunch(bool enable) = 0;
		virtual void disableChecksum(bool disable) = 0; 
		virtual void disableSpeedLimiter(bool disable) = 0; 
		virtual void disableVssCheck(bool disable) = 0; 
		virtual void disableTempCheck(bool disable) = 0; 
		virtual void disablePsCheck(bool disable) = 0; 
		virtual void disableCelCheck(bool disable) = 0; 
		virtual void disableKnockSensor(bool disable) = 0;
		virtual void disableCodes(bool disable) = 0;

		//return rpm
		virtual int getVtecEngage(int loc);
		virtual int getVtecDisengage(int loc);
		virtual int getRevlimit();
		virtual int getRevRestart();
		virtual int getLaunchRpm() = 0;
		virtual int getLaunchSpeed() = 0;

		//bools dont propigate well, so 1==true 0==false
		virtual int isKnockSensorDisabled() = 0;
		virtual int isChecksumDisabled() = 0;
		virtual int isOxygenSensorsDisabled() = 0;
		virtual int is2ndOxygenSensorDisabled() = 0;
		virtual int isSpeedLimiterDisabled() = 0;
		virtual int isVssCheckDisabled() = 0;
		virtual int isTempCheckDisabled() = 0;
		virtual int isCelCheckDisabled() = 0;
		virtual int isPsCheckDisabled() = 0;
		virtual int isLaunchDisabled() = 0;

		//returns value in the rpm scalar
		virtual int getRpmScalarValue(int row, bool fuel, bool vtec); 

		//returns 1 if the code for vector disables is in the rom
		virtual int isCodesDisabled() = 0;

		//return negative for fail, 1 for success
		virtual int addCelDisable(int code); //return negative for fail, 1 for success
//		virtual int* getCelDisableVec();

		//return negative for fail, 1 for success
		virtual int removeCelDisable(int code); //return negative for fail, 1 for success

		//gets the location of the disabled vector
		virtual int getCelVectorLocation()
		{return celvecloc;};

		//is the input code a bit disable? If not then its a vector disable
		virtual int isCodeBitDisable(int code) = 0;

		//disable the damn code via bit disable. true disables the code
		virtual int codeBitDisable(int code, bool disable) = 0;

		//is this code bit disabled? 1 if yes, 0 if no, -1 if code is vec disable 
		virtual int isCodeBitDisabled(int code) = 0;


		virtual double** getLambdaMap(int index);
		virtual ByteChange* setAfrCellEx(int x, int y, double val, int index);

	};
}

//	ostream& operator<<(ostream& out, VT0Ecu& ecu);

#endif