//This adds a Datlogger definition definition to BRE

using System;
using System.Windows.Forms;
using System.Collections;
using System.Drawing;

using bre20;
//css_reference ..\bre.exe;
//css_reference ..\Gauge.dll;

namespace BensScripts
{
	public class NGLogger
	{
		child romWind;
		public static object AppliesTo(){return "NG";}
		public static object ScriptTitle(){return "NG Logger Defn by Ben Ogle";}
		public object ScriptMain(params object[] args)
		{
			romWind = (child)args[1];
			//subscribe to the event
			romWind.subscribeToEvent("OnDataloggerOpened", "BensScripts.NGLogger.log_Opened");
			return null;
		}
		public object log_Opened(params object[] args)
		{	//when the logger is opened, then we add our rule
			romWind.addLoggerRule(new NGLoggerDef(romWind.getRom()));
			return null;
		}
	}//end class

	public class NGLoggerDef: Rule
	{
		Logger log;
		bool sae,oldsae;

		int mapsensor; //must figure this out...

	//	RomRep.HondaEcu ecu;

		//save a reference to our keys
		LoggerKey rpm, cell, boost, vac, tps, speed;
		Gauge.Gauge grpm,gboost,gvac,gtps,gspeed;

		//need type, mapsensor type,
		public NGLoggerDef(RomRep.HondaEcu rom)
		{
		//	ecu = rom;
		//	dev = Common.isMatch("dev",rom.getType(),false);
			mapsensor = RomRep.Pm6Ng.getMapSensor(rom.getFile());
		}
		public override void setupRule(Form frm)
		{
			log = ((Logger)frm);

			sae = oldsae = log.isSae();

			log.setParity(0);
			log.setBaud(38400);
			log.setSendMode(Logger.SENDMODE_BYTE);

			//here we set up the rpm LoggerKey
			rpm = new LoggerKey("Rpm",true,true);//
			cell = new LoggerKey("Cell",false,true);//
			boost = new LoggerKey("Boost",true,true);
			vac = new LoggerKey("Vacuum",true,true);
			tps = new LoggerKey("TPS",true,true);//
			speed = new LoggerKey("Speed",true,true);//

			int maxboost = 20;
			if(mapsensor > 1)
				maxboost = 30;

			string tpsunit = "KPH";
			if(sae)
				tpsunit = "MPH";

			gboost = log.addGauge("Boost","psi",120,296,0,0,maxboost,10,0);
			gvac = log.addGauge("Vacuum","InHg",120,176,0,0,30,10,0);
			gtps = log.addGauge("TPS","%",120,56,0,0,100,10,0);
			gspeed = log.addGauge("Speed", tpsunit,200,240,120,0,160,8,0);
			grpm = log.addGauge("Rpm","x1000",200,32,120,0,8,8,0);

			//add it to the logger
			log.getLoggerData().addLoggerKey(rpm);
			log.getLoggerData().addLoggerKey(cell);
			log.getLoggerData().addLoggerKey(boost);
			log.getLoggerData().addLoggerKey(vac);
			log.getLoggerData().addLoggerKey(tps);
			log.getLoggerData().addLoggerKey(speed);
		}
		public override bool executeRule(Form frm)
		{	
			//get the results values from the hash table
			//the names/labels/keys are from loggerCmd.ini
			Hashtable hash = log.getLoggerData().getHashtable();
			float fval;
			string str;
			bool vtech = false;
 
			//RPM
			int val = ((Integer)hash["rpm low"]).Value;
			int val2= ((Integer)hash["rpm high"]).Value;
			val = RomRep.Pm6Ng.getRevConstant()/(val + 256*(val2));
			//set the value for the output
			grpm.CurrentValue = (float)val/1000.0F;
			rpm.Value = val.ToString();

			//TPS
			val = ((Integer)hash["tps"]).Value;
			val = val-50;
			if(val<0) val = 0;
			fval = ((float)val/2.0F);
			gtps.CurrentValue = fval;
			tps.Value = fval.ToString("0.00")+"%";

			//index
			int x = ((Integer)hash["x"]).Value;
			int index = 0;
			if(x > 14)
			{
				index = 1;
				x-=15;
			}
			log.setMapIndex(index);

			//SPEED
			val = ((Integer)hash["speed"]).Value;
			fval = 0.6436F; //metric
			str = "KPH";
			if(sae)
			{
				fval = .4F;
				str = "MPH";
			}
			val = (int)((float)val*fval);
			gspeed.CurrentValue = (float)val;
			speed.Value = val.ToString() + str;

			//FUEL CELL

			val = ((Integer)hash["rpm image"]).Value;
			val2 = ((Integer)hash["map image"]).Value;

			int rinterp = val%16;
			int cinterp = val2%16;

			val = val/16;
			//	val2 = val2/16;

			cell.Value = val2.ToString()+":"+ (val2/16).ToString() +":"+x.ToString()+ "," + val.ToString();
			log.setCurrentColumn(x);
			log.setCurrentFuelRow(val);
			log.setCurrentIgnitionRow(val);

			setWeights(hash, val2, cinterp, val, rinterp);

			//BOOST
			val = ((Integer)hash["map"]).Value;
			fval = RomRep.Engine.voltsToPsi(RomRep.Engine.rawToVolts(val),mapsensor);
			if(fval <0)
				fval = 0.0F;
			gboost.CurrentValue = fval;
			boost.Value = fval.ToString("0.00") + "psi";

			//VACUUM
			val2 = RomRep.Engine.voltsToInHg(RomRep.Engine.rawToVolts(val),mapsensor);
			if(fval>0)
				val2 = 0;
			gvac.CurrentValue = (float)val2;
			vac.Value = val2.ToString() + "inhg";

			return true;
		}
		public override bool updateRule(Form frm)
		{
			oldsae = sae;
			sae = log.isSae();
			if(oldsae != sae)
			{
				if(sae) makeSae();
				else makeMetric();
			}
			return true;
		}
		private void makeMetric()
		{
			gspeed.Units = "KPH";
		}
		private void makeSae()
		{
			gspeed.Units = "MPH";
		}
		//weights for the fuel maps...
		private void setWeights(Hashtable hash, int col, int colinterp, int row, int rowinterp)
		{
			//compute avg
			//need to take interp into account...
			float curcolpercent = (float)(15-colinterp)/15.0F;
			float nextcolpercent = (float)(colinterp)/15.0F;

			float currowpercent = (float)(15-rowinterp)/15.0F;
			float nextrowpercent = (float)(rowinterp)/15.0F;

			if(currowpercent > 1.0F)
				currowpercent = 1.0F;
			if(nextrowpercent > 1.0F)
				nextrowpercent = 1.0F;

			log.setCellWeight(0,0,curcolpercent * currowpercent);
			log.setCellWeight(1,0,nextcolpercent * currowpercent);
			log.setCellWeight(1,1,nextcolpercent * nextrowpercent);
			log.setCellWeight(0,1,curcolpercent * nextrowpercent);
		}

		public override bool executeReplay(string[] keys, Hashtable values)
		{
			//could update the gauges here as well... Oh well. me == lazy
			cell.Value = (string)values["x"] + ", " + (string)values["fuely"];
			return true;
		}
		//this is what we are modifying
		public override string modifiedFields(){return "NG base";}
	}
}//end namespace